@@ -1,11 +1,11 @@
 //
 // File:        BabelConfiguration.java
 // Package:     gov.llnl.babel
-// Revision:    @(#) $Revision: 6212 $
-// Date:        $Date: 2007-10-31 17:07:17 -0700 (Wed, 31 Oct 2007) $
+// Revision:    @(#) $Revision: 6494 $
+// Date:        $Date: 2008-09-04 16:24:45 -0700 (Thu, 04 Sep 2008) $
 // Description: Store configuration info shared by parsers and backend
 // 
-// Copyright (c) 2000-2004, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2007, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -71,14 +71,10 @@ public class BabelConfiguration implements Cloneable
   public  static final String POST_EXCEPTION = "sidl.PostViolation";
   public  static final String INV_EXCEPTION  = "sidl.InvViolation";
 
+  public  static final String CONTRACT_ENFORCER = "sidl.Enforcer";
+  public  static final String CONTRACT_POLICY   = "sidl.EnfPolicy";
 
   public static final String OPTIONAL_RMI = "WITH_RMI";
-  /*
-   * Overall level of assertion statistics support.
-   */
-  public  static final String NO_CHECKING         = "0";
-  public  static final String SUPPORT_CHECKING    = "1";
-  public  static final String ADVANCED_CHECKING   = "2";
 
   /**
    * Static options -- not dependent on command line settings
@@ -92,7 +88,7 @@ public class BabelConfiguration implements Cloneable
    * User options -- extractable from the command line via UserOptions.
    * Each of these must be copied in clone().
    */
-  private String  d_assertion_level        = SUPPORT_CHECKING;
+  private boolean d_suppress_contracts     = false;
   private boolean d_generate_hooks         = false;
   private boolean d_package_subdirs        = false;
   private boolean d_language_subdir        = false;
@@ -112,6 +108,7 @@ public class BabelConfiguration implements Cloneable
   private boolean d_rename_splicers        = false;
   private boolean d_cca_mode               = false; 
   private boolean d_protect_last_time_mod  = true;
+  private boolean d_genmakefile            = false;
 
   /*
    * State variables that are deliberately not copied during clone.
@@ -161,9 +158,10 @@ public class BabelConfiguration implements Cloneable
    * Make a copy of the configuration.
    */
   public Object clone()
+    throws CloneNotSupportedException
   {
-    BabelConfiguration result = new BabelConfiguration();
-    result.d_assertion_level = this.d_assertion_level;
+    BabelConfiguration result = (BabelConfiguration)super.clone();
+    result.d_suppress_contracts = this.d_suppress_contracts;
     result.d_generate_hooks = this.d_generate_hooks;
     result.d_package_subdirs = this.d_package_subdirs;
     result.d_language_subdir = this.d_language_subdir;
@@ -181,6 +179,9 @@ public class BabelConfiguration implements Cloneable
     result.d_suppress_ior = this.d_suppress_ior;
     result.d_suppress_stub = this.d_suppress_stub;
     result.d_rename_splicers = this.d_rename_splicers;
+    result.d_cca_mode = this.d_cca_mode;
+    result.d_protect_last_time_mod = this.d_protect_last_time_mod;
+    result.d_genmakefile = this.d_genmakefile;
     return result;
   }
 
@@ -265,6 +266,13 @@ public class BabelConfiguration implements Cloneable
     return OPTIONAL_RMI;
   }
 
+  /**
+   * The full name of the most basic exception type.
+   */
+  public static String getFundamentalException() { 
+    return FUND_EXCEPTION;
+  }
+
   /**
    * The full name of the exception type implicit in all methods.
    */
@@ -272,6 +280,30 @@ public class BabelConfiguration implements Cloneable
     return RUNTIME_EXCEPTION;
   }
 
+  /**
+   * The full name of the Invariant Violation type implicit in all 
+   * methods of classes with invariants.
+   */
+  public static String getInvariantViolation() { 
+    return INV_EXCEPTION;
+  }
+
+  /**
+   * The full name of the Precondition Violation type implicit in all 
+   * methods with preconditions.
+   */
+  public static String getPreconditionViolation() { 
+    return PRE_EXCEPTION;
+  }
+
+  /**
+   * The full name of the Postcondition Violation type implicit in all 
+   * methods with postconditions.
+   */
+  public static String getPostconditionViolation() { 
+    return POST_EXCEPTION;
+  }
+
   public static String getLangSpecific(){ return LANGSPECIFIC_EXCEPTION; }
   public static String getNotImplemented(){ return NOTIMPLEMENTED_EXCEPTION; }
   public static String getMemoryAllocationException(){ return MEMORYALLOCATION_EXCEPTION; }
@@ -306,6 +338,20 @@ public class BabelConfiguration implements Cloneable
     return d_makefile;
   }
 
+  /**
+   * Should a Makefile be generated.
+   */
+  public boolean getGenMakefile() {
+    return d_genmakefile;
+  }
+
+  /**
+   * Determine whether a Makefile should be generated.
+   */
+  void setGenMakefile(boolean value) {
+    d_genmakefile = value;
+  }
+
   /**
    * Return the maximum array dimension.
    */
@@ -398,24 +444,17 @@ public class BabelConfiguration implements Cloneable
   }
 
   /**
-   * Set the level of the assertion generation user option.
+   * Set contract suppression to <code>true</code> or <code>false</code>.
    */
-  public void setAssertionLevel(String lvl) {
-    d_assertion_level = lvl;
+  public void setSuppressContracts(boolean suppress) {
+    d_suppress_contracts = suppress;
   }
 
   /**
-   * Return the level of assertion checking.
+   * Return whether or not contract enforcement code should be generated.
    */
-  public String getAssertionLevel() {
-    return d_assertion_level;
-  }
-  
-  /**
-   * Return whether or not user has indicated assertions should be generated.
-   */
-  public boolean generateAssertions() {
-    return (!d_assertion_level.equals(NO_CHECKING));
+  public boolean generateContracts() {
+    return !d_suppress_contracts;
   }
   
   /**
