@@ -2,8 +2,8 @@
 // File:        Generator.java
 // Package:     gov.llnl.babel
 // Copyright:   (c) 2007 Lawrence Livermore National Security, LLC
-// Revision:    @(#) $Revision: 6212 $
-// Date:        $Date: 2007-10-31 17:07:17 -0700 (Wed, 31 Oct 2007) $
+// Revision:    @(#) $Revision: 6535 $
+// Date:        $Date: 2008-10-01 16:12:56 -0700 (Wed, 01 Oct 2008) $
 // Description: Manager parsing and generation of files
 // 
 
@@ -86,6 +86,7 @@ public class Generator {
       if (f.exists() && f.isFile() && f.canRead()) {
         d_astList.add(parseURL(f.toURL().toString()));
         d_context.getSymbolTable().setInputFromSIDL();
+        d_context.getSIDLFiles().add(url);
       } else if (f.exists() && f.isDirectory()) {
         resolveSymbolByString(url);
       } else if (url.indexOf(':') != -1) {
@@ -223,7 +224,7 @@ public class Generator {
         d_context.getSymbolTable().resolveSymbol("sidl");
         d_context.getSymbolTable().resolveAllReferences();
         d_context.getSymbolTable().resolveAllParents();
-        d_context.getSymbolTable().validateAssertions();
+        d_context.getSymbolTable().validateContracts();
       }
       catch (Exception ex) {
         d_msgs.addMsg(convertExceptionToMsg(ex));
