@@ -1,15 +1,14 @@
 //
 // File:        ResourceLoader.java
 // Package:     gov.llnl.babel
-// Copyright:   (c) 2003 The Regents of the University of California
-// Revision:    @(#) $Revision: 4434 $
-// Date:        $Date: 2005-03-17 09:05:29 -0800 (Thu, 17 Mar 2005) $
+// Copyright:   (c) 2003 Lawrence Livermore National Security, LLC
+// Revision:    @(#) $Revision: 6211 $
+// Date:        $Date: 2007-10-29 21:09:40 -0700 (Mon, 29 Oct 2007) $
 // Description: Resource loader that works with Kaffe or Sun's JDK
 // 
 
 package gov.llnl.babel;
 
-import gov.llnl.babel.Version;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
@@ -86,9 +85,11 @@ public class ResourceLoader {
     }
     if (jarFile != null) {
       JarEntry entry = jarFile.getJarEntry(name);
-      InputStream is = jarFile.getInputStream(entry);
-      if (is != null) {
-        return is;   
+      if (entry != null) {
+        InputStream is = jarFile.getInputStream(entry);
+        if (is != null) {
+          return is;   
+        }
       }
     }
     throw new IOException("Cannot get a ClassLoader to load " + name);
