@@ -1,10 +1,10 @@
 //
 // File:        UserOptions.java
 // Package:     gov.llnl.babel
-// Revision:     @(#) $Id: UserOptions.java 6212 2007-11-01 00:07:17Z epperly $
+// Revision:     @(#) $Id: UserOptions.java 6482 2008-08-21 22:50:53Z epperly $
 // Description: singleton Map for configuring general, user-related options
 // 
-// Copyright (c) 2000-2004, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2007, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -153,6 +153,21 @@ public class UserOptions implements CommandLineExtension
     }
   }
 
+  public class GenerateMakefileSwitch extends BasicSwitch {
+    public GenerateMakefileSwitch() {
+      super(0, "makefile",
+            "Generate a sample GNU Makefile");
+    }
+    final public void processCommandSwitch(String optarg)
+    {
+      if (d_context.getConfig().makeGlueSubdirs()) {
+        System.err.println
+          ("Babel: Warning: the --makefile option does not work correctly with the --hide-glue option.");
+      }
+      d_context.getConfig().setGenMakefile(true);
+    }
+  }
+
   public class GenerateSubdirSwitch extends BasicSwitch {
     public GenerateSubdirSwitch() { 
       super('g', "generate-subdirs",
@@ -184,8 +199,10 @@ public class UserOptions implements CommandLineExtension
     {
       d_context.getSymbolTable().setTargetIsXML(false);
       d_context.getConfig().setParseCheckOnly(true);
-      if ((d_gen == null) && (d_numRequired++ > 0)) {
-        throw new InvalidOptionException("Only one of the following options or its equivalent is allowed: --help, --version, --client, --server, --text or --parse-check unless you use the --multi option.");
+      if (d_gen == null) {
+        if (d_numRequired++ > 0) {
+          throw new InvalidOptionException("Only one of the following options or its equivalent is allowed: --help, --version, --client, --server, --text or --parse-check unless you use the --multi option.");
+        }
       }
       else {
         if (!d_gen.resolveSymbols()) {
@@ -216,6 +233,10 @@ public class UserOptions implements CommandLineExtension
             "Put \"glue\" (not modifiable by user) generate code in a glue/ subdirectory.");
     }
     final public void processCommandSwitch(String optarg) {
+      if (d_context.getConfig().getGenMakefile()) {
+        System.err.println
+          ("Babel: Warning: the --hide-glube option makes the Makefile generated by the --makefile option incorrect.");
+      }
       d_context.getConfig().setMakeGlueSubdirs(true);
     }
   }
@@ -240,7 +261,7 @@ public class UserOptions implements CommandLineExtension
     {
       if (d_gen == null ) ++d_numRequired;
       d_versionSet = true;
-      System.out.println( "Babel version " + Version.VERSION );
+      System.out.println( "Babel version " + Version.getFullVersion());
     }
   }
 
@@ -371,32 +392,15 @@ public class UserOptions implements CommandLineExtension
     public String getArgumentName() { return d_argName; }
   }
 
-  public class AssertionLevelSwitch extends RequiredArgSwitch
+  public class SuppressContractsSwitch extends BasicSwitch
   {
-    public AssertionLevelSwitch() {
-      super('a', "assertion-level", 
-            "Generate SIDL-specified assertion checking code. NOTE: 0=None, 1=Support, and 2=Advanced.", "lvl");
+    public SuppressContractsSwitch() {
+      super(0, "suppress-contracts", 
+            "Refrain from generating contract enforcement from SIDL specs.");
     }
 
-    /**
-     * Check if the specified assertion checking level is supported.
-     */
-    private boolean validAssertionLevel( String alvl ) { 
-      return  BabelConfiguration.NO_CHECKING.equals(alvl) ||
-        BabelConfiguration.SUPPORT_CHECKING.equals(alvl) ||
-        BabelConfiguration.ADVANCED_CHECKING.equals(alvl);
-    }
-
-    final public void processCommandSwitch(String optarg) 
-      throws InvalidArgumentException
-    {
-      if ((optarg != null) && validAssertionLevel(optarg)) {
-        d_context.getConfig().setAssertionLevel(optarg);
-      }
-      else {
-        throw new InvalidArgumentException
-          ("The assertion level must be 0, 1, or 2.");
-      }
+    final public void processCommandSwitch(String optarg) {
+      d_context.getConfig().setSuppressContracts(true);
     }
   }
 
