@@ -41,7 +41,8 @@ public class Name extends ASTNode {
   }
 
   public boolean equals(Object o) {
-    return ((Name)o).toString().equals(this.toString());
+      return ((o != null) && (o instanceof Name)) ?
+        (((Name)o).toString().equals(this.toString())) : false;
   }
 
   public int hashCode() {
