@@ -1,10 +1,10 @@
 //
 // File:        C.java
 // Package:     gov.llnl.babel.backend.c
-// Revision:    @(#) $Id: C.java 6196 2007-10-24 23:42:37Z epperly $
+// Revision:    @(#) $Id: C.java 6482 2008-08-21 22:50:53Z epperly $
 // Description: common C binding routines shared by C code generators
 //
-// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -198,6 +198,10 @@ public class C
     return id.getFullName().replace('.', '_');
   }
 
+  public static String getInlineDecl(SymbolID id) {
+    return getSymbolName(id).toUpperCase() + "_INLINE_DECL";
+  }
+
   /**
    * Convert a SIDL enumerated type into its symbol name, which is
    * "enum " followed by the symbol name followed by "__enum".
