@@ -1,10 +1,10 @@
 //
 // File:        ImplHeader.java
 // Package:     gov.llnl.babel.backend.c
-// Revision:    @(#) $Id: ImplHeader.java 6201 2007-10-26 06:15:35Z kumfert $
+// Revision:    @(#) $Id: ImplHeader.java 6585 2008-11-07 16:55:36Z epperly $
 // Description: generate C implementation header to a pretty writer stream
 //
-// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2007, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -30,14 +30,14 @@
 
 package gov.llnl.babel.backend.c;
 
-import gov.llnl.babel.Context;
+import gov.llnl.babel.backend.c.C;
 import gov.llnl.babel.backend.CodeConstants;
 import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.backend.CodeSplicer;
 import gov.llnl.babel.backend.IOR;
 import gov.llnl.babel.backend.Utilities;
-import gov.llnl.babel.backend.c.C;
 import gov.llnl.babel.backend.writers.LanguageWriterForC;
+import gov.llnl.babel.Context;
 import gov.llnl.babel.symbols.Argument;
 import gov.llnl.babel.symbols.Class;
 import gov.llnl.babel.symbols.Extendable;
@@ -143,7 +143,6 @@ public class ImplHeader {
     }
     i = null;
 
-    d_writer.println();
     d_splicer.splice(includes, d_writer, "include files");
     d_writer.println();
   }
@@ -336,7 +335,7 @@ public class ImplHeader {
     while (i.hasNext()) {
       Method m = (Method)i.next();
       if (!m.isAbstract()) {
-        if (IOR.supportHooks(cls, context)) {
+        if (IOR.generateHookMethods(cls, context)) {
           Method pre = m.spawnPreHook();
           Method post = m.spawnPostHook();
           writeMethodDecl(lw, false, pre, cls, context);
@@ -376,30 +375,32 @@ public class ImplHeader {
 
     for (Iterator i = fconnectSIDs.iterator(); i.hasNext(); ) {
       SymbolID l_id = (SymbolID) i.next();
-      lw.println("extern " + C.getSymbolObjectPtr(l_id) + " " +C.getImplFConnectName(cls,l_id) + 
-                     "(const char* url, sidl_bool ar, sidl_BaseInterface *_ex);");
+      lw.print("extern " + C.getSymbolObjectPtr(l_id) + " ");
+      lw.print(C.getImplFConnectName(cls, l_id) + "(");
+      lw.println("const char* url, sidl_bool ar, sidl_BaseInterface *_ex);");
     }
+    lw.printlnUnformatted("#endif /*WITH_RMI*/");
 
     for (Iterator i = serializeSIDs.iterator(); i.hasNext(); ) {
       SymbolID l_id = (SymbolID) i.next();
       lw.println("extern void");
-      lw.println(IOR.getSkelSerializationName(cls,l_id, true) +
-                 "(const " +
-                 IOR.getStructName(l_id) + " *strct, " +
-                 "struct sidl_rmi_Return__object *pipe, const char *name, sidl_bool copyArg, " +
-                 IOR.getExceptionFundamentalType() + "*exc);");
+      lw.print(IOR.getSkelSerializationName(cls, l_id, true) + "(");
+      lw.print("const " + IOR.getStructName(l_id) + " *strct, ");
+      lw.print("struct sidl_rmi_Return__object *pipe, ");
+      lw.print("const char *name, sidl_bool copyArg, ");
+      lw.println(IOR.getExceptionFundamentalType() + " *exc);");
     }
     for (Iterator i = deserializeSIDs.iterator(); i.hasNext(); ) {
       SymbolID l_id = (SymbolID) i.next();
       lw.println("extern void");
-      lw.println(IOR.getSkelSerializationName(cls,l_id, false) +
-                 "(" +
-                 IOR.getStructName(l_id) + " *strct, " +
-                 "struct sidl_rmi_Call__object *pipe, const char *name, sidl_bool copyArg, " +
-                 IOR.getExceptionFundamentalType() + "*exc);");
+      lw.print(IOR.getSkelSerializationName(cls, l_id, false) + "(");
+      lw.print(IOR.getStructName(l_id) + " *strct, ");
+      lw.print("struct sidl_rmi_Call__object *pipe, ");
+      lw.print("const char *name, ");
+      lw.print("sidl_bool copyArg, ");
+      lw.println(IOR.getExceptionFundamentalType() + " *exc);");
     }
 
-   lw.printlnUnformatted("#endif /*WITH_RMI*/");
 
   }
 
@@ -423,30 +424,26 @@ public class ImplHeader {
     if(cls.hasOverwrittenMethods()) {
       //function from the Impl file that sets the superEPV
       lw.println("extern void " + C.getObjectName(cls) + "__superEPV(");
-      lw.println("struct " + C.getObjectName(cls.getParentClass())
-        + "__epv*);");
+      lw.print("struct " + C.getObjectName(cls.getParentClass()));
+      lw.println("__epv*);");
       lw.println();
     }
     writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.LOAD, cls, context), 
                     cls, context);
-    writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.CONSTRUCTOR, cls, context), 
+    writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.CONSTRUCTOR, 
+                                                    cls, context), 
                     cls, context);
-    writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.CONSTRUCTOR2, cls, context), 
+    writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.CONSTRUCTOR2, 
+                                                    cls, context), 
                     cls, context);
-
-    writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.DESTRUCTOR, cls, context),  
+    writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.DESTRUCTOR, 
+                                                    cls, context),  
                     cls, context);
-    if (IOR.supportAssertions(cls, context)) {
-      if (cls.hasStaticMethod(true)) {
-        writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.CHECK_ERROR, cls, 
-                        context, true), cls, context);
-      }
-      writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.CHECK_ERROR, cls, context), 
-                      cls, context);
-    }
   }
 
-  public Collection getLocalReferences(Extendable ext) throws CodeGenerationException{
+  public Collection getLocalReferences(Extendable ext) 
+    throws CodeGenerationException
+  {
     Set refs = ext.getObjectDependencies();
     refs.add(ext); 
     return refs;
@@ -489,6 +486,7 @@ public class ImplHeader {
      final String fullname = cls.getFullName();
     Collection localRefs = getLocalReferences(cls);
     if (d_context.getConfig().getRenameSplicers()) {
+      d_splicer.renameSymbol(fullname + "._misc", fullname + "._hmisc");
       d_splicer.renameSymbol(fullname + "._includes", 
                              fullname + "._hincludes");
     }
