@@ -1,11 +1,11 @@
 //
 // File:        SkelSource.java
 // Package:     gov.llnl.babel.backend.c
-// Revision:    @(#) $Revision: 6212 $
-// Date:        $Date: 2007-10-31 17:07:17 -0700 (Wed, 31 Oct 2007) $
+// Revision:    @(#) $Revision: 6585 $
+// Date:        $Date: 2008-11-07 08:55:36 -0800 (Fri, 07 Nov 2008) $
 // Description: generate C skeleton based for a particular class/interface
 //
-// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2007, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -31,14 +31,14 @@
 
 package gov.llnl.babel.backend.c;
 
-import gov.llnl.babel.Context;
+import gov.llnl.babel.backend.c.C;
 import gov.llnl.babel.backend.CodeConstants;
 import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.backend.IOR;
 import gov.llnl.babel.backend.Utilities;
-import gov.llnl.babel.backend.c.C;
 import gov.llnl.babel.backend.writers.LanguageWriter;
 import gov.llnl.babel.backend.writers.LanguageWriterForC;
+import gov.llnl.babel.Context;
 import gov.llnl.babel.symbols.Argument;
 import gov.llnl.babel.symbols.AssertionExpression;
 import gov.llnl.babel.symbols.CExprString;
@@ -83,13 +83,12 @@ public class SkelSource {
     "sidl_row_major_order"
   };
 
-  private static final String s_self = Utilities.s_self;
-
-  private static final String s_epv = "epv";
-  private static final String s_pre_epv = "pre_epv";
-  private static final String s_post_epv = "post_epv";
-  private static final String s_sepv = "sepv";
-  private static final String s_pre_sepv = "pre_sepv";
+  private static final String s_self      = Utilities.s_self;
+  private static final String s_epv       = "epv";
+  private static final String s_pre_epv   = "pre_epv";
+  private static final String s_post_epv  = "post_epv";
+  private static final String s_sepv      = "sepv";
+  private static final String s_pre_sepv  = "pre_sepv";
   private static final String s_post_sepv = "post_sepv";
 
 
@@ -121,7 +120,7 @@ public class SkelSource {
                    context);
     }
 
-    ImplHeader.writeBuiltinDecls(writer,cls, context);
+    ImplHeader.writeBuiltinDecls(writer, cls, context);
     ImplHeader.writeMethodDecls(writer, cls, true, context);
     ImplHeader.writeMethodDecls(writer, cls, false, context);
 
@@ -133,7 +132,7 @@ public class SkelSource {
     }
     writer.openCxxExtern();
     writeCallLoad(cls,writer);
-    writeRMIAccessFunctions(cls,writer, context);
+    writeRMIAccessFunctions(cls, writer, context);
     
     writeGetDataPointer(cls, writer);
     writeSetDataPointer(cls, writer);
@@ -148,7 +147,7 @@ public class SkelSource {
     writer.generateInclude(IOR.getHeaderFile(cls), false);
     writer.generateInclude(C.getHeaderFile(cls), false);
     writer.printlnUnformatted("#include <stddef.h>");
-   if(cls.hasOverwrittenMethods()) {
+   if (cls.hasOverwrittenMethods()) {
       writer.generateInclude(IOR.getHeaderFile(cls), false);
       writer.printlnUnformatted("#ifdef SIDL_DYNAMIC_LIBRARY");
       writer.printlnUnformatted("#include <stdio.h>");
@@ -159,10 +158,10 @@ public class SkelSource {
       //function from the Impl file that sets the superEPV
       writer.println();
       writer.println("extern void " + C.getObjectName(cls) + "__superEPV(");
-      writer.println("struct " 
-       + C.getObjectName(cls.getParentClass()) + "__epv*);");
+      writer.print("struct " + C.getObjectName(cls.getParentClass()));
+      writer.println("__epv*);");
 
-      StubSource.generateGetExternals(cls,writer, context);
+      StubSource.generateGetExternals(cls, writer, context);
     }
     writer.println();
 
@@ -174,8 +173,9 @@ public class SkelSource {
       SymbolID l_id = (SymbolID) i.next();
       writer.generateInclude(C.getHeaderFile(l_id), true);
     }
+    writer.printlnUnformatted("#endif /* WITH_RMI */");
 
-    Set serializeSIDs = IOR.getStructSymbolIDs(cls, true);
+    Set serializeSIDs   = IOR.getStructSymbolIDs(cls, true);
     Set deserializeSIDs = IOR.getStructSymbolIDs(cls, false);
 
     if (!serializeSIDs.isEmpty()) {
@@ -199,7 +199,6 @@ public class SkelSource {
       writer.generateInclude(C.getHeaderFile(l_id), true);
       writer.println();
     }
-    writer.printlnUnformatted("#endif /* WITH_RMI */");
   }
 
   /**
@@ -219,16 +218,18 @@ public class SkelSource {
                                        Context context) 
     throws CodeGenerationException
   {
-    Iterator i = cls.getMethods(false).iterator();
-    while (i.hasNext()){
+    Iterator i              = cls.getMethods(false).iterator();
+    boolean  genHookMethods = IOR.generateHookMethods(cls, context);
+
+    while (i.hasNext()) {
       Method m = (Method)i.next();
       if (C.methodNeedsSkel(m)) {
-        if (IOR.supportHooks(cls, context)) {
-          Method pre = m.spawnPreHook();
+        if (genHookMethods) {
+          Method pre  = m.spawnPreHook();
           Method post = m.spawnPostHook();
-          writeSkelMethod(cls, pre, writer, context);
-          writeSkelMethod(cls, m,  writer, context);
-          writeSkelMethod(cls, post,writer, context);
+          writeSkelMethod(cls, pre,  writer, context);
+          writeSkelMethod(cls, m,    writer, context);
+          writeSkelMethod(cls, post, writer, context);
         } else {
           writeSkelMethod(cls, m, writer, context);
         }
@@ -270,9 +271,8 @@ public class SkelSource {
     
     writer.println(method_name + "(");
     writer.tab();
-    IOR.generateArgumentList(writer, context,
-                             s_self, false, id, m, true, true, false, 
-                             true, false, false, false);
+    IOR.generateArgumentList(writer, context, s_self, false, id, m, true, 
+                             true, false, true, false, false, false);
     writer.backTab();
     writer.println(")");
 
@@ -283,76 +283,76 @@ public class SkelSource {
       writer.println(getReturnString(methodReturn, context) + " _return;");
       if (methodReturn.hasArrayOrderSpec()) {
         hasOrderSpec = hasOrderSpec || !methodReturn.isRarray();
-        writer.println(getReturnString(methodReturn, context) + " _return_proxy;");
+        writer.print(getReturnString(methodReturn, context));
+        writer.println(" _return_proxy;");
       }
     }
-    List extArgs = Utilities.extendArgs(id, m, false, context);
-    Iterator args = extArgs.iterator();
-    Map index_args = m.getRarrayInfo();
+    List     extArgs    = Utilities.extendArgs(id, m, false, context);
+    Iterator args       = extArgs.iterator();
+    Map      index_args = m.getRarrayInfo();
 
     while (args.hasNext()) {
       Argument arg = (Argument)args.next();
       //declare ensured array and ensure it
       if (arg.hasArrayOrderSpec()) {
         final Type argType = arg.getType();
-        hasOrderSpec = hasOrderSpec || 
-          (!argType.isRarray() && (arg.getMode() != Argument.IN));
-        writer.print(getReturnString(argType, context) +
-                     " " + arg.getFormalName() 
-          + "_proxy");
+        hasOrderSpec = hasOrderSpec 
+                     || (!argType.isRarray() && (arg.getMode() != Argument.IN));
+        writer.print(getReturnString(argType, context) + " ");
+        writer.print(arg.getFormalName() + "_proxy");
         if (arg.getMode()!= Argument.OUT) {
-          writer.println(" = " + C.getEnsureArray(argType.getArrayType())
-            + "(" + getArgValue(arg) + ", " + argType.getArrayDimension() + ", "
-            + s_ensureOrderConstant[argType.getArrayOrder()] + ");");
+          writer.print(" = " + C.getEnsureArray(argType.getArrayType()));
+          writer.print("(" + getArgValue(arg) + ", ");
+          writer.print(argType.getArrayDimension() + ", ");
+          writer.println(s_ensureOrderConstant[argType.getArrayOrder()] + ");");
         } else {
           writer.println(" = " + C.NULL + ";");
         }
       }
       //declare rarray proxies.
-      if(arg.getType().isRarray()) {
-        writer.print(IOR.getArgumentWithFormal(arg, context, 
-                                               false, true, false) 
-          + C.RAW_ARRAY_EXT + " = ");
+      if (arg.getType().isRarray()) {
+        writer.print(IOR.getArgumentWithFormal(arg, context, false, true, 
+                                               false));
+        writer.print(C.RAW_ARRAY_EXT + " = ");
         writer.println(arg.getFormalName() + "_proxy->d_firstElement;");
       }
-      if ((arg.getMode() != Argument.IN) &&
-          (arg.getType().getDetailedType() == Type.ENUM)) {
-        writer.print(C.getEnumName(arg.getType().getSymbolID()) +
-                     " _proxy_" + arg.getFormalName());
+      if (  (arg.getMode() != Argument.IN) 
+         && (arg.getType().getDetailedType() == Type.ENUM)  ) 
+      {
+        writer.print(C.getEnumName(arg.getType().getSymbolID()));
+        writer.print(" _proxy_" + arg.getFormalName());
         if (arg.getMode() == Argument.INOUT) {
-          writer.print(" = (" +
-                       C.getEnumName(arg.getType().getSymbolID()) +
-                       ")*" + arg.getFormalName());
+          writer.print(" = (" + C.getEnumName(arg.getType().getSymbolID()));
+          writer.print(")*" + arg.getFormalName());
         }
         writer.println(";");
       }
     }
     //Create rarray index variables and initialize them.
-    for(Iterator i = index_args.values().iterator(); i.hasNext();) {
+    for (Iterator i = index_args.values().iterator(); i.hasNext();) {
       //This is more than a little ugly, but I all I want is the first
       //element off the collection of array dimension that define
       //the index's value.
-      HashSet index_collection = (HashSet) i.next();
-      Iterator tmp = index_collection.iterator();
-      Method.RarrayInfo info = (Method.RarrayInfo) tmp.next();
-      Argument index = info.index;
-      int dim = info.dim;
+      HashSet           index_collection = (HashSet) i.next();
+      Iterator          tmp              = index_collection.iterator();
+      Method.RarrayInfo info             = (Method.RarrayInfo) tmp.next();
+      Argument          index            = info.index;
+      int               dim              = info.dim;
+      String            upper_func_name  = "sidlLength";
 
-      String upper_func_name = "sidlLength";
-
-      if(index.getType().getType() == Type.LONG)
+      if (index.getType().getType() == Type.LONG) {
         writer.print("int64_t ");
-      else
+      } else {
         writer.print("int32_t ");
+      }
       writer.print(index.getFormalName() + " = ");
-      writer.print(CExprString.
-                   toCString(Inverter.invertExpr
-                             ((AssertionExpression)
-                              info.rarray.getType().
-                              getArrayIndexExprs().get(dim),
-                              upper_func_name + "(" +
-                              info.rarray.getFormalName() + "_proxy," + 
-                              dim + ")", context)));
+      writer.print(CExprString.toCString(Inverter.invertExpr
+                                         ((AssertionExpression)
+                                         info.rarray.getType().
+                                         getArrayIndexExprs().get(dim),
+                                         upper_func_name + "(" 
+                                         + info.rarray.getFormalName() 
+                                         + "_proxy," + dim + ")", context)));
       writer.println(";");
     }
 
@@ -360,9 +360,10 @@ public class SkelSource {
     while (args.hasNext()) {
       Argument arg = (Argument)args.next();
       if (  arg.hasArrayOrderSpec() 
-            && arg.getMode() == Argument.INOUT ) {
-        writer.println(C.getDelRefArray(arg.getType().getArrayType()) 
-          + "(" + getArgValue(arg) + ");");
+         && arg.getMode() == Argument.INOUT ) 
+      {
+        writer.print(C.getDelRefArray(arg.getType().getArrayType()));
+        writer.println("(" + getArgValue(arg) + ");");
         if (!arg.getType().isRarray()) {
           writer.println(getArgValue(arg) + " = NULL;");
         }
@@ -377,8 +378,7 @@ public class SkelSource {
       writer.tab();
     }
     //Write Impl call
-    writer.println(C.getMethodImplName(id, m.getLongMethodName()) 
-      + "(");
+    writer.println(C.getMethodImplName(id, m.getLongMethodName()) + "(");
     writer.tab();
     List callArgs = Utilities.extendArgs(id, m, true, context);
     args = callArgs.iterator();
@@ -391,8 +391,9 @@ public class SkelSource {
         writer.print(arg.getFormalName() + "_proxy");
       } else if (arg.getType().isRarray()) {
         writer.print(arg.getFormalName() + C.RAW_ARRAY_EXT);
-      } else if ((arg.getMode() != Argument.IN) &&
-                 (arg.getType().getDetailedType() == Type.ENUM)) {
+      } else if (  (arg.getMode() != Argument.IN) 
+                && (arg.getType().getDetailedType() == Type.ENUM)  ) 
+      {
         writer.print("&_proxy_" + arg.getFormalName());
       } else {
         writer.print(arg.getFormalName());
@@ -417,35 +418,38 @@ public class SkelSource {
         final Type argType = arg.getType();
         if (!argType.isRarray()) {
           if ((arg.getMode() != Argument.IN)) {
-            writer.println(getArgValue(arg) + " = " 
-                      + C.getEnsureArray(argType.getArrayType()) + "(" 
-                      + arg.getFormalName() + "_proxy, " 
-                      + arg.getType().getArrayDimension() + ", " 
-                      + s_ensureOrderConstant[argType.getArrayOrder()] + ");");
-            writer.println(C.getDelRefArray(arg.getType().getArrayType()) 
-                           + "(" + arg.getFormalName() + "_proxy);");
+            writer.print(getArgValue(arg) + " = ");
+            writer.print(C.getEnsureArray(argType.getArrayType()) + "(");
+            writer.print(arg.getFormalName() + "_proxy, ");
+            writer.print(arg.getType().getArrayDimension() + ", ");
+            writer.print(s_ensureOrderConstant[argType.getArrayOrder()]);
+            writer.println(");");
+            writer.print(C.getDelRefArray(arg.getType().getArrayType()));
+            writer.println("(" + arg.getFormalName() + "_proxy);");
           }
         }
         else if (arg.getMode() == Argument.INOUT) {
-          writer.println(getArgValue(arg) + " = " + 
-                         arg.getFormalName() + "_proxy;");
+          writer.print(getArgValue(arg) + " = " + arg.getFormalName());
+          writer.println("_proxy;");
           writer.println(arg.getFormalName() + "_proxy = NULL;");
         }
       }
       else if ((arg.getMode() != Argument.IN) &&
                (arg.getType().getDetailedType() == Type.ENUM)) {
-        writer.println("*" + arg.getFormalName() + " = _proxy_" +
-                       arg.getFormalName() + ";");
+        writer.print("*" + arg.getFormalName() + " = _proxy_");
+        writer.println(arg.getFormalName() + ";");
       }
     }
     if (useReturn) {
       if (methodReturn.hasArrayOrderSpec()) {
-        writer.println("_return = " 
-          + C.getEnsureArray(methodReturn.getArrayType()) + "(_return_proxy, "
-          + methodReturn.getArrayDimension() + ", "
-          + s_ensureOrderConstant[methodReturn.getArrayOrder()] + ");");
-        writer.println(C.getDelRefArray(methodReturn.getArrayType()) 
-          + "(_return_proxy);");
+        writer.print("_return = ");
+        writer.print(C.getEnsureArray(methodReturn.getArrayType()));
+        writer.print("(_return_proxy, " + methodReturn.getArrayDimension());
+        writer.print(", ");
+        writer.print(s_ensureOrderConstant[methodReturn.getArrayOrder()]);
+        writer.println(");");
+        writer.print(C.getDelRefArray(methodReturn.getArrayType()));
+        writer.println("(_return_proxy);");
       }
     }
     if (hasOrderSpec && !m.getThrows().isEmpty()) {
@@ -462,12 +466,13 @@ public class SkelSource {
     args = extArgs.iterator();
     while (args.hasNext()) {
       Argument arg = (Argument)args.next();
-      if ((arg.getType().isRarray() && 
-                (arg.getMode() == Argument.INOUT)) ||
-               (arg.getType().hasArrayOrderSpec() &&
-                (arg.getMode() == Argument.IN))) {
-        writer.println(C.getDelRefArray(arg.getType().getArrayType()) 
-                       + "(" + arg.getFormalName() + "_proxy);");
+      if (  (  arg.getType().isRarray() 
+            && (arg.getMode() == Argument.INOUT) ) 
+         || (  arg.getType().hasArrayOrderSpec() 
+            && (arg.getMode() == Argument.IN) )  ) 
+      {
+        writer.print(C.getDelRefArray(arg.getType().getArrayType()));
+        writer.println("(" + arg.getFormalName() + "_proxy);");
       }
     }
     if (useReturn) {
@@ -484,8 +489,7 @@ public class SkelSource {
    * at the start of this class.  Symbol types and array types require
    * special processing.
    */
-  private static String getReturnString(Type type,
-                                        Context context)
+  private static String getReturnString(Type type, Context context)
     throws CodeGenerationException
   {
     return IOR.getReturnString(type, context, false, false);
@@ -496,11 +500,12 @@ public class SkelSource {
    */
   private static void writeGetDataPointer(Class cls, LanguageWriter writer) {
     writer.println(C.getDataName(cls) + "*");
-    writer.println(C.getDataGetName(cls) + "(" + C.getObjectName(cls) + " self)");
+    writer.print(C.getDataGetName(cls) + "(" + C.getObjectName(cls));
+    writer.println(" self)");
     writer.println("{");
     writer.tab();
-    writer.println("return (" + C.getDataName(cls) + "*)" 
-      + "(self ? self->d_data : " + C.NULL + ");");
+    writer.print("return (" + C.getDataName(cls) + "*)");
+    writer.println("(self ? self->d_data : " + C.NULL + ");");
     writer.backTab();
     writer.println("}");
     writer.println();
@@ -539,34 +544,37 @@ public class SkelSource {
                                          Context context)
     throws CodeGenerationException
   {
+    SymbolID id = cls.getSymbolID();
     writer.openCxxExtern();
     writer.println("void");
-    if(IOR.supportHooks(cls, context)) {
-      writer.println(C.getSetEPVName(cls) + "(" + IOR.getEPVName(cls) + " *"+s_epv+",");
+    /*
+     * The following assumes (basic) hook EPV option is a super set of 
+     * hook methods.  That is, if hook methods are to be generated,
+     * then so will the hook EPVs.  Otherwise, we'd have to checked for
+     * both generation of hook EPV and hook methods.
+     */
+    writer.print(C.getSetEPVName(cls) + "(" + IOR.getEPVName(cls) + " *");
+    writer.print(s_epv);
+    if (IOR.generateHookEPVs(cls, context)) {
+      writer.println(",");
       writer.tab();
-      writer.println(IOR.getPreEPVName(cls) + " *"+s_pre_epv+", "+
-                     IOR.getPostEPVName(cls) + " *"+s_post_epv+")");
+      writer.println(IOR.getPreEPVName(cls) + " *" + s_pre_epv + ", ");
+      writer.println(IOR.getPostEPVName(cls) + " *" + s_post_epv);
       writer.backTab();
-    } else {
-      writer.println(C.getSetEPVName(cls) + "(" + IOR.getEPVName(cls) + " *"+s_epv+")");
     }
+    writer.println(")");
     writer.println("{");
     writer.tab();
-    initializeMethodPointer(writer, IOR.getBuiltinMethod(IOR.CONSTRUCTOR, cls,
-                                                         context),
+    initializeMethodPointer(writer, 
+                            IOR.getBuiltinMethod(IOR.CONSTRUCTOR, id, context),
                             cls, context);
-    initializeMethodPointer(writer, IOR.getBuiltinMethod(IOR.CONSTRUCTOR2, cls,
+    initializeMethodPointer(writer, IOR.getBuiltinMethod(IOR.CONSTRUCTOR2, id,
                                                          context),
                             cls, context);
 
-    initializeMethodPointer(writer, IOR.getBuiltinMethod(IOR.DESTRUCTOR, cls,
-                                                         context), 
+    initializeMethodPointer(writer, 
+                            IOR.getBuiltinMethod(IOR.DESTRUCTOR, id, context), 
                             cls, context);
-    if (IOR.supportAssertions(cls, context)) {
-      initializeMethodPointer(writer, IOR.getBuiltinMethod(IOR.CHECK_ERROR, cls,
-                                                           context),
-                              cls, context);
-    }
     Iterator i = cls.getMethods(false).iterator();
     while (i.hasNext()) {
       Method m = (Method)i.next();
@@ -574,9 +582,9 @@ public class SkelSource {
     }
     writer.println();
     //if we have overwritten methods, initialize the super pointer
-    if(cls.hasOverwrittenMethods()) {
-      writer.println(C.getObjectName(cls) + "__superEPV(_getExternals()->"
-        + "getSuperEPV());");
+    if (cls.hasOverwrittenMethods()) {
+      writer.print(C.getObjectName(cls) + "__superEPV(_getExternals()->");
+      writer.println("getSuperEPV());");
     }
     writer.backTab();
     writer.println("}");
@@ -614,8 +622,8 @@ public class SkelSource {
     throws CodeGenerationException
   {
 
-    Set fconnectSIDs = IOR.getFConnectSymbolIDs(cls);
-    Set serializeSIDs = IOR.getStructSymbolIDs(cls, true);
+    Set fconnectSIDs    = IOR.getFConnectSymbolIDs(cls);
+    Set serializeSIDs   = IOR.getStructSymbolIDs(cls, true);
     Set deserializeSIDs = IOR.getStructSymbolIDs(cls, false);
     //Set fcastSIDs = IOR.getFCastSymbolIDs(cls);
 
@@ -623,23 +631,29 @@ public class SkelSource {
     
     for (Iterator i = fconnectSIDs.iterator(); i.hasNext(); ) {
       SymbolID l_id = (SymbolID) i.next();
-      writer.println(C.getSymbolObjectPtr(l_id) + " " + IOR.getSkelFConnectName(cls,l_id)+ 
-                     "(const char* url, sidl_bool ar, sidl_BaseInterface *_ex) { ");
+      writer.print(C.getSymbolObjectPtr(l_id) + " ");
+      writer.print(IOR.getSkelFConnectName(cls,l_id));
+      writer.print("(const char* url, sidl_bool ar, sidl_BaseInterface *_ex)");
+      writer.println(" { ");
       writer.tab();
-      writer.println("return "+ IOR.getRemoteConnectName(l_id) + "(url, ar, _ex);");
-      //      writer.println("return "+C.getImplFConnectName(cls,l_id)+"(url, ar, _ex);");
+      writer.print("return "+ IOR.getRemoteConnectName(l_id));
+      writer.println("(url, ar, _ex);");
+      //writer.print("return " + C.getImplFConnectName(cls,l_id));
+      //writer.println("(url, ar, _ex);");
       writer.backTab();
       writer.println("}");
       writer.println();
     }
+    writer.printlnUnformatted("#endif /*WITH_RMI*/");
 
     /*
     for (Iterator i = fcastSIDs.iterator(); i.hasNext(); ) {
       SymbolID l_id = (SymbolID) i.next();
-      writer.println(C.getSymbolObjectPtr(l_id) + " " + IOR.getSkelFCastName(cls,l_id)+ 
-                     "(void* bi, sidl_BaseInterface *_ex) { ");
+      writer.print(C.getSymbolObjectPtr(l_id) + " ");
+      writer.print(IOR.getSkelFCastName(cls, l_id));
+      writer.println("(void* bi, sidl_BaseInterface *_ex) { ");
       writer.tab();
-      writer.println("return "+C.getImplFCastName(cls,l_id)+"(bi, _ex);");
+      writer.println("return " + C.getImplFCastName(cls, l_id) + "(bi, _ex);");
       writer.backTab();
       writer.println("}");
       writer.println();
@@ -651,19 +665,21 @@ public class SkelSource {
       writer.println("void");
       writer.println(IOR.getSkelSerializationName(cls,l_id, true));
       writer.tab();
-      writer.println("(const " +
-                       IOR.getStructName(l_id) + " *strct, " +
-                       "struct sidl_rmi_Return__object *pipe, const char *name, sidl_bool copyArg, " +
-                       IOR.getExceptionFundamentalType() + "*exc)");
+      writer.print("(const " + IOR.getStructName(l_id) + " *strct, ");
+      writer.print("struct sidl_rmi_Return__object *pipe, const char *name, ");
+      writer.print("sidl_bool copyArg, " + IOR.getExceptionFundamentalType());
+      writer.println("*exc)");
       writer.backTab();
       writer.println("{");
       writer.tab();
-      writer.println("struct sidl_io_Serializer__object *__pipe = sidl_io_Serializer__cast(pipe, exc);");
+      writer.print("struct sidl_io_Serializer__object *__pipe = ");
+      writer.println("sidl_io_Serializer__cast(pipe, exc);");
       writer.println("if (*exc) return;");
       writer.println("if (__pipe) {");
       writer.tab();
       writer.println(C.getExceptionType() + " __throwaway__;");
-      writer.println(IOR.getSymbolName(l_id) + "__serialize(strct, __pipe, name, copyArg, exc);");
+      writer.print(IOR.getSymbolName(l_id));
+      writer.println("__serialize(strct, __pipe, name, copyArg, exc);");
       writer.println("sidl_io_Serializer_deleteRef(__pipe, &__throwaway__);");
       writer.backTab();
       writer.println("}");
@@ -677,18 +693,21 @@ public class SkelSource {
       writer.println("void");
       writer.println(IOR.getSkelSerializationName(cls, l_id, false));
       writer.tab();
-      writer.println("(" + IOR.getStructName(l_id) + " *strct, " +
-                       "struct sidl_rmi_Call__object *pipe, const char *name, sidl_bool copyArg, " +
-                       IOR.getExceptionFundamentalType() + " *exc)");
+      writer.print("(" + IOR.getStructName(l_id) + " *strct, ");
+      writer.print("struct sidl_rmi_Call__object *pipe, const char *name, ");
+      writer.print("sidl_bool copyArg, " + IOR.getExceptionFundamentalType());
+      writer.println(" *exc)");
       writer.backTab();
       writer.println("{");
       writer.tab();
-      writer.println("struct sidl_io_Deserializer__object *__pipe = sidl_io_Deserializer__cast(pipe, exc);");
+      writer.print("struct sidl_io_Deserializer__object *__pipe = ");
+      writer.println("sidl_io_Deserializer__cast(pipe, exc);");
       writer.println("if (*exc) return;");
       writer.println("if (__pipe) {");
       writer.tab();
       writer.println(C.getExceptionType() + " __throwaway__;");
-      writer.println(IOR.getSymbolName(l_id) + "__deserialize(strct, __pipe, name, copyArg, exc);");
+      writer.print(IOR.getSymbolName(l_id));
+      writer.println("__deserialize(strct, __pipe, name, copyArg, exc);");
       writer.println("sidl_io_Deserializer_deleteRef(__pipe, &__throwaway__);");
       writer.backTab();
       writer.println("}");
@@ -697,16 +716,12 @@ public class SkelSource {
       writer.println();
     }
 
-   writer.printlnUnformatted("#endif /*WITH_RMI*/");
-
-
   }
 
   /**
    * For non-<code>static</code> methods, write an assignment statement to
-   * initialize a particular membor of the entry point
-   * vector. Initialization of <code>static</code> methods appears
-   * elsewhere.
+   * initialize a particular member of the entry point vector.
+   * Initialization of <code>static</code> methods appears elsewhere.
    *
    * @param writer     the output writer to which the assignment statement
    *                   is written.
@@ -714,32 +729,55 @@ public class SkelSource {
    * @param id         the name of class that owns the method
    */
   private static void initializeMethodPointer(LanguageWriter writer,
-                                              Method m, SymbolID id,
+                                              Method m, Class cls,
                                               Context context)
     throws CodeGenerationException
   {
     final String methodName = m.getLongMethodName();
+    final String ename      = IOR.getVectorEntry(methodName);
+    final SymbolID id       = cls.getSymbolID();
 
     switch (m.getDefinitionModifier()) {
     case Method.FINAL:
     case Method.NORMAL:
-      String ename = IOR.getVectorEntry(methodName);
       String sname = C.getMethodSkelName(id, m);
-      if (IOR.supportHooks(id, context) & (!IOR.isBuiltinMethod(methodName)) ) {
-        Method pre = m.spawnPreHook();
+      if (!IOR.isBuiltinMethod(methodName)) {
+        Method pre  = m.spawnPreHook();
         Method post = m.spawnPostHook();
-        writeMethodAssignment(writer, s_pre_epv, IOR.getVectorEntry(pre.getLongMethodName()), 
-                              C.getMethodSkelName(id, pre));
-        writeMethodAssignment(writer, s_epv, ename, sname);
-        writeMethodAssignment(writer, s_post_epv, IOR.getVectorEntry(post.getLongMethodName()), 
-                              C.getMethodSkelName(id, post));
+
+        /*
+         * Order is (currently) critical!  If hook methods (i.e.,
+         * pre- and post-hooks) are being generated then be sure to
+         * initialize the EPV pointers to the skel/impl methods!  
+         * Otherwise, if we're just making sure to include the 
+         * basic hook skeletons (but not the implementations), then
+         * initialize the EPV pointers to null.
+         */
+        if (IOR.generateHookMethods(cls, context)) {
+          writeMethodAssignment(writer, s_pre_epv, 
+                                IOR.getVectorEntry(pre.getLongMethodName()), 
+                                C.getMethodSkelName(id, pre));
+          writeMethodAssignment(writer, s_epv, ename, sname);
+          writeMethodAssignment(writer, s_post_epv, 
+                                IOR.getVectorEntry(post.getLongMethodName()), 
+                                C.getMethodSkelName(id, post));
+        } else if (IOR.generateHookEPVs(cls, context)) {
+          writeMethodAssignment(writer, s_pre_epv, 
+                                IOR.getVectorEntry(pre.getLongMethodName()), 
+                                C.NULL);
+          writeMethodAssignment(writer, s_epv, ename, sname);
+          writeMethodAssignment(writer, s_post_epv, 
+                                IOR.getVectorEntry(post.getLongMethodName()), 
+                                C.NULL);
+        } else {
+          writeMethodAssignment(writer, s_epv, ename, sname);
+        }
       } else {
         writeMethodAssignment(writer, s_epv, ename, sname);
       }
       break;
     case Method.ABSTRACT:
-      writeMethodAssignment(writer, s_epv, IOR.getVectorEntry(methodName), 
-                            C.NULL);
+      writeMethodAssignment(writer, s_epv, ename, C.NULL);
       break;
     default:
       /* do nothing */
@@ -749,7 +787,7 @@ public class SkelSource {
 
   /**
    * Write a function to initialize entries in the static entry point vector
-   * for a particular class. This will generate an assignment statement for
+   * for a particular class.  This will generate an assignment statement for
    * each <code>static</code> method defined locally in the class.
    *
    * @param cls    the class for which an routine will be written.
@@ -760,49 +798,34 @@ public class SkelSource {
                                           Context context)
     throws CodeGenerationException
   {
+    SymbolID id = cls.getSymbolID();
+
     writer.openCxxExtern();
     writer.println("void");
-    if(IOR.supportHooks(cls, context)) {
-      writer.println(C.getSetSEPVName(cls) + "(" + IOR.getSEPVName(cls) + " *"+s_sepv+",");
+
+    /*
+     * The following assumes if hook methods are to be generated, so are 
+     * hook EPVs.  Otherwise, we'd have to checked for generation 
+     * of both hook EPVs and methods.
+     */
+    writer.print(C.getSetSEPVName(cls) + "(" + IOR.getSEPVName(cls) + " *");
+    writer.print(s_sepv);
+    if (IOR.generateHookEPVs(cls, context)) {
+      writer.println(",");
       writer.tab();
-      writer.println(IOR.getPreSEPVName(cls) + " *"+s_pre_sepv+", "+
-                     IOR.getPostSEPVName(cls) + " *"+s_post_sepv+")");
+      writer.println(IOR.getPreSEPVName(cls) + " *" + s_pre_sepv + ", ");
+      writer.println(IOR.getPostSEPVName(cls) + " *" + s_post_sepv + ")");
       writer.backTab();
     } else {
-      writer.println(C.getSetSEPVName(cls) + "(" + IOR.getSEPVName(cls) 
-      + " *" + s_sepv + ")");
-
+      writer.println(")");
     }
 
     writer.println("{");
     writer.tab();
-    if (IOR.supportAssertions(cls, context)) {
-      writeMethodAssignment(writer, s_sepv, 
-        IOR.getVectorEntry(IOR.getBuiltinName(IOR.CHECK_ERROR, true)), 
-        C.getMethodSkelName(cls, IOR.getBuiltinMethod(IOR.CHECK_ERROR, cls, 
-                                                     context, true)));
-    }
     Iterator i = cls.getMethods(false).iterator();
     while (i.hasNext()) {
       Method m = (Method) i.next();
-      if (m.isStatic()) {
-        String name     = m.getLongMethodName();
-        String skelName = C.getMethodSkelName(cls,m);
-        if (IOR.supportHooks(cls, context) && 
-            (!IOR.isBuiltinMethod(name,true))) {
-          Method pre = m.spawnPreHook();
-          Method post = m.spawnPostHook();
-          writeMethodAssignment(writer, s_pre_sepv, IOR.getVectorEntry(pre.getLongMethodName()), 
-                                C.getMethodSkelName(cls, pre));
-          writeMethodAssignment(writer, s_sepv, IOR.getVectorEntry(name), 
-                                skelName);
-          writeMethodAssignment(writer, s_post_sepv, IOR.getVectorEntry(post.getLongMethodName()), 
-                                C.getMethodSkelName(cls, post));
-        } else {
-          writeMethodAssignment(writer, s_sepv, IOR.getVectorEntry(name), 
-                                skelName);
-        }
-      }
+      initializeStaticMethodPointer(writer, m, cls, context);
     }
     writer.backTab();
     writer.println("}");
@@ -810,6 +833,69 @@ public class SkelSource {
     writer.println();
   }
 
+  /**
+   * For <code>static</code> methods, write an assignment statement to
+   * initialize a particular member of the entry point vector.
+   * Initialization of non-<code>static</code> methods appears elsewhere.
+   *
+   * @param writer     the output writer to which the assignment statement
+   *                   is written.
+   * @param m          a description of the method to initialize
+   * @param cls        the class that owns the method
+   */
+  private static void initializeStaticMethodPointer(LanguageWriter writer,
+                                                    Method m, Class cls,
+                                                    Context context)
+    throws CodeGenerationException
+  {
+    final String methodName = m.getLongMethodName();
+    final String ename      = IOR.getVectorEntry(methodName);
+    final SymbolID id       = cls.getSymbolID();
+
+    switch (m.getDefinitionModifier()) {
+    case Method.STATIC:
+      String sname = C.getMethodSkelName(id, m);
+      if (!IOR.isBuiltinMethod(methodName)) {
+        Method pre  = m.spawnPreHook();
+        Method post = m.spawnPostHook();
+
+        /*
+         * Order is (currently) critical!  If hook methods (i.e.,
+         * pre- and post-hooks) are being generated then be sure to
+         * initialize the EPV pointers to the skel/impl methods!  
+         * Otherwise, if we're just making sure to include the 
+         * basic hook skeletons (but not the implementations), then
+         * initialize the EPV pointers to null.
+         */
+        if (IOR.generateHookMethods(cls, context)) {
+          writeMethodAssignment(writer, s_pre_sepv, 
+                                IOR.getVectorEntry(pre.getLongMethodName()), 
+                                C.getMethodSkelName(id, pre));
+          writeMethodAssignment(writer, s_sepv, ename, sname);
+          writeMethodAssignment(writer, s_post_sepv, 
+                                IOR.getVectorEntry(post.getLongMethodName()), 
+                                C.getMethodSkelName(id, post));
+        } else if (IOR.generateHookEPVs(cls, context)) {
+          writeMethodAssignment(writer, s_pre_sepv, 
+                                IOR.getVectorEntry(pre.getLongMethodName()), 
+                                C.NULL);
+          writeMethodAssignment(writer, s_sepv, ename, sname);
+          writeMethodAssignment(writer, s_post_sepv, 
+                                IOR.getVectorEntry(post.getLongMethodName()), 
+                                C.NULL);
+        } else {
+          writeMethodAssignment(writer, s_sepv, ename, sname);
+        }
+      } else {
+        writeMethodAssignment(writer, s_sepv, ename, sname);
+      }
+      break;
+    default:
+      /* do nothing */
+      break;
+    }
+  }
+
   /**
    * Write the function assignment for the specified var and method name
    * to the specified value.
