@@ -1,10 +1,10 @@
 //
 // File:        StubHeader.java
 // Package:     gov.llnl.babel.backend.c
-// Revision:    @(#) $Id: StubHeader.java 6196 2007-10-24 23:42:37Z epperly $
+// Revision:    @(#) $Id: StubHeader.java 6482 2008-08-21 22:50:53Z epperly $
 // Description: write C client (i.e., stub) header to a pretty writer stream
 //
-// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2007, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -30,13 +30,13 @@
 
 package gov.llnl.babel.backend.c;
 
-import gov.llnl.babel.Context;
 import gov.llnl.babel.backend.CodeConstants;
 import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.backend.IOR;
-import gov.llnl.babel.backend.Utilities;
 import gov.llnl.babel.backend.rmi.RMIStubHeader;
+import gov.llnl.babel.backend.Utilities;
 import gov.llnl.babel.backend.writers.LanguageWriterForC;
+import gov.llnl.babel.Context;
 import gov.llnl.babel.symbols.Enumeration;
 import gov.llnl.babel.symbols.Extendable;
 import gov.llnl.babel.symbols.Method;
@@ -74,9 +74,9 @@ public class StubHeader {
   /**
    * Indices associated with the special, stub-only built-in methods.
    */
-  public final static int DUMP_STATS   = 0;
-  public final static int SET_CHECKING = 1;
-  public final static int SET_HOOKS    = 2;
+  public final static int DUMP_STATS    = 0;
+  public final static int SET_CONTRACTS = 1;
+  public final static int SET_HOOKS     = 2;
 
   /**
    * Maximum number of rarray arguments allowed in an inlined method.
@@ -126,8 +126,8 @@ public class StubHeader {
    *               be written. This will not be closed.
    */
   public StubHeader(LanguageWriterForC writer, Context context) {
-    d_writer = writer;
-    d_context = context;
+    d_writer      = writer;
+    d_context     = context;
     d_comment_all = !context.getConfig().getCommentLocalOnly();
   }
 
@@ -183,7 +183,7 @@ public class StubHeader {
    * @param sym the <code>Symbol</code> whose header will be written.
    */
   private void generatePrologue(Symbol sym) {
-     String         header = C.getHeaderFile(sym);
+     String header = C.getHeaderFile(sym);
 
     d_writer.writeBanner(sym, header, false, 
                          CodeConstants.C_DESC_STUB_PREFIX + sym.getFullName());
@@ -207,6 +207,20 @@ public class StubHeader {
     d_writer.closeHeaderGuard();
   }
 
+  private void generateInlinePrologue(SymbolID id)
+  {
+    final String decl = C.getInlineDecl(id);
+    d_writer.printlnUnformatted("#if !defined(" + decl +
+                     ") && defined(SIDL_C_HAS_INLINE)");
+    d_writer.printlnUnformatted
+      ("#if (defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 199901L)) || defined(__INTEL_COMPILER) || !defined(__GNUC__)");
+    d_writer.printlnUnformatted("#define " + decl + " inline");
+    d_writer.printlnUnformatted("#else");
+    d_writer.printlnUnformatted("#define " + decl + " extern inline");
+    d_writer.printlnUnformatted("#endif");
+    d_writer.printlnUnformatted("#endif /* inline preprocessor definitions */");
+  }
+
   /**
    * Generate the C header for a SIDL enumerated type.  This method simply
    * includes the IOR enumerated type header file and typedefs that type
@@ -260,6 +274,7 @@ public class StubHeader {
     generateIncludes(ext);
     d_writer.openCxxExtern();
 
+    generateInlinePrologue(ext);
     /*
      * Output standard function signatures for all methods.
      */
@@ -320,8 +335,8 @@ public class StubHeader {
     d_writer.println("NULL (if any).");
     d_writer.endBlockComment(true);
     d_writer.println("void");
-    d_writer.println(IOR.getSymbolName(strct) + "__init(" +
-                     IOR.getStructName(strct) + "* strct);");
+    d_writer.print(IOR.getSymbolName(strct) + "__init(");
+    d_writer.println(IOR.getStructName(strct) + "* strct);");
 
     d_writer.println();
     d_writer.beginBlockComment(true);
@@ -329,8 +344,8 @@ public class StubHeader {
     d_writer.println("inside a struct. It does not free the struct itself.");
     d_writer.endBlockComment(true);
     d_writer.println("void");
-    d_writer.println(IOR.getSymbolName(strct) + "__destroy(" +
-                     IOR.getStructName(strct) + "* strct,");
+    d_writer.print(IOR.getSymbolName(strct) + "__destroy(");
+    d_writer.println(IOR.getStructName(strct) + "* strct,");
     d_writer.tab();
     d_writer.println("struct sidl_BaseInterface__object **exception);");
     d_writer.backTab();
@@ -345,8 +360,8 @@ public class StubHeader {
     d_writer.println("On exit, dest is an independent copy of src.");
     d_writer.endBlockComment(true);
     d_writer.println("void");
-    d_writer.println(IOR.getSymbolName(strct) + "__copy(const " +
-                     IOR.getStructName(strct) + "* src, ");
+    d_writer.print(IOR.getSymbolName(strct) + "__copy(const ");
+    d_writer.println(IOR.getStructName(strct) + "* src, ");
     d_writer.tab();
     d_writer.println(IOR.getStructName(strct) + "* dest,");
     d_writer.println("struct sidl_BaseInterface__object **exception);");
@@ -359,8 +374,8 @@ public class StubHeader {
     d_writer.println("can be safely overridden without any prior action");
     d_writer.endBlockComment(true);
     d_writer.println("void");
-    d_writer.println(IOR.getSymbolName(strct) + 
-                     "__deserialize(" + IOR.getStructName(strct) + " *arg,");
+    d_writer.print(IOR.getSymbolName(strct));
+    d_writer.println("__deserialize(" + IOR.getStructName(strct) + " *arg,");
     d_writer.tab();
     d_writer.println("struct sidl_io_Deserializer__object *pipe,");
     d_writer.println("const char *name,");
@@ -373,8 +388,8 @@ public class StubHeader {
     d_writer.println("inside a struct. It does not free the struct itself.");
     d_writer.endBlockComment(true);
     d_writer.println("void");
-    d_writer.println(IOR.getSymbolName(strct) +
-                     "__serialize(const "+ IOR.getStructName(strct) + " *arg,");
+    d_writer.print(IOR.getSymbolName(strct));
+    d_writer.println("__serialize(const "+ IOR.getStructName(strct) + " *arg,");
     d_writer.tab();
     d_writer.println("struct sidl_io_Serializer__object *pipe,");
     d_writer.println("const char *name,");
@@ -432,12 +447,12 @@ public class StubHeader {
       }
       d_writer.println();
     }
-    d_writer.generateInclude(C.getHeaderFile(
-                                             Utilities.lookupSymbol(d_context, "sidl.rmi.Call")), 
-                             true);
-    d_writer.generateInclude(C.getHeaderFile(
-                                             Utilities.lookupSymbol(d_context, "sidl.rmi.Return")), 
-                             true);
+    d_writer.generateInclude(
+      C.getHeaderFile(Utilities.lookupSymbol(d_context, "sidl.rmi.Call")),
+                      true);
+    d_writer.generateInclude(
+      C.getHeaderFile(Utilities.lookupSymbol(d_context, "sidl.rmi.Return")), 
+                      true);
     d_writer.printlnUnformatted("#ifdef SIDL_C_HAS_INLINE");
     d_writer.generateInclude(IOR.getHeaderFile(ext), true);
     d_writer.printlnUnformatted("#endif /* SIDL_C_HAS_INLINE */");
@@ -456,19 +471,17 @@ public class StubHeader {
   }
 
   /**
-   * Generate prototypes for the specified assertion method(s).
+   * Generate prototypes for the specified interface contract method(s).
    */
-  private void generateAssertionPrototypes(SymbolID id, boolean doStatic) 
+  private void generateContractPrototypes(SymbolID id, boolean doStatic) 
     throws CodeGenerationException 
   {
-    generateBuiltinPrototypes(id, SET_CHECKING, doStatic);
+    generateBuiltinPrototypes(id, SET_CONTRACTS, doStatic);
     generateBuiltinPrototypes(id, DUMP_STATS, doStatic);
   }
 
-  public static boolean stubInlineMethod(Method m)
-  {
-    return
-      !(m.isStatic() || (m.numRarray() > MAX_RARRAY_INLINE));
+  public static boolean stubInlineMethod(Method m) {
+    return !(m.isStatic() || (m.numRarray() > MAX_RARRAY_INLINE));
   }
 
   private void generateMethodStub(SymbolID id,
@@ -480,12 +493,14 @@ public class StubHeader {
     final boolean inlineMethod = stubInlineMethod(method);
     generateMethodSignature(id, isInterface, method, docComment, inlineMethod);
     if (inlineMethod) {
-      d_writer.printlnUnformatted("#ifdef SIDL_C_HAS_INLINE");
+      d_writer.printlnUnformatted("#ifdef " +
+                                  C.getInlineDecl(id));
       StubSource.generateMethodStubBody(d_writer, d_context,
                                         id, isInterface, method);
       d_writer.printlnUnformatted("#else");
       d_writer.println(";");
-      d_writer.printlnUnformatted("#endif /* SIDL_C_HAS_INLINE */");
+      d_writer.printlnUnformatted("#endif /* " +
+                                  C.getInlineDecl(id) + " */");
       d_writer.println();
     }
   }
@@ -508,18 +523,18 @@ public class StubHeader {
     if (!ext.isAbstract()) {
       d_writer.writeComment("Constructor function for the class.", true);
       d_writer.println(C.getSymbolObjectPtr(ext));
-      d_writer.println(C.getFullMethodName(ext, "_create") + "(sidl_BaseInterface* _ex);");
+      d_writer.print(C.getFullMethodName(ext, "_create"));
+      d_writer.println("(sidl_BaseInterface* _ex);");
       d_writer.println();
 
       if(!IOR.isSIDLSymbol(ext)) {
-        d_writer.writeComment("Wraps up the private data struct pointer (" +
-                              C.getDataName(ext) +
-                              ") passed in rather than running the constructor.", 
-                              true);
+        d_writer.writeComment("Wraps up the private data struct pointer (" 
+                             + C.getDataName(ext) + ") passed in rather "
+                             + "than running the constructor.", true);
         d_writer.println(C.getSymbolName(ext));
         
-        d_writer.println(C.getFullMethodName(ext, "_wrapObj") 
-                         + "(void * data, sidl_BaseInterface *_ex);");
+        d_writer.print(C.getFullMethodName(ext, "_wrapObj"));
+        d_writer.println("(void * data, sidl_BaseInterface *_ex);");
       }
 
       d_writer.println();
@@ -533,34 +548,30 @@ public class StubHeader {
     d_writer.writeComment("RMI constructor function for the class.", true);
     d_writer.println(C.getSymbolName(ext));
     
-    d_writer.println(C.getFullMethodName(ext, "_createRemote") 
-                     + "(const char * url, sidl_BaseInterface *_ex);");
+    d_writer.print(C.getFullMethodName(ext, "_createRemote"));
+    d_writer.println("(const char * url, sidl_BaseInterface *_ex);");
     
     d_writer.println();
     
-    d_writer.writeComment("RMI connector function for the class.(addrefs)", true);
+    d_writer.writeComment("RMI connector function for the class.(addrefs)", 
+                          true);
     d_writer.println(C.getSymbolName(ext));
     
-    d_writer.println(C.getFullMethodName(ext, "_connect") 
-                     + "(const char *, sidl_BaseInterface *_ex);");
+    d_writer.print(C.getFullMethodName(ext, "_connect"));
+    d_writer.println("(const char *, sidl_BaseInterface *_ex);");
     d_writer.println();                 
  
     d_writer.println();
     d_writer.printlnUnformatted("#endif /*WITH_RMI*/");
 
     /*
-     * Output the assertion and/or hooks method(s) if appropriate.
+     * Output the built-in contract method(s).
      */
     boolean hasStatic = ext.hasStaticMethod(true);
-    if (!ext.isAbstract() && !ext.isInterface()) {
-      if (IOR.supportAssertions(ext, d_context)) {
-        if (hasStatic) {
-          generateAssertionPrototypes(ext, true);
-        }
-        generateAssertionPrototypes(ext, false);
-      }
-
+    if (hasStatic) {
+      generateContractPrototypes(ext, true);
     }
+    generateContractPrototypes(ext, false);
 
     /*
      * Output each of the method prototypes from the interface or class.
@@ -569,14 +580,17 @@ public class StubHeader {
     Collection methods = ext.getMethods(true);
     for (Iterator m = methods.iterator(); m.hasNext(); ) {
       Method method = (Method) m.next();
-      generateMethodStub(ext, isIfc, method, d_comment_all || ext.isLocal(method));
+      generateMethodStub(ext, isIfc, method, 
+                         d_comment_all || ext.isLocal(method));
       d_writer.println();
       if ( method.getCommunicationModifier() == Method.NONBLOCKING ) { 
         Method send = method.spawnNonblockingSend();
-        generateMethodSignature(ext, isIfc, send, d_comment_all || ext.isLocal(method), false);
+        generateMethodSignature(ext, isIfc, send, 
+                                d_comment_all || ext.isLocal(method), false);
         d_writer.println();
         Method recv = method.spawnNonblockingRecv();
-        generateMethodSignature(ext, isIfc, recv, d_comment_all || ext.isLocal(method), false);
+        generateMethodSignature(ext, isIfc, recv, 
+                                d_comment_all || ext.isLocal(method), false);
         d_writer.println();        
       }
     }
@@ -584,7 +598,8 @@ public class StubHeader {
     /*
      * Output the cast methods for the class or interface.
      */
-    d_writer.writeComment("Cast method for interface and class type conversions.", true);
+    d_writer.writeComment("Cast method for interface and class type "
+                         + "conversions.", true);
     d_writer.println(C.getSymbolObjectPtr(ext));
     d_writer.println(C.getFullMethodName(ext, "_cast") + "(");
     d_writer.tab();
@@ -614,28 +629,27 @@ public class StubHeader {
     //m_exec = C.getSExecMethod();
     //generateMethodSignature(id, isIfc, m_exec, true);
 
-    Method m_getURL = IOR.getBuiltinMethod(IOR.GETURL,ext, d_context,false);
+    Method m_getURL = IOR.getBuiltinMethod(IOR.GETURL, ext, d_context, false);
     generateMethodStub(ext, isIfc, m_getURL, true);
 
-    Method m_raddRef = IOR.getBuiltinMethod(IOR.RADDREF,ext, d_context,false);
+    Method m_raddRef = IOR.getBuiltinMethod(IOR.RADDREF, ext, d_context, false);
     generateMethodStub(ext, isIfc, m_raddRef, true);
 
-    Method m_isRemote = IOR.getBuiltinMethod(IOR.ISREMOTE,ext, d_context,false);
+    Method m_isRemote = IOR.getBuiltinMethod(IOR.ISREMOTE, ext, d_context,
+                                             false);
     generateMethodStub(ext, isIfc, m_isRemote, true);
 
-    Method m_isLocal = IOR.getBuiltinMethod(IOR.ISREMOTE,ext, d_context,false);
+    Method m_isLocal = IOR.getBuiltinMethod(IOR.ISREMOTE, ext, d_context, 
+                                            false);
     m_isLocal.setMethodName("_isLocal", "");
     generateMethodSignature(ext, isIfc, m_isLocal, true, false);
-    if(IOR.supportHooks(ext, d_context)) {
-      Method m_hook = IOR.getBuiltinMethod(IOR.HOOKS,ext, d_context,false);
-      generateMethodStub(ext, isIfc, m_hook, true);
-      
-      if (hasStatic) {
-        //Static set_hooks
-        Method m_s_hook = IOR.getBuiltinMethod(IOR.HOOKS,ext, d_context,true);
-        generateMethodStub(ext, isIfc, m_s_hook, true);
-      }
+
+    if (hasStatic) {
+      Method m_s_hook = IOR.getBuiltinMethod(IOR.HOOKS, ext, d_context, true);
+      generateMethodStub(ext, isIfc, m_s_hook, true);
     }
+    Method m_hook = IOR.getBuiltinMethod(IOR.HOOKS, ext, d_context, false);
+    generateMethodStub(ext, isIfc, m_hook, true);
   }
   
 
@@ -670,15 +684,14 @@ public class StubHeader {
       d_writer.writeComment(method, true);
     }
     if (isInline) {
-      d_writer.println("SIDL_C_INLINE_DECL");
+      d_writer.println(C.getInlineDecl(id));
     }
     d_writer.println(getReturnString(method.getReturnType()));
     d_writer.print(C.getFullMethodName(id, method));
     d_writer.println("(");
     d_writer.tab();
-    C.generateArgumentList(d_writer, d_context,
-                           s_self, isInterface, id, method, true, 
-                           true, false, true, false, true, true);
+    C.generateArgumentList(d_writer, d_context, s_self, isInterface, id, 
+                           method, true, true, false, true, false, true, true);
     d_writer.println(isInline ? ")" : ");");
     d_writer.backTab();
   }
@@ -706,11 +719,13 @@ public class StubHeader {
     String cmt  = null;
     String desc = doStatic ? "static " : "";
     switch (type) {
-      case SET_CHECKING:
-        cmt = "Method to set the " + desc + "assertion checking level.";
-        break;
       case DUMP_STATS:
-        cmt = "Method to dump " + desc +  "assertion checking statistics.";
+        cmt = "Method to dump " + desc;
+        cmt += "interface contract enforcement statistics.";
+        break;
+      case SET_CONTRACTS:
+        cmt = "Method to enable/disable " + desc;
+        cmt += "interface contract enforcement.";
         break;
       case SET_HOOKS:
         cmt = "Method to enable/disable " + desc +  "hooks execution.";
@@ -726,15 +741,15 @@ public class StubHeader {
                                               SymbolID id, boolean doStatic,
                                               String terminator)
   {
-    String suffix = doStatic ? IOR.s_static_suffix : "";
+    String suffix   = doStatic ? IOR.s_static_suffix : "";
     String basename = null;
     switch (type) {
-      case SET_CHECKING:
-        basename = "_set_checking";
-        break;
       case DUMP_STATS:
         basename = "_dump_stats";
         break;
+      case SET_CONTRACTS:
+        basename = "_set_contracts";
+        break;
       case SET_HOOKS:
         basename = "_set_hooks";
         break;
@@ -746,22 +761,20 @@ public class StubHeader {
       lw.println(C.getFullSelfDecl(id) + ",");
     }
     switch (type) {
-      case SET_CHECKING:
-        lw.println("int32_t level,");
-        lw.println("double  rate,");
-        lw.println("int32_t resetCounters,");
-        lw.println(IOR.getExceptionFundamentalType() +
-                   "*_ex)" + terminator);
-        break;
       case DUMP_STATS:
         lw.println("const char* filename,");
-        lw.println(IOR.getExceptionFundamentalType() +
-                   "*_ex)" + terminator);
+        lw.println("const char* prefix,");
+        lw.println(IOR.getExceptionFundamentalType() + "*_ex)" + terminator);
+        break;
+      case SET_CONTRACTS:
+        lw.println("sidl_bool   enable,");
+        lw.println("const char* enfFilename,");
+        lw.println("sidl_bool   resetCounters,");
+        lw.println(IOR.getExceptionFundamentalType() + "*_ex)" + terminator);
         break;
       case SET_HOOKS:
-        lw.println("int32_t on,");
-        lw.println(IOR.getExceptionFundamentalType() +
-                   "*_ex)" + terminator);
+        lw.println("sidl_bool enable,");
+        lw.println(IOR.getExceptionFundamentalType() + "*_ex)" + terminator);
         break;
     }
     lw.backTab();
@@ -775,14 +788,14 @@ public class StubHeader {
   public static String getBuiltinArgList(int type) {
     String args = null;
     switch (type) {
-      case SET_CHECKING:
-        args = "level, rate, resetCounters, _ex";
-        break;
       case DUMP_STATS:
-        args = "filename, _ex";
+        args = "filename, prefix, _ex";
+        break;
+      case SET_CONTRACTS:
+        args = "enable, enfFilename, resetCounters, _ex";
         break;
       case SET_HOOKS:
-        args = "on, _ex";
+        args = "enable, _ex";
         break;
     }
     return args;
