@@ -2,11 +2,11 @@
 // File:        ArrayModule.java
 // Package:     gov.llnl.babel.backend.fortran
 // Copyright:   (c) 2003 Lawrence Livermore National Security, LLC
-// Revision:    @(#) $Revision: 6196 $
-// Date:        $Date: 2007-10-24 16:42:37 -0700 (Wed, 24 Oct 2007) $
+// Revision:    @(#) $Revision: 6521 $
+// Date:        $Date: 2008-09-17 16:21:01 -0700 (Wed, 17 Sep 2008) $
 // Description: Create a F90 module to provide stubs for arrays
 // 
-// Copyright (c) 2003, The Regents of the University of Calfornia.
+// Copyright (c) 2003, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -280,9 +280,18 @@ public class ArrayModule {
     d_lw.println("logical function " + getPrivateName("is_null", dim) +
                  "(array)");
     d_lw.tab();
+    if (Fortran.hasBindC(d_context)) {
+      d_lw.println("use,intrinsic :: iso_c_binding");
+      d_lw.println("use :: sidl");
+    }
     d_lw.println(getImplArrayType(dim) + ", intent(in) :: array");
-    d_lw.println(getPrivateName("is_null", dim) +
+    if (Fortran.hasBindC(d_context)) {
+      d_lw.println(getPrivateName("is_null", dim) +
+                 " = (.not. c_associated(array%d_array))");
+    } else {
+      d_lw.println(getPrivateName("is_null", dim) +
                  " = (array%d_array .eq. 0)");
+    }
     d_lw.backTab();
     d_lw.println("end function " + getPrivateName("is_null", dim));
     d_lw.println();
@@ -293,9 +302,18 @@ public class ArrayModule {
     d_lw.println("logical function " + getPrivateName("not_null", dim) +
                  "(array)");
     d_lw.tab();
+    if (Fortran.hasBindC(d_context)) {
+      d_lw.println("use,intrinsic :: iso_c_binding");
+      d_lw.println("use :: sidl");
+    }
     d_lw.println(getImplArrayType(dim) + ", intent(in) :: array");
-    d_lw.println(getPrivateName("not_null", dim) +
+    if (Fortran.hasBindC(d_context)) {
+      d_lw.println(getPrivateName("not_null", dim) +
+                 " = c_associated(array%d_array)");
+    } else {
+      d_lw.println(getPrivateName("not_null", dim) +
                  " = (array%d_array .ne. 0)");
+    }
     d_lw.backTab();
     d_lw.println("end function " + getPrivateName("not_null", dim));
     d_lw.println();
@@ -307,7 +325,11 @@ public class ArrayModule {
                  "(array)");
     d_lw.tab();
     d_lw.println(getImplArrayType(dim) + ", intent(out) :: array");
-    d_lw.println("array%d_array = 0");
+    if (Fortran.hasBindC(d_context)) {
+      d_lw.println("array%d_array=C_NULL_PTR");
+    } else {
+      d_lw.println("array%d_array = 0");
+    }
     if (d_access) {
       d_lw.println("nullify(array%d_data)");
     }
@@ -546,6 +568,40 @@ public class ArrayModule {
         d_lw.println();
       }
     }
+    // If we're using the F2003 BindC module, convert from c_ptr to a sidl array
+    // for all arrays EXCEPT interface arrays
+    if (d_id.isSymbolBindC() && Fortran.hasBindC(d_context)){
+        for(int dim = 1; dim <= s_maxArray; ++dim) {
+          generateCastFromPtr(d_type, dim);
+        }
+        d_lw.println();
+    }
+  }
+
+  private void generateCastFromPtr(int type, int dim)
+    throws UnsupportedEncodingException
+  {
+    String typeString = null;
+    typeString = "CptrTosidl_"+BabelConfiguration.arrayType(d_type);
+    
+    String funcname = d_mang.shortName("cast" + typeString + dim+"d",d_privateSuffix);
+
+    final String function = d_mang.shortName
+      ("sidl_"+BabelConfiguration.arrayType(d_type)+"__array_vcast", "_m");
+
+    String newTypeName = typeString+"_"+dim+"d";
+
+    d_lw.println("subroutine " + funcname + "(oldType, newType)");
+    d_lw.tab();
+    d_lw.println("use,intrinsic :: iso_c_binding");
+    d_lw.println("type(c_ptr), intent(in) :: oldType");
+    d_lw.println("type(sidl_"+BabelConfiguration.arrayType(d_type)+"_"+dim+"d), intent(out) :: newType");
+    d_lw.println("external " + function);
+    d_lw.println("call "+function+"(oldType,"+dim+",newType)");
+
+    d_lw.backTab();
+    d_lw.println("end subroutine " + funcname);  
+    d_lw.println();
   }
 
   //Doesn't exist for non-base types! (except BaseInterface)
@@ -559,7 +615,6 @@ public class ArrayModule {
 
     d_lw.println("subroutine " + funcname + "(oldType, newType)");
     d_lw.tab();    
-    //d_lw.generateUse("sidl_"+BabelConfiguration.arrayType(type)+"_array");
     d_lw.println("type(sidl__array), intent(in) :: oldType");
     d_lw.println("type("+oldTypeName+"), intent(out) :: newType");
     
@@ -568,7 +623,11 @@ public class ArrayModule {
     if(d_casttype == BASEINTERFACE) {
       d_lw.println("if(sidl__array_dimen_m(oldType%d_array) .ne. "+dim+" .or. sidl__array_type_m(oldType%d_array) .ne. "+type+") then");
       d_lw.tab();
-      d_lw.println("newType%d_array=0");
+    if (Fortran.hasBindC(d_context)){
+        d_lw.println("newType%d_array=C_NULL_PTR");
+      } else {
+        d_lw.println("newType%d_array=0");
+      }
       d_lw.backTab();
       d_lw.println("else");
       d_lw.tab();
@@ -597,30 +656,14 @@ public class ArrayModule {
     
     String funcname = d_mang.shortName("cast" + typeString + dim+"dToGeneric", d_privateSuffix);
     String newTypeName = typeString+"_"+dim+"d";
-    //String extfunc = d_mang.shortArrayName("sidl", "cast", "_m");  
 
     d_lw.println("subroutine " + funcname + "(oldType, newType)");
     d_lw.tab();
     d_lw.println("type(sidl__array), intent(out) :: newType");
     d_lw.println("type("+newTypeName+"), intent(in) :: oldType");
 
-    //There is no BaseInterface array cast, so do that manually
-    //if(d_casttype == BASEINTERFACE) {
-    //  d_lw.println("if(dimen_p(oldType) .ne. +"dim"+ .or. type_p(oldType) .ne. "+type+") then");
-    //  d_lw.tab();
-    //  d_lw.println("set_null(newType)");
-    // d_lw.backTab();
-    // d_lw.println("else");
-    // d_lw.tab();
-
     //Anything can get cast to a generic array.  That's no big deal.
     d_lw.println("newType%d_array = oldType%d_array");
-    //  d_lw.backTab();
-    //d_lw.println("end if");
-    //} else {
-    //d_lw.println("external " + extfunc);
-    //d_lw.println("call " + extfunc + "(oldType%d_array, newType)");
-    // }
     
     d_lw.backTab();
     d_lw.println("end subroutine " + funcname);  
@@ -656,7 +699,6 @@ public class ArrayModule {
     throws UnsupportedEncodingException
   {
     generateQuery("dimen", d_intType, dim);
-    //generateQuery("type", d_intType, dim);
     generateIndexedQuery("stride", dim);
     generateIndexedQuery("lower", dim);
     generateIndexedQuery("upper", dim);
@@ -857,6 +899,7 @@ public class ArrayModule {
       else
         d_lw.println();
     }
+    d_lw.backTab();
     
     if(d_casttype != USERDEFINED) {
       d_lw.println();
@@ -873,7 +916,23 @@ public class ArrayModule {
       }
     }
     d_lw.backTab();
+    //interface if F2003 BindC is used
+    if (d_id.isSymbolBindC() && Fortran.hasBindC(d_context)) {
+      d_lw.println();
+      d_lw.println("private :: &");
+      d_lw.tab(); 
+      //Functions that cast TO a generic array
+      for(int dim = 1; dim <= s_maxArray; ++dim) {
+        d_lw.print(d_mang.shortName("castCptrTo" + typeString+dim+"d",
+                                    d_privateSuffix));
+        if(dim < s_maxArray) //So the last declaration doesn't end with a &
+          d_lw.println(", &"); 
+        else
+          d_lw.println();
+      }
+    }
     d_lw.backTab();
+    d_lw.println();
 
     //Interface declaration
 
@@ -908,6 +967,22 @@ public class ArrayModule {
       }
     }
 
+    //interface if F2003 BindC is used
+    if (d_id.isSymbolBindC() && Fortran.hasBindC(d_context)){
+      d_lw.backTab();
+      d_lw.println("module procedure &");
+      d_lw.tab();
+      //Functions that cast TO a generic array
+      for(int dim = 1; dim <= s_maxArray; ++dim) {
+        d_lw.print(d_mang.shortName("castCptrTo" + typeString+dim+"d",
+                                    d_privateSuffix));
+        if(dim < s_maxArray) //So the last declaration doesn't end with a &
+          d_lw.println(", &"); 
+        else
+          d_lw.println();
+      }
+    }
+
     d_lw.backTab();
     d_lw.backTab();
     d_lw.println("end interface");
@@ -985,7 +1060,8 @@ public class ArrayModule {
     int i;
     if (d_is_base_type) {
       TypeModule.writeArrayType(d_lw, d_id, 
-                                (d_access ? d_implDataType : null));
+                                (d_access ? d_implDataType : null),
+                                d_context);
     }
     if(d_is_generic_array) {  //The methods available in generic arrays.
       for(i = 0; i < s_generic_methods.length ; ++i){
@@ -1082,7 +1158,6 @@ public class ArrayModule {
     try {
       d_lw.println();
       if (d_is_generic_array) {
-        //d_lw.printlnUnformatted("#include \"sidl_array_fAbbrev.h\"");      
         d_lw.generateInclude("sidl_array_fAbbrev.h");      
       }
       d_lw.println("module " + Fortran.getArrayModule(d_id));
@@ -1095,6 +1170,10 @@ public class ArrayModule {
         }
       }
       d_lw.generateUse("sidl_array_type", emptyMap); //Everyone needs the generic module!
+      if (d_id.isSymbolBindC() && Fortran.hasBindC(d_context)){
+        d_lw.println("use,intrinsic :: iso_c_binding");
+      }
+
       d_lw.println();
       emptyMap = null;
       
