@@ -1,10 +1,10 @@
 //
 // File:        Fortran.java
 // Package:     gov.llnl.babel.backend.fortran
-// Revision:    @(#) $Revision: 6205 $
+// Revision:    @(#) $Revision: 6482 $
 // Description: Collection of static methods for the FORTRAN binding
 //
-// Copyright (c) 2000-2002, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2002, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -326,6 +326,15 @@ public class Fortran implements CodeConstants
     return result;
   }
 
+  static public String structSerializeStub(SymbolID id,
+                                           boolean serialize)
+  {
+    return id.getFullName().replace('.', '_') +
+      (serialize ? "__serialize_c" : "__deserialize_c");
+  }
+
+  
+
   static public String arrayIndices(List indices)
   {
     StringBuffer result = new StringBuffer();
