@@ -1,10 +1,10 @@
-//
+////
 // File:        GenerateClient.java
 // Package:     gov.llnl.babel.backend.fortran
-// Revision:    @(#) $Revision: 6203 $
+// Revision:    @(#) $Revision: 6525 $
 // Description: generate C code to allow FORTRAN to access BABEL
 //
-// Copyright (c) 2000-2002, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2002, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -37,12 +37,14 @@ import gov.llnl.babel.ResourceLoader;
 import gov.llnl.babel.backend.CodeConstants;
 import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.backend.CodeGenerator;
+import gov.llnl.babel.backend.CodeSplicer;
 import gov.llnl.babel.backend.Utilities;
 import gov.llnl.babel.backend.fortran.ModuleSource;
 import gov.llnl.babel.backend.fortran.StubDoc;
 import gov.llnl.babel.backend.fortran.StubSource;
 import gov.llnl.babel.backend.writers.LanguageWriterForC;
 import gov.llnl.babel.backend.writers.LanguageWriterForFortran;
+import gov.llnl.babel.symbols.Struct;
 import gov.llnl.babel.symbols.Symbol;
 import gov.llnl.babel.symbols.SymbolID;
 import gov.llnl.babel.symbols.SymbolTable;
@@ -51,6 +53,7 @@ import gov.llnl.babel.symbols.Version;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.PrintWriter;
+import java.security.NoSuchAlgorithmException;
 import java.io.Writer;
 import java.util.Iterator;
 import java.util.Set;
@@ -186,22 +189,55 @@ public class GenerateClient implements CodeGenerator
           forWriter.writeBanner(pkg, "sidl.F90", false,
                                 CodeConstants.C_FORTRAN_MODULE_PREFIX +
                                 id.getFullName());
-          forWriter.println();
-          forWriter.println("module sidl");
-          forWriter.tab();
-          forWriter.writeCommentLine("Define types for common numeric types");
-          forWriter.println("integer, parameter :: sidl_int=selected_int_kind(9)");
-          forWriter.println("integer, parameter :: sidl_long=selected_int_kind(18)");
-          forWriter.println("integer, parameter :: sidl_opaque=selected_int_kind(18)");
-          forWriter.println("integer, parameter :: sidl_arrayptr=selected_int_kind(18)");
-          forWriter.println("integer, parameter :: sidl_iorptr=selected_int_kind(18)");
-          
-          forWriter.println("integer, parameter :: sidl_enum=selected_int_kind(18)");
-          forWriter.println("integer, parameter :: sidl_dcomplex=selected_real_kind(15,307)");
-          forWriter.println("integer, parameter :: sidl_fcomplex=selected_real_kind(6,37)");
-          forWriter.println("integer, parameter :: sidl_double=selected_real_kind(15,307)");
-          forWriter.println("integer, parameter :: sidl_float=selected_real_kind(6,37)");
-          forWriter.println();
+          if (Fortran.hasBindC(d_context)) {
+            forWriter.printlnUnformatted("#include \"babel_config.h\"");
+            forWriter.println();
+            forWriter.println("module sidl");
+            forWriter.tab();
+            forWriter.println("use, intrinsic :: iso_c_binding");
+            forWriter.writeCommentLine("Define types for common numeric types");
+            forWriter.println("integer, parameter :: sidl_int=c_int32_t");
+            forWriter.println("integer, parameter :: sidl_long=c_int64_t");
+            forWriter.println("integer, parameter :: sidl_opaque=c_int64_t");
+            forWriter.println("integer, parameter :: sidl_arrayptr=c_int64_t");
+            forWriter.println("integer, parameter :: sidl_iorptr=c_int64_t");
+            forWriter.println("integer, parameter :: sidl_enum=c_int64_t");
+            forWriter.println("integer, parameter :: sidl_dcomplex=c_double_complex");
+            forWriter.println("integer, parameter :: sidl_fcomplex=c_float_complex");
+            forWriter.println("integer, parameter :: sidl_double=c_double");
+            forWriter.println("integer, parameter :: sidl_float=c_float");
+            forWriter.println();
+            forWriter.printlnUnformatted("#ifndef SIDL_HAS_FORTRAN_C_ASSOC");
+            forWriter.println("interface");
+            forWriter.tab();
+            forWriter.println("logical function c_associated(ptr) bind(c)");
+            forWriter.tab();
+            forWriter.println("use, intrinsic :: iso_c_binding");
+            forWriter.println("type(c_ptr) :: ptr");
+            forWriter.backTab();
+            forWriter.println("end function c_associated");
+            forWriter.backTab();
+            forWriter.println("end interface");
+            forWriter.printlnUnformatted("#endif");
+          }
+          else {
+            forWriter.println();
+            forWriter.println("module sidl");
+            forWriter.tab();
+            forWriter.writeCommentLine("Define types for common numeric types");
+            forWriter.println("integer, parameter :: sidl_int=selected_int_kind(9)");
+            forWriter.println("integer, parameter :: sidl_long=selected_int_kind(18)");
+            forWriter.println("integer, parameter :: sidl_opaque=selected_int_kind(18)");
+            forWriter.println("integer, parameter :: sidl_arrayptr=selected_int_kind(18)");
+            forWriter.println("integer, parameter :: sidl_iorptr=selected_int_kind(18)");
+            
+            forWriter.println("integer, parameter :: sidl_enum=selected_int_kind(18)");
+            forWriter.println("integer, parameter :: sidl_dcomplex=selected_real_kind(15,307)");
+            forWriter.println("integer, parameter :: sidl_fcomplex=selected_real_kind(6,37)");
+            forWriter.println("integer, parameter :: sidl_double=selected_real_kind(15,307)");
+            forWriter.println("integer, parameter :: sidl_float=selected_real_kind(6,37)");
+            forWriter.println();
+          }
           forWriter.writeCommentLine("Define generic array types");
           for(int i = 0 ; i < s_generic_array_types.length; ++i) {
             forWriter.println("integer, parameter :: sidl_" + 
@@ -217,6 +253,38 @@ public class GenerateClient implements CodeGenerator
             pw.close();
           }
         }
+        if (Fortran.hasBindC(d_context)) {
+          pw = null;
+          try {
+            pw = d_context.getFileManager().createFile(id, Type.PACKAGE,
+                                                       "STUBSRCS",
+                                                       "sidl_fStub.c");
+            LanguageWriterForC writer= new LanguageWriterForC(pw, d_context);
+            writer.writeBanner(pkg, "sidl_fStub.c", false,
+                                CodeConstants.C_DESC_STUB_PREFIX +
+                                id.getFullName());
+            writer.printlnUnformatted("#include \"babel_config.h\"");
+            writer.printlnUnformatted("#ifndef SIDL_HAS_FORTRAN_C_ASSOC");
+            writer.printlnUnformatted("#include \"sidlfortran.h\"");
+            writer.printlnUnformatted("#ifdef __cplusplus");
+            writer.println("extern \"C\"");
+            writer.printlnUnformatted("#endif");
+            writer.println("SIDL_F90_Bool");
+            writer.println("c_associated(void **ptr) {");
+            writer.tab();
+            writer.println("return (*ptr) ? SIDL_F90_TRUE : SIDL_F90_FALSE;");
+            writer.backTab();
+            writer.println("}");
+            writer.println();
+            writer.printlnUnformatted("#endif");
+          }
+          finally {
+            if (pw != null) {
+              pw.flush();
+              pw.close();
+            }
+          }
+        }
       }
       else {
         try {
@@ -352,7 +420,7 @@ public class GenerateClient implements CodeGenerator
                                                   "TYPEMODULESRCS",
                                                   filename);
         forWriter = new LanguageWriterForFortran(pw, d_context);
-        TypeModule.generateCode(null, forWriter);
+        TypeModule.generateCode(null, forWriter,d_context);
         //am.generateStub();
         forWriter.close();
 
@@ -455,7 +523,7 @@ public class GenerateClient implements CodeGenerator
           mw        = d_context.getFileManager().
             createFile(id, type, "TYPEMODULESRCS", tf);
           forWriter = new LanguageWriterForFortran(mw, d_context);
-          TypeModule.generateCode(symbol, forWriter);
+          TypeModule.generateCode(symbol, forWriter,d_context);
           forWriter.close();
         } else {
           String df = Fortran.getStubDocFile(id);
@@ -509,12 +577,12 @@ public class GenerateClient implements CodeGenerator
           ModuleSource.generateCode(symbol, forWriter, d_context);
           am.generateStub();
           forWriter.close();
-//
+
           String tf = Fortran.getTypeFile(id);
           mw        = d_context.getFileManager().
             createFile(id, type, "TYPEMODULESRCS", tf);
           forWriter = new LanguageWriterForFortran(mw, d_context);
-          TypeModule.generateCode(symbol, forWriter);
+          TypeModule.generateCode(symbol, forWriter,d_context);
         }
         else {
           f = Fortran.getEnumStubFile(id);
@@ -536,11 +604,17 @@ public class GenerateClient implements CodeGenerator
           mw.close();
         }
       }
+      break;
     case Symbol.STRUCT:
       try {
-        //String h = Fortran.getHeaderFile(id);
-        //pw       = d_context.getFileManager().createFile(id, type, 
-                                                        //"STUBHDRS", h);
+        {
+          nw = d_context.getFileManager().
+            createFile(id, type, "STUBSRCS", Fortran.getStubFile(id));
+          LanguageWriterForC writer = new LanguageWriterForC(nw, d_context);
+          StubSource.generateCode(symbol, writer, d_context);
+          writer.close();
+        }
+        nw = null;
         if (Fortran.needsAbbrev(d_context)) {
           String nf = Fortran.getStubNameFile(id);
           nw        = d_context.getFileManager().
@@ -549,12 +623,27 @@ public class GenerateClient implements CodeGenerator
           AbbrevHeader.generateCode(symbol, writer, d_context);
           writer.close();
         }
-        
-        //LanguageWriterForC writer = new LanguageWriterForC(pw);
-        //StubHeader.generateCode(symbol, writer);
-        //writer.close();
 
-      } finally {
+        Struct structSym = (Struct)symbol;
+        String f = Fortran.getStructModuleFile(id, d_context);
+        CodeSplicer splicer =
+          d_context.getFileManager().getCodeSplicer(id, type, f, false, true);
+        pw       = d_context.getFileManager().
+          createFile(id, type, "TYPEMODULESRCS", f);
+        LanguageWriterForFortran writer = 
+          new LanguageWriterForFortran(pw, d_context);
+        StructModule.generateCode(structSym, writer, splicer, d_context);
+        writer.close();
+
+      } 
+        catch ( IOException ex ) { 
+          throw new CodeGenerationException( ex.getMessage() );
+        } 
+        catch ( NoSuchAlgorithmException nsae) {
+        throw new CodeGenerationException( "NoSuchAlgorithmException: " +
+                                           nsae.getMessage());
+      }
+      finally {
         if (nw != null) {
           nw.close();
         }
