@@ -1,10 +1,10 @@
 //
 // File:        ImplSource.java
 // Package:     gov.llnl.babel.backend.fortran
-// Revision:    @(#) $Revision: 6205 $
+// Revision:    @(#) $Revision: 6482 $
 // Description: Generate a skeleton implementation in Fortran
 //
-// Copyright (c) 2000-2002, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2002, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -45,9 +45,12 @@ import gov.llnl.babel.backend.mangler.NameMangler;
 import gov.llnl.babel.backend.mangler.NonMangler;
 import gov.llnl.babel.backend.writers.LanguageWriterForFortran;
 import gov.llnl.babel.symbols.Argument;
+import gov.llnl.babel.symbols.AssertionExpression;
+import gov.llnl.babel.symbols.CExprString;
 import gov.llnl.babel.symbols.Class;
 import gov.llnl.babel.symbols.Extendable;
 import gov.llnl.babel.symbols.Method;
+import gov.llnl.babel.symbols.Struct;
 import gov.llnl.babel.symbols.Symbol;
 import gov.llnl.babel.symbols.SymbolID;
 import gov.llnl.babel.symbols.SymbolTable;
@@ -84,6 +87,13 @@ public class ImplSource
 
   private Context              d_context;
 
+  /**
+    * Keep track of the local splicer declarations that are written
+    * for rarrays since we don't want multiple splicer blocks.
+    */
+
+   private static HashSet spliceWritten = null;
+
   /**
    * This is the output device.
    */
@@ -143,7 +153,32 @@ public class ImplSource
   }
 
   /**
-   * Return the argument declaratoin, tailored to the specific flavor
+   * Return the function declaration, tailored to BindC.
+   *
+   * @param a    the argument
+   * @param MethodName    the name of the current function name
+   *
+   * @return     the argument declaration string
+   *
+   * Function return types in Fortran are the name of the function itself and
+   * as of 070208 this should only be called for 
+   *   1) primitive SIDL types.
+   *   2) From within a logical branch that selects only BindC.
+   *
+   */
+  public String getFunctionRetTypeDecl(Argument a,String methodName)
+    throws CodeGenerationException
+  {
+    final Type argType = a.getType();
+    StringBuffer result = 
+      new StringBuffer(Fortran.getReturnString(argType, d_context));
+      result.append(" :: ");
+      result.append(methodName);
+    return result.toString();
+  }
+
+  /**
+   * Return the argument declaration, tailored to the specific flavor
    * of FORTRAN.
    *
    * @param a    the argument
@@ -156,20 +191,13 @@ public class ImplSource
     final Type argType = a.getType();
     StringBuffer result = 
       new StringBuffer(Fortran.getReturnString(argType, d_context));
-    if (Fortran.isFortran90(d_context) ) {
+    if (Fortran.isFortran90(d_context) || Fortran.hasBindC(d_context) ) {
       if (argType.isRarray()) {
         result.append(", dimension");
         result.append(Fortran.arrayIndices(argType.getArrayIndexExprs()));
       }
       result.append(" :: ");
       result.append(a.getFormalName());
-    } else if (Fortran.hasBindC(d_context) ) {
-      if (argType.isRarray()) {
-        result.append(", dimension");
-        result.append(Fortran.arrayIndices(argType.getArrayIndexExprs()));
-      } 
-      result.append(" :: ");
-      result.append(a.getFormalName());
     } else {
       result.append(' ');
       result.append(a.getFormalName());
@@ -318,7 +346,7 @@ public class ImplSource
     return result;
   }
 
-  private void addUseForReferences(Method m,
+  public void addUseForReferences(Method m,
                                    SymbolID id)
     throws CodeGenerationException
   {
@@ -397,28 +425,50 @@ public class ImplSource
                                  SymbolID id)
     throws CodeGenerationException
   {
+    spliceWritten = new HashSet();
     List extendedArgs    = StubSource.extendArgs(id, m, d_context, true);
     String splicerTag    = id.getFullName() + '.' + m.getLongMethodName();
     String splicerTagUse = splicerTag + s_use;
+    String splicerTagLocal = splicerTag+".local_declarations";
     String methodImplName = Fortran.getMethodImplName(id, m, d_mang, 
                                                       d_context);
+    boolean gotFunc=isFortranFunction(m.getReturnType());
 
     d_lw.println();
     d_lw.println();
 
     d_lw.writeComment(m, false);
-    if (Fortran.isFortran90(d_context) || Fortran.hasBindC(d_context)) {
-      d_lw.print("recursive ");
-    }
-    d_lw.print("subroutine " + methodImplName + "(");
-    Iterator i = extendedArgs.iterator();
-    while (i.hasNext()) {
-      d_lw.print(((Argument)i.next()).getFormalName());
-      if (i.hasNext()) {
-        d_lw.print(", ");
+
+    if (gotFunc && Fortran.hasBindC(d_context)) {
+      /**
+        * Right now functions are only for the F03 binding 
+      **/
+        d_lw.print("function " + methodImplName + "(");
+        Iterator i = extendedArgs.iterator();
+        while (i.hasNext()) {
+          String argName = ((Argument)i.next()).getFormalName();
+          if (!(argName.equals(Fortran.s_return))) {
+            d_lw.print(argName);
+            if (i.hasNext()) {
+              d_lw.print(", ");
+            }
+          }
+        }
+        d_lw.println(")");
+    } else {
+      if (Fortran.isFortran90(d_context) || Fortran.hasBindC(d_context)) {
+        d_lw.print("recursive ");
       }
+      d_lw.print("subroutine " + methodImplName + "(");
+      Iterator i = extendedArgs.iterator();
+      while (i.hasNext()) {
+        d_lw.print(((Argument)i.next()).getFormalName());
+        if (i.hasNext()) {
+          d_lw.print(", ");
+        }
+      }
+      d_lw.println(")");
     }
-    d_lw.println(")");
 
     if (Fortran.hasBindC(d_context)){
       d_lw.tab();
@@ -434,38 +484,122 @@ public class ImplSource
     }
 
     d_lw.println("implicit none");
-    i = Fortran.reorderArguments(extendedArgs).iterator();
+    Iterator i = Fortran.reorderArguments(extendedArgs).iterator();
     while (i.hasNext()) {
       Argument a = (Argument)i.next();
       String decl = getArgumentDeclaration(a); 
-
       if (Fortran.isFortran90(d_context) || Fortran.hasBindC(d_context)) {
         //Retarded special case to make ctor2 work
         if(m.equals(IOR.getBuiltinMethod(IOR.CONSTRUCTOR2,id,d_context)) &&
-           a.getType().getType() == Type.OPAQUE && 
-           a.getFormalName().equals("private_data")) {
+          a.getType().getType() == Type.OPAQUE && 
+          a.getFormalName().equals("private_data")) {
           d_lw.println("type(" + id.getFullName().replace('.', '_') + 
-                       "_wrap" + ") :: private_data");
+                      "_wrap" + ") :: private_data");
         } else {
-          d_lw.println(decl);
-          d_lw.writeCommentLine(a.getModeString());
+          //Some ugly logic--for later cleanup-SPM
+          if (gotFunc && Fortran.hasBindC(d_context)) {
+            if (!a.getFormalName().equals(Fortran.s_return)){
+              d_lw.println(decl);
+            } else {
+              String fDecl = getFunctionRetTypeDecl(a,methodImplName); 
+              d_lw.println(fDecl);
+            }  
+          } else {
+            d_lw.println(decl);
+          }
+        d_lw.writeCommentLine(a.getModeString());
         }
-      }
-      else {
+      } else {
         d_lw.writeCommentLine(a.getArgumentString());
         d_lw.println(decl);
       }
     }
+
+    d_lw.println();
+    d_lw.println();
+
+    /** Additional code to handle the case where we are using F2003 (bindC) struct
+     *  support and there is a raw-array within that struct.  In this case we
+     *  need to convert the c pointer into a fortran pointer--SPM 
+     * 
+     *  First pass for the declarations **/
+    i = Fortran.reorderArguments(extendedArgs).iterator();
+    while (i.hasNext()) {
+      Argument a = (Argument)i.next();
+      String decl = getArgumentDeclaration(a); 
+      if (Fortran.hasBindC(d_context)){
+        Type type = a.getType();
+        int t=type.getDetailedType();
+        switch(t) {
+        case Type.ARRAY:
+          /*no-op*/
+          break;
+        case Type.STRUCT:
+          SymbolTable table = d_context.getSymbolTable();
+          Symbol sym = table.lookupSymbol(type.getSymbolID());
+          if (sym != null) {
+            if (sym instanceof Struct) {
+              Struct strct=(Struct)sym;
+              generateImplDeclaration(strct,a,splicerTagLocal,methodImplName);
+            }
+          }
+          break;
+        default:
+          /*no-op*/
+          break;
+        } 
+      }
+    }
+
+
+    /** Second pass for the C_F_Pointer calls **/
+    i = Fortran.reorderArguments(extendedArgs).iterator();
+    while (i.hasNext()) {
+      Argument a = (Argument)i.next();
+      String decl = getArgumentDeclaration(a); 
+      if (Fortran.hasBindC(d_context)){
+        Type type = a.getType();
+        int t=type.getDetailedType();
+        switch(t) {
+        case Type.ARRAY:
+          /*no-op*/
+          break;
+        case Type.STRUCT:
+          SymbolTable table = d_context.getSymbolTable();
+          Symbol sym = table.lookupSymbol(type.getSymbolID());
+          if (sym != null) {
+            if (sym instanceof Struct) {
+              Struct strct=(Struct)sym;
+              generateImplCall(strct,a,splicerTagLocal);
+            }
+          }
+          break;
+        default:
+          /*no-op*/
+          break;
+        } 
+      }
+    }
+
     d_lw.println();
     if (Fortran.isFortran90(d_context) || Fortran.hasBindC(d_context)) {
       d_lw.backTab();
     }
 
     spliceInitialContent(m, splicerTag);
-    if (Fortran.isFortran90(d_context) || Fortran.hasBindC(d_context)) {
-      d_lw.println("end subroutine " + methodImplName);
-    } else {
-      d_lw.println("end");
+    if (gotFunc && Fortran.hasBindC(d_context)){
+      /**
+        * Right now functions are only for the F03 binding 
+      **/
+      if (Fortran.hasBindC(d_context)){
+        d_lw.println("end function " + methodImplName);
+      } 
+    }else {
+      if (Fortran.isFortran90(d_context) || Fortran.hasBindC(d_context)) {
+        d_lw.println("end subroutine " + methodImplName);
+      } else {
+        d_lw.println("end");
+      }
     }
   }
 
@@ -612,7 +746,7 @@ public class ImplSource
     while (i.hasNext()) {
       Method m = (Method)i.next();
       if (!m.isAbstract()) {
-        if ( (IOR.supportHooks(cls, d_context)) && (!m.isBuiltIn()) ) {
+        if ( (IOR.generateHookMethods(cls, d_context)) && (!m.isBuiltIn()) ) {
           extendAndGenerate(m.spawnPreHook(), id);
           extendAndGenerate(m, id);
           extendAndGenerate(m.spawnPostHook(false, false), id);
@@ -664,4 +798,155 @@ public class ImplSource
     ImplSource source = new ImplSource(writer, splicer, context);
     source.generateCode(cls);
   }
+
+public static boolean isInt (String s) {
+
+  /**
+   * If a String can be parsed into an Integer return true.
+   * Used to check r-array indices.
+  */
+
+   try {
+     int i = Integer.parseInt(s.trim());
+     return true;
+   } catch (NumberFormatException nfe) {
+     return false;
+   }
+  }
+
+private void generateImplDeclaration(Struct strct,
+                                     Argument a,
+                                     String splicerTagLocal,
+                                     String methodImplName)
+  {
+
+    String strctName=a.getFormalName();
+
+    Iterator ii = strct.getItems().iterator(); 
+    while (ii.hasNext()){
+
+      final Struct.Item item = (Struct.Item)ii.next();
+
+      Type tt = item.getType();
+
+      /* ttype is the Type integer of the Array */
+      int ttype = tt.getDetailedType();
+      if (ttype==Type.ARRAY){
+        Type aType = tt.getArrayType();
+        if (null != aType){
+          if (tt.isRarray()){
+            Iterator indexVar = tt.getArrayIndexExprs().iterator();
+            AssertionExpression ae = (AssertionExpression)indexVar.next();
+            String checkInt = ae.toString();
+            if (!isInt(checkInt)){
+              if (tt.isNumericArray()){
+
+                /* Only want one local splicer block per method iff a
+                struct w/rarray is present */
+                if(!spliceWritten.contains(methodImplName)) {
+                  /* splicer block so implementor can add local declarations */
+                  d_splicer.splice (splicerTagLocal, d_lw, "local declarations");
+                  d_lw.println();
+                  spliceWritten.add(methodImplName);
+                }
+
+                Babel2BindCType bbc = new Babel2BindCType();
+                Integer objInt = new Integer(aType.getBasicType());
+      
+                StringBuffer ptrString=new StringBuffer();
+                ptrString.append(bbc.getBindCType(objInt)+",dimension(:");                       
+    
+                int dims=tt.getArrayDimension();
+    
+                for(int ij = 1 ; ij < dims; ij++){
+                  ptrString.append(",:"); 
+                }
+                String fptr=strctName+"_"+item.getName()+"_fptr";
+                ptrString.append("),pointer :: "+fptr);
+    
+                d_lw.println(ptrString.toString());
+                d_lw.println();
+
+              }
+            }
+          }
+        }
+      }
+    }
+  }
+
+private void generateImplCall(Struct strct,
+                                     Argument a,
+                                     String splicerTagLocal)
+  throws CodeGenerationException 
+  {
+    String strctName=a.getFormalName();
+
+    /* Now take care of C_F_POINTER calls */
+
+    Iterator ii = strct.getItems().iterator(); 
+    while (ii.hasNext()){
+      final Struct.Item item = (Struct.Item)ii.next();
+      Type tt = item.getType();
+
+
+      /* ttype is the Type integer of the Array */
+      int ttype = tt.getDetailedType();
+      if (ttype==Type.ARRAY){
+        Type aType = tt.getArrayType();
+        if (null != aType){
+          if (tt.isRarray()){
+            if (tt.isNumericArray()){
+              Iterator indexVar = tt.getArrayIndexExprs().iterator();
+              AssertionExpression ae = (AssertionExpression)indexVar.next();
+              String checkInt = ae.toString();
+              if (!isInt(checkInt)){
+
+                String fptr=strctName+"_"+item.getName()+"_fptr";
+                final int dims=tt.getArrayDimension();
+
+                d_lw.println("call C_F_POINTER(CPTR="+strctName+"%"+item.getName()+", & "); 
+                d_lw.println("                 FPTR="+fptr+", & "); 
+
+                StringBuffer dimString = new StringBuffer();
+                dimString.append("                 SHAPE=(/");
+
+                dimString.append("DIM");
+
+                for(int ij = 1; ij < dims && indexVar.hasNext(); ij++ ) {
+                  dimString.append(",DIM");
+                }
+
+                dimString.append("/))"); 
+
+                d_lw.println(dimString.toString()); 
+                d_lw.println(); 
+              }
+            }else{
+              throw new CodeGenerationException("Babel: Error Rarrays in Structs with BindC must have a numeric Type."); 
+            }
+          }
+        }
+      }
+    }
+  }
+
+
+  /**
+   * Utility function that checks if we have a fortran function
+   * that returns primitve types.
+   *
+   * @param Type is m.getReturnType()
+   */
+  public static boolean isFortranFunction(Type check)
+  {
+    boolean gotFunc=false;
+    if (check.isF03FuncRetType()) {
+      gotFunc=true;
+    } else {
+      gotFunc=false;
+    }
+    return gotFunc;
+  }
+
 }
