@@ -1,10 +1,10 @@
 //
 // File:        ModuleSource.java
 // Package:     gov.llnl.babel.backend.fortran
-// Revision:    @(#) $Revision: 6205 $
+// Revision:    @(#) $Revision: 6494 $
 // Description: Generate a module file for FORTRAN 90 clients
 //
-// Copyright (c) 2002-2003, The Regents of the University of Calfornia.
+// Copyright (c) 2002-2003, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -181,11 +181,21 @@ public class ModuleSource {
       d_lw.print(" " + s_intent_spec[a.getMode()]);
     }
     if (argType.isRarray()) {
+      Iterator indexVar = argType.getArrayIndexExprs().iterator();
+      AssertionExpression ae = (AssertionExpression)indexVar.next();
       final int dimen = argType.getArrayDimension();
       d_lw.print(", dimension(:");
-      for(int j = 1; j < dimen; ++j) {
+      int count = 1;
+      while(indexVar.hasNext()) {
+        ae = (AssertionExpression)indexVar.next();
         d_lw.print(", :");
+        count++;
       }
+
+      if (count != dimen){
+        throw new CodeGenerationException("Babel: Error in Rarray dimensions. Dimensions="+dimen+" of type don't match those given in name "+a.getFormalName()+" (dimensions="+count+").");
+      }
+
       d_lw.print(")");
     }
     d_lw.println(" :: " + a.getFormalName());
@@ -259,10 +269,11 @@ public class ModuleSource {
       declareArgument(a, true);
     }
     // declare array indices
-    i = indexMap.keySet().iterator();
+    i = indexMap.entrySet().iterator();
     while (i.hasNext()) {
-      String indexVar = (String)i.next();
-      Collection choice = (Collection)indexMap.get(indexVar);
+      Map.Entry ent = (Map.Entry)i.next();
+      String indexVar = (String)ent.getKey();
+      Collection choice = (Collection)ent.getValue();
       if (choice.size() > 0) {
         Method.RarrayInfo info = (Method.RarrayInfo)choice.iterator().next();
         declareArgument(info.index, false);
@@ -270,10 +281,11 @@ public class ModuleSource {
     }
     d_lw.println();
     d_lw.println("external " + stubName);
-    i = indexMap.keySet().iterator();
+    i = indexMap.entrySet().iterator();
     while (i.hasNext()) {
-      String indexVar = (String)i.next();
-      Collection choice = (Collection)indexMap.get(indexVar);
+      Map.Entry ent = (Map.Entry)i.next();
+      String indexVar = (String)ent.getKey();
+      Collection choice = (Collection)ent.getValue();
       if (choice.size() > 0) {
         Method.RarrayInfo info = (Method.RarrayInfo)choice.iterator().next();
         d_lw.print(indexVar + " = ");
@@ -797,6 +809,20 @@ public class ModuleSource {
     writeMethodInterface(ext, bi);
     interfacesWritten.add(bi.getShortMethodName());
 
+/* TODO/ToDo/TBD: Correct?
+    bi = IOR.getBuiltinMethod(IOR.CONTRACTS, id, d_context, false);
+    bi.setMethodName("set_contracts");
+    d_lw.println();
+    writeMethodInterface(ext, bi);
+    interfacesWritten.add(bi.getShortMethodName());
+
+    bi = IOR.getBuiltinMethod(IOR.DUMP_STATS, id, d_context, false);
+    bi.setMethodName("dump_stats");
+    d_lw.println();
+    writeMethodInterface(ext, bi);
+    interfacesWritten.add(bi.getShortMethodName());
+*/
+
     if(ext.hasStaticMethod(true)) {
       bi = IOR.getBuiltinMethod(IOR.HOOKS,id,d_context,true);
       bi.setMethodName("set_hooks_static");
@@ -804,6 +830,19 @@ public class ModuleSource {
       writeMethodInterface(ext, bi, true);
       interfacesWritten.add(bi.getShortMethodName());
 
+/* TODO/ToDo/TBD:  Correct?
+      bi = IOR.getBuiltinMethod(IOR.CONTRACTS, id, d_context, true);
+      bi.setMethodName("set_contracts_static");
+      d_lw.println();
+      writeMethodInterface(ext, bi, true);
+      interfacesWritten.add(bi.getShortMethodName());
+
+      bi = IOR.getBuiltinMethod(IOR.DUMP_STATS, id, d_context, true);
+      bi.setMethodName("dump_stats_static");
+      d_lw.println();
+      writeMethodInterface(ext, bi, true);
+      interfacesWritten.add(bi.getShortMethodName());
+*/
     }
 
   }
@@ -834,6 +873,7 @@ public class ModuleSource {
     d_lw.println();
     writeMethodStub(bi,id,"_s",false,true);
 
+/* TODO/ToDo/TBD:  RESUME DUPS HERE */
     bi = IOR.getBuiltinMethod(IOR.HOOKS,id,d_context,false);
     bi.setMethodName("set_hooks");
     d_lw.println();
