@@ -1,10 +1,10 @@
 //
 // File:        SkelSource.java
 // Package:     gov.llnl.babel.backend.fortran
-// Revision:    @(#) $Revision: 6196 $
+// Revision:    @(#) $Revision: 6482 $
 // Description: Generate to allow the IOR to call FORTRAN implementations
 //
-// Copyright (c) 2000-2002, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2002, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -39,6 +39,7 @@ import gov.llnl.babel.backend.IOR;
 import gov.llnl.babel.backend.Utilities;
 import gov.llnl.babel.backend.c.C;
 import gov.llnl.babel.backend.fortran.Fortran;
+import gov.llnl.babel.backend.fortran.ImplSource;
 import gov.llnl.babel.backend.fortran.StubSource;
 import gov.llnl.babel.backend.mangler.FortranMangler;
 import gov.llnl.babel.backend.mangler.NameMangler;
@@ -48,6 +49,8 @@ import gov.llnl.babel.backend.writers.LanguageWriterForC;
 import gov.llnl.babel.symbols.Argument;
 import gov.llnl.babel.symbols.AssertionExpression;
 import gov.llnl.babel.symbols.CExprString;
+import gov.llnl.babel.symbols.Class;
+import gov.llnl.babel.symbols.Extendable;
 import gov.llnl.babel.symbols.Inverter;
 import gov.llnl.babel.symbols.Method;
 import gov.llnl.babel.symbols.Symbol;
@@ -154,10 +157,11 @@ public class SkelSource {
    *   a catch all exception to indicate problems in the code generation
    *   phase.
    */
-  private void generateExtern(String name, List args)
+  private void generateExtern(String name, List args, Method m)
     throws CodeGenerationException
   {
-    StubSource.generateSignature(d_writer, name, args, d_context);
+    boolean gotFunc=ImplSource.isFortranFunction(m.getReturnType());
+    StubSource.generateSignature(d_writer, name, args, d_context, m, gotFunc);
     d_writer.println(";");
   }
 
@@ -524,6 +528,61 @@ public class SkelSource {
     }
   }
 
+  /**
+   * Deal with Function args for FORTRAN function call.
+   *
+   * @param args   the list of arguments.
+   *
+   *  This should only be called if this is BindC AND it's a function
+   */
+  private void listFuncArgs(List args) {
+    Iterator i = args.iterator();
+    while (i.hasNext()) {
+      Argument a    = (Argument)i.next();
+      Type     t    = a.getType();
+      String   name = a.getFormalName();
+
+      if (!Fortran.s_return.equals(name)){
+      if (StubSource.hasProxy(t) || (Fortran.s_return.equals(name))) {
+        switch(t.getDetailedType()) {
+        case Type.STRING:
+          d_writer.println(Fortran.getFortranPrefix(d_context) + "_STR_LOCAL_ARG(" 
+            + s_proxy + name + ")");
+          d_writer.println(Fortran.getFortranPrefix(d_context) + "_STR_NEAR_LEN(" 
+            + s_proxy + name + ")");
+          break;
+        case Type.CHAR:
+            d_writer.printlnUnformatted(StubSource.charCheck(d_context));
+            d_writer.println(Fortran.getFortranPrefix(d_context) + "_STR_LOCAL_ARG(" 
+              + s_proxy + name + ")");
+            d_writer.println(Fortran.getFortranPrefix(d_context) + "_STR_NEAR_LEN(" 
+              + s_proxy + name + ")");
+            d_writer.printlnUnformatted("#else");
+            d_writer.println("&" + s_proxy + name);
+            d_writer.printlnUnformatted("#endif");
+          break;
+        case Type.ARRAY:
+          if (t.isRarray()) {
+            d_writer.print(s_proxy + name + "->d_firstElement");
+            break;
+          }
+          // fall through to default intended
+        default:
+            d_writer.print("&" + s_proxy + name);
+          break;
+        }
+      } else {
+          if (a.getMode() == Argument.IN && t.getDetailedType() != Type.STRUCT) {
+            d_writer.print("&");
+          }
+          d_writer.print(name);
+        }
+        if (i.hasNext()) {
+          d_writer.println(",");
+        }
+      }
+    }
+  }
   /**
    * List each normal argument for the FORTRAN subroutine call.
    *
@@ -564,7 +623,7 @@ public class SkelSource {
           break;
         }
       } else {
-        if (a.getMode() == Argument.IN) {
+        if (a.getMode() == Argument.IN && t.getDetailedType() != Type.STRUCT) {
           d_writer.print("&");
         }
         d_writer.print(name);
@@ -575,6 +634,33 @@ public class SkelSource {
     }
   }
 
+  /**
+   * Deal with characters as Function return types.  This should ONLY be called
+   * if the F2003 binding is being used and we are writing a function in the fSkel
+   * files.
+   *
+   * @param args  the list of arguments.
+   */
+  private void listStringLengthFuncArgs(List args)
+  {
+    Iterator i = args.iterator();
+    while (i.hasNext()) {
+      Argument a    = (Argument)i.next();
+      Type     t    = a.getType();
+      String   name = a.getFormalName();
+      if (!Fortran.s_return.equals(name)){
+        switch (t.getDetailedType()) {
+          case Type.STRING:
+            d_writer.println("#error Strings not supported as F03 function return types");
+            break;
+          case Type.CHAR:
+            d_writer.println("#error Characters not supported as F03 function return types");
+            break;
+        }
+      }
+    }
+  }
+
   /**
    * On some compilers, the string lengths are passed as extra arguments at
    * the end of the argument list.  List the length of each string argument.
@@ -618,14 +704,23 @@ public class SkelSource {
    *   a catch all exception to indicate problems in the code generation
    *   phase.
    */
-  private void callMethod(String methodName, List args)
+  private void callMethod(String methodName, List args,boolean gotFunc)
     throws CodeGenerationException
   {
+    if (gotFunc && Fortran.hasBindC(d_context)){
+      d_writer.print("_proxy_retval = ");
+    }
+
     d_writer.println(Fortran.getFortranSymbol(d_context) + "(" + methodName.toLowerCase()
       + "," + methodName.toUpperCase() + "," + methodName + ")(");
     d_writer.tab();
-    listNormalArguments(args);
-    listStringLengthArguments(args);
+    if (gotFunc && Fortran.hasBindC(d_context)){
+      listFuncArgs(args);
+      listStringLengthFuncArgs(args);
+    } else {
+      listNormalArguments(args);
+      listStringLengthArguments(args);
+    }
     d_writer.println(");");
     d_writer.backTab();
   }
@@ -752,6 +847,12 @@ public class SkelSource {
     case Type.SYMBOL:
       throw new CodeGenerationException(pre + "Unable to process " 
                   + argType.getTypeString() + ' ' + argName);
+    case Type.STRUCT:
+      if (argName.equals(Fortran.s_return)) {
+        writeArgValue(argName, mode);
+        d_writer.println(" = &" + s_proxy + argName + ";");
+      }
+      break;
     default:
       if (argName.equals(Fortran.s_return)) {
         writeArgValue(argName, mode);
@@ -831,14 +932,44 @@ public class SkelSource {
    */
   private void declareReturn(Type t) throws CodeGenerationException {
     if (Type.VOID != t.getDetailedType()) {
-      d_writer.print(getReturnString(t) + " " + Fortran.s_return);
-      if (Utilities.isPointer(t)) {
-        d_writer.print(" = NULL");
-      }
+      switch (t.getDetailedType()) {
+        case Type.STRUCT:
+          //Structs as return values should be pointers.
+          d_writer.print(getReturnString(t) + " *" + Fortran.s_return+"=NULL");
+          break;
+        default:
+          d_writer.print(getReturnString(t) + " " + Fortran.s_return);
+          if (Utilities.isPointer(t)) {
+            d_writer.print(" = NULL");
+          }
+          break;
+        }
+
       d_writer.println(";"); 
     }
   }
 
+  /**
+   * The statement that actually returns the return value of the method.
+   * 
+   * @param t  the type of the method's return value.
+   * @exception gov.llnl.babel.backend.CodeGenerationException
+   *   a catch all exception to indicate problems in the code generation
+   *   phase.
+   */
+  private void addReturnStatement(Type t) throws CodeGenerationException {
+    if (Type.VOID != t.getDetailedType()) {
+      switch (t.getDetailedType()) {
+        case Type.STRUCT:
+          d_writer.println("return *" + Fortran.s_return + ";");
+          break;
+        default:
+          d_writer.println("return " + Fortran.s_return + ";");
+          break;
+      }
+    }
+  }
+
   /**
    * This manages the generation of a skeleton function for a particular
    * method.
@@ -856,6 +987,9 @@ public class SkelSource {
                               SymbolID id)
     throws CodeGenerationException
   {
+
+    boolean gotFunc=ImplSource.isFortranFunction(m.getReturnType());
+
     d_writer.println("static " + getReturnString(m.getReturnType()));
     d_writer.println(Fortran.getMethodSkelName(id, m) + "(");
     d_writer.tab();
@@ -875,15 +1009,13 @@ public class SkelSource {
     declareIndices(m.getRarrayInfo());
     initializeProxies(m,args);
     callMethod(Fortran.getMethodImplName(id, m, d_mang, d_context), 
-               argsWithIndices);
+               argsWithIndices,gotFunc);
     checkExceptionBlock(m);
     processOutgoing(args);
     endExceptionBlock(m);
     releaseIncomingRefs(args);
     freeResources(m, args);
-    if (Type.VOID != m.getReturnType().getDetailedType()) {
-      d_writer.println("return " + Fortran.s_return + ";");
-    }
+    addReturnStatement(m.getReturnType());
     d_writer.backTab();
     d_writer.println("}");
   }
@@ -894,7 +1026,7 @@ public class SkelSource {
     * at the start of this class.  Symbol types and array types require
     * special processing.
     */
-  private String getReturnString(Type type)
+  public String getReturnString(Type type)
      throws CodeGenerationException
    {
      return IOR.getReturnString(type, d_context, true, false);
@@ -919,7 +1051,7 @@ public class SkelSource {
     d_writer.println();
     d_writer.println();
     generateExtern(Fortran.getMethodImplName(id, m, d_mang, d_context), 
-                   extendedArgsWithIndices);
+                   extendedArgsWithIndices, m);
     d_writer.println();
     generateMethod(extendedArgs, extendedArgsWithIndices, m, id);
   }
@@ -930,9 +1062,7 @@ public class SkelSource {
    * 
    * @param cls   the class object.
    */
-  private Collection extendMethods(gov.llnl.babel.symbols.Class cls) 
-    throws CodeGenerationException 
-  {
+  private Collection extendMethods(Class cls) throws CodeGenerationException {
     Collection localMethods = cls.getMethods(false);
     final SymbolID id = cls.getSymbolID();
     ArrayList  extendedMethods = new ArrayList(localMethods.size()+2);
@@ -953,21 +1083,22 @@ public class SkelSource {
    * @param cls    the class for which an routine will be written.
    * @param writer the output writer to which the routine will be written.
    */
-  private static void writeInitializeSEPV(SymbolID id, 
-                                          Collection methods,
+  private static void writeInitializeSEPV(Class              cls, 
+                                          Collection         methods,
                                           LanguageWriterForC writer,
-                                          Context context)
+                                          Context            context)
      throws CodeGenerationException
   {
+    SymbolID id = cls.getSymbolID();
     writer.openCxxExtern();
     writer.println("void");
-    writer.print(IOR.getSetSEPVName(id) + "(" + IOR.getSEPVName(id) + " * " 
-       + s_sepv);
-    if (IOR.supportHooks(id, context )) {
+    writer.print(IOR.getSetSEPVName(id) + "(" + IOR.getSEPVName(id) + " * ");
+    writer.print(s_sepv);
+    if (IOR.generateHookMethods(cls, context)) {
       writer.println(",");
       writer.tab();
       writer.println(IOR.getPreSEPVName(id) + " *" + s_pre_sepv + ", ");
-      writer.println(IOR.getPostSEPVName(id) + " *" + s_post_sepv + ")");
+      writer.println(IOR.getPostSEPVName(id) + " *" + s_post_sepv + ") ");
       writer.backTab();
     } else {
       writer.print(") ");
@@ -978,7 +1109,7 @@ public class SkelSource {
     while (i.hasNext()) {
       Method m = (Method)i.next();
       if (m.isStatic()) {
-        initializeMethodPointer(writer, m, id, true, context);
+        initializeMethodPointer(writer, m, cls, true, context);
       }
     }
     writer.backTab();
@@ -1116,22 +1247,23 @@ public class SkelSource {
    * @param writer     the output writer to which the assignment statement
    *                   is written.
    * @param m          a description of the method to initialize
-   * @param id         the name of class that owns the method
+   * @param cls        the class that owns the method
    * @param doStatic   TRUE if initializing a static epv; FALSE otherwise
    */
   private static void initializePointers(LanguageWriter writer, 
-                                         Method m,
-                                         SymbolID id, 
-                                         boolean doStatic,
-                                         Context context)
+                                         Method         m,
+                                         Class          cls,
+                                         boolean        doStatic,
+                                         Context        context)
      throws CodeGenerationException
   {
+    final SymbolID id       = cls.getSymbolID();
     final String methodName = m.getLongMethodName();
     final String ename      = IOR.getVectorEntry(methodName);
     final String sname      = Fortran.getMethodSkelName(id, m);
     String epvStr           = doStatic ? s_sepv : s_epv;
 
-    if ( IOR.supportHooks(id, context) && (!m.isBuiltIn()) ) {
+    if ( IOR.generateHookMethods(cls, context) && (!m.isBuiltIn()) ) {
       String preEpv  = doStatic ? s_pre_sepv : s_pre_epv;
       String postEpv = doStatic ? s_post_sepv : s_post_epv;
       Method hook    = m.spawnPreHook();
@@ -1155,22 +1287,23 @@ public class SkelSource {
    * @param writer     the output writer to which the assignment statement
    *                   is written.
    * @param m          a description of the method to initialize
-   * @param id         the name of class that owns the method
+   * @param cls        the class that owns the method
    * @param doStatic   TRUE if initializing a static epv; FALSE otherwise
    */
-  private static void initializeMethodPointer(LanguageWriter writer, Method m,
-                                              SymbolID id, 
-                                              boolean doStatic,
-                                              Context context)
+  private static void initializeMethodPointer(LanguageWriter writer, 
+                                              Method         m,
+                                              Class          cls,
+                                              boolean        doStatic,
+                                              Context        context)
      throws CodeGenerationException
   {
     if (doStatic) {
-      initializePointers (writer, m, id, doStatic, context);
+      initializePointers(writer, m, cls, doStatic, context);
     } else {
       switch (m.getDefinitionModifier()) {
         case Method.FINAL:
         case Method.NORMAL:
-          initializePointers (writer, m, id, doStatic, context);
+          initializePointers(writer, m, cls, doStatic, context);
           break;
         case Method.ABSTRACT:
           String methodName = m.getLongMethodName();
@@ -1192,22 +1325,23 @@ public class SkelSource {
    * @param cls    the class for which an routine will be written.
    * @param writer the output writer to which the routine will be written.
    */
-  private static void writeInitializeEPV(SymbolID id, 
-                                         Collection methods,
+  private static void writeInitializeEPV(Class              cls,
+                                         Collection         methods,
                                          LanguageWriterForC writer, 
-                                         boolean isSuper,
-                                         Context context) 
+                                         boolean            isSuper,
+                                         Context            context) 
      throws CodeGenerationException
   {
+    SymbolID id = cls.getSymbolID();
     writer.openCxxExtern();
     writer.println("void");
-    writer.print(IOR.getSetEPVName(id) + "(" + IOR.getEPVName(id) + " * " 
-       + s_epv);
-    if (IOR.supportHooks(id, context)) {
+    writer.print(IOR.getSetEPVName(id) + "(" + IOR.getEPVName(id) + " * ");
+    writer.print(s_epv);
+    if (IOR.generateHookMethods(cls, context)) {
       writer.println(",");
       writer.tab();
       writer.println(IOR.getPreEPVName(id) + " *" + s_pre_epv + ", ");
-      writer.println(IOR.getPostEPVName(id) + " *" + s_post_epv + ")");
+      writer.println(IOR.getPostEPVName(id) + " *" + s_post_epv + ") ");
       writer.backTab();
     } else {
       writer.print(") ");
@@ -1218,7 +1352,7 @@ public class SkelSource {
     while (i.hasNext()) {
       Method m = (Method)i.next();
       if (!m.isStatic()) {
-        initializeMethodPointer(writer, m, id, false, context);
+        initializeMethodPointer(writer, m, cls, false, context);
       }
     }
 
@@ -1238,9 +1372,10 @@ public class SkelSource {
     final String implMethod = Fortran.getMethodImplName(id, m, d_mang,
                                                         d_context);
     List extendedArgs = StubSource.extendArgs(id, m, d_context, false);
+    Type type=m.getReturnType();
     try {
       d_writer.pushLineBreak(false);
-      StubSource.generateMethodSymbol(d_writer, implMethod, d_context);
+      StubSource.generateMethodSymbol(d_writer, implMethod, d_context,m.getReturnType(),false);
       d_writer.tab();
       d_writer.println("int64_t *self,");
       d_writer.println("void    *private_data,");
@@ -1291,9 +1426,7 @@ public class SkelSource {
    *   a catch all exception to indicate problems in the code generation
    *   phase.
    */
-  public void generateCode(gov.llnl.babel.symbols.Class cls)
-    throws CodeGenerationException
-  {
+  public void generateCode(Class cls) throws CodeGenerationException {
     final String pre = "fortran.SkelSource.generateCode: ";
     Collection methods = extendMethods(cls);
     final SymbolID id = cls.getSymbolID();
@@ -1317,7 +1450,7 @@ public class SkelSource {
     while (i.hasNext()) {
       Method m = (Method)i.next();
       if (!m.isAbstract()) {
-        if (IOR.supportHooks(cls, d_context) && (!m.isBuiltIn()) ) {
+        if (IOR.generateHookMethods(cls, d_context) && (!m.isBuiltIn()) ) {
           extendAndGenerate(m.spawnPreHook(), id);
           extendAndGenerate(m, id);
           extendAndGenerate(m.spawnPostHook(false, false), id);
@@ -1337,23 +1470,23 @@ public class SkelSource {
     Method load_meth = IOR.getBuiltinMethod(IOR.LOAD, id, d_context);
     List load_args = StubSource.extendArgs(id, load_meth, d_context, false);
     generateExtern(Fortran.getMethodImplName(id, load_meth, d_mang, d_context),
-                   load_args);
+                   load_args, load_meth);
     d_writer.println();
     d_writer.println("void " + IOR.getSymbolName(id) + "__call_load(void) { ");
     d_writer.tab();
     d_writer.println("int64_t _proxy_exception;");
     callMethod(Fortran.getMethodImplName(id,load_meth,d_mang, d_context),
-               load_args);
+               load_args,false);
     d_writer.backTab();
     d_writer.println("}");
 
     d_writer.println();
     d_writer.println();
-    writeInitializeEPV(id, methods, d_writer, cls.hasOverwrittenMethods(), 
+    writeInitializeEPV(cls, methods, d_writer, cls.hasOverwrittenMethods(), 
                        d_context);
     if (cls.hasStaticMethod(true)) {
       d_writer.println();
-      writeInitializeSEPV(id, methods, d_writer, d_context);
+      writeInitializeSEPV(cls, methods, d_writer, d_context);
       d_writer.println();
     }
 
@@ -1379,9 +1512,9 @@ public class SkelSource {
    * @exception java.security.NoSuchAlgorithmException
    *   problem with the name mangler.
    */
-  public static void generateCode(gov.llnl.babel.symbols.Class cls,
+  public static void generateCode(Class              cls,
                                   LanguageWriterForC writer,
-                                  Context context)
+                                  Context            context)
     throws CodeGenerationException, NoSuchAlgorithmException
   {
     SkelSource source = new SkelSource(writer, context);
@@ -1394,13 +1527,12 @@ public class SkelSource {
    * @param cls    the class for which an routine will be written.
    * @param writer the output writer to which the routine will be written.
    */
-  private void writeRMIAccessFunctions(gov.llnl.babel.symbols.Class cls) 
+  private void writeRMIAccessFunctions(Class cls) 
     throws CodeGenerationException
   {
     SymbolID id = cls.getSymbolID();
 
     Set fconnectSIDs = IOR.getFConnectSymbolIDs(cls);
-    //    Set fcastSIDs = IOR.getFCastSymbolIDs(cls);
     d_writer.printlnUnformatted("#ifdef WITH_RMI");
 
     for (Iterator i = fconnectSIDs.iterator(); i.hasNext(); ) {        
@@ -1413,21 +1545,58 @@ public class SkelSource {
       d_writer.println("}");
       d_writer.println();
     }
-    
-    /*
-    for (Iterator i = fcastSIDs.iterator(); i.hasNext(); ) {
-      SymbolID destination_id = (SymbolID) i.next();      
-      d_writer.println(C.getSymbolObjectPtr(destination_id) + " " + IOR.getSkelFCastName(id,destination_id)+ 
-                       "(void* bi, "+ IOR.getExceptionFundamentalType() + "*_ex) { ");
-      d_writer.tab();
-      d_writer.println("return "+C.getFullMethodName(destination_id, "_rmicast")+"(bi, _ex);");
-      d_writer.backTab();
-      d_writer.println("}");
-      d_writer.println();
-    }
-    */
+
     d_writer.printlnUnformatted("#endif /*WITH_RMI*/");
    
+    Iterator i = IOR.getStructSymbolIDs(cls, true).iterator();
+    while(i.hasNext()) {
+      writeStructSerialize(cls, (SymbolID)i.next(), true);
+    }
+    i = IOR.getStructSymbolIDs(cls, false).iterator();
+    while(i.hasNext()) {
+      writeStructSerialize(cls, (SymbolID)i.next(), false);
+    }
   }
 
+
+  private void writeStructSerialize(Extendable ext,
+                                    SymbolID structid,
+                                    boolean serialize)
+    throws CodeGenerationException
+  {
+    final String pipeType = 
+      (serialize ? "sidl_io_Serializer" : "sidl_io_Deserializer");
+    StubSource.writeStructSerializeSig(d_writer, structid, serialize);
+    d_writer.println("void");
+    d_writer.println(IOR.getSkelSerializationName(ext, structid, serialize) +
+                     "(");
+    d_writer.tab();
+    d_writer.println((serialize ? "const " : "") +
+                     IOR.getStructName(structid) + " *strct,");
+    d_writer.println("struct sidl_rmi_" +
+                     (serialize ? "Return" : "Call") +
+                     "__object *pipe,");
+    d_writer.println("const char *name,");
+    d_writer.println("sidl_bool copy,");
+    d_writer.println("sidl_BaseInterface *_ex)");
+    d_writer.backTab();
+    d_writer.println("{");
+    d_writer.tab();
+    d_writer.println("sidl_BaseInterface throwaway_exception;");
+    d_writer.println("sidl_BaseInterface bipipe = (sidl_BaseInterface)pipe;");
+    d_writer.println("struct " + pipeType + "__object *uppipe =");
+    d_writer.tab();
+    d_writer.println("(struct " + pipeType + "__object *)");
+    d_writer.println("(*bipipe->d_epv->f__cast)(bipipe->d_object,");
+    d_writer.println("\"" + pipeType.replace('_','.') + "\",");
+    d_writer.println("&throwaway_exception);");
+    d_writer.backTab();
+    d_writer.println(Fortran.structSerializeStub(structid, serialize) + "(");
+    d_writer.tab();
+    d_writer.println("strct, uppipe, name, copy, _ex);");
+    d_writer.backTab();
+    d_writer.backTab();
+    d_writer.println("}");
+    d_writer.println();
+  }
 }
