@@ -1,10 +1,10 @@
 //
 // File:        StubSource.java
 // Package:     gov.llnl.babel.backend.fortran
-// Revision:    @(#) $Revision: 6205 $
+// Revision:    @(#) $Revision: 6596 $
 // Description: Generate code to allow FORTRAN calls to BABEL
 //
-// Copyright (c) 2000-2003, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2003, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -38,6 +38,8 @@ import gov.llnl.babel.backend.IOR;
 import gov.llnl.babel.backend.Utilities;
 import gov.llnl.babel.backend.c.C;
 import gov.llnl.babel.backend.fortran.Fortran;
+import gov.llnl.babel.backend.mangler.FortranMangler;
+import gov.llnl.babel.backend.mangler.NameMangler;
 import gov.llnl.babel.backend.mangler.NonMangler;
 import gov.llnl.babel.backend.rmi.RMI;
 import gov.llnl.babel.backend.rmi.RMIStubSource;
@@ -52,9 +54,11 @@ import gov.llnl.babel.symbols.Comment;
 import gov.llnl.babel.symbols.Enumeration;
 import gov.llnl.babel.symbols.Extendable;
 import gov.llnl.babel.symbols.Method;
+import gov.llnl.babel.symbols.Struct;
 import gov.llnl.babel.symbols.Symbol;
 import gov.llnl.babel.symbols.SymbolID;
 import gov.llnl.babel.symbols.Type;
+import java.security.NoSuchAlgorithmException;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.HashSet;
@@ -209,7 +213,8 @@ public class StubSource {
    */
   public static boolean declareArgument(LanguageWriter writer, String argName, 
                                         Type argType, boolean needComma,
-                                        Context context)
+                                        Context context,
+                                        Method method)
     throws CodeGenerationException
   {
     final String pre = "fortran.StubSource.declareArgument: ";
@@ -236,11 +241,22 @@ public class StubSource {
     case Type.LONG:
     case Type.CLASS:
     case Type.INTERFACE:
-    case Type.STRUCT:
       writer.print(Fortran.getFortranTypeInC(argType, context) +
                    " *" + argName);
       needComma = true;
       break;
+    case Type.STRUCT:
+      //breaking out STRUCT case for STRUCTS that are function return vars.
+        Type returnType = method.getReturnType();
+        if (returnType.getDetailedType() == Type.STRUCT) {
+          writer.print( IOR.getReturnString(argType, context, true, false) +
+                    " *" + argName);
+        } else {
+          writer.print(Fortran.getFortranTypeInC(argType, context) +
+                    " *" + argName);
+        }
+      needComma = true;
+      break;
     case Type.OPAQUE:
       writer.print(Fortran.getFortranTypeInC(argType, context) +
                    " *" + argName);
@@ -279,9 +295,24 @@ public class StubSource {
    */
   public static void generateMethodSymbol(LanguageWriter writer,
                                           String methodName,
-                                          Context context)
+                                          Context context,
+                                          Type type,
+                                          boolean fromSkelFunc)
+    throws CodeGenerationException
   {
-    writer.println("void");
+    if (fromSkelFunc && Fortran.hasBindC(context)){
+      switch(type.getDetailedType()) {
+      case Type.BOOLEAN:
+        writer.println(Fortran.getFortranPrefix(context) + 
+                       "_Bool ");
+        break;
+      default:
+        writer.println(IOR.getReturnString(type,context, true, false));
+        break;
+      }
+    } else {
+      writer.println("void");
+    }
     writer.println(Fortran.getFortranSymbol(context) + 
                    "(" + methodName.toLowerCase() 
                    + ',' + methodName.toUpperCase() + ',' + methodName + ")");
@@ -306,16 +337,27 @@ public class StubSource {
    */
   private static boolean declareNormalArguments(LanguageWriter writer,
                                                 Iterator i,
-                                                Context context)
+                                                Context context,
+                                                Method method,
+                                                boolean gotFunc)
     throws CodeGenerationException
   {
     boolean needComma = false;
     while (i.hasNext()) {
       needComma = comma(writer, needComma);
       Argument a = (Argument)i.next();
-      needComma = 
-        declareArgument(writer, a.getFormalName(), a.getType(), needComma,
-                        context);
+      String argName = a.getFormalName();
+      if (gotFunc && Fortran.hasBindC(context)) {
+        if ( !(argName.equals(Fortran.s_return))){
+          needComma = 
+            declareArgument(writer, a.getFormalName(), a.getType(), needComma,
+                          context, method);
+        }
+      } else {
+        needComma = 
+          declareArgument(writer, a.getFormalName(), a.getType(), needComma,
+                        context, method);
+      }
     }
     return needComma;
   }
@@ -349,7 +391,7 @@ public class StubSource {
           writer.printlnUnformatted(charCheck(context));
         }
         writer.println(Fortran.getFortranPrefix(context) +
-                       "_STR_FAR_LEN_DECL(" 
+                      "_STR_FAR_LEN_DECL(" 
           + a.getFormalName() + ")");
         if (Type.CHAR == t) {
           writer.printlnUnformatted("#endif");
@@ -368,25 +410,30 @@ public class StubSource {
    * @param methodName the name of the function.
    * @param arguments   a list of {@link gov.llnl.babel.symbols.Argument
    *                   Argument} objects.
+   * @param fromSkelFunc   Need different logic to deal with Fortran Functions in
+   *                       the skel file.
    * @exception gov.llnl.babel.backend.CodeGenerationException
    *    something went wrong -- probably an unsupported type.
    */
   public static void generateSignature(LanguageWriter writer,
                                        String methodName, 
                                        List arguments,
-                                       Context context)
+                                       Context context,
+                                       Method method,
+                                       boolean fromSkelFunc)
     throws CodeGenerationException
   {
     try {
       writer.pushLineBreak(false);
-      generateMethodSymbol(writer, methodName, context);
+      generateMethodSymbol(writer, methodName, context,method.getReturnType(),fromSkelFunc);
+
       if (arguments.isEmpty()) {
         writer.print("void");
       } else {
         boolean needComma;
         writer.tab();
         needComma = declareNormalArguments(writer, arguments.iterator(), 
-                                           context);
+                                           context, method,fromSkelFunc);
         needComma =
           declareExtraArguments(writer, arguments.iterator(), needComma,
                                 context);
@@ -742,10 +789,22 @@ public class StubSource {
    * Generates the super methods
    */
   private void generateSuperMethods(Class cls)  throws CodeGenerationException {
+    NameMangler mang;
+    try {
+      if (Fortran.needsAbbrev(d_context)) {
+        mang = new FortranMangler(AbbrevHeader.MAXNAME,
+                                  AbbrevHeader.MAXUNMANGLED);
+      }
+      else {
+        mang = new NonMangler();
+      }
+    } catch (NoSuchAlgorithmException ex) {
+      throw new CodeGenerationException("Caught NoSuchAlgorithm exception from new FortranMangler");
+    }
     Collection methods = cls.getOverwrittenClassMethods();
     for (Iterator m = methods.iterator(); m.hasNext(); ) {
       Method method = (Method) m.next();
-      extendAndGenerateSuper(cls, method, cls.getSymbolID(), false);
+      extendAndGenerateSuper(cls, method, cls.getSymbolID(), false, mang);
     }
   }
 
@@ -847,7 +906,9 @@ public class StubSource {
         }
       } else {
         if (Argument.IN == a.getMode()) {
-          d_writer.print("*");
+          if(a.getType().getDetailedType()!=Type.STRUCT) {
+            d_writer.print("*");
+          } 
         }
         d_writer.print(varName);
       }
@@ -995,7 +1056,7 @@ public class StubSource {
     throws CodeGenerationException 
   {
     d_writer.writeComment(m, false);
-    generateSignature(d_writer, name, argumentsWithIndices, d_context);
+    generateSignature(d_writer, name, argumentsWithIndices, d_context,m,false);
     d_writer.println();
     d_writer.println("{");
     d_writer.tab();
@@ -1140,6 +1201,8 @@ public class StubSource {
     writer.generateSystemInclude("stdlib.h");
     writer.generateSystemInclude("string.h");
     writer.generateInclude("sidlfortran.h", false);
+    writer.generateInclude("sidl_io_Serializer.h", true);
+    writer.generateInclude("sidl_io_Deserializer.h", true);
     if (Fortran.isFortran90(context) || Fortran.hasBindC(context)) {
       writer.generateInclude("sidlf90array.h", true);
     }
@@ -1211,10 +1274,10 @@ public class StubSource {
    *  a catch all exception for problems in the code generation phase.
    */
   private void extendAndGenerateSuper(Class cls, Method m, SymbolID id,
-                                      boolean isInterface) 
+                                      boolean isInterface, NameMangler mang) 
     throws CodeGenerationException
   {
-    String name = Fortran.getMethodSuperName(id, m, new NonMangler(), d_context);
+    String name = Fortran.getMethodSuperName(id, m, mang, d_context);
     List extendedArgs = extendArgs(id, m, d_context, false);
     List extendedArgsWithIndices = extendArgs(id, m, d_context, true);
     generateMethod(cls, name, extendedArgs, extendedArgsWithIndices,
@@ -1388,7 +1451,7 @@ public class StubSource {
     d_writer.println();
     d_writer.println();
     d_writer.writeComment("Remote Constructor for the class.", false);
-    generateSignature(d_writer, name, argumentsWithIndicies, d_context);
+    generateSignature(d_writer, name, argumentsWithIndicies, d_context,m,false);
     d_writer.println();
     d_writer.println("{");
     d_writer.tab();
@@ -1429,7 +1492,7 @@ public class StubSource {
     String cName = C.getFullMethodName(ext.getSymbolID(), "_remoteConnect");
 
     d_writer.writeComment("Remote Connector for the class.", false);
-    generateSignature(d_writer, name, argumentsWithIndicies, d_context);
+    generateSignature(d_writer, name, argumentsWithIndicies, d_context,m,false);
     d_writer.println();
     d_writer.println("{");
     d_writer.tab();
@@ -1453,7 +1516,7 @@ public class StubSource {
     List extendedArgs = extendArgs(id, m, d_context, false);
     String name = Fortran.getMethodStubName(id, m, d_context);
     d_writer.writeComment(m, false);
-    generateSignature(d_writer, name, extendedArgs, d_context);
+    generateSignature(d_writer, name, extendedArgs, d_context,m,false);
     d_writer.println();
     d_writer.println("{");
     d_writer.tab();
@@ -1520,7 +1583,9 @@ public class StubSource {
                    args, args, m, id, isInterface, false);
   }
 
-  private void generateRMIForwardDecls(Extendable ext) {
+  private void generateRMIForwardDecls(Extendable ext) 
+    throws CodeGenerationException
+  {
     SymbolID id = ext.getSymbolID();
 
     d_writer.printlnUnformatted("#ifdef WITH_RMI");
@@ -1536,6 +1601,41 @@ public class StubSource {
                      + C.getFullMethodName(id,"_IHConnect")  
                      + "(struct sidl_rmi_InstanceHandle__object *instance, struct sidl_BaseInterface__object **_ex);");
     d_writer.printlnUnformatted("#endif /*WITH_RMI*/");
+
+    Iterator i = IOR.getStructSymbolIDs(ext, true).iterator();
+    while (i.hasNext()) {
+      final SymbolID sid = (SymbolID)i.next();
+      writeStructSerializeSig(d_writer, sid, true);
+      d_writer.printlnUnformatted("#define RMI_" +
+                                  IOR.getSymbolName(sid) + "_serialize(strct, pipe, name, copyArg, exc) { \\");
+      d_writer.printlnUnformatted("  sidl_io_Serializer __pipe = sidl_io_Serializer__cast((pipe), (exc)); SIDL_CHECK(*(exc)); \\");
+      d_writer.printlnUnformatted("  if (__pipe) { \\");
+      d_writer.printlnUnformatted("    sidl_BaseInterface __throwaway__; \\");
+      d_writer.printlnUnformatted("    " +     
+                                  IOR.getSymbolName(sid) + 
+                                  "__serialize_c((strct),__pipe, (name), (copyArg), (exc)); \\");
+      d_writer.printlnUnformatted("    sidl_io_Serializer_deleteRef(__pipe, &__throwaway__); \\");
+      d_writer.printlnUnformatted("  } \\");
+      d_writer.printlnUnformatted("}");
+      d_writer.println();
+    }
+    i = IOR.getStructSymbolIDs(ext, false).iterator();
+    while (i.hasNext()) {
+      final SymbolID sid = (SymbolID)i.next();
+      writeStructSerializeSig(d_writer, sid, false);
+      d_writer.printlnUnformatted("#define RMI_" +
+                                  IOR.getSymbolName(sid) + "_deserialize(strct, pipe, name, copyArg, exc) { \\");
+      d_writer.printlnUnformatted("  sidl_io_Deserializer __pipe = sidl_io_Deserializer__cast((pipe), (exc)); SIDL_CHECK(*(exc)); \\");
+      d_writer.printlnUnformatted("  if (__pipe) { \\");
+      d_writer.printlnUnformatted("    sidl_BaseInterface __throwaway__; \\");
+      d_writer.printlnUnformatted("    " +     
+                                  IOR.getSymbolName(sid) + 
+                                  "__deserialize_c((strct),__pipe, (name), (copyArg), (exc)); \\");
+      d_writer.printlnUnformatted("    sidl_io_Deserializer_deleteRef(__pipe, &__throwaway__); \\");
+      d_writer.printlnUnformatted("  } \\");
+      d_writer.printlnUnformatted("}");
+      d_writer.println();
+    }
   }
 
   private void generateBuiltins(Extendable ext)
@@ -1690,6 +1790,55 @@ public class StubSource {
     }
   }
 
+  private Set referencedStructs(Struct strct)
+    throws CodeGenerationException
+  {
+    HashSet result = new HashSet();
+    Iterator i = strct.getSymbolReferences().iterator();
+    while (i.hasNext()) {
+      SymbolID id = (SymbolID)i.next();
+      Symbol sym = Utilities.lookupSymbol(d_context, id);
+      if (sym instanceof Struct) {
+        result.add(sym);
+      }
+    }
+    return result;
+  }
+
+  private void generateStruct(Struct strct)
+    throws CodeGenerationException
+  {
+    LanguageWriterForC cwriter = (LanguageWriterForC)d_writer;
+    d_writer.writeBanner(strct, Fortran.getStubFile(strct), false,
+                         CodeConstants.C_DESC_STUB_PREFIX + 
+                         strct.getFullName());
+    d_writer.writeComment(strct, false);
+    cwriter.generateInclude(IOR.getHeaderFile(strct), false);
+    cwriter.printlnUnformatted("#include <string.h>");
+    cwriter.printlnUnformatted("#include <stdlib.h>");
+    cwriter.generateInclude("sidl_BaseInterface_IOR.h", true);
+    cwriter.generateInclude("sidl_io_Serializer.h", true);
+    cwriter.generateInclude("sidl_io_Deserializer.h", true);
+    cwriter.generateInclude("sidl_Exception.h", true);
+    Set refs = referencedStructs(strct);
+    Iterator i = refs.iterator();
+    while (i.hasNext()) {
+      final SymbolID id = (SymbolID)i.next();
+      writeStructSerializeSig(d_writer, id, true);
+      d_writer.printlnUnformatted("#define RMI_" +
+                                  IOR.getSymbolName(id) + "_serialize " +
+                                  IOR.getSymbolName(id) + "__serialize_c");
+      writeStructSerializeSig(d_writer, id, false);
+      d_writer.printlnUnformatted("#define RMI_" +
+                                  IOR.getSymbolName(id) + "_deserialize " +
+                                  IOR.getSymbolName(id) + "__deserialize_c");
+    }
+    d_writer.println();
+    writeStructSerialize(strct, true);
+    writeStructSerialize(strct, false);
+  }
+    
+
   /**
    * Generate a C file to provide FORTRAN stubs for a sidl
    * object/interface. The stubs allow FORTRAN clients to make calls on
@@ -1706,6 +1855,9 @@ public class StubSource {
   public void generateCode(Symbol symbol) throws CodeGenerationException {
     final String pre = "fortran.StubSource.generateCode: ";
     switch(symbol.getSymbolType()) {
+    case Symbol.STRUCT:
+      generateStruct((Struct)symbol);
+      break;
     case Symbol.CLASS:
     case Symbol.INTERFACE:
       if (d_writer instanceof LanguageWriterForC) {
@@ -1746,5 +1898,41 @@ public class StubSource {
     source.generateCode(ext);
   }
 
-  
+  public static void writeStructSerializeSig(LanguageWriter writer,
+                                             SymbolID structid,
+                                             boolean serialize)
+    throws CodeGenerationException
+  {
+    final String pipeType = 
+      (serialize ? "sidl_io_Serializer" : "sidl_io_Deserializer");
+    writer.printlnUnformatted("#ifdef __cplusplus");
+    writer.println("extern \"C\"");
+    writer.printlnUnformatted("#endif /* __cplusplus */");
+    writer.println("void");
+    writer.println(Fortran.structSerializeStub(structid, serialize) + "(");
+    writer.tab();
+    writer.println((serialize ? "const " : "") + 
+                   IOR.getStructName(structid) + " *arg,");
+    writer.println("struct " + pipeType + "__object *pipe,");
+    writer.println("const char *,");
+    writer.println("sidl_bool,");
+    writer.println("sidl_BaseInterface *);");
+    writer.backTab();
+    writer.println();
+  }
+
+  private void writeStructSerialize(Struct strct,
+                                    boolean serialize)
+    throws CodeGenerationException
+  {
+    writeStructSerializeSig(d_writer, strct, serialize);
+    if (serialize) {
+      gov.llnl.babel.backend.c.StubSource.
+        writeStructSerialize(d_writer, strct, "_c");
+    }
+    else {
+      gov.llnl.babel.backend.c.StubSource.
+        writeStructDeserialize(d_writer, strct, "_c", d_context);
+    }
+  }
 }
