@@ -1,10 +1,10 @@
 //
 // File:        ClientJNI.java
 // Package:     gov.llnl.babel.backend.jdk
-// Revision:    @(#) $Id: ClientJNI.java 6201 2007-10-26 06:15:35Z kumfert $
+// Revision:    @(#) $Id: ClientJNI.java 6482 2008-08-21 22:50:53Z epperly $
 // Description: write Java client (stub) JNI code that links Java with the IOR
 //
-// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -236,7 +236,7 @@ public class ClientJNI {
      * If this is a class with static methods, then output a pointer to the
      * static EPV structure that will be initialized at class loading.
      */
-    if (ext.hasStaticMethod(false)) {
+    if (ext.hasStaticMethod(true)) {
       d_writer.writeComment("External reference to static EPV.", false);
       d_writer.println("static const "
                        + IOR.getSEPVName(id)
@@ -261,12 +261,12 @@ public class ClientJNI {
       d_writer.backTab();
       d_writer.println("{");
       d_writer.tab();
-      d_writer.println("(void) env;");
-      d_writer.println("(void) cls;");
       d_writer.println(IOR.getExceptionFundamentalType() +
                        "_ex = NULL;");
       d_writer.println(
                        "jlong _res = POINTER_TO_JLONG((*s_external->createObject)(NULL, &_ex));");
+      d_writer.println("(void) env;");
+      d_writer.println("(void) cls;");
       generateCheckRuntimeException(Type.LONG);
       d_writer.println("return _res;");
       d_writer.backTab();
@@ -285,11 +285,9 @@ public class ClientJNI {
       d_writer.backTab();
       d_writer.println("{");
       d_writer.tab();
-      d_writer.println("(void) env;");
-      d_writer.println("(void) cls;");
-
       d_writer.println("jlong _res = 0;");
 
+
       d_writer.println();
       d_writer.printlnUnformatted("#ifdef WITH_RMI");
       d_writer.println();
@@ -303,6 +301,8 @@ public class ClientJNI {
       d_writer.println();
       d_writer.printlnUnformatted("#endif /*WITH_RMI*/");
       d_writer.println();
+      d_writer.println("(void) env;");
+      d_writer.println("(void) cls;");
 
       d_writer.println("return _res;");
 
@@ -323,8 +323,6 @@ public class ClientJNI {
         d_writer.backTab();
         d_writer.println("{");
         d_writer.tab();
-        d_writer.println("(void) env;");
-        d_writer.println("(void) cls;");
         d_writer.println(IOR.getExceptionFundamentalType() +
                          "_ex = NULL;");
         d_writer.println("jlong _res = 0;");
@@ -332,6 +330,8 @@ public class ClientJNI {
         
         d_writer.println(IOR.getObjectName(id) +
                          " *_ptr = (*s_external->createObject)(g_data, &_ex);");
+        d_writer.println("(void) env;");
+        d_writer.println("(void) cls;");
         generateCheckRuntimeException(Type.LONG);
         
         //d_writer.println("(*_ptr->d_epv->f_addRef)(_ptr, &_ex);");
@@ -354,8 +354,6 @@ public class ClientJNI {
     d_writer.backTab();
     d_writer.println("{");
     d_writer.tab();
-    d_writer.println("(void) env;");
-    d_writer.println("(void) cls;");
     d_writer.println("jlong _res = 0;");
 
     d_writer.println();
@@ -372,6 +370,8 @@ public class ClientJNI {
     d_writer.println();
     d_writer.printlnUnformatted("#endif /*WITH_RMI*/");
     d_writer.println();
+    d_writer.println("(void) env;");
+    d_writer.println("(void) cls;");
     
     d_writer.println("return _res;");
     d_writer.backTab();
@@ -481,7 +481,7 @@ public class ClientJNI {
 
 
     final int nmethods = countMethods(ext) + (ext.isAbstract() ? 3 : 5) + 
-      numOverwritten + numNonblocking + (ext.hasStaticMethod(false) ? 1 : 0) +
+      numOverwritten + numNonblocking + (ext.hasStaticMethod(true) ? 1 : 0) +
       ((!IOR.isSIDLSymbol(id) && !ext.isAbstract()) ? 1 : 0);
     
     d_writer.writeComment("Register JNI methods with the Java JVM.", false);
@@ -524,7 +524,7 @@ public class ClientJNI {
       /*
        * If this class has a static method, then initialize the static pointer.
        */
-      if (ext.hasStaticMethod(false)) {
+      if (ext.hasStaticMethod(true)) {
         d_writer.println("s_sepv = (*(s_external->getStaticEPV))();");
         d_writer.println();
       }
@@ -568,7 +568,7 @@ public class ClientJNI {
       generateRegistryEntry(IOR.getBuiltinMethod(IOR.HOOKS, id, d_context, false),
                             idx, false);
       ++idx;      
-      if (ext.hasStaticMethod(false)) {
+      if (ext.hasStaticMethod(true)) {
         generateRegistryEntry(IOR.getBuiltinMethod(IOR.HOOKS, id, d_context, true),
                               idx, false);
         ++idx;      
