@@ -1,10 +1,10 @@
 //
 // File:        Java.java
 // Package:     gov.llnl.babel.backend.jdk
-// Revision:    @(#) $Id: Java.java 6196 2007-10-24 23:42:37Z epperly $
+// Revision:    @(#) $Id: Java.java 6482 2008-08-21 22:50:53Z epperly $
 // Description: common Java binding routines shared by Java code generators
 //
-// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -149,6 +149,7 @@ public class Java {
     s_java_arr.put(s_float,    "sidl.Float$Array#");
     s_java_arr.put(s_int,      "sidl.Integer$Array#");
     s_java_arr.put(s_long,     "sidl.Long$Array#");
+    s_java_arr.put(s_enum,     "sidl.Enum$Array#");
     s_java_arr.put(s_opaque,   "sidl.Opaque$Array#");
     s_java_arr.put(s_string,   "sidl.String$Array#");
 
@@ -527,7 +528,15 @@ public class Java {
    *
    */
   public static String getEnsureArray(Type arrayType) {
-    return "sidl_" + arrayType.getTypeString() + "__array_ensure";
+    switch(arrayType.getDetailedType()) {
+    case Type.ENUM:
+      return "sidl_long__array_ensure";
+    case Type.CLASS:
+    case Type.INTERFACE:
+      return "sidl_interface__array_ensure";
+    default:
+      return "sidl_" + arrayType.getTypeString() + "__array_ensure";
+    }
   }
     
   /**
@@ -949,7 +958,9 @@ public class Java {
         if (null != arrayType) {
           arrayTypeInteger = new Integer(arrayType.getDetailedType());
           
-          arrayTypeString = arrayType.isSymbol() 
+          arrayTypeString = 
+            (arrayType.getDetailedType() == Type.CLASS ||
+             arrayType.getDetailedType() == Type.INTERFACE)
             ? arrayType.getSymbolID().getFullName() 
             : ((String)s_java_arr.get(arrayTypeInteger)).
                 replace('#', Character.forDigit(type.getArrayDimension(), 10));
@@ -1001,7 +1012,9 @@ public class Java {
         if (null != arrayType) {
           arrayTypeInteger = new Integer(arrayType.getDetailedType());
           
-          arrayTypeString = arrayType.isSymbol() 
+          arrayTypeString = 
+            (arrayType.getDetailedType() == Type.CLASS ||
+             arrayType.getDetailedType() == Type.INTERFACE)
             ? arrayType.getSymbolID().getFullName() 
             : ((String)s_java_arr.get(arrayTypeInteger)).
             replace('#', Character.forDigit(type.getArrayDimension(), 10));
@@ -1090,7 +1103,9 @@ public class Java {
       if (null != arrayType) {
         arrayTypeInteger = new Integer(arrayType.getDetailedType());
       
-        arrayTypeString = arrayType.isSymbol() 
+        arrayTypeString = 
+          (arrayType.getDetailedType() == Type.CLASS ||
+           arrayType.getDetailedType() == Type.INTERFACE)
           ?  (arrayType.getSymbolID().getFullName() + "$Array" 
              + type.getArrayDimension()) 
           : ((String)s_java_arr.get(arrayTypeInteger)).
@@ -1172,7 +1187,9 @@ public class Java {
         if (null != arrayType) {
           arrayTypeInteger = new Integer(arrayType.getDetailedType());
         
-          arrayTypeString = arrayType.isSymbol() 
+          arrayTypeString = 
+            (arrayType.getDetailedType() == Type.CLASS ||
+             arrayType.getDetailedType() == Type.INTERFACE)
             ? arrayType.getSymbolID().getFullName() 
             : ((String)s_java_arr.get(arrayTypeInteger)).
             replace('#', Character.forDigit(type.getArrayDimension(), 10));
