@@ -1,10 +1,10 @@
 //
 // File:        ServerJNI.java
 // Package:     gov.llnl.babel.backend.jdk
-// Revision:    @(#) $Id: ServerJNI.java 6201 2007-10-26 06:15:35Z kumfert $
+// Revision:    @(#) $Id: ServerJNI.java 6482 2008-08-21 22:50:53Z epperly $
 // Description: write Java server (skel) JNI code that links Java with the IOR
 //
-// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -207,8 +207,8 @@ public class ServerJNI {
     for (Iterator m = methods.iterator(); m.hasNext();) {
       Method method = (Method) m.next();
       if (d_ext.isInterface() || !method.isAbstract()) {
-        if (IOR.supportHooks(d_ext, d_context)) {
-          Method pre = method.spawnPreHook();
+        if (IOR.generateHookMethods(d_ext, d_context)) {
+          Method pre  = method.spawnPreHook();
           Method post = method.spawnPostHook();
           generateMethod(pre);
           generateMethod(method);
@@ -303,8 +303,8 @@ public class ServerJNI {
       if (!method.isAbstract()) {
         d_writer.println("jmethodID "
                          + Java.getJavaServerMethodName(method) + "MID;");
-        if (IOR.supportHooks(d_ext, d_context)) {
-          Method pre = method.spawnPreHook();
+        if (IOR.generateHookMethods(d_ext, d_context)) {
+          Method pre  = method.spawnPreHook();
           Method post = method.spawnPostHook();
           d_writer.println("jmethodID "
                            + Java.getJavaServerMethodName(pre) + "MID;");
@@ -341,8 +341,8 @@ public class ServerJNI {
     for (Iterator m = methods.iterator(); m.hasNext();) {
       Method method = (Method) m.next();
       if (!method.isAbstract()) {
-        if (IOR.supportHooks(d_ext, d_context)) {
-          Method pre = method.spawnPreHook();
+        if (IOR.generateHookMethods(d_ext, d_context)) {
+          Method pre  = method.spawnPreHook();
           Method post = method.spawnPostHook();
           String preDescriptor = Java.getJavaServerSignature(pre);
           String postDescriptor = Java.getJavaServerSignature(post);
@@ -530,14 +530,16 @@ public class ServerJNI {
     SymbolID id = d_ext.getSymbolID();
     d_writer.openCxxExtern();
     d_writer.println("void");
-    if(IOR.supportHooks(id, d_context)) {
-      d_writer.println(C.getSetEPVName(id) + "(" + IOR.getEPVName(id) + " *"+s_epv+",");
+    d_writer.print(C.getSetEPVName(id) + "(" + IOR.getEPVName(id) + " *");
+    d_writer.print(s_epv);
+    if (IOR.generateHookMethods(d_ext, d_context)) {
+      d_writer.println(",");
       d_writer.tab();
-      d_writer.println(IOR.getPreEPVName(id) + " *"+s_pre_epv+", "+
-                     IOR.getPostEPVName(id) + " *"+s_post_epv+")");
+      d_writer.print(IOR.getPreEPVName(id) + " *" + s_pre_epv + ", ");
+      d_writer.println(IOR.getPostEPVName(id) + " *" + s_post_epv + ")");
       d_writer.backTab();
     } else {
-      d_writer.println(C.getSetEPVName(id) + "(" + IOR.getEPVName(id) + " *"+s_epv+")");
+      d_writer.println(")");
     }
     d_writer.println("{");
     d_writer.tab();
@@ -547,9 +549,12 @@ public class ServerJNI {
     d_writer.println();
 
     d_writer.writeCommentLine("initialize builtin methods");
-    initializeEPVMethodPointer(IOR.getBuiltinMethod(IOR.CONSTRUCTOR, id, d_context), id, s_epv);
-    initializeEPVMethodPointer(IOR.getBuiltinMethod(IOR.CONSTRUCTOR2, id, d_context), id, s_epv); 
-    initializeEPVMethodPointer(IOR.getBuiltinMethod(IOR.DESTRUCTOR, id, d_context), id, s_epv);
+    initializeEPVMethodPointer(IOR.getBuiltinMethod(IOR.CONSTRUCTOR, id, 
+                                                    d_context), id, s_epv);
+    initializeEPVMethodPointer(IOR.getBuiltinMethod(IOR.CONSTRUCTOR2, id, 
+                                                    d_context), id, s_epv); 
+    initializeEPVMethodPointer(IOR.getBuiltinMethod(IOR.DESTRUCTOR, id, 
+                                                    d_context), id, s_epv);
     d_writer.println();
 
     d_writer.writeCommentLine("initialize local methods");
@@ -559,8 +564,8 @@ public class ServerJNI {
       if ((m.getDefinitionModifier() == Method.NORMAL)
           || (m.getDefinitionModifier() == Method.FINAL)) {
         initializeEPVMethodPointer(m, id, s_epv);
-          if (IOR.supportHooks(d_ext, d_context)) {
-            Method pre = m.spawnPreHook();
+          if (IOR.generateHookMethods(d_ext, d_context)) {
+            Method pre  = m.spawnPreHook();
             Method post = m.spawnPostHook();
             initializeEPVMethodPointer(pre, id, s_pre_epv);
             initializeEPVMethodPointer(post, id, s_post_epv);
@@ -581,16 +586,16 @@ public class ServerJNI {
     SymbolID id = d_ext.getSymbolID();
     d_writer.openCxxExtern();
     d_writer.println("void");
-    if(IOR.supportHooks(id, d_context)) {
-      d_writer.println(C.getSetSEPVName(id) + "(" + IOR.getSEPVName(id) + " *"+s_sepv+",");
+    d_writer.print(C.getSetSEPVName(id) + "(" + IOR.getSEPVName(id));
+    d_writer.print(" *" + s_sepv);
+    if (IOR.generateHookMethods(d_ext, d_context)) {
+      d_writer.println(",");
       d_writer.tab();
-      d_writer.println(IOR.getPreSEPVName(id) + " *"+s_pre_sepv+", "+
-                     IOR.getPostSEPVName(id) + " *"+s_post_sepv+")");
+      d_writer.print(IOR.getPreSEPVName(id) + " *" + s_pre_sepv + ", ");
+      d_writer.println(IOR.getPostSEPVName(id) + " *" + s_post_sepv + ")");
       d_writer.backTab();
     } else {
-      d_writer.println(C.getSetSEPVName(id) + "(" + IOR.getSEPVName(id) 
-      + " *" + s_sepv + ")");
-
+      d_writer.println(")");
     }
 
     d_writer.println("{");
@@ -606,8 +611,8 @@ public class ServerJNI {
       Method m = (Method) i.next();
       if (m.getDefinitionModifier() == Method.STATIC) {
         initializeEPVMethodPointer(m, id, s_sepv);
-        if (IOR.supportHooks(d_ext, d_context)) {
-          Method pre = m.spawnPreHook();
+        if (IOR.generateHookMethods(d_ext, d_context)) {
+          Method pre  = m.spawnPreHook();
           Method post = m.spawnPostHook();
           initializeEPVMethodPointer(pre, id,s_pre_sepv);
           initializeEPVMethodPointer(post, id,s_post_sepv);
