@@ -1,11 +1,11 @@
 //
 // File:        Python.java
 // Package:     gov.llnl.babel.backend.python
-// Revision:    @(#) $Revision: 6201 $
-// Date:        $Date: 2007-10-25 23:15:35 -0700 (Thu, 25 Oct 2007) $
+// Revision:    @(#) $Revision: 6482 $
+// Date:        $Date: 2008-08-21 15:50:53 -0700 (Thu, 21 Aug 2008) $
 // Description: Python naming conventions shared by Python code generators
 // 
-// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -39,6 +39,7 @@ import gov.llnl.babel.backend.CodeSplicer;
 import gov.llnl.babel.backend.IOR;
 import gov.llnl.babel.backend.Utilities;
 import gov.llnl.babel.backend.c.C;
+import gov.llnl.babel.backend.writers.LanguageWriter;
 import gov.llnl.babel.backend.writers.LanguageWriterForC;
 import gov.llnl.babel.backend.writers.LanguageWriterForPython;
 import gov.llnl.babel.symbols.Argument;
@@ -549,12 +550,15 @@ public class Python {
       return "sidl_generic_borrow_python_array";
     }
     else {
-      if (arrayType.getType() != Type.SYMBOL) {
-        return "sidl_" + arrayType.getTypeString() + "__borrow_python_array";
-      }
-      else {
+      switch (arrayType.getDetailedType()) {
+      case Type.INTERFACE:
+      case Type.CLASS:
         return arrayType.getSymbolID().getFullName().replace('.', '_')
           + "__convert_python_array";
+      case Type.ENUM:
+        return "sidl_long__borrow_python_array";
+      default:
+        return "sidl_" + arrayType.getTypeString() + "__borrow_python_array";
       }
     }
   }
@@ -565,12 +569,13 @@ public class Python {
         + "__convert_generic_array";
     }
     else {
-      if (arrayType.getType() != Type.SYMBOL) {
-        return "sidl_python_borrow_array";
-      }
-      else {
+      switch(arrayType.getDetailedType()) {
+      case Type.INTERFACE:
+      case Type.CLASS:
         return arrayType.getSymbolID().getFullName().replace('.', '_')
           + "__convert_sidl_array";
+      default:
+        return "sidl_python_borrow_array";
       }
     }
   }
@@ -581,23 +586,24 @@ public class Python {
       return "sidl_generic_borrow_python_array";
     }
     else {
-      if (elemType.getType() != Type.SYMBOL) {
-        if (arrayType.getArrayOrder() == Type.COLUMN_MAJOR) {
-          return "sidl_" + elemType.getTypeString() + 
-            "__clone_python_array_column";
-        }
-        else if (arrayType.getArrayOrder() == Type.ROW_MAJOR) {
-          return "sidl_" + elemType.getTypeString() + 
-            "__clone_python_array_row";
-        }
-        else {
-          return "sidl_" + elemType.getTypeString() + 
-            "__borrow_python_array";
-        }
-      }
-      else {
+      switch(elemType.getDetailedType()) {
+      case Type.CLASS:
+      case Type.INTERFACE:
         return elemType.getSymbolID().getFullName().replace('.', '_')
           + "__convert_python_array";
+      default:
+        final String atype = 
+          (elemType.getDetailedType() == Type.ENUM) ? "long" :
+          elemType.getTypeString();
+        if (arrayType.getArrayOrder() == Type.COLUMN_MAJOR) {
+          return "sidl_" + atype + "__clone_python_array_column";
+        }
+        else if (arrayType.getArrayOrder() == Type.ROW_MAJOR) {
+          return "sidl_" + atype + "__clone_python_array_row";
+        }
+        else {
+          return "sidl_" + atype + "__borrow_python_array";
+        }
       }
     }
   }
@@ -608,16 +614,33 @@ public class Python {
         + "__convert_generic_array";
     }
     else {
-      if (arrayType.getType() != Type.SYMBOL) {
-        return "sidl_python_clone_array";
-      }
-      else {
+      switch(arrayType.getDetailedType()) {
+      case Type.CLASS:
+      case Type.INTERFACE:
         return arrayType.getSymbolID().getFullName().replace('.', '_')
           + "__convert_sidl_array";
+      default:
+        return "sidl_python_clone_array";
       }
     }
   }
 
+  public static void leavePython(LanguageWriter lw)
+  {
+    lw.printlnUnformatted("#if (PY_VERSION_HEX >= 0x02040000)");
+    lw.println("Py_BEGIN_ALLOW_THREADS");
+    lw.println("sidl_Python_LogUnlock(__func__, __FILE__, __LINE__);");
+    lw.printlnUnformatted("#endif /* Python 2.4 or later */");
+  }
+
+  public static void resumePython(LanguageWriter lw)
+  {
+    lw.printlnUnformatted("#if (PY_VERSION_HEX >= 0x02040000)");
+    lw.println("Py_END_ALLOW_THREADS");
+    lw.println("sidl_Python_LogRelock(__func__, __FILE__, __LINE__);");
+    lw.printlnUnformatted("#endif /* Python 2.4 or later */");
+  }
+
   public static String getDestroyArray(Type arrayType) {
     return "sidl_python_deleteRef_array";
   }
