@@ -1,11 +1,11 @@
 //
 // File:        PythonClientCSource.java
 // Package:     gov.llnl.babel.backend.python
-// Revision:    @(#) $Revision: 6210 $
-// Date:        $Date: 2007-10-29 16:12:37 -0700 (Mon, 29 Oct 2007) $
+// Revision:    @(#) $Revision: 6494 $
+// Date:        $Date: 2008-09-04 16:24:45 -0700 (Thu, 04 Sep 2008) $
 // Description: Generate the C source file for a Python for an extendable
 // 
-// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -58,6 +58,7 @@ import java.util.Collection;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.LinkedList;
+import java.util.List;
 import java.util.Set;
 
 
@@ -166,6 +167,8 @@ public class PythonClientCSource extends PythonClientC {
     d_lw.printlnUnformatted("#endif");
     addInclude("sidl_Loader.h", true);
     addInclude("sidl_header.h", true);
+    addInclude("sidl_String.h", true);
+    addInclude("sidl_Python.h", true);
     addInclude("sidl_interface_IOR.h", true);
     addInclude(Python.getCHeaderPath(d_context.getSymbolTable().lookupSymbol("sidl.rmi.NetworkException"), "Module"), false);
     Iterator i = Utilities.convertIdsToSymbols(d_context,
@@ -223,6 +226,10 @@ public class PythonClientCSource extends PythonClientC {
                    IOR.getExternalName(d_ext.getSymbolID()) + 
                    " *_implEPV = NULL;");
       d_lw.println();
+
+      if (!BabelConfiguration.isSIDLBaseClass(d_ext)) {
+        writeLoadImpl();
+      }
     }
   }
 
@@ -275,8 +282,13 @@ public class PythonClientCSource extends PythonClientC {
       d_lw.tab();
       d_lw.println(IOR.getExceptionFundamentalType() +
                    "_exception;");
+      if (!BabelConfiguration.isSIDLBaseClass(d_ext)) {
+        d_lw.println("if (!_loadClassImpl()) return -1;");
+      }
+      Python.leavePython(d_lw);
       d_lw.print("optarg = (*(_implEPV->createObject))");
       d_lw.println("(NULL,&_exception);");
+      Python.resumePython(d_lw);
       d_lw.println("if (_exception) {");
       d_lw.tab();
       d_lw.println("sidl_RuntimeException__import();");
@@ -294,9 +306,14 @@ public class PythonClientCSource extends PythonClientC {
       d_lw.tab();
       d_lw.println(IOR.getExceptionFundamentalType() +
                    "_exception;");
+      if (!BabelConfiguration.isSIDLBaseClass(d_ext)) {
+        d_lw.println("if (!_loadClassImpl()) return -1;");
+      }
       d_lw.println("Py_INCREF(implObj);");
+      Python.leavePython(d_lw);
       d_lw.print("optarg = (*(_implEPV->createObject))");
       d_lw.println("((void*)implObj,&_exception);");
+      Python.resumePython(d_lw);
       d_lw.println("if (_exception) {");
       d_lw.tab();
       d_lw.println("sidl_RuntimeException__import();");
@@ -309,7 +326,6 @@ public class PythonClientCSource extends PythonClientC {
       d_lw.backTab();
       d_lw.println("}");
       d_lw.backTab();
-      d_lw.backTab();
       d_lw.println("}");
 
       d_lw.println();
@@ -320,11 +336,19 @@ public class PythonClientCSource extends PythonClientC {
       d_lw.print(IOR.getExceptionFundamentalType());
       d_lw.println("_exception = NULL;");
       String cName = C.getFullMethodName(d_ext.getSymbolID(), "_remoteCreate");
+      Python.leavePython(d_lw);
       d_lw.println("optarg = "+cName+"(url,&_exception);");
+      Python.resumePython(d_lw);
       d_lw.println("if (_exception) {");
       d_lw.tab();
       d_lw.println("sidl_rmi_NetworkException__import();");
+
+      d_lw.println("{");
+      d_lw.tab();
       processExceptions(Python.createRemoteMethod(d_ext, d_context));
+      d_lw.println("}"); 
+      d_lw.backTab();
+      
       d_lw.println("return -1;");
       d_lw.backTab();      
       d_lw.println("}"); 
@@ -621,6 +645,22 @@ public class PythonClientCSource extends PythonClientC {
     d_lw.println("};");
   }
 
+  private void loadSEPV(Method m)
+  {
+    if (m.isStatic() && !BabelConfiguration.isSIDLBaseClass(d_ext)) {
+      d_lw.println("if (_loadClassImpl()) {");
+      d_lw.tab();
+      d_lw.println("_sepv = (*_implEPV->getStaticEPV)();");
+      d_lw.backTab();
+      d_lw.println("}");
+      d_lw.println("else {");
+      d_lw.tab();
+      d_lw.println("return NULL;");
+      d_lw.backTab();
+      d_lw.println("}");
+    }
+  }
+
 
   /**
    * Write code to call the sidl method using the static entry point vector
@@ -632,6 +672,7 @@ public class PythonClientCSource extends PythonClientC {
    */
   private void callMethod(Method m, boolean isInterface) {
     boolean addComma = false;
+    Python.leavePython(d_lw);
     if (m.getReturnType().getType() != Type.VOID) {
       d_lw.print("_return = ");
     }
@@ -680,6 +721,7 @@ public class PythonClientCSource extends PythonClientC {
       addComma = true;
     }
     d_lw.println(");");
+    Python.resumePython(d_lw);
   }
 
   /**
@@ -698,6 +740,7 @@ public class PythonClientCSource extends PythonClientC {
     Object [] exceptions = m.getThrows().toArray();
     Arrays.sort(exceptions, new LevelComparator(symtab));
     int i;
+
     /* declare exception pointers */
     for(i = 0; i < exceptions.length ; ++i) {
       d_lw.print(IOR.getObjectName((SymbolID)exceptions[i]));
@@ -740,9 +783,11 @@ public class PythonClientCSource extends PythonClientC {
       d_lw.print(Python.getExceptionType(sym));
       d_lw.println(", _obj);");
       d_lw.println("Py_XDECREF(_obj);");
+      Python.leavePython(d_lw);
       d_lw.println("(*(_exception->d_epv->" +
                    IOR.getVectorEntry("deleteRef") +
                    "))(_exception->d_object, &throwaway_exception);");
+      Python.resumePython(d_lw);
       d_lw.println("Py_XDECREF(_eargs);");
       d_lw.backTab();
       d_lw.println("}");
@@ -751,12 +796,17 @@ public class PythonClientCSource extends PythonClientC {
 
   void cleanOutgoingArrays(Method m )
   {
+    boolean first = true;
     Iterator i = m.getArgumentList().iterator();
     while (i.hasNext()) {
       Argument arg = (Argument)i.next();
       if (Argument.IN != arg.getMode()) {
         switch(arg.getType().getDetailedType()) {
         case Type.ARRAY:
+          if (first) {
+            Python.leavePython(d_lw);
+            first = false;
+          }
           d_lw.println(Python.getDestroyArray(arg.getType()) +
                        "((struct sidl__array *)" +
                        arg.getFormalName() + ");");
@@ -767,6 +817,9 @@ public class PythonClientCSource extends PythonClientC {
         }
       }
     }
+    if (!first) {
+      Python.resumePython(d_lw);
+    }
   }
 
   /**
@@ -802,13 +855,17 @@ public class PythonClientCSource extends PythonClientC {
         d_lw.println("free((void *)_return);");
       }
       else if (Type.ARRAY == m.getReturnType().getType()) {
+        Python.leavePython(d_lw);
         destroyArrayArg(m.getReturnType().getArrayType(), "_return");
+        Python.resumePython(d_lw);
       }
       else if (Type.STRUCT == m.getReturnType().getDetailedType()) {
         Symbol sym = Utilities.lookupSymbol(d_context,
                                             m.getReturnType().getSymbolID());
+        Python.leavePython(d_lw);
         d_lw.println(Python.getStructDestroy(sym) +
                      "(&_return);");
+        Python.resumePython(d_lw);
       }
       cleanOutgoingArrays(m);
     }
@@ -833,6 +890,7 @@ public class PythonClientCSource extends PythonClientC {
       d_lw.tab();
       d_lw.println(IOR.getExceptionFundamentalType() +
                    "throwaway_exception;");
+      Python.leavePython(d_lw);
       if (d_ext.isInterface()) {
         d_lw.println("(*(_self_ior->d_epv->" +
                      IOR.getVectorEntry("deleteRef") +
@@ -843,6 +901,7 @@ public class PythonClientCSource extends PythonClientC {
                      IOR.getVectorEntry("deleteRef") +
                      "))(_self_ior, &throwaway_exception);");
       }
+      Python.resumePython(d_lw);
       d_lw.backTab();
       d_lw.println("}");
       d_lw.backTab();
@@ -865,8 +924,20 @@ public class PythonClientCSource extends PythonClientC {
       d_lw.println("}");
     }
     //Reset the sepv if we have to (a little hackish)
+/* TODO/ToDo/TBD:  Is the following correct?
+    boolean isSetHooks     = IOR.getBuiltinMethod(IOR.HOOKS, 
+                                 d_ext.getSymbolID(), d_context, true);
+    boolean isSetContracts = IOR.getBuiltinMethod(IOR.CONTRACTS, 
+                                 d_ext.getSymbolID(), d_context, true);
+    boolean isDumpStats    = IOR.getBuiltinMethod(IOR.DUMP_STATS, 
+                                 d_ext.getSymbolID(), d_context, true); 
+    if (isSetHooks || isSetContracts || isDumpStats) {
+*/
     if(m.equals(IOR.getBuiltinMethod(IOR.HOOKS, d_ext.getSymbolID(), 
                                      d_context, true))) {
+      if (!BabelConfiguration.isSIDLBaseClass(d_ext)) {
+        d_lw.println("_loadClassImpl();");
+      }
       d_lw.println("if(_implEPV) {");
       d_lw.tab();
       d_lw.println("_sepv = (*_implEPV->getStaticEPV)();");
@@ -924,6 +995,7 @@ public class PythonClientCSource extends PythonClientC {
   private void cleanUpArguments(Method m)
     throws CodeGenerationException
   {
+    boolean first = true;
     Iterator i = m.getArgumentList().iterator();
     while (i.hasNext()) {
       Argument a = (Argument)i.next();
@@ -934,12 +1006,20 @@ public class PythonClientCSource extends PythonClientC {
           Extendable ext = 
             (Extendable)Utilities.lookupSymbol(d_context,
                                                a.getType().getSymbolID());
+          if (first) {
+            Python.leavePython(d_lw);
+            first = false;
+          }
           d_lw.println(Python.getExtendableDeref(ext) +
                        "(" + a.getFormalName() + ");");
         }
         break;
       case Type.ARRAY:
         if (Argument.IN == a.getMode()) {
+          if (first) {
+            Python.leavePython(d_lw);
+            first = false;
+          }
           destroyArrayArg(a.getType().getArrayType(),
                           a.getFormalName());
         }
@@ -947,11 +1027,18 @@ public class PythonClientCSource extends PythonClientC {
       case Type.STRUCT:
         Symbol sym = Utilities.lookupSymbol(d_context,
                                             a.getType().getSymbolID());
+        if (first) {
+          Python.leavePython(d_lw);
+          first = false;
+        }
         d_lw.println(Python.getStructDestroy(sym) +
                      "(&" + a.getFormalName() + ");");
         break;
       }
     }
+    if (!first) {
+      Python.resumePython(d_lw);
+    }
   }
 
   /**
@@ -1039,7 +1126,6 @@ public class PythonClientCSource extends PythonClientC {
     declareArguments(m);
     translator.declareProxies();
     listArgumentNames(m);
-    translator.setBorrowArrays(true);
     translator.setConvertIncoming(true);
     d_lw.println("int _okay;");
     writeImports(getNonParentRefs(m));
@@ -1049,10 +1135,10 @@ public class PythonClientCSource extends PythonClientC {
     d_lw.println("if (_okay) {");
     d_lw.tab();
     declareReturnValue(m, translator);
+    loadSEPV(m);
     translator.convertIncomingArguments(false);
     callMethod(m, isInterface);
     translator.setConvertIncoming(false);
-    translator.setBorrowArrays(false);
     translator.convertOutgoingArguments(true);
     processReturnValues(m, translator);
     cleanUpArguments(m);
@@ -1075,7 +1161,6 @@ public class PythonClientCSource extends PythonClientC {
     declareArguments(m);
     translator.declareProxies();
     listArgumentNames(m);
-    translator.setBorrowArrays(true);
     translator.setConvertIncoming(true);
     d_lw.println("int _okay;");
     writeImports(getNonParentRefs(m));
@@ -1086,9 +1171,10 @@ public class PythonClientCSource extends PythonClientC {
     d_lw.tab();
     declareReturnValue(m, translator);
     translator.convertIncomingArguments(false);
+    Python.leavePython(d_lw);
     d_lw.println("self = "+cName+"(url,1,&_exception);");
+    Python.resumePython(d_lw);
     translator.setConvertIncoming(false);
-    translator.setBorrowArrays(false);
     translator.convertOutgoingArguments(true);
     processReturnValues(m, translator);
     cleanUpArguments(m);
@@ -1137,6 +1223,18 @@ public class PythonClientCSource extends PythonClientC {
       localMethods.add(IOR.getBuiltinMethod(IOR.HOOKS,id,d_context,true));
     }
 
+/* TODO/ToDo/TBD...are the following correct?
+    localMethods.add(IOR.getBuiltinMethod(IOR.CONTRACTS, id, d_context, false));
+    localMethods.add(IOR.getBuiltinMethod(IOR.DUMP_STATS, id, d_context, 
+                                          false));
+    if(d_ext.hasStaticMethod(false) && includeStatics) {
+      localMethods.add(IOR.getBuiltinMethod(IOR.CONTRACTS, id, d_context, 
+                                            true));
+      localMethods.add(IOR.getBuiltinMethod(IOR.DUMP_STATS, id, d_context, 
+                                            true));
+    }
+*/
+
     return localMethods;
   }
 
@@ -1308,6 +1406,7 @@ public class PythonClientCSource extends PythonClientC {
     d_lw.println();
   }
 
+/* TODO/ToDo/TBD:  Remove the following if the new alternate works. */
   private void generateSetHooksTableEntry() throws CodeGenerationException{
     SymbolID id = d_ext.getSymbolID();
     Method m = IOR.getBuiltinMethod(IOR.HOOKS,id, d_context,true);
@@ -1322,6 +1421,26 @@ public class PythonClientCSource extends PythonClientC {
   }
 
 
+  private void generateBuiltinEntry(int iorType) throws CodeGenerationException{
+    if (  (iorType == IOR.HOOKS) || (iorType == IOR.CONTRACTS)
+       || (iorType == IOR.DUMP_STATS) ) 
+    {
+
+      SymbolID id = d_ext.getSymbolID();
+
+      Method m = IOR.getBuiltinMethod(iorType, id, d_context, true);
+      d_lw.print("{ \"");
+      d_lw.print(m.getLongMethodName());
+      d_lw.print("\", (PyCFunction)");
+      d_lw.print(Python.getStubMethod(id, m));
+      d_lw.println(",");
+      d_lw.println("(METH_VARARGS | METH_KEYWORDS),");
+      documentMethod(m);
+      d_lw.println(" },");
+    }
+  }
+
+
   /**
    * Write the static method virtual function table for the Python module.
    * This table is needed to initialize the Python C extension module.
@@ -1338,6 +1457,13 @@ public class PythonClientCSource extends PythonClientC {
     if(d_ext.hasStaticMethod(false)) {
       generateSetHooksTableEntry();
     }
+/* TODO/ToDo/TBD...Are the following right?
+    if(d_ext.hasStaticMethod(false)) {
+      generateBuiltinEntry(IOR.HOOKS);
+      generateBuiltinEntry(IOR.CONTRACTS);
+      generateBuiltinEntry(IOR.DUMP_STATS);
+    }
+*/
     d_lw.println("{ NULL, NULL }");
     d_lw.backTab();
     d_lw.println("};");
@@ -1715,9 +1841,11 @@ public class PythonClientCSource extends PythonClientC {
     d_lw.println("if (*sidlobj) {");
     d_lw.tab();
     d_lw.println("struct sidl_BaseInterface__object *throwaway_exception;");
+    Python.leavePython(d_lw);
     d_lw.println("(*((*sidlobj)->d_epv->f_deleteRef))((*sidlobj)" 
                  + (d_ext.isInterface() ? "->d_object" : "") +
                  ", &throwaway_exception);");
+    Python.resumePython(d_lw);
     d_lw.backTab();
     d_lw.println("}");
     d_lw.print("*sidlobj = sidl_Cast(obj, \"");
@@ -1758,19 +1886,35 @@ public class PythonClientCSource extends PythonClientC {
                       "we have already imported all the nescessary modules",false);
     d_lw.println("static int lang_inited = 0;");  
     d_lw.println();
+    d_lw.println("static void _loadDependentMods(void) {");
+    d_lw.tab();
+    d_lw.println("PyGILState_STATE _gstate;");
+    d_lw.printlnUnformatted("#if (PY_VERSION_HEX >= 0x02040000)");
+    d_lw.println("_gstate = PyGILState_Ensure();");
+    d_lw.println("sidl_Python_LogGILEnsure(__func__, __FILE__, __LINE__, (int)_gstate);");
+    d_lw.printlnUnformatted("#endif /* Python 2.4 or later */");
+
+    for (Iterator i = dependencies.iterator(); i.hasNext(); ) {
+      SymbolID destination_id = (SymbolID) i.next();
+      d_lw.println(Python.getImport(destination_id.getFullName())+"();");
+    }
+    d_lw.println(Python.getImport("sidl.rmi.Ticket")+"();");
+    if (hasStruct()) {
+      d_lw.println(Python.getImport("sidl.rmi.Invocation")+"();");
+    }
+    d_lw.printlnUnformatted("#if (PY_VERSION_HEX >= 0x02040000)");
+    d_lw.println("PyGILState_Release(_gstate);");
+    d_lw.println("sidl_Python_LogGILRelease(__func__, __FILE__, __LINE__, (int)_gstate);");
+    d_lw.printlnUnformatted("#endif /* Python 2.4 or later */");
+    d_lw.backTab();
+    d_lw.println("}");
+    d_lw.println();
     try {
       d_lw.pushLineBreak(false);
       d_lw.println("#define "+RMI.LangSpecificInit()+" if(!lang_inited) { \\");
       d_lw.tab();
       d_lw.println("lang_inited = 1; \\");
-      for (Iterator i = dependencies.iterator(); i.hasNext(); ) {
-        SymbolID destination_id = (SymbolID) i.next();
-        d_lw.println(Python.getImport(destination_id.getFullName())+"(); \\");
-      }
-      d_lw.println(Python.getImport("sidl.rmi.Ticket")+"(); \\");
-      if (hasStruct()) {
-        d_lw.println(Python.getImport("sidl.rmi.Invocation")+"(); \\");
-      }
+      d_lw.println("_loadDependentMods(); \\");
       d_lw.backTab();
       d_lw.println("}");
     }
@@ -1802,7 +1946,9 @@ public class PythonClientCSource extends PythonClientC {
     d_lw.println("struct sidl_BaseInterface__object *asInt = (struct sidl_BaseInterface__object *)sidlobj;");
     d_lw.println(IOR.getExceptionFundamentalType() +
                  "throwaway_exception;");
+    Python.leavePython(d_lw);
     d_lw.println("(*asInt->d_epv->f_deleteRef)(asInt->d_object, &throwaway_exception);");
+    Python.resumePython(d_lw);
     d_lw.backTab();
     d_lw.println("}");
     d_lw.println("return FALSE;");
@@ -1828,7 +1974,9 @@ public class PythonClientCSource extends PythonClientC {
     d_lw.println("int result = 0;");
     d_lw.println("if (*sidlarray) {");
     d_lw.tab();
+    Python.leavePython(d_lw);
     d_lw.println("sidl__array_deleteRef((struct sidl__array *)*sidlarray);");
+    Python.resumePython(d_lw);
     d_lw.println("*sidlarray = NULL;");
     d_lw.backTab();
     d_lw.println("}");
@@ -1868,10 +2016,12 @@ public class PythonClientCSource extends PythonClientC {
     d_lw.backTab();
     d_lw.println("if (*sidlarray && !result) {");
     d_lw.tab();
+    Python.leavePython(d_lw);
     d_lw.println("sidl_interface__array_deleteRef(");
     d_lw.tab();
     d_lw.println("(struct  sidl_interface__array *)*sidlarray);");
     d_lw.backTab();
+    Python.resumePython(d_lw);
     d_lw.println("*sidlarray = NULL;");
     d_lw.backTab();
     d_lw.println("}");
@@ -2125,6 +2275,121 @@ public class PythonClientCSource extends PythonClientC {
     
   }
 
+  private void writeLoadImpl() {
+    final SymbolID id = d_ext.getSymbolID();
+    d_lw.println("static void *");
+    d_lw.println("_loadClassImpl(void) {");
+    d_lw.tab();
+    d_lw.println("if (!_implEPV) {");
+    d_lw.tab();
+    d_lw.writeCommentLine("Load the implementation.");
+    d_lw.println(IOR.getExceptionFundamentalType() +
+                 "throwaway_exception;");
+    d_lw.println("sidl_DLL dll = NULL;");
+    d_lw.println("char *errmsg = NULL;");
+    d_lw.println(IOR.getExternalName(id) + "*(*_extFunc)(void) = NULL;");
+    d_lw.writeCommentLine("Try search global namespace first");
+    Python.leavePython(d_lw);
+    d_lw.println("dll = sidl_DLL__create(&throwaway_exception);");
+    d_lw.println("if (dll && sidl_DLL_loadLibrary(dll, \"main:\", TRUE, FALSE, &throwaway_exception)) {");
+    d_lw.tab();
+    d_lw.println("_extFunc = (" + IOR.getExternalName(id) + "*(*)(void))");
+    d_lw.tab();
+    d_lw.println("sidl_DLL_lookupSymbol(dll, \"" +
+                 IOR.getExternalFunc(id) + "\", &throwaway_exception);");
+    d_lw.backTab();
+    d_lw.println("if ((!throwaway_exception) && _extFunc) {");
+    d_lw.tab();
+    d_lw.println("_implEPV = (*_extFunc)();");
+    d_lw.backTab();
+    d_lw.println("}");
+    d_lw.backTab();
+    d_lw.println("}");
+    d_lw.println("if (dll) sidl_DLL_deleteRef(dll, &throwaway_exception);");
+    
+    d_lw.println("if (!_implEPV) {");
+    d_lw.tab();
+    d_lw.println("dll = sidl_Loader_findLibrary(\"" + 
+                 id.getFullName() + "\",");
+    d_lw.tab();
+    d_lw.println("\"ior/impl\", sidl_Scope_SCLSCOPE,");
+    d_lw.println("sidl_Resolve_SCLRESOLVE, &throwaway_exception);");
+    d_lw.backTab();
+    d_lw.println("if (dll) {");
+    d_lw.tab();
+    d_lw.println("_extFunc = (" + IOR.getExternalName(id) + "*(*)(void))");
+    d_lw.tab();
+    d_lw.println("sidl_DLL_lookupSymbol(dll, \"" +
+                 IOR.getExternalFunc(id) + "\", &throwaway_exception);");
+    d_lw.backTab();
+    d_lw.println("if (!throwaway_exception && _extFunc) {");
+    d_lw.tab();
+    d_lw.println("_implEPV = (*_extFunc)();");
+    d_lw.backTab();
+    d_lw.println("}");
+    d_lw.println("else {");
+    d_lw.tab();
+    try {
+      d_lw.pushLineBreak(false);
+      d_lw.println("char *name = sidl_DLL_getName(dll, &throwaway_exception);");
+      d_lw.println("if (throwaway_exception) {");
+      d_lw.tab();
+      d_lw.println("name = sidl_String_strdup(\"(unknown)\");");
+      d_lw.backTab();
+      d_lw.println("}");
+      d_lw.println("errmsg = sidl_String_concat3(");
+      d_lw.tab();
+      d_lw.println("\"babel: The shared library containing SIDL type " +
+                   id.getFullName() + ", \",");
+      d_lw.println("name,");
+      d_lw.println("\", was loaded but is missing a required symbol (" +
+                   IOR.getExternalFunc(id) + ").\");");
+      d_lw.backTab();
+      d_lw.println("sidl_String_free(name);");
+    }
+    finally {
+      d_lw.popLineBreak();
+    }
+    d_lw.backTab();
+    d_lw.println("}");
+    d_lw.println("sidl_DLL_deleteRef(dll, &throwaway_exception);");
+    d_lw.backTab();
+    d_lw.println("}");
+    d_lw.println("else {");
+    d_lw.tab();
+    try {
+      d_lw.pushLineBreak(false);
+      d_lw.println("errmsg = sidl_String_strdup(\"babel: Unable to find or load a shared library containing the implementation of SIDL type " +
+                   id.getFullName() + ". To get more debugging information trying setting the SIDL_DEBUG_DLOPEN environment variable and rerunning.\");");
+    }
+    finally {
+      d_lw.popLineBreak();
+    }
+    d_lw.backTab();
+    d_lw.println("}");
+    d_lw.backTab();
+    d_lw.println("}");
+    Python.resumePython(d_lw);
+    d_lw.println("if (!_implEPV) {");
+    d_lw.tab();
+    d_lw.println("if (errmsg) PyErr_SetString(PyExc_RuntimeError, errmsg);");
+    try {
+      d_lw.pushLineBreak(false);
+      d_lw.println("else PyErr_SetString(PyExc_RuntimeError, \"Unexpected error loading SIDL implementation for " + id.getFullName() + ".\");");
+    }
+    finally {
+      d_lw.popLineBreak();
+    }
+    d_lw.println("sidl_String_free(errmsg);");
+    d_lw.backTab();
+    d_lw.println("}");
+    d_lw.backTab();
+    d_lw.println("}");
+    d_lw.println("return (void *)_implEPV;");
+    d_lw.backTab();
+    d_lw.println("}");
+  }
+
   /**
    * Write the C extension module's initialization function. This 
    * function is called when the module is loaded into memory or
@@ -2153,10 +2418,6 @@ public class PythonClientCSource extends PythonClientC {
     d_lw.println("_NUM];");
     d_lw.println(IOR.getExceptionFundamentalType() +
                  "throwaway_exception;");
-    if (!(isBaseClass || d_ext.isInterface())){
-      d_lw.println("sidl_DLL dll = NULL;");
-      d_lw.println(IOR.getExternalName(id) + "*(*_extFunc)(void) = NULL;");
-    }
     if (docComment) {
       d_lw.print("module = Py_InitModule3(\"");
     }
@@ -2243,84 +2504,29 @@ public class PythonClientCSource extends PythonClientC {
       d_lw.backTab();
       d_lw.println("}");
     }
-    if (!d_ext.isInterface()) {
-      if (isBaseClass) {
-        d_lw.println("_implEPV = " + IOR.getExternalFunc(id)
-                     + "();");
+    if (!d_ext.isInterface() && isBaseClass) {
+      d_lw.println("_implEPV = " +
+                   IOR.getExternalFunc(id) + "();");
+      if (d_hasStaticMethods) {
+        d_lw.println("_sepv = (*_implEPV->getStaticEPV)();");
       }
-      else {
-        d_lw.writeCommentLine("Load the implementation after initializing the module.");
-        d_lw.writeCommentLine("Try search global namespace first");
-        d_lw.println("dll = sidl_DLL__create(&throwaway_exception);");
-        d_lw.println("if (dll && sidl_DLL_loadLibrary(dll, \"main:\", TRUE, FALSE, &throwaway_exception)) {");
-        d_lw.tab();
-        d_lw.println("_extFunc = (" + IOR.getExternalName(id) + "*(*)(void))");
-        d_lw.tab();
-        d_lw.println("sidl_DLL_lookupSymbol(dll, \"" +
-                     IOR.getExternalFunc(id) + "\", &throwaway_exception);");
-        d_lw.backTab();
-        d_lw.println("if (_extFunc) {");
-        d_lw.tab();
-        d_lw.println("_implEPV = (*_extFunc)();");
-        d_lw.backTab();
-        d_lw.println("}");
-        d_lw.backTab();
-        d_lw.println("}");
-        d_lw.println("if (dll) sidl_DLL_deleteRef(dll, &throwaway_exception);");
-        
-        d_lw.println("if (!_implEPV) {");
-        d_lw.tab();
-        d_lw.println("dll = sidl_Loader_findLibrary(\"" + 
-                     id.getFullName() + "\",");
-        d_lw.tab();
-        d_lw.println("\"ior/impl\", sidl_Scope_SCLSCOPE,");
-        d_lw.println("sidl_Resolve_SCLRESOLVE, &throwaway_exception);");
-        d_lw.backTab();
-        d_lw.println("if (dll) {");
-        d_lw.tab();
-        d_lw.println("_extFunc = (" + IOR.getExternalName(id) + "*(*)(void))");
-        d_lw.tab();
-        d_lw.println("sidl_DLL_lookupSymbol(dll, \"" +
-                     IOR.getExternalFunc(id) + "\", &throwaway_exception);");
-        d_lw.backTab();
-        d_lw.println("if (_extFunc) {");
-        d_lw.tab();
-        d_lw.println("_implEPV = (*_extFunc)();");
-        d_lw.backTab();
-        d_lw.println("}");
-        d_lw.println("sidl_DLL_deleteRef(dll, &throwaway_exception);");
-        d_lw.backTab();
-        d_lw.println("}");
-        d_lw.backTab();
-        d_lw.println("}");
-      }
-      if (d_hasStaticMethods){
-        d_lw.println("if (_implEPV) {");
-        d_lw.tab();
-        d_lw.print("_sepv");
-        d_lw.println(" = (*_implEPV->getStaticEPV)();");
-        d_lw.println("if (PyErr_Occurred()) {");
-        d_lw.tab();
+      d_lw.println("if (PyErr_Occurred()) {");
+      d_lw.tab();
+      try {
+        d_lw.pushLineBreak(false);
         d_lw.print("Py_FatalError(\"Cannot initialize Python module ");
         d_lw.print(d_ext.getFullName());
         d_lw.println(".\");");
-        d_lw.backTab();
-        d_lw.println("}");
-        d_lw.backTab();
-        d_lw.println("}");
-        d_lw.println("else {");
       }
-      else {
-        d_lw.println("if (!_implEPV) {");
+      finally{
+        d_lw.popLineBreak();
       }
-      d_lw.tab();
-      d_lw.println("Py_FatalError(\"Cannot load implementation for sidl" +
-                   (d_ext.isInterface() ? " interface " : " class ") +
-                   id.getFullName() + "\");");
       d_lw.backTab();
       d_lw.println("}");
     }
 
+    copyInheritedStatics();
+
     d_lw.println();
     d_lw.printlnUnformatted("#ifdef WITH_RMI");
     d_lw.println();
@@ -2337,6 +2543,73 @@ public class PythonClientCSource extends PythonClientC {
 
   }
 
+  private List inheritedStatics() 
+  {
+    HashSet staticMethods = new HashSet(d_ext.getStaticMethods(true));
+    staticMethods.removeAll(d_ext.getStaticMethods(false));
+    return Utilities.sort(staticMethods);
+  }
+
+  public void writeInheritedStatics()
+  {
+    List staticMethods = inheritedStatics();
+    if (!staticMethods.isEmpty()) {
+      Iterator i = staticMethods.iterator();
+      d_lw.println("static const char *s_inherited_statics[] = {");
+      d_lw.tab();
+      while (i.hasNext()) {
+        Method m = (Method)i.next();
+        d_lw.println("\"" + m.getShortMethodName() + "\",");
+      }
+      d_lw.println("NULL");
+      d_lw.backTab();
+      d_lw.println("};");
+      d_lw.println();
+    }
+  }
+
+  public void copyInheritedStatics()
+  {
+    List staticMethods = inheritedStatics();
+    if (!staticMethods.isEmpty()) {
+      d_lw.println("{");
+      d_lw.tab();
+      d_lw.println("PyObject *pmod = PyImport_ImportModule(\"" +
+                   ((Class)d_ext).getParentClass().getFullName() +
+                   "\");");
+      d_lw.println("PyObject *pdict = (pmod ? PyModule_GetDict(pmod) : NULL);");
+      d_lw.println("if (pdict) {");
+      d_lw.tab();
+      d_lw.println("const char * const *namep = s_inherited_statics;");
+      d_lw.println("while (*namep) {");
+      d_lw.tab();
+      d_lw.println("PyObject *meth = PyDict_GetItemString(pdict, *namep);");
+      d_lw.println("if (!meth || PyDict_SetItemString(dict, *namep, meth)) {");
+      d_lw.tab();
+      d_lw.println("Py_DECREF(pmod);");
+      d_lw.println("pmod = NULL;");
+      try {
+        d_lw.pushLineBreak(false);
+        d_lw.println("PyErr_SetString(PyExc_RuntimeError, \"Unable to initialize static method for class " +
+                     d_ext.getFullName() + ".\");");
+      }
+      finally {
+        d_lw.popLineBreak();
+      }
+      d_lw.println("break;");
+      d_lw.backTab();
+      d_lw.println("}");
+      d_lw.println("++namep;");
+      d_lw.backTab();
+      d_lw.println("}");
+      d_lw.backTab();
+      d_lw.println("}");
+      d_lw.println("Py_XDECREF(pmod);");
+      d_lw.backTab();
+      d_lw.println("}");
+    }
+  }
+
   /**
    * Generate the source file for the extendable with which this object was
    * created.
@@ -2385,6 +2658,7 @@ public class PythonClientCSource extends PythonClientC {
       overwriteRenamedMethods();
       writePythonType();
       externalMethods();
+      writeInheritedStatics();
       initModule();
     }
     finally {
