@@ -3,11 +3,11 @@
 // Package:     gov.llnl.babel.backend.python
 // Copyright:   (c) 2007 Lawrence Livermore National Security, LLC
 // Release:     $Name$
-// Revision:    @(#) $Revision: 6201 $
-// Date:        $Date: 2007-10-25 23:15:35 -0700 (Thu, 25 Oct 2007) $
+// Revision:    @(#) $Revision: 6482 $
+// Date:        $Date: 2008-08-21 15:50:53 -0700 (Thu, 21 Aug 2008) $
 // Description: Generate C source file for Python wrapper for struct
 // 
-// Copyright (c) 2007, The Regents of the University of Calfornia.
+// Copyright (c) 2007, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -122,6 +122,7 @@ public class PythonClientStructSource extends PythonClientC {
     addInclude("sidlStruct.h", true);
     addInclude("sidlOps.h", true);
     addInclude("sidlObjA.h", true);
+    addInclude("sidl_Python.h", true);
     if (d_struct.hasType(Type.ARRAY)) {
       addInclude("sidlPyArrays.h", true);
     }
@@ -346,10 +347,12 @@ public class PythonClientStructSource extends PythonClientC {
                  ", _eargs);");
     d_lw.println("PyErr_SetObject(sidl_RuntimeException__type, _obj);");
     d_lw.println("Py_XDECREF(_obj);");
+    Python.leavePython(d_lw);
     d_lw.println("(*(exception->d_epv->f_deleteRef))(exception->d_object,");
     d_lw.tab();
     d_lw.println("&throwaway_exception);");
     d_lw.backTab();
+    Python.resumePython(d_lw);
     d_lw.println("Py_XDECREF(_eargs);");
     d_lw.backTab();
     d_lw.println("}");
@@ -547,8 +550,10 @@ public class PythonClientStructSource extends PythonClientC {
                          name + ");");
         break;
       case Type.ARRAY:
+        Python.leavePython(d_lw);
         d_lw.println("sidl__array_deleteRef((struct sidl__array *)dest->" 
                      + name + ");");
+        Python.resumePython(d_lw);
         d_lw.println("dest->" + name +  " = (" +
                          IOR.getReturnString(t, d_context) +
                          ")");
