@@ -1,11 +1,11 @@
 //
 // File:        PythonServerCSource.java
 // Package:     gov.llnl.babel.backend.python
-// Revision:    @(#) $Revision: 6212 $
-// Date:        $Date: 2007-10-31 17:07:17 -0700 (Wed, 31 Oct 2007) $
+// Revision:    @(#) $Revision: 6482 $
+// Date:        $Date: 2008-08-21 15:50:53 -0700 (Thu, 21 Aug 2008) $
 // Description: Generate skeleton's to link to a Python implementation
 // 
-// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -200,7 +200,7 @@ public class PythonServerCSource {
         d_lw.println(IOR.getExceptionFundamentalType() + " *_exception");
         needComma = true;
       }
-    } else {
+   } else {
       /* if no argument put void */
       d_lw.print("void");
     }
@@ -215,6 +215,7 @@ public class PythonServerCSource {
     d_lw.println("PyObject *_pfunc;");
     d_lw.println("PyObject *_args;");
     d_lw.println("PyObject *_result;");
+    d_lw.println("PyGILState_STATE _gstate;");
   }
 
   private void getFunction(boolean isStatic, String  methodName) {
@@ -237,7 +238,9 @@ public class PythonServerCSource {
     d_lw.println("static int _import = 1;");
     d_lw.println("if (_import) {");
     d_lw.tab();
+    d_lw.println("PyGILState_STATE _gstate;");
     d_lw.println("_import = 0;");
+    acquirePythonLock();
     d_lw.println("import_SIDLObjA();");
     d_lw.println("import_SIDLPyArrays();");
     d_lw.println(Python.getExtendableImport(d_cls) + "();");
@@ -263,6 +266,7 @@ public class PythonServerCSource {
     if(usesGeneric == true)
       d_lw.println(Python.getImport(BabelConfiguration.getBaseInterface())
                    + "();");
+    releasePythonLock();
     d_lw.backTab();
     d_lw.println("}");
     d_lw.backTab();
@@ -270,9 +274,26 @@ public class PythonServerCSource {
     d_lw.println();
   }
 
-  private void cleanupRarrays(Collection arguments)
+  private boolean hasRarray(Collection arguments)
   {
     Iterator i = arguments.iterator();
+    while (i.hasNext()) {
+      Argument arg = (Argument)i.next();
+      if ((arg.getMode() == Argument.INOUT) &&
+          arg.getType().isRarray()) {
+        return true;
+      }
+    }
+    return false;
+  }
+
+  private void cleanupRarrays(Collection arguments)
+  {
+    boolean thread = hasRarray(arguments);
+    Iterator i = arguments.iterator();
+    if (thread) {
+      Python.leavePython(d_lw);
+    }
     while (i.hasNext()) {
       Argument arg = (Argument)i.next();
       if ((arg.getMode() == Argument.INOUT) &&
@@ -281,6 +302,9 @@ public class PythonServerCSource {
                      arg.getFormalName() + "));");
       }
     }
+    if (thread) {
+      Python.resumePython(d_lw);
+    }
   }
 
   private void handleExceptions(Method m, Set throwTypes)
@@ -438,6 +462,7 @@ public class PythonServerCSource {
   }
 
   private void releaseInOutParameters(Method m) throws CodeGenerationException {
+    boolean first = true;
     Symbol sym;
     Iterator i = m.getArgumentList().iterator();
     while (i.hasNext()) {
@@ -446,17 +471,29 @@ public class PythonServerCSource {
         final Type t = arg.getType();
         switch (t.getDetailedType()) {
         case Type.STRUCT:
+          if (first) {
+            Python.leavePython(d_lw);
+            first = false;
+          }
           sym = (Symbol)
             Utilities.lookupSymbol(d_context, t.getSymbolID());
           d_lw.println(Python.getStructDestroy(sym) +
                        "(" + arg.getFormalName() + ");");
           break;
         case Type.STRING:
+          if (first) {
+            Python.leavePython(d_lw);
+            first = false;
+          }
           d_lw.println("free((void *)*" + arg.getFormalName() + ");");
           d_lw.println("*" + arg.getFormalName() + " = NULL;");
           break;
         case Type.INTERFACE:
         case Type.CLASS:
+          if (first) {
+            Python.leavePython(d_lw);
+            first = false;
+          }
           sym = (Symbol)
             Utilities.lookupSymbol(d_context, t.getSymbolID());
           d_lw.println(Python.getExtendableDeref(sym) + "(*" +
@@ -465,6 +502,11 @@ public class PythonServerCSource {
           break;
         case Type.ARRAY:
           if (!t.isRarray()) {
+            if (first) {
+              Python.leavePython(d_lw);
+              first = false;
+            }
+            
             d_lw.println(Python.getDestroyArray(t.getArrayType()) + 
                          "((struct sidl__array *)(*" + 
                          arg.getFormalName() + "));");
@@ -474,6 +516,28 @@ public class PythonServerCSource {
         }
       }
     }
+    if (!first) {
+      Python.resumePython(d_lw);
+    }
+  }
+
+  /**
+   * Write code to acquire the Python GIL lock.
+   */
+  private void acquirePythonLock()
+  {
+    d_lw.printlnUnformatted("#if (PY_VERSION_HEX >= 0x02040000)");
+    d_lw.println("_gstate = PyGILState_Ensure();");
+    d_lw.println("sidl_Python_LogGILEnsure(__func__, __FILE__, __LINE__, (int)_gstate);");
+    d_lw.printlnUnformatted("#endif /* Python 2.4 or later */");
+  }
+
+  private void releasePythonLock()
+  {
+    d_lw.printlnUnformatted("#if (PY_VERSION_HEX >= 0x02040000)");
+    d_lw.println("PyGILState_Release(_gstate);");
+    d_lw.println("sidl_Python_LogGILRelease(__func__, __FILE__, __LINE__, (int)_gstate);");
+    d_lw.printlnUnformatted("#endif /* Python 2.4 or later */");
   }
 
   /**
@@ -489,12 +553,12 @@ public class PythonServerCSource {
                                                       false, false);
     writeMethodSignature(m);
     trans.setConvertIncoming(true);
-    trans.setBorrowArrays(true);
     d_lw.println("{");
     d_lw.tab();
     declareLocalVariables();
     declareReturnVariable(m);
     initializeOutVariables(m);
+    acquirePythonLock();
     getFunction(m.isStatic(), m.getLongMethodName());
     trans.declareProxies();
     trans.convertIncomingArguments(true);
@@ -517,7 +581,6 @@ public class PythonServerCSource {
       d_lw.println("_result = _tmp;");
     }
     d_lw.print("_okay = ");
-    trans.setBorrowArrays(false);
     trans.convertPythonToSidl("_result");
     d_lw.println("if (_okay) {");
     d_lw.tab();
@@ -525,6 +588,7 @@ public class PythonServerCSource {
     d_lw.backTab();
     d_lw.println("}");
     cleanupReferences(m);
+    releasePythonLock();
     if (m.getReturnType().getDetailedType() != Type.VOID) {
       d_lw.println("return " + TranslateArguments.RETURN_VAR + ";");
     }
@@ -548,7 +612,7 @@ public class PythonServerCSource {
       Method m = (Method)i.next();
       if (!m.isAbstract()) {
         convertMethod(m);
-        if (IOR.supportHooks(d_cls, d_context)) {
+        if (IOR.generateHookMethods(d_cls, d_context)) {
           Method pre = m.spawnPreHook();
           Method post = m.spawnPostHook();
           convertMethod(pre);
@@ -559,21 +623,22 @@ public class PythonServerCSource {
   }
 
   private void writeInitializeSEPV(Collection methods) 
-    throws CodeGenerationException {
+    throws CodeGenerationException 
+  {
     SymbolID id = d_cls.getSymbolID();
     Iterator i = methods.iterator();
     d_lw.openCxxExtern();
     d_lw.println("void");
-    if(IOR.supportHooks(id, d_context)) {
-      d_lw.println(Python.getSetSEPVName(id) + "(" + IOR.getSEPVName(id) + " *"+s_sepv+",");
+    d_lw.print(Python.getSetSEPVName(id) + "(" + IOR.getSEPVName(id) + " *"
+      + s_sepv);
+    if (IOR.generateHookEPVs(d_cls, d_context)) {
+      d_lw.println(",");
       d_lw.tab();
-      d_lw.println(IOR.getPreSEPVName(id) + " *"+s_pre_sepv+", "+
-                   IOR.getPostSEPVName(id) + " *"+s_post_sepv+")");
+      d_lw.println(IOR.getPreSEPVName(id) + " *" + s_pre_sepv + ", "
+        + IOR.getPostSEPVName(id) + " *" + s_post_sepv + ")");
       d_lw.backTab();
     } else {
-      d_lw.println(Python.getSetSEPVName(id) + "(" + IOR.getSEPVName(id) 
-                   + " *" + s_sepv + ")");
-
+      d_lw.println(")");
     }
 
     d_lw.println("{");
@@ -583,13 +648,7 @@ public class PythonServerCSource {
     while (i.hasNext()) {
       Method m = (Method)i.next();
       if (m.isStatic()) {
-        initializeEPVPointer(id, m, s_sepv);
-        if (IOR.supportHooks(d_cls, d_context)) {
-          Method pre = m.spawnPreHook();
-          Method post = m.spawnPostHook();
-          initializeEPVPointer(id, pre, s_pre_sepv);
-          initializeEPVPointer(id, post, s_post_sepv);
-        }
+        initializeSEPVPointer(d_cls, m);
       }
     }
     d_lw.backTab();
@@ -598,64 +657,166 @@ public class PythonServerCSource {
     d_lw.println();
   }
 
-  private void writeInitializeEPV(Collection methods) 
-    throws CodeGenerationException {
-    SymbolID id = d_cls.getSymbolID();
-    Iterator i = methods.iterator();
-    d_lw.openCxxExtern();
+  private void initializeSEPVPointer(Class cls, Method m) 
+        throws CodeGenerationException
+  {
+    final String methodName = m.getLongMethodName();
+    final String ename      = IOR.getVectorEntry(methodName);
+    final SymbolID id       = cls.getSymbolID();
 
-    d_lw.println("void");
-    if(IOR.supportHooks(id, d_context)) {
-      d_lw.println(Python.getSetEPVName(id) + "(" + IOR.getEPVName(id) + " *"+s_epv+",");
-      d_lw.tab();
-      d_lw.println(IOR.getPreEPVName(id) + " *"+s_pre_epv+", "+
-                   IOR.getPostEPVName(id) + " *"+s_post_epv+")");
-      d_lw.backTab();
-    } else {
-      d_lw.println(Python.getSetEPVName(id) + "(" + IOR.getEPVName(id) + " *"+s_epv+")");
-    }
-    d_lw.println("{");
-    d_lw.tab();
-    d_lw.println("_importModules();");
-    initializeEPVPointer(id, IOR.getBuiltinMethod(IOR.CONSTRUCTOR, id, d_context), s_epv);
-    initializeEPVPointer(id, IOR.getBuiltinMethod(IOR.CONSTRUCTOR2, id, d_context), s_epv);
-    initializeEPVPointer(id, IOR.getBuiltinMethod(IOR.DESTRUCTOR, id, d_context), s_epv);
-
-    while (i.hasNext()) {
-      Method m = (Method)i.next();
-      if(!m.isStatic()) {
-        initializeEPVPointer(id, m, s_epv);
-        if (IOR.supportHooks(d_cls, d_context)) {
-          Method pre = m.spawnPreHook();
-          Method post = m.spawnPostHook();
-          initializeEPVPointer(id, pre, s_pre_epv);
-          initializeEPVPointer(id, post, s_post_epv);
-        }
-      }
-    }
-    d_lw.backTab();
-    d_lw.println("}");
-    d_lw.closeCxxExtern();
-    d_lw.println();
-  }
-
-  private void initializeEPVPointer(SymbolID id, Method m, String epvname) {
     switch(m.getDefinitionModifier()) {
-    case Method.FINAL:
-    case Method.NORMAL:
     case Method.STATIC:
-      d_lw.println(epvname+"->" + IOR.getVectorEntry(m.getLongMethodName()) + " = "
-                   + Python.getSkelMethod(id, m) + ";");
+      String sname = Python.getSkelMethod(id, m);
+      if (!IOR.isBuiltinMethod(methodName)) {
+        Method pre   = m.spawnPreHook();
+        Method post  = m.spawnPostHook();
+
+        /*
+         * Order is (currently) critical!  If hook methods (i.e.,
+         * pre- and post-hooks) are being generated then be sure to
+         * initialize the EPV pointers to the skel/impl methods!
+         * Otherwise, if we're just making sure to include the
+         * basic hook skeletons (but not the implementations), then
+         * initialize the EPV pointers to null.
+         */
+        if (IOR.generateHookMethods(cls, d_context)) {
+          writeMethodAssignment(s_pre_sepv, 
+                                IOR.getVectorEntry(pre.getLongMethodName()), 
+                                Python.getSkelMethod(id, pre));
+          writeMethodAssignment(s_sepv, ename, sname);
+          writeMethodAssignment(s_post_sepv, 
+                                IOR.getVectorEntry(post.getLongMethodName()), 
+                                Python.getSkelMethod(id, post));
+        } else if (IOR.generateHookEPVs(cls, d_context)) {
+          writeMethodAssignment(s_pre_sepv, 
+                                IOR.getVectorEntry(pre.getLongMethodName()), 
+                                "NULL");
+          writeMethodAssignment(s_sepv, ename, sname);
+          writeMethodAssignment(s_post_sepv, 
+                                IOR.getVectorEntry(post.getLongMethodName()), 
+                                "NULL");
+        } else {
+          writeMethodAssignment(s_sepv, ename, sname);
+        }
+      } else {
+        writeMethodAssignment(s_sepv, ename, sname);
+      }
       break;
-    case Method.ABSTRACT:
-      d_lw.println(epvname+"->" + IOR.getVectorEntry(m.getLongMethodName()) 
-                   + " = NULL;");
     default:
       /* do nothing */
       break;
     }
   }
 
+  private void writeInitializeEPV(Collection methods) 
+    throws CodeGenerationException 
+  {
+    SymbolID id = d_cls.getSymbolID();
+    Iterator i = methods.iterator();
+    d_lw.openCxxExtern();
+
+    d_lw.println("void");
+    d_lw.print(Python.getSetEPVName(id) + "(" + IOR.getEPVName(id) + " *"
+      + s_epv);
+    if (IOR.generateHookEPVs(d_cls, d_context)) {
+      d_lw.println(",");
+      d_lw.tab();
+      d_lw.println(IOR.getPreEPVName(id) + " *" + s_pre_epv + ", "
+        + IOR.getPostEPVName(id) + " *" + s_post_epv + ")");
+      d_lw.backTab();
+    } else {
+      d_lw.println(")");
+    }
+    d_lw.println("{");
+    d_lw.tab();
+    d_lw.println("_importModules();");
+    initializeEPVPointer(d_cls, IOR.getBuiltinMethod(IOR.CONSTRUCTOR, id, 
+                         d_context));
+    initializeEPVPointer(d_cls, IOR.getBuiltinMethod(IOR.CONSTRUCTOR2, id, 
+                         d_context));
+    initializeEPVPointer(d_cls, IOR.getBuiltinMethod(IOR.DESTRUCTOR, id, 
+                         d_context));
+
+    while (i.hasNext()) {
+      Method m = (Method)i.next();
+      if(!m.isStatic()) {
+        initializeEPVPointer(d_cls, m);
+      }
+    }
+    d_lw.backTab();
+    d_lw.println("}");
+    d_lw.closeCxxExtern();
+    d_lw.println();
+  }
+
+  private void initializeEPVPointer(Class cls, Method m)
+    throws CodeGenerationException 
+  {
+    final String methodName = m.getLongMethodName();
+    final String ename      = IOR.getVectorEntry(methodName);
+    final SymbolID id       = cls.getSymbolID();
+
+    switch(m.getDefinitionModifier()) {
+    case Method.FINAL:
+    case Method.NORMAL:
+      String sname = Python.getSkelMethod(id, m);
+      if (!IOR.isBuiltinMethod(methodName)) {
+        Method pre   = m.spawnPreHook();
+        Method post  = m.spawnPostHook();
+
+        /*
+         * Order is (currently) critical!  If hook methods (i.e.,
+         * pre- and post-hooks) are being generated then be sure to
+         * initialize the EPV pointers to the skel/impl methods!
+         * Otherwise, if we're just making sure to include the
+         * basic hook skeletons (but not the implementations), then
+         * initialize the EPV pointers to null.
+         */
+        if (IOR.generateHookMethods(cls, d_context)) {
+          writeMethodAssignment(s_pre_epv, 
+                                IOR.getVectorEntry(pre.getLongMethodName()), 
+                                Python.getSkelMethod(id, pre));
+          writeMethodAssignment(s_epv, ename, sname);
+          writeMethodAssignment(s_post_epv, 
+                                IOR.getVectorEntry(post.getLongMethodName()), 
+                                Python.getSkelMethod(id, post));
+        } else if (IOR.generateHookEPVs(cls, d_context)) {
+          writeMethodAssignment(s_pre_epv, 
+                                IOR.getVectorEntry(pre.getLongMethodName()), 
+                                "NULL");
+          writeMethodAssignment(s_epv, ename, sname);
+          writeMethodAssignment(s_post_epv, 
+                                IOR.getVectorEntry(post.getLongMethodName()), 
+                                "NULL");
+        } else {
+          writeMethodAssignment(s_epv, ename, sname);
+        }
+      } else {
+        writeMethodAssignment(s_epv, ename, sname);
+      }
+      break;
+    case Method.ABSTRACT:
+        writeMethodAssignment(s_epv, ename, "NULL");
+    default:
+      /* do nothing */
+      break;
+    }
+  }
+
+  /**
+   * Write the function assignment for the specified var and method name
+   * to the specified value.
+   *
+   * @param var        the pointer variable representing the EPV.
+   * @param mname      the method name.
+   * @param value      the desired value, or RHS.
+   */
+  private void writeMethodAssignment(String var, String mname, String value)
+  {
+    d_lw.println(var + "->" + mname + " = " + value + ";");
+  }
+
+
   private boolean listSerialize(List ids,
                                 boolean serialize,
                                 boolean output_comma)
@@ -736,7 +897,7 @@ public class PythonServerCSource {
     extendedMethods.add(IOR.getBuiltinMethod(IOR.CONSTRUCTOR2, id, d_context));
     extendedMethods.add(IOR.getBuiltinMethod(IOR.DESTRUCTOR, id, d_context));
 
-    if (IOR.supportHooks(d_cls, d_context)) {
+    if (IOR.generateHookMethods(d_cls, d_context)) {
       for(Iterator i = localMethods.iterator(); i.hasNext();) {
         Method m = (Method) i.next();
         Method pre = m.spawnPreHook();
@@ -759,6 +920,7 @@ public class PythonServerCSource {
     d_lw.tab();
     declareLocalVariables();
     d_lw.println("*_exception = NULL;");
+    acquirePythonLock();
     getFunction(true, id.getShortName());
     d_lw.println("_args = Py_BuildValue(\"(O&)\", (void *)");
     d_lw.tab();
@@ -800,6 +962,7 @@ public class PythonServerCSource {
     d_lw.println("*_exception = sidl_Handle_Unexpected(\"_ctor\");");
     d_lw.backTab();
     d_lw.println("}");
+    releasePythonLock();
     d_lw.backTab();
     d_lw.println("}");
     d_lw.println();
@@ -823,9 +986,12 @@ public class PythonServerCSource {
     writeMethodSignature(dtor);
     d_lw.println("{");
     d_lw.tab();
+    d_lw.println("PyGILState_STATE _gstate;");
     d_lw.println("*_exception = NULL;");
+    acquirePythonLock();
     d_lw.writeCommentLine("Remove only reference to Python object");
     d_lw.println("Py_XDECREF(((PyObject *)self->d_data));");
+    releasePythonLock();
     d_lw.println("sidl_Python_DecGlobalRef();");
     d_lw.backTab();
     d_lw.println("}");
@@ -853,7 +1019,7 @@ public class PythonServerCSource {
       writeConstructor();
       writeDestructor();
       writeInitializeEPV(methods);
-      if (d_cls.hasStaticMethod(false)) {
+      if (d_cls.hasStaticMethod(true)) {
         writeInitializeSEPV(methods);
       }
       writeRMIAccessFunctions(d_cls); 
