@@ -1,11 +1,11 @@
 //
 // File:        TranslateArguments.java
 // Package:     gov.llnl.babel.backend.pythong
-// Revision:    @(#) $Revision: 6196 $
-// Date:        $Date: 2007-10-24 16:42:37 -0700 (Wed, 24 Oct 2007) $
+// Revision:    @(#) $Revision: 6482 $
+// Date:        $Date: 2008-08-21 15:50:53 -0700 (Thu, 21 Aug 2008) $
 // Description: Manage the translation of parameters to/from Python
 // 
-// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -50,7 +50,7 @@ public class TranslateArguments {
   private Context d_context;
   private boolean d_convertIncoming = true;
   private boolean d_localVars;
-  private boolean d_borrowArrays = false;
+  private boolean d_borrowArrays = true;
   private boolean d_isClient;
   public static final String RETURN_VAR = "_return";
 
@@ -77,16 +77,6 @@ public class TranslateArguments {
     return d_convertIncoming;
   }
 
-  public final void setBorrowArrays(boolean borrowArrays)
-  {
-    d_borrowArrays = borrowArrays;
-  }
-
-  public final boolean isBorrowArrays()
-  {
-    return d_borrowArrays;
-  }
-  
   public static boolean hasProxy(Type ty, 
                                  boolean inout,
                                  boolean isClient)
@@ -107,11 +97,13 @@ public class TranslateArguments {
   }
 
   private void translateArg(int type,
+                            int arrayType,
                             String var,
                             String getValue,
                             boolean sidlToPython,
                             boolean readOnly,
-                            boolean isRarray)
+                            boolean isRarray,
+                            int arrayOrder)
   {
     switch (type) {
     case Type.FCOMPLEX:
@@ -151,18 +143,29 @@ public class TranslateArguments {
     case Type.STRING:
       if (!sidlToPython && !readOnly) {
         d_lw.println(getValue + var + " = (" + getValue + var +
-                     " ? strcpy(malloc(strlen(" +
+                     " ? strcpy((char*)malloc(strlen(" +
                      getValue + var + ")+1), " +
                      getValue + var + ") : NULL);");
       }
       break;
     case Type.ARRAY:
       if (!(sidlToPython || readOnly || d_isClient) && isRarray) {
-        d_lw.println("sidl_python_copy((const struct sidl__array *)" +
+        d_lw.println("sidl_python_copy((struct sidl__array *)" +
                      proxyVariable(var) + ", (struct sidl__array *)(*" +
                      var + "));");
+        Python.leavePython(d_lw);
         d_lw.println("sidl_python_deleteRef_array((struct sidl__array *)(" +
                      proxyVariable(var) + "));");
+        Python.resumePython(d_lw);
+      }
+      else if (!(sidlToPython || isRarray) && 
+               (arrayOrder != Type.UNSPECIFIED)) {
+        d_lw.println(getValue + var + " = (struct sidl_" +
+                     Type.getTypeName(arrayType) + "__array *)" +
+                     "sidl_python_ensure((struct sidl__array *)(" +
+                     getValue + var + 
+                     ((arrayOrder == Type.COLUMN_MAJOR) 
+                      ? "), 1);" : "), 0);"));
       }
       break;
     }
@@ -446,6 +449,12 @@ public class TranslateArguments {
     }
   }
 
+  private int getArrayType(Type t)
+  {
+    Type arrayType = t.getArrayType();
+    return (arrayType != null) ? arrayType.getDetailedType() : Type.VOID;
+  }
+
   public void convertIncomingArguments(boolean sidlToPython)
   {
     if (d_convertIncoming) {
@@ -457,9 +466,11 @@ public class TranslateArguments {
           final String getValue = 
             (mode == Argument.INOUT && !d_localVars) ? "*" : "";
           translateArg(arg.getType().getDetailedType(), 
+                       getArrayType(arg.getType()),
                        arg.getFormalName(), getValue, sidlToPython,
                        mode == Argument.IN,
-                       arg.getType().isRarray());
+                       arg.getType().isRarray(),
+                       Type.UNSPECIFIED);
         }
       }
     }
@@ -471,7 +482,10 @@ public class TranslateArguments {
       final String getValue = (d_localVars ? "" : "*");
       final int returnType = d_m.getReturnType().getDetailedType();
       if (returnType != Type.VOID) {
-        translateArg(returnType, RETURN_VAR, "", sidlToPython, false, false);
+        translateArg(returnType, 
+                     getArrayType(d_m.getReturnType()),
+                     RETURN_VAR, "", sidlToPython, false, false,
+                     d_m.getReturnType().getArrayOrder());
       }
       Iterator i = d_m.getArgumentList().iterator();
       while (i.hasNext()) {
@@ -479,8 +493,10 @@ public class TranslateArguments {
         final int mode = arg.getMode();
         if (mode != Argument.IN) {
           translateArg(arg.getType().getDetailedType(), 
+                       getArrayType(arg.getType()),
                        arg.getFormalName(), getValue, sidlToPython, false,
-                       arg.getType().isRarray());
+                       arg.getType().isRarray(),
+                       arg.getType().getArrayOrder());
         }
       }
     }
