@@ -2,9 +2,9 @@
 // File:        RMI.java
 // Package:     gov.llnl.babel.backend.rmi
 // Copyright:   (c) 2004 The Lawrence Livermore National Security, LLC
-// Revision:     @(#) $Id: RMI.java 6201 2007-10-26 06:15:35Z kumfert $
+// Revision:     @(#) $Id: RMI.java 6509 2008-09-11 17:31:27Z epperly $
 // Description: 
-// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -165,8 +165,8 @@ public class RMI {
     if(mode != Argument.IN) {
       var_prefix ="*";
     }
-    url_string = new String("sidl_BaseInterface__getURL" + "((sidl_BaseInterface)" 
-                            + var_prefix + var + ", _ex)");
+    url_string = "sidl_BaseInterface__getURL" + "((sidl_BaseInterface)" 
+      + var_prefix + var + ", _ex)";
 
     if(isCopy) {
       if(RMI.isSerializable(type, context)) {
