@@ -1,10 +1,10 @@
 //
-// File:        IORHeader.java
+// File:        RMIIORHeader.java
 // Package:     gov.llnl.babel.backend.ior
-// Revision:    @(#) $Id: RMIIORHeader.java 6201 2007-10-26 06:15:35Z kumfert $
+// Revision:    @(#) $Id: RMIIORHeader.java 6482 2008-08-21 22:50:53Z epperly $
 // Description: generate IOR header to a pretty writer stream
 //
-// Copyright (c) 2000-2004, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2007, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -99,13 +99,13 @@ public class RMIIORHeader {
          case Symbol.PACKAGE:
            break;
          case Symbol.CLASS:
-           declareRMIAcessorFunctions((Class) symbol);
+           declareRMIAccessorFunctions((Class) symbol);
            defineRemoteStructure((Extendable) symbol); 
            break;
          case Symbol.INTERFACE:
            Class cls = ((Interface)symbol).generateAnonymousClass();
            generateExtendable(cls);
-           //declareRMIAcessorFunction((Extendable) symbol);
+           //declareRMIAccessorFunction((Extendable) symbol);
            break;
          }
       }
@@ -203,10 +203,13 @@ public class RMIIORHeader {
        * Declare exported symbols and forward declarations.
        */
       generateExportedSymbols(ext);
+
       /*
-       * Generate method indices, if necessary.
+       * Generate invariant clause data, method indices, and method
+       * precondition and postcondition clause data, if necessary.
        */
       if (!ext.isInterface()) {
+        generateInvClauseData(ext);
         generateMethodDescData(ext);
       }
 
@@ -218,13 +221,26 @@ public class RMIIORHeader {
       }
       generateEPV(ext, false);
 
+/*
+ * TODO:  This is in the IOR, should it be in here too?
+ *
+      if (IOR.generateHookMethods(ext, d_context)) {
+        generatePreHooksEPV(ext, false);
+        generatePostHooksEPV(ext, false);
+        if (ext.hasStaticMethod(true)) {
+          generatePreHooksEPV(ext, true);
+          generatePostHooksEPV(ext, true);
+        }
+      }
+ */
+
       /*
        * Generate the class or interface object itself
        */
       if (ext.isInterface()) {
          generateInterfaceObject((Interface) ext);
       } else {
-         generateControls(ext);
+         IOR.generateControlNStats(d_writer, ext, d_context);
          generateClassObject((Class) ext);
          //generateExternalStruct(ext);
          //IORSource.generateExternalSignature(d_writer, ext, ";");
@@ -335,7 +351,9 @@ public class RMIIORHeader {
       if (ext.hasStaticMethod(true)) {
          d_writer.println("extern " + sepv + "*");
          d_writer.println(IOR.getStaticsName(id) + "(void);");
-         if (IOR.supportAssertions(ext) || IOR.supportHooks(ext)) {
+         if (   IOR.generateContractChecks(ext, d_context) 
+             || IOR.generateHookMethods(ext, d_context)  ) 
+         {
            d_writer.println();
            d_writer.println("extern " + sepv + "*");
            d_writer.println(IOR.getLocalStaticsName(id) + "(int type);");
@@ -361,13 +379,13 @@ public class RMIIORHeader {
          d_writer.println("static const int " 
            + IOR.getStaticTypeOption(id, IOR.SET_PUBLIC) + " = 0;");
          int i = 1;
-         if (IOR.supportAssertions(ext)) {
+         if (IOR.generateContractChecks(ext, d_context)) {
             d_writer.println("static const int " 
               + IOR.getStaticTypeOption(id, IOR.SET_ASSERTIONS) + " = " + i 
               + ";");
             i++;
          }
-         if (IOR.supportHooks(ext)) {
+         if (IOR.generateHookMethods(ext, d_context)) {
             d_writer.println("static const int " 
               + IOR.getStaticTypeOption(id, IOR.SET_HOOKS) + " = " + i 
               + ";");
@@ -451,36 +469,28 @@ public class RMIIORHeader {
    * 
    * @param m           the method to be generated
    * @param self        a string with the type of the object
-   * @param addHooks    indicator of whether hooks should be added.
-   * @param alreadySeen set of method names already generated.
-   *                    Each element is a <code>String</code>.
    * @exception gov.llnl.babel.backend.CodeGenerationException
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  private void generateEPVEntry(Method m, String self, String suffix) 
+  private void generateEPVEntry(Method m, String self)
     throws CodeGenerationException
   {
-    if (suffix.equals("")) {
-      d_writer.print(IOR.getReturnString(m.getReturnType(), 
-                                         d_context, true, false));
-    } else {
-      d_writer.print("void");
-    }
+    d_writer.print(IOR.getReturnString(m.getReturnType(), 
+                                       d_context, true, false));
     d_writer.print(" (*");
-    d_writer.print(IOR.getVectorEntry(m.getLongMethodName()) + suffix);
+    d_writer.print(IOR.getVectorEntry(m.getLongMethodName()));
     d_writer.println(")(");
     d_writer.tab();
 
     /* 
      * Generate the argument list.
      */
-    boolean has_throws = !m.getThrows().isEmpty() && (suffix.equals(""));
+    boolean has_throws = !m.getThrows().isEmpty();
     List args = m.getArgumentList();
-    IOR.generateArguments(d_writer, d_context,
-                          self, args, m.isStatic(), has_throws, 
-        (suffix.equals(IOR.GENERIC_POST_SUFFIX) ? m.getReturnType() : null), 
-                        true, true, false, false);
+    IOR.generateArguments(d_writer, d_context, self, args, m.isStatic(), 
+                          has_throws, m.getReturnType(),
+                          true, true, false, false);
     d_writer.println(");");
     d_writer.backTab();
   }
@@ -491,27 +501,19 @@ public class RMIIORHeader {
    * 
    * @param m           the method to be generated
    * @param self        a string with the type of the object
-   * @param addHooks    indicator of whether hooks should be added.
    * @param alreadySeen set of method names already generated.
    *                    Each element is a <code>String</code>.
    * @exception gov.llnl.babel.backend.CodeGenerationException
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  private void generateEPVMethod(Method m, String self, boolean addHooks, 
-                                 Set alreadySeen)
+  private void generateEPVMethod(Method m, String self, Set alreadySeen)
     throws CodeGenerationException
   {
     String name = m.getLongMethodName();
     if (!alreadySeen.contains(name)) {
       alreadySeen.add(name);
-      if (addHooks) {
-        generateEPVEntry(m, self, IOR.GENERIC_PRE_SUFFIX);
-        generateEPVEntry(m, self, "");
-        generateEPVEntry(m, self, IOR.GENERIC_POST_SUFFIX);
-      } else {
-        generateEPVEntry(m, self, "");
-      }
+      generateEPVEntry(m, self);
     }
   }
 
@@ -525,15 +527,13 @@ public class RMIIORHeader {
    * @param self               the type used for the self pointer.
    * @param doStatic           should <code>static</code> or
    *                           non-<code>static</code> entries be listed?
-   * @param addHooks           should hooks be listed?
    * @param methodsAlreadySeen set of method names already included.
    * @exception gov.llnl.babel.backend.CodeGenerationException
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
   private void generateLocalEntries(Extendable ext, String self, 
-                                    boolean doStatic, boolean addHooks,
-                                    Set methodsAlreadySeen)
+                                    boolean doStatic, Set methodsAlreadySeen)
     throws CodeGenerationException
   {
     /*
@@ -550,12 +550,12 @@ public class RMIIORHeader {
     }
     while (m.hasNext()) {
       Method method = (Method) m.next();
-      generateEPVMethod(method, self, addHooks, methodsAlreadySeen);
+      generateEPVMethod(method, self, methodsAlreadySeen);
       if (method.getCommunicationModifier() == Method.NONBLOCKING) { 
       	Method send = method.spawnNonblockingSend();
-      	generateEPVMethod(send, self, addHooks, methodsAlreadySeen );
+      	generateEPVMethod(send, self, methodsAlreadySeen );
       	Method recv = method.spawnNonblockingRecv();
-      	generateEPVMethod( recv, self, addHooks, methodsAlreadySeen );
+      	generateEPVMethod( recv, self, methodsAlreadySeen );
       }
     }
   }
@@ -582,10 +582,8 @@ public class RMIIORHeader {
   {
     final SymbolID id          = ext.getSymbolID();
     final int      numBuiltins = (ext.isInterface() 
-                                 ? IOR.INTERFACE_BUILTIN_METHODS
-                                 : IOR.CLASS_BUILTIN_METHODS);
-    //final boolean  doAsserts   = IOR.supportAssertions(ext);
-    //final boolean  doHooks     = IOR.supportHooks(ext);
+                                 ? IOR.INTERFACE_BUILT_IN_METHODS
+                                 : IOR.CLASS_BUILT_IN_METHODS);
 
     d_writer.writeCommentLine("Implicit builtin methods");
 
@@ -595,21 +593,19 @@ public class RMIIORHeader {
     if (doStatic) {
       for(int i = 0; i < numBuiltins; ++i){
         if (  IOR.hasStaticBuiltin(i)
-           && (  (IOR.isBuiltinAssert(i)  && 
-                  IOR.supportAssertions(ext, d_context))
-              || ((i == IOR.HOOKS) && IOR.supportHooks(ext, d_context)) ) ) {
+           && (  !IOR.isBuiltinContractMeth(i) 
+              || IOR.generateContractChecks(ext, d_context)  ) ) {
           Method b = IOR.getBuiltinMethod(i, id, d_context, doStatic);
-          generateEPVMethod(b, self, false, alreadySeen); 
+          generateEPVMethod(b, self, alreadySeen); 
         }
       }
     } else {
       for(int i = 0; i < numBuiltins; ++i){
+        /* d_writer.writeCommentLine(""+i); */
         if (  IOR.isBuiltinBasic(i) 
-           || (IOR.isBuiltinAssert(i)  && 
-               IOR.supportAssertions(ext,d_context))
-           || ((i == IOR.HOOKS) && IOR.supportHooks(ext, d_context)) ) {
+           || IOR.generateContractChecks(ext, d_context) ) {
           Method b = IOR.getBuiltinMethod(i, id, d_context, doStatic);
-          generateEPVMethod(b, self, false, alreadySeen); 
+          generateEPVMethod(b, self, alreadySeen); 
         }
       }
     }
@@ -625,8 +621,6 @@ public class RMIIORHeader {
    * @param self               the type used for the self pointer.
    * @param doStatic           should <code>static</code> or
    *                           non-<code>static</code> entries be listed?
-   * @param addHooks           should the hook method entries be 
-   *                           listed?
    * @param methodsAlreadySeen set of method names already included.
    * @param extAlreadySeen     set of extendables whose methods have
    *                           already been included
@@ -635,15 +629,14 @@ public class RMIIORHeader {
    *    violations of the data type invariants.
    */
   private void generateParentEntries(Extendable ext, String self,
-                                     boolean doStatic, boolean addHooks,
-                                     Set methodsAlreadySeen,
+                                     boolean doStatic, Set methodsAlreadySeen,
                                      Set extAlreadySeen) 
     throws CodeGenerationException
   {
     if (ext instanceof gov.llnl.babel.symbols.Class) {
       generateEPVEntries
         (((gov.llnl.babel.symbols.Class)ext).getParentClass(), self, doStatic, 
-        addHooks, methodsAlreadySeen, extAlreadySeen);
+        methodsAlreadySeen, extAlreadySeen);
     }
     /*
      * It is critical that we always visit parent interfaces in a canonical
@@ -653,7 +646,7 @@ public class RMIIORHeader {
     Object[] parents = ext.getParentInterfaces(false).toArray();
     Arrays.sort(parents, new SortComparator());
     for(int i = 0; i < parents.length; ++i){
-      generateEPVEntries((Extendable)parents[i], self, doStatic, addHooks,
+      generateEPVEntries((Extendable)parents[i], self, doStatic, 
                          methodsAlreadySeen, extAlreadySeen);
     }
   }
@@ -677,8 +670,6 @@ public class RMIIORHeader {
    * @param self               the type used for the self pointer.
    * @param doStatic           should <code>static</code> or
    *                           non-<code>static</code> entries be listed?
-   * @param addHooks           should the hook method entries be
-   *                           listed?
    * @param methodsAlreadySeen set of method names already included.
    * @param extAlreadySeen     a set of extendables whose methodss have
    *                           already been included.
@@ -687,36 +678,67 @@ public class RMIIORHeader {
    *    violations of the data type invariants.
    */
   private void generateEPVEntries(Extendable ext, String self, boolean doStatic,
-                                  boolean addHooks, Set methodsAlreadySeen,
-                                  Set extAlreadySeen) 
+                                  Set methodsAlreadySeen, Set extAlreadySeen) 
     throws CodeGenerationException 
   {
     if ((ext != null) && !extAlreadySeen.contains(ext)) {
       if (hasParents(ext)) {
-        /* 
-         * It is assumed parent entries should not have pre/post method
-         * hooks added.  If that is not the case, then extra work must be 
-         * done to ensure pre/post methods are created and EPV entries 
-         * properly initialized for each.
-         */
-        generateParentEntries(ext, self, doStatic, false, 
-                              methodsAlreadySeen, extAlreadySeen);
+        generateParentEntries(ext, self, doStatic, methodsAlreadySeen, 
+                              extAlreadySeen);
       }
-      generateLocalEntries(ext, self, doStatic, addHooks, methodsAlreadySeen);
+      generateLocalEntries(ext, self, doStatic, methodsAlreadySeen);
       extAlreadySeen.add(ext);
     }
   }
+
+
+   /**
+    * Generate invariant clause data for contract clause checking purposes.
+    */
+   private void generateInvClauseData(Extendable ext)
+     throws CodeGenerationException
+   {
+     if (IOR.generateContractChecks(ext, d_context)) {
+        d_writer.writeComment("Define invariant clause data for interface "
+          + "contract enforcement.", false);
+
+        SymbolID id = ext.getSymbolID();
+
+/*
+ * ToDo...If the following are made conditional again, be sure the
+ * corresponding changes are made to IORSource.java and, perhaps even,
+ * the runtime.
+ */
+//        if (ext.hasInvariants(true)) {
+          d_writer.print("static " + IOR.getInvDescDataStruct(id));
+          d_writer.println(" {");
+          d_writer.tab();
+          d_writer.println("int    " + IOR.D_INV_COMPLEXITY + ";");
+          d_writer.println("double " + IOR.D_EXEC_INV_TIME + ";");
+          d_writer.backTab();
+          d_writer.println("} " + IOR.getInvDescDataName(id) + " = {");
+          d_writer.tab();
+          d_writer.println(ext.getInvDefaultComplexity() + ", 0.0,");
+          d_writer.backTab();
+          d_writer.println("};");
+          d_writer.println();
+//        }
+     }
+   }
   
 
    /**
     * Generate the method description data, including indices to facilitate 
-    * access to the corresponding control information for assertion checking 
-    * purposes.
+    * access to the corresponding control information for interface contract 
+    * enforcement purposes.
     */
-   private void generateMethodDescData(Extendable ext) throws CodeGenerationException {
-     if (IOR.supportAssertions(ext, d_context)) {
-        d_writer.writeComment("Define method description data for assertion "
-          + "control", false);
+   private void generateMethodDescData(Extendable ext) 
+     throws CodeGenerationException 
+   {
+     if (IOR.generateContractChecks(ext, d_context)) {
+        d_writer.writeComment("Define method description data for interface "
+          + "contract enforcement.", false);
+
         int      num      = 0;
         SymbolID id       = ext.getSymbolID();
         List     methods  = (List)ext.getMethods(true);
@@ -803,9 +825,15 @@ public class RMIIORHeader {
       d_writer.tab();
       Set methodsAlreadySeen = new HashSet();
       Set extAlreadySeen = new HashSet();
+
+      /*
+       * NOTE:  The new IORHeader.java supports "From Clauses".
+       * Is this new behavior needed here as well?
+       */
+
       generateBuiltinEntries(ext, self, doStatic, methodsAlreadySeen);
-      generateEPVEntries(ext, self, doStatic, IOR.supportHooks(ext, d_context),
-                         methodsAlreadySeen, extAlreadySeen);
+      generateEPVEntries(ext, self, doStatic, methodsAlreadySeen, 
+                         extAlreadySeen);
       methodsAlreadySeen = null;
       extAlreadySeen = null;
       d_writer.backTab();
@@ -813,6 +841,157 @@ public class RMIIORHeader {
       d_writer.println();
    }
 
+
+   /**
+    * Generate the pre EPV (entry point vector for for pre hooks)
+    * data structure for the methods
+    * in the interface or class.  If the flag is true, then generate an EPV
+    * for the static members of the class.
+    *
+    * The EPV must be ordered correctly!  The parent entries must appear
+    * before child entries.  This is required for runtime features.
+    */
+/*
+ * TBD:  This is in IOR, does it belong here too?
+ *
+   private void generatePreHooksEPV(Extendable ext, boolean doStatic)
+     throws CodeGenerationException
+   {
+*/
+      /*
+       * Generate the name for this entry point vector as well as a pointer
+       * to "self" for this structure.  For classes, self will be the object
+       * structure whereas for interfaces it is void*.
+       */
+/*
+      final SymbolID id = ext.getSymbolID();
+      String self = null;
+      if (!doStatic) {
+        if (ext.isInterface()) {
+           self = "void* self";
+        } else {
+           self = IOR.getObjectName(id) + "* self";
+        }
+      }
+
+*/
+      /*
+       * Write a comment block to document the EPV structure.
+       */
+/*
+      String desc = doStatic ? "static " : "";
+      d_writer.writeComment("Declare the " + desc
+        + "method pre hooks entry point vector.", false);
+
+*/
+      /*
+       * Begin definition of the EPV structure and increase tab indent.
+       */
+/*
+      if (doStatic) {
+         d_writer.print(IOR.getPreSEPVName(id));
+         d_writer.print(IOR.getPreSEPVName(id));
+      } else {
+         d_writer.print(IOR.getPreEPVName(id));
+      }
+      d_writer.println(" {");
+      d_writer.tab();
+
+      Set methodsAlreadySeen = new HashSet();
+      Iterator m = null;
+      if (doStatic) {
+        m = ext.getStaticMethods(false).iterator();
+      } else {
+        m = ext.getNonstaticMethods(false).iterator();
+      }
+
+      while (m.hasNext()) {
+        Method method = (Method) m.next();
+        Method pre = method.spawnPreHook();
+        generateEPVEntry(pre, self);
+      }
+
+      d_writer.backTab();
+      d_writer.println("};");
+      d_writer.println();
+   }
+
+*/
+   /**
+    * Generate the post EPV (entry point vector for for pre hooks)
+    * data structure for the methods
+    * in the interface or class.  If the flag is true, then generate an EPV
+    * for the static members of the class.
+    *
+    * The EPV must be ordered correctly!  The parent entries must appear
+    * before child entries.  This is required for runtime features.
+    */
+/*
+ * TBD:  This is in IOR, does it belong here too?
+ *
+   private void generatePostHooksEPV(Extendable ext, boolean doStatic)
+     throws CodeGenerationException
+   {
+*/
+      /*
+       * Generate the name for this entry point vector as well as a pointer
+       * to "self" for this structure.  For classes, self will be the object
+       * structure whereas for interfaces it is void*.
+       */
+/*
+      final SymbolID id = ext.getSymbolID();
+      String self = null;
+      if (!doStatic) {
+        if (ext.isInterface()) {
+           self = "void* self";
+        } else {
+           self = IOR.getObjectName(id) + "* self";
+        }
+      }
+
+*/
+      /*
+       * Write a comment block to document the EPV structure.
+       */
+/*
+      String desc = doStatic ? "static " : "";
+      d_writer.writeComment("Declare the " + desc
+        + "method post hooks entry point vector.", false);
+
+*/
+      /*
+       * Begin definition of the EPV structure and increase tab indent.
+       */
+/*
+      if (doStatic) {
+        d_writer.print(IOR.getPostSEPVName(id));
+      } else {
+        d_writer.print(IOR.getPostEPVName(id));
+      }
+      d_writer.println(" {");
+      d_writer.tab();
+
+      Set methodsAlreadySeen = new HashSet();
+      Iterator m = null;
+      if (doStatic) {
+        m = ext.getStaticMethods(false).iterator();
+      } else {
+        m = ext.getNonstaticMethods(false).iterator();
+     }
+
+      while (m.hasNext()) {
+        Method method = (Method) m.next();
+        Method post = method.spawnPostHook();
+        generateEPVEntry(post, self);
+      }
+
+      d_writer.backTab();
+      d_writer.println("};");
+      d_writer.println();
+   }
+*/
+
+                         
    /*
     * Generate the object data structure for an interface.  This structure
     * contains an EPV, a pointer to the object implementation, and an integer
@@ -841,56 +1020,6 @@ public class RMIIORHeader {
       d_writer.println();
    }
 
-   /**
-    * Generate the different versions of the assertion controls data structure.
-    */
-   private void generateControls(Extendable ext) throws CodeGenerationException {
-      boolean doAsserts = IOR.supportAssertions(ext, d_context);
-      boolean doHooks   = IOR.supportHooks(ext, d_context);
-
-      if (doAsserts || doHooks) {
-         d_writer.writeComment("Define the controls structure.", false);
-
-         SymbolID id  = ext.getSymbolID();
-         int      num = ext.getNumberOfMethods(true);
-         d_writer.println();
-         d_writer.println(IOR.getControlsStruct(id) + " {");
-         d_writer.tab();
-         if (doAsserts) {
-           d_writer.println("int32_t " + IOR.D_CHECKS + ";");
-           d_writer.println("double  " + IOR.D_RATE + ";");
-           d_writer.println(IOR.getMethodControlsStruct(id) + " {");
-           d_writer.printlnUnformatted("#ifdef " + IOR.S_FULL_STATS_MACRO);
-           d_writer.tab();
-           d_writer.println("int32_t " + IOR.D_CALLS + ";");
-           d_writer.println("int32_t " + IOR.D_COUNTDOWN + ";");
-           d_writer.println("int32_t " + IOR.D_METHOD_EXCEPT + ";");
-           d_writer.println("int32_t " + IOR.D_SUCCESSES + ";");
-           d_writer.println("int32_t " + IOR.D_TRIES + ";");
-           d_writer.printlnUnformatted("#else /* !" + IOR.S_FULL_STATS_MACRO 
-             + " */");
-           d_writer.println("int32_t " + IOR.D_COUNTDOWN + ";");
-           d_writer.printlnUnformatted("#endif /* " + IOR.S_FULL_STATS_MACRO 
-             + " */");
-           d_writer.backTab();
-           d_writer.println("} " + IOR.D_METHOD_CONTROLS + "[" + num + "];");
-         }
-         if (doHooks) {
-           d_writer.println("int     " + IOR.D_HOOKS + ";");
-         }
-         d_writer.backTab();
-         d_writer.println("};");
-      }
-   }
-
-//   /**
-//    * Generate a version of the structure based on whether or not all of
-//    * the statistics are to be retained.
-//    */
-//   private void generateControlsStats(SymbolID id, int num, boolean doAsserts,
-//                                      boolean doHooks, boolean all) 
-//   {
-//   }
 
    /**
     * Generate the object data structure for a class.  This structure
@@ -906,7 +1035,7 @@ public class RMIIORHeader {
        * data structure.
        */
       List unique = Utilities.sort(Utilities.getUniqueInterfaceIDs(cls));
-      int width = Utilities.getWidth(unique) + "struct __object".length();
+      int width   = Utilities.getWidth(unique) + "struct __object".length();
 
       /*
        * Extract information about the parent class (if it exists).
@@ -926,20 +1055,19 @@ public class RMIIORHeader {
       /*
        * Extract information about the EPV structure and its width.
        */
-      boolean addAsserts = IOR.supportAssertions(cls, d_context);
-      boolean addHooks   = IOR.supportHooks(cls, d_context);
-      String  ctrls      = null;
+      boolean addContracts = IOR.generateContractEPVs(cls, d_context);
+      String  ctrls        = null;
 
       SymbolID id  = cls.getSymbolID();
       String   epv = IOR.getEPVName(id) + "*";
       int      len = epv.length();
       if (len > width) { width = len; }
-      if (addAsserts || addHooks) {
-        ctrls = IOR.getControlsStruct(id);
+      if (addContracts) {
+        ctrls = IOR.getControlsNStatsStruct(id);
         len   = ctrls.length();
         if (len > width) { width = len; }
       }
-      width = width + 1;
+      width++;
 
       /*
        * Output the object data structure.  Start with the parent class
@@ -964,14 +1092,14 @@ public class RMIIORHeader {
        */
       d_writer.printAligned(epv, width);
       d_writer.println(IOR.getEPVVar(IOR.PUBLIC_EPV) + ";");
-      if (IOR.supportBaseEPVAttr(cls, d_context)) {
+      if (IOR.generateBaseEPVAttr(cls, d_context)) {
         d_writer.printAligned(epv, width);
         d_writer.println(IOR.getEPVVar(IOR.BASE_EPV) + ";");
       }
 
-      if (addAsserts || addHooks) {
+      if (addContracts) {
         d_writer.printAligned(ctrls, width);
-        d_writer.println(IOR.D_CONTROLS + ";");
+        d_writer.println(IOR.D_CSTATS + ";");
       }
 
       d_writer.printAligned("void*", width);
@@ -988,30 +1116,28 @@ public class RMIIORHeader {
    * @param cls    the class for which an routine will be written.
    * @param writer the output writer to which the routine will be written.
    */
-  private void declareRMIAcessorFunctions(Class cls) 
+  private void declareRMIAccessorFunctions(Class cls) 
     throws CodeGenerationException
   {
-    SymbolID id = cls.getSymbolID();
-    Set fconnectSIDs = IOR.getFConnectSymbolIDs(cls);
-    Set fcastSIDs = IOR.getFCastSymbolIDs(cls);
-    Set serializeSIDs = IOR.getStructSymbolIDs(cls, true);
+    SymbolID id         = cls.getSymbolID();
+    Set fconnectSIDs    = IOR.getFConnectSymbolIDs(cls);
+    Set fcastSIDs       = IOR.getFCastSymbolIDs(cls);
+    Set serializeSIDs   = IOR.getStructSymbolIDs(cls, true);
     Set deserializeSIDs = IOR.getStructSymbolIDs(cls, false);
     for (Iterator i = fconnectSIDs.iterator(); i.hasNext(); ) {
       SymbolID l_id = (SymbolID) i.next();
-      d_writer.println(C.getSymbolObjectPtr(l_id) + " " + 
-		       IOR.getSkelFConnectName(id,l_id)+ 
-		       "(const char* url, sidl_bool ar, " +
-                       IOR.getExceptionFundamentalType() + 
-                       "*_ex);");
+      d_writer.print(C.getSymbolObjectPtr(l_id) + " ");
+      d_writer.print(IOR.getSkelFConnectName(id,l_id));
+      d_writer.print("(const char* url, sidl_bool ar, ");
+      d_writer.println(IOR.getExceptionFundamentalType() + " *_ex);");
     }
 
     for (Iterator i = fcastSIDs.iterator(); i.hasNext(); ) {
       SymbolID l_id = (SymbolID) i.next();
-      d_writer.println(C.getSymbolObjectPtr(l_id) + " " + 
-		       IOR.getSkelFCastName(id,l_id)+ 
-                     "(void *bi, " +
-                       IOR.getExceptionFundamentalType() +
-                       "*_ex);");
+      d_writer.print(C.getSymbolObjectPtr(l_id) + " ");
+      d_writer.print(IOR.getSkelFCastName(id,l_id));
+      d_writer.print("(void *bi, ");
+      d_writer.println(IOR.getExceptionFundamentalType() + "*_ex);");
 
       d_writer.println();
     }
@@ -1021,12 +1147,12 @@ public class RMIIORHeader {
     }
     for (Iterator i = serializeSIDs.iterator(); i.hasNext(); ) {
       SymbolID l_id = (SymbolID) i.next();
-      d_writer.println("void " +
-                       IOR.getSkelSerializationName(id, l_id, true) +
-                       "(const " +
-                       IOR.getStructName(l_id) + "*strct, " +
-                       "struct sidl_rmi_Return__object *pipe, const char *name, sidl_bool copyArg, " +
-                       IOR.getExceptionFundamentalType() + " *exc);");
+      d_writer.print("void ");
+      d_writer.print(IOR.getSkelSerializationName(id, l_id, true));
+      d_writer.print("(const " + IOR.getStructName(l_id) + " *strct, ");
+      d_writer.print("struct sidl_rmi_Return__object *pipe, ");
+      d_writer.print("const char *name, sidl_bool copyArg, ");
+      d_writer.println(IOR.getExceptionFundamentalType() + " *exc);");
 
       d_writer.println();
     }
@@ -1036,12 +1162,12 @@ public class RMIIORHeader {
     }
     for (Iterator i = deserializeSIDs.iterator(); i.hasNext(); ) {
       SymbolID l_id = (SymbolID) i.next();
-      d_writer.println("void " +
-                       IOR.getSkelSerializationName(id, l_id, false) +
-                       "(" +
-                       IOR.getStructName(l_id) + "*strct, " +
-                       "struct sidl_rmi_Call__object *pipe, const char *name, sidl_bool copyArg, " +
-                       IOR.getExceptionFundamentalType() + " *exc);");
+      d_writer.print("void ");
+      d_writer.print(IOR.getSkelSerializationName(id, l_id, false) + "(");
+      d_writer.print(IOR.getStructName(l_id) + " *strct, ");
+      d_writer.print("struct sidl_rmi_Call__object *pipe, ");
+      d_writer.print("const char *name, sidl_bool copyArg, ");
+      d_writer.println(IOR.getExceptionFundamentalType() + " *exc);");
 
       d_writer.println();
     }
