@@ -1,10 +1,10 @@
 //
 // File:        RMIIORSource.java
 // Package:     gov.llnl.babel.backend.rmi
-// Revision:    @(#) $Id: RMIIORSource.java 6212 2007-11-01 00:07:17Z epperly $
+// Revision:    @(#) $Id: RMIIORSource.java 6482 2008-08-21 22:50:53Z epperly $
 // Description: generate IOR implementation source to a pretty writer stream
 //
-// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -80,7 +80,7 @@ public class RMIIORSource {
 
   static {
     s_longestBuiltin = 0;
-    for(int j = 0; j < IOR.CLASS_BUILTIN_METHODS; ++j) {
+    for(int j = 0; j < IOR.CLASS_BUILT_IN_METHODS; ++j) {
       String mname = IOR.getBuiltinName(j);
       if (mname.length() > s_longestBuiltin) {
         s_longestBuiltin = mname.length();
