@@ -1,10 +1,10 @@
 //
 // File: RMIStubSource.java
 // Package: gov.llnl.babel.backend.rmi
-// Revision: @(#) $Id: RMIStubSource.java 6201 2007-10-26 06:15:35Z kumfert $
+// Revision: @(#) $Id: RMIStubSource.java 6482 2008-08-21 22:50:53Z epperly $
 // Description: generate IOR stub source to a pretty writer stream
 //
-// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -81,6 +81,12 @@ public class RMIStubSource {
   private static final String s_set_hooksBuiltin = IOR
       .getBuiltinName(IOR.HOOKS);
 
+  private static final String s_set_contractsBuiltin = IOR
+      .getBuiltinName(IOR.CONTRACTS);
+
+  private static final String s_dump_statsBuiltin = IOR
+      .getBuiltinName(IOR.DUMP_STATS);
+
   private Context d_context;
 
   /**
@@ -103,7 +109,7 @@ public class RMIStubSource {
 
   static {
     s_longestBuiltin = 0;
-    for (int j = 0; j < IOR.CLASS_BUILTIN_METHODS; ++j) {
+    for (int j = 0; j < IOR.CLASS_BUILT_IN_METHODS; ++j) {
       String mname = IOR.getBuiltinName(j);
       if (mname.length() > s_longestBuiltin) {
         s_longestBuiltin = mname.length();
@@ -269,6 +275,8 @@ public class RMIStubSource {
     generateRemoteAddRefFunction(ext);
     generateIsLocalIsRemote(cls);
     generateIsSetHooks(cls);
+    generateIsSetContracts(cls);
+    generateIsDumpStats(cls);
     generateRemoteExecFunction(ext);
     generateRemoteMethodBodies(ext);
     generateRemoteInitEPV(ext);
@@ -318,6 +326,8 @@ public class RMIStubSource {
     generateRemoteAddRefFunction(anonCls);
     generateIsLocalIsRemote(anonCls);
     generateIsSetHooks(anonCls);
+    generateIsSetContracts(anonCls);
+    generateIsDumpStats(anonCls);
     generateRemoteExecFunction(anonCls);
     generateRemoteMethodBodies(anonCls);
     generateRemoteInitEPV(anonCls);
@@ -697,7 +707,8 @@ public class RMIStubSource {
     d_writer.println("static sidl_bool");
     d_writer.println("remote_" + IOR.getRemoteIsRemoteName(id) + "(");
     d_writer.println("    struct " + my_symbolName + "__object* self, ");
-    d_writer.println("    "+IOR.getObjectName(BabelConfiguration.getBaseInterface())+"* *_ex) {");
+    d_writer.println("    "
+      + IOR.getObjectName(BabelConfiguration.getBaseInterface()) + "* *_ex) {");
     d_writer.tab();
     d_writer.println("*_ex = NULL;");
     d_writer.println("return TRUE;");
@@ -705,11 +716,32 @@ public class RMIStubSource {
     d_writer.println("}");
     d_writer.println("");
   }
-  private void generateIsSetHooks(Extendable ext) throws CodeGenerationException {
-    Method method = IOR.getBuiltinMethod(IOR.HOOKS,ext.getSymbolID(), 
-                                         d_context,false);
+
+  private void generateIsSetHooks(Extendable ext) throws CodeGenerationException
+  {
+    Method method = IOR.getBuiltinMethod(IOR.HOOKS, ext.getSymbolID(), 
+                                         d_context, false);
     generateSingleRemoteMethodBody(ext, method);
   }
+
+  private void generateIsSetContracts(Extendable ext) 
+     throws CodeGenerationException
+  {
+    Method method = IOR.getBuiltinMethod(IOR.CONTRACTS, ext.getSymbolID(), 
+                                         d_context, false);
+    comment("Contract enforcement has not been implemented for remote use.");
+    generateSingleRemoteMethodBody(ext, method);
+  }
+
+  private void generateIsDumpStats(Extendable ext) 
+     throws CodeGenerationException
+  {
+    Method method = IOR.getBuiltinMethod(IOR.DUMP_STATS, ext.getSymbolID(), 
+                                         d_context, false);
+    comment("Contract enforcement has not been implemented for remote use.");
+    generateSingleRemoteMethodBody(ext, method);
+  }
+
   /**
    * Generate remote method bodies for all non-static user-defined methods in an
    * extendable.
@@ -781,7 +813,11 @@ public class RMIStubSource {
       }
       try {
         d_writer.pushLineBreak(false);
-        d_writer.println("SIDL_THROW(*_ex, sidl_PreViolation, \"local method called on remote object\");");
+        /*
+         * TODO:  Is it really correct to use the Precondition violation?
+         */
+        d_writer.print("SIDL_THROW(*_ex, sidl_PreViolation, ");
+        d_writer.println("\"local method called on remote object\");");
       }
       finally {
         d_writer.popLineBreak();
@@ -824,8 +860,12 @@ public class RMIStubSource {
     d_writer.println("r_obj->d_refcount++;");
     d_writer.printUnformatted("#ifdef SIDL_DEBUG_REFCOUNT\n");
     d_writer.pushLineBreak(false);
-    d_writer.println("fprintf(stderr, \"babel: addRef %p new count %d (type %s)\\n\","+
-             "r_obj, r_obj->d_refcount, \"" + id.getFullName() + " Remote Stub\");"); 
+    d_writer.println("fprintf(stderr, "
+      + "\"babel: addRef %p new count %d (type %s)\\n\",");
+    d_writer.tab();
+    d_writer.println("r_obj, r_obj->d_refcount, ");
+    d_writer.println("\"" + id.getFullName() + " Remote Stub\");"); 
+    d_writer.backTab();
     d_writer.popLineBreak();
     d_writer.printUnformatted("#endif /* SIDL_DEBUG_REFCOUNT */ \n");
     d_writer.println(IOR.getUnlockStaticGlobalsMacroName() + ";");
@@ -1021,13 +1061,20 @@ public class RMIStubSource {
     d_writer.println();
     if (!outDisallowed ) { 
         d_writer.println("_be = sidl_rmi_Response_getExceptionThrown(_rsvp, _ex);SIDL_CHECK(*_ex);");
-        d_writer.println("if(_be != NULL) {");
+        d_writer.println("if (_be != NULL) {");
         d_writer.tab();
-        d_writer.println(IOR.getObjectName(BabelConfiguration.getBaseInterface())+"* throwaway_exception = NULL;");
-        d_writer.printlnUnformatted("sidl_BaseException_addLine(_be, \"Exception unserialized from " + ext.getFullName()+"."
-                         + method.getLongMethodName() +".\", &throwaway_exception);"); 
-        d_writer.println("*_ex = ("+IOR.getObjectName(BabelConfiguration.getBaseInterface())+
-                         "*) "+IOR.getSymbolName(BabelConfiguration.getBaseInterface())+"__cast(_be,&throwaway_exception);");
+        d_writer.println(IOR.getObjectName(BabelConfiguration.getBaseInterface())
+          + "* throwaway_exception = NULL;");
+        d_writer.println("sidl_BaseException_addLine(_be, ");
+        d_writer.tab();
+        d_writer.printlnUnformatted("      \"Exception unserialized from " 
+          + ext.getFullName() + "." + method.getLongMethodName() +".\",");
+        d_writer.println("&throwaway_exception);"); 
+        d_writer.backTab();
+        d_writer.println("*_ex = (" 
+           + IOR.getObjectName(BabelConfiguration.getBaseInterface())
+           + "*) " + IOR.getSymbolName(BabelConfiguration.getBaseInterface())
+           + "__cast(_be,&throwaway_exception);");
         d_writer.println("goto EXIT;");
         d_writer.backTab();
         d_writer.println("}");
@@ -1041,10 +1088,10 @@ public class RMIStubSource {
       // ( sidl.rmi.Ticket is generated as a local instance by the protocol)
       comment("extract return value");
       RMI
-          .unpackType(d_writer, d_context,
-                      ext, "sidl_rmi_Response", "_rsvp", returnType,
-              "_retval", "_retval", Argument.IN, method.isReturnCopy(), true,
-              false);
+        .unpackType(d_writer, d_context,
+                    ext, "sidl_rmi_Response", "_rsvp", returnType,
+                    "_retval", "_retval", Argument.IN, method.isReturnCopy(), true,
+                    false);
       d_writer.println();
     }
 
@@ -1054,9 +1101,9 @@ public class RMIStubSource {
         Argument arg = (Argument) a.next();
         if (arg.getMode() != Argument.IN) {
           RMI
-              .unpackArg(d_writer, d_context,
-                         ext, "sidl_rmi_Response", "_rsvp", arg,
-                  false);
+            .unpackArg(d_writer, d_context,
+                       ext, "sidl_rmi_Response", "_rsvp", arg,
+                       false);
         }
       }
       d_writer.println();
@@ -1126,15 +1173,15 @@ public class RMIStubSource {
     /*
      * Output standard methods.
      */
-    boolean doHooks = false;
-
-    for (int j = 0; j < IOR.CLASS_BUILTIN_METHODS; j++) {
-      if (IOR.isBuiltinBasic(j) || ((j == IOR.HOOKS) && doHooks)) {
+    for (int j = 0; j < IOR.CLASS_BUILT_IN_METHODS; j++) {
+      if (IOR.isBuiltinBasic(j)) {
         String mname = IOR.getBuiltinName(j);
         d_writer.print("epv->");
         d_writer.printAligned(IOR.getVectorEntry(mname), mwidth);
-        if ((j == IOR.CONSTRUCTOR) || (j == IOR.DESTRUCTOR)
-            || (j == IOR.CHECK_ERROR) || (j == IOR.CONSTRUCTOR2)) {
+        if (   (j == IOR.CONSTRUCTOR) 
+            || (j == IOR.DESTRUCTOR)
+            || (j == IOR.CONSTRUCTOR2)  ) 
+        {
           d_writer.println(" = " + C.NULL + ";");
         } else {
           d_writer.println(" = " + getRemoteMethodName(id, mname) + ';');
@@ -1148,12 +1195,10 @@ public class RMIStubSource {
      */
     for (Iterator i = methods.iterator(); i.hasNext();) {
       Method method = (Method) i.next();
-      assignRemoteEPVPointer(name, method, mwidth, doHooks);
+      assignRemoteEPVPointer(name, method, mwidth);
       if (method.getCommunicationModifier() == Method.NONBLOCKING) {
-        assignRemoteEPVPointer(name, method.spawnNonblockingSend(), mwidth,
-            doHooks);
-        assignRemoteEPVPointer(name, method.spawnNonblockingRecv(), mwidth,
-            doHooks);
+        assignRemoteEPVPointer(name, method.spawnNonblockingSend(), mwidth);
+        assignRemoteEPVPointer(name, method.spawnNonblockingRecv(), mwidth);
       }
     }
 
@@ -1188,28 +1233,13 @@ public class RMIStubSource {
    *          The method in the EPV to initialzie
    * @param mwidth
    *          formatting parameter related to width of longest method name
-   * @param doHooks
-   *          trigger for extra semantic code generation
    */
-  private void assignRemoteEPVPointer(String name, Method method, int mwidth,
-      boolean doHooks) {
+  private void assignRemoteEPVPointer(String name, Method method, int mwidth) {
     String mname = method.getLongMethodName();
     String ename = IOR.getVectorEntry(mname);
-    if (doHooks) {
-      d_writer.print("epv->");
-      d_writer.printAligned(ename + IOR.GENERIC_PRE_SUFFIX, mwidth);
-      d_writer.println(" = " + C.NULL + ";");
-      d_writer.print("epv->");
-      d_writer.printAligned(ename, mwidth);
-      d_writer.println(" = remote_" + name + "_" + mname + ";");
-      d_writer.print("epv->");
-      d_writer.printAligned(ename + IOR.GENERIC_POST_SUFFIX, mwidth);
-      d_writer.println(" = " + C.NULL + ";");
-    } else {
-      d_writer.print("epv->");
-      d_writer.printAligned(ename, mwidth);
-      d_writer.println(" = remote_" + name + "_" + mname + ";");
-    }
+    d_writer.print("epv->");
+    d_writer.printAligned(ename, mwidth);
+    d_writer.println(" = remote_" + name + "_" + mname + ";");
   }
 
   /**
@@ -1687,7 +1717,7 @@ public class RMIStubSource {
      */
     SymbolID id = cls.getSymbolID();
     String epv = IOR.getEPVName(id);
-    int epvType = remote ? IOR.EPV_REMOTE : IOR.EPV_NEW;
+    int epvType = remote ? IOR.EPV_REMOTE : IOR.EPV_MINE;
 
     int width = epv.length() + 1;
     int w = Utilities.getWidth(parents) + "struct __epv*".length();
@@ -1742,7 +1772,7 @@ public class RMIStubSource {
       }
 
       /*
-       * Generate the assignments for the cast and delete functions.
+       * Generate the assignments for the built-in methods.
        */
       String estring = "e" + String.valueOf(e) + "->";
       String vecEntry = IOR.getVectorEntry(s_castBuiltin);
@@ -1750,37 +1780,62 @@ public class RMIStubSource {
       d_writer.print(estring);
       d_writer.printAligned(vecEntry, mwidth);
       d_writer.print(" = (void* (*)(" + self);
-      d_writer.print(",const char*, "+IOR.getObjectName(BabelConfiguration.getBaseInterface())+"**)) epv->");
+      d_writer.print(", const char*, "
+        + IOR.getObjectName(BabelConfiguration.getBaseInterface())
+        + "**)) epv->");
       d_writer.print(vecEntry);
       d_writer.println(";");
 
       vecEntry = IOR.getVectorEntry(s_deleteBuiltin);
       d_writer.print(estring);
       d_writer.printAligned(vecEntry, mwidth);
-      d_writer.println(" = (void (*)(" + self + ","+IOR.getObjectName(BabelConfiguration.getBaseInterface())+
-                       "**)) epv->" + vecEntry + ";");
+      d_writer.println(" = (void (*)(" + self + ", "
+        + IOR.getObjectName(BabelConfiguration.getBaseInterface())
+        + "**)) epv->" + vecEntry + ";");
 
       vecEntry = IOR.getVectorEntry(s_getURLBuiltin);
       d_writer.print(estring);
       d_writer.printAligned(vecEntry, mwidth);
-      d_writer.println(" = (char* (*)(" + self + ","+IOR.getObjectName(BabelConfiguration.getBaseInterface())+"**)) epv->" + vecEntry + ";");
+      d_writer.println(" = (char* (*)(" + self + ", "
+        + IOR.getObjectName(BabelConfiguration.getBaseInterface())
+        + "**)) epv->" + vecEntry + ";");
 
       vecEntry = IOR.getVectorEntry(s_raddRefBuiltin);
       d_writer.print(estring);
       d_writer.printAligned(vecEntry, mwidth);
-      d_writer.println(" = (void (*)(" + self + ","+IOR.getObjectName(BabelConfiguration.getBaseInterface())+"**)) epv->" + vecEntry + ";");
+      d_writer.println(" = (void (*)(" + self + ", "
+        + IOR.getObjectName(BabelConfiguration.getBaseInterface())
+        + "**)) epv->" + vecEntry + ";");
 
       vecEntry = IOR.getVectorEntry(s_isRemoteBuiltin);
       d_writer.print(estring);
       d_writer.printAligned(vecEntry, mwidth);
-      d_writer.println(" = (sidl_bool (*)(" + self + ","+IOR.getObjectName(BabelConfiguration.getBaseInterface())+"**)) epv->" + vecEntry
-          + ";");
+      d_writer.println(" = (sidl_bool (*)(" + self + ", "
+        + IOR.getObjectName(BabelConfiguration.getBaseInterface())
+        + "**)) epv->" + vecEntry + ";");
 
       vecEntry = IOR.getVectorEntry(s_set_hooksBuiltin);
       d_writer.print(estring);
       d_writer.printAligned(vecEntry, mwidth);
-      d_writer.println(" = (void (*)(" + self + ",int32_t, "+IOR.getObjectName(BabelConfiguration.getBaseInterface())+"**)) epv->" + vecEntry
-          + ";");
+      d_writer.println(" = (void (*)(" + self + ", sidl_bool, "
+        + IOR.getObjectName(BabelConfiguration.getBaseInterface())
+        + "**)) epv->" + vecEntry + ";");
+
+      vecEntry = IOR.getVectorEntry(s_set_contractsBuiltin);
+      d_writer.print(estring);
+      d_writer.printAligned(vecEntry, mwidth);
+      d_writer.println(" = (void (*)(" + self 
+        + ", sidl_bool, const char*, sidl_bool, "
+        + IOR.getObjectName(BabelConfiguration.getBaseInterface())
+        + "**)) epv->" + vecEntry + ";");
+
+      vecEntry = IOR.getVectorEntry(s_dump_statsBuiltin);
+      d_writer.print(estring);
+      d_writer.printAligned(vecEntry, mwidth);
+      d_writer.println(" = (void (*)(" + self 
+        + ", const char*, const char*, "
+        + IOR.getObjectName(BabelConfiguration.getBaseInterface())
+        + "**)) epv->" + vecEntry + ";");
 
 
       /*
