@@ -1,10 +1,10 @@
 //
 // File:        SidlSource.java
 // Package:     gov.llnl.babel.backend.sidl
-// Revision:    @(#) $Revision: 6212 $
+// Revision:    @(#) $Revision: 6482 $
 // Description: Generate a sidl file for contents of symbol table
 //
-// Copyright (c) 2002-2003, The Regents of the University of Calfornia.
+// Copyright (c) 2002-2003, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -34,12 +34,15 @@ package gov.llnl.babel.backend.sidl;
 import gov.llnl.babel.BabelConfiguration;
 import gov.llnl.babel.Context;
 import gov.llnl.babel.backend.CodeGenerationException;
+import gov.llnl.babel.backend.fortran.Fortran;
+import gov.llnl.babel.backend.fortran.ImplSource;
 import gov.llnl.babel.backend.sidl.ScopedName;
 import gov.llnl.babel.backend.sidl.Sidl;
 import gov.llnl.babel.backend.writers.LanguageWriter;
 import gov.llnl.babel.backend.writers.LanguageWriterForSidl;
 import gov.llnl.babel.backend.Utilities;
 import gov.llnl.babel.symbols.Argument;
+import gov.llnl.babel.symbols.AssertionExpression;
 import gov.llnl.babel.symbols.Assertion;
 import gov.llnl.babel.symbols.Attributes;
 import gov.llnl.babel.symbols.Class;
@@ -53,6 +56,7 @@ import gov.llnl.babel.symbols.Struct;
 import gov.llnl.babel.symbols.Symbol;
 import gov.llnl.babel.symbols.SymbolID;
 import gov.llnl.babel.symbols.SymbolUtilities;
+import gov.llnl.babel.symbols.Type;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
@@ -86,6 +90,7 @@ public class SidlSource {
     d_context = context;
   }
 
+
   /**
    * Generate the SIDL for a struct.
    *
@@ -95,6 +100,7 @@ public class SidlSource {
    *   phase.
    */
   public void generateCode(Struct st)
+    throws CodeGenerationException
   {
     d_lw.writeComment(st, true, false);
     generateCustomAttrs(st, null);
@@ -103,12 +109,60 @@ public class SidlSource {
     Iterator i = st.getItems().iterator();
     while(i.hasNext()) {
       Struct.Item item = (Struct.Item)i.next();
-      d_lw.println(item.getType().getTypeString() + " " + item.getName() + ";");
+
+      Type t = item.getType();
+      int type = t.getDetailedType();
+      Integer objInt = new Integer(type);
+      if (type == Type.ARRAY) {
+        final Type arrayType = t.getArrayType();
+        if (null != arrayType) {
+          if (t.isRarray()) {
+            if (t.isNumericArray()){
+              generateIndices(item);
+            }
+          } else {
+            d_lw.println(item.getType().getTypeString() + " " + item.getName() + ";");
+          }
+        }
+      } else {
+        d_lw.println(item.getType().getTypeString() + " " + item.getName() + ";");
+      }
     }
     d_lw.backTab();
     d_lw.println("}");
   }
 
+  private void generateIndices(Struct.Item e)
+    throws CodeGenerationException
+    {
+
+      Type t = e.getType();
+      Type aType = t.getArrayType();
+      Integer objInt = new Integer(aType.getBasicType());
+      StringBuffer rawString = new StringBuffer();
+      Iterator indexVar = t.getArrayIndexExprs().iterator();
+      AssertionExpression ae = (AssertionExpression)indexVar.next();
+      String checkInt = ae.toString();
+      rawString.append(e.getType().getTypeString() + " " +e.getName() + "("+checkInt);
+
+      final int dims=t.getArrayDimension();
+
+      int count=1;
+      while(indexVar.hasNext()) {
+        ae = (AssertionExpression)indexVar.next();
+        checkInt = ae.toString();
+        rawString.append(","+checkInt);
+        count++;
+      }
+      rawString.append(");");
+      d_lw.println(rawString.toString());
+
+      if (count != dims){
+        throw new CodeGenerationException("Babel: Error in Rarray dimensions. Dimensions="+dims+" of type don't matc    h those given in name "+e.getName()+" (dimensions="+count+").");
+      }
+    }
+
+
   private void generateCustomAttrs(Attributes a, String [] builtins)
   {
     if (a.getAttributes().size() > 0) {
@@ -204,7 +258,7 @@ public class SidlSource {
    * Return indication of whether or not the symbol name is the same as
    * the base interface.
    *
-   * @param name  the symbol name being checked
+   * @param name the symbol name being checked
    * @return     true if the name equals the base interface name; else, false
    */
   public static boolean isBaseInterfaceName(String name) {
@@ -283,15 +337,55 @@ public class SidlSource {
   /**
    * Print the assertion.
    *
-   * @param lw           the language writer to which the signature is printed
+   * @param lw  The language writer to which the signature is printed.
+   * @param as  The assertion whose contents are being printed.
    */
-  public void printAssertion(LanguageWriter lw, Assertion as)
+  private void printAssertion(LanguageWriter lw, Assertion as)
   {
     Comment cmt = as.getComment();
     lw.writeComment(cmt, true);
     lw.println(as.toString() + ";");
   }
 
+  /**
+   * Print the list of assertions associated with a contract clause.
+   *
+   * @param lw      The language writer to which the signature is printed.
+   * @param type    The name of the contract clause.
+   * @param clause  The list of assertions contained in the clause.
+   */
+  private void printClause(LanguageWriter lw, String typeStr, List clause)
+  {
+    if (clause.size() > 0) {
+      lw.tab();
+      lw.println(typeStr);
+      lw.tab();
+      for (Iterator i = clause.iterator(); i.hasNext(); ) {
+        printAssertion(lw, (Assertion) i.next());
+      }
+      lw.backTab();
+      lw.backTab();
+    }
+  }
+
+  /**
+   * Print the interface contract clause contents.
+   *
+   * @param lw     The language writer to which the signature is printed.
+   * @param type   The type of clause (see Assertion.java).
+   * @param clause The list of assertions contained in the clause.
+   *                   
+   */
+  private void printContractClause(LanguageWriter lw, int type, List clause)
+  {
+    if ( Assertion.isInvariant(type)
+       || Assertion.isPrecondition(type)
+       || Assertion.isPostcondition(type) ) 
+    {
+      printClause(lw, Assertion.getTypeName(type), clause);
+    }
+  }
+
 
   /**
    * Print the signature of the method, optionally including the definition
@@ -299,19 +393,19 @@ public class SidlSource {
    * <code>LanguageWriter</code>.  Also, optionally abbreviate the type if 
    * in specified package.
    *
-   * @param lw           the language writer to which the signature is printed
-   * @param ext          The Extendable this method is being generated for
-   * @param meth         the method whose signature is being printed
-   * @param parent_pkg   the string containing the parent package.  When not
-   *                     null, it is used to strip the package from the return
-   *                     string if it is in the specified package.
+   * @param lw          the language writer to which the signature is printed
+   * @param ext         the Extendable for which this method is being generated
+   * @param meth        the method whose signature is being printed
+   * @param parent_pkg  the string containing the parent package.  When not
+   *                    null, it is used to strip the package from the return
+   *                    string if it is in the specified package.
    */
   public void printSignaturePlus(LanguageWriter lw, Extendable ext,
-                             Method meth, String parent_pkg)
+                                 Method meth, String parent_pkg)
   {
     lw.print(meth.getSignaturePreface(ext.isInterface(), parent_pkg) + "(");
 
-    int       increased_tabs = 0;
+    int  increased_tabs = 0;
     List args           = meth.getArgumentListWithIndices();
     for (Iterator a = args.iterator(); a.hasNext(); ) {
       // ToDo...Once ScopedName's getScopedName() differs from the
@@ -325,7 +419,7 @@ public class SidlSource {
           lw.tab();
           lw.tab();
           lw.tab();
-          increased_tabs = increased_tabs + 3;
+          increased_tabs = 3;
         }
       }
     }
@@ -336,13 +430,14 @@ public class SidlSource {
       lw.println();
       if (increased_tabs > 0) {
         lw.backTab();
-        increased_tabs = increased_tabs - 1;
+        increased_tabs--;
       } else {
         lw.tab();
         lw.tab();
         increased_tabs = 2;
       }
       lw.print("throws ");
+      boolean indentedThrows = false;
       for (Iterator t = exceptions.iterator(); t.hasNext(); ) {
         SymbolID sid = (SymbolID) t.next();
         String fname = sid.getFullName();
@@ -353,58 +448,53 @@ public class SidlSource {
         }
         if (t.hasNext()) {
           lw.println(",");
+          if (!indentedThrows) {
+            lw.tab();
+            indentedThrows = true;
+          }
         }
       }
+      if (indentedThrows) {
+        lw.backTab();
+        indentedThrows = false;
+      }
     }
 
-    if(ext.methodWasRenamed(meth)) {
+    if (ext.methodWasRenamed(meth)) {
       Method oldM = ext.getRenamedMethod(meth);
       SymbolID sid = ext.getRenamedMethodSymbolID(oldM);
       lw.print(" from " + sid.getFullName()+"."+oldM.getShortMethodName());
       String extension = oldM.getNameExtension();
       if ((extension != null) && (extension != "")) {
-        lw.print(" ["+extension+"] ");
+        lw.print(" [" + extension + "] ");
       }
     }
 
-    boolean   noAssertions  = true;
-    List preconditions = meth.getPreconditions();
-    if (!preconditions.isEmpty()) {
+    boolean noContract = true;
+    List    preClause  = meth.getPreClause();
+    if (!preClause.isEmpty()) {
       lw.println(";");
       for (int i=increased_tabs; i > 1; i--) {
         lw.backTab();
-        increased_tabs = increased_tabs - 1;
+        increased_tabs--;
       }
-      noAssertions = false;
-      lw.tab();
-      lw.println("require ");
-      lw.tab();
-      for (Iterator i = preconditions.iterator(); i.hasNext(); ) {
-        printAssertion(lw, (Assertion) i.next());
-      }
-      lw.backTab();
-      lw.backTab();
+      noContract = false;
+      printContractClause(lw, Assertion.REQUIRE, preClause);
     }
-    List postconditions = meth.getPostconditions();
-    if (!postconditions.isEmpty()) {
-      if (noAssertions) {
-        noAssertions = false;
+
+    List postClause = meth.getPostClause();
+    if (!postClause.isEmpty()) {
+      if (noContract) {
+        noContract = false;
         lw.println(";");
         for (int i=increased_tabs; i > 1; i--) {
           lw.backTab();
-          increased_tabs = increased_tabs - 1;
+          increased_tabs--;
         }
       }
-      lw.tab();
-      lw.println("ensure ");
-      lw.tab();
-      for (Iterator i = postconditions.iterator(); i.hasNext(); ) {
-        printAssertion(lw, (Assertion) i.next());
-      }
-      lw.backTab();
-      lw.backTab();
+      printContractClause(lw, Assertion.ENSURE, postClause);
     }
-    if (noAssertions) {
+    if (noContract) {
       lw.println(";");
     }
     for (int i=increased_tabs; i > 0; i--) {
@@ -421,6 +511,9 @@ public class SidlSource {
    * @exception  gov.llnl.babel.backend.CodeGenerationException
    *   a catch all exception to indicate problems in the code generation
    *   phase.
+   *
+   * Assumes the method is sandwiched between the printing of open
+   * and close parentheses.
    */
   public void printMethods(Extendable ext)
     throws CodeGenerationException
@@ -428,8 +521,6 @@ public class SidlSource {
     Collection methods = ext.getMethods(false);
     String     name    = Sidl.getSymbolName(ext.getSymbolID());
 
-    d_lw.println("{");
-    d_lw.tab();
     for (Iterator m = methods.iterator(); m.hasNext(); ) {
       Method meth = (Method)m.next();
       d_lw.writeComment(meth, true, false);
@@ -439,8 +530,6 @@ public class SidlSource {
         d_lw.println();
       }
     }
-    d_lw.backTab();
-    d_lw.println("}");
   }
 
 
@@ -455,10 +544,10 @@ public class SidlSource {
   public void generateCode(Class cls)
     throws CodeGenerationException
   {
-    String builtinAttrs[] = { "abstract" };
-    SymbolID sid = cls.getSymbolID();
-    String sname = sid.getShortName();
-    String scope = SymbolUtilities.getParentPackage(sid.getFullName());
+    String   builtinAttrs[] = { "abstract" };
+    SymbolID sid   = cls.getSymbolID();
+    String   sname = sid.getShortName();
+    String   scope = SymbolUtilities.getParentPackage(sid.getFullName());
 
     d_lw.println();
     d_lw.writeComment(cls, true, false);
@@ -484,15 +573,20 @@ public class SidlSource {
     if ( printed_extends ) {
       d_lw.backTab();
     }
+    d_lw.println("{");
+    d_lw.tab();
+    printContractClause(d_lw, Assertion.INVARIANT, cls.getInvClause(false));
     printMethods((Extendable)cls);
+    d_lw.backTab();
+    d_lw.println("}");
   }
 
 
   /**
    * Generate the SIDL for the interface.
    * 
-   * @param iface  the SIDL interface to be written.
-   * @exception  gov.llnl.babel.backend.CodeGenerationException
+   * @param iface the SIDL interface to be written.
+   * @exception   gov.llnl.babel.backend.CodeGenerationException
    *   a catch all exception to indicate problems in the code generation
    *   phase.
    */
@@ -506,7 +600,12 @@ public class SidlSource {
     generateCustomAttrs(iface, null);
     d_lw.print("interface " + name + " ");
     printParentInterfaces((Extendable)iface);
+    d_lw.println("{");
+    d_lw.tab();
+    printContractClause(d_lw, Assertion.INVARIANT, iface.getInvClause(false));
     printMethods((Extendable)iface);
+    d_lw.backTab();
+    d_lw.println("}");
   }
 
 
@@ -534,10 +633,10 @@ public class SidlSource {
     throws CodeGenerationException
   {
     final String builtinAttrs[] = { "final" };
-    Collection symids  = pkg.getOrderedSymbolReferences();
-    SymbolID id        = pkg.getSymbolID();
-    String name        = id.getShortName();
-    String vstr        = null;
+    Collection   symids = pkg.getOrderedSymbolReferences();
+    SymbolID     id     = pkg.getSymbolID();
+    String       name   = id.getShortName();
+    String       vstr   = null;
 
     /*
      * First generate the requires/versions and imports statements but only
@@ -571,9 +670,6 @@ public class SidlSource {
       SymbolID sid  = (SymbolID)s.next();
       Symbol sym    = d_context.getSymbolTable().lookupSymbol(sid);
       switch (sym.getSymbolType()) { 
-      case Symbol.STRUCT:
-        generateCode((Struct)sym);
-        break;
         case Symbol.CLASS:
           generateCode((Class)sym);
           break;
@@ -588,6 +684,9 @@ public class SidlSource {
           generateCode((Package)sym);
           d_lw.println();
           break;
+        case Symbol.STRUCT:
+          generateCode((Struct)sym);
+          break;
         default:
           throw new CodeGenerationException("Unknown/Invalid Symbol Type "
                          + "detected during SIDL Package generation");
