@@ -4,7 +4,7 @@
 // Revision:    @(#) $$
 // Description: common C++ binding routines shared by C++ code generators
 //
-// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -91,6 +91,8 @@ public class Cxx {
   public final static int FILE_ROLE_SKEL = 2;
   public final static int FILE_ROLE_STUB = 3;
 
+  public final static String NULL = "NULL";
+
 
   /*  The indices for the following MUST correspond to the role values above */
   public final static String FILE_ROLE_SUFFIX[] =
@@ -1023,13 +1025,10 @@ public class Cxx {
       for(Iterator j = argTypes.iterator(); j.hasNext();) {
         SymbolID argID = (SymbolID) j.next();
         Symbol symbol = (Symbol) Utilities.lookupSymbol(context, argID);
-        if (symbol.getSymbolType() == Type.ENUM) {
+        if (symbol.getSymbolType() == Type.ENUM || symbol.getSymbolType() == Type.STRUCT ) {
           includes.add(argID);
         } 
       }
-      if (method.getReturnType().getDetailedType() == Type.STRUCT) {
-        includes.add(method.getReturnType().getSymbolID());
-      }
     }
     
     // remove self Header file?  If invoked from Stub yes, 
@@ -1269,7 +1268,8 @@ public class Cxx {
                                               Method method, 
                                               String altcomment,
                                               int role, 
-                                              boolean rarrays)
+                                              boolean rarrays,
+                                              boolean hasInvariants)
     throws CodeGenerationException
   {
     if ( method == null ) { return; }
@@ -1303,11 +1303,13 @@ public class Cxx {
       writer.println();
       writer.backTab();
       writer.println(")");
-      generateThrowsList( writer, method, canThrowNullIORException, context );
+      generateThrowsList( writer, method, canThrowNullIORException, 
+                          hasInvariants, context );
       writer.println(";");
     } else { 
       writer.print("() ");
-      generateThrowsList( writer, method, canThrowNullIORException, context );
+      generateThrowsList( writer, method, canThrowNullIORException, 
+                          hasInvariants, context );
       writer.print(";");
     }
     writer.println();
@@ -1318,7 +1320,8 @@ public class Cxx {
                                                     Method method, 
                                                     String altcomment,
                                                     int role, boolean isSuper, 
-                                                    boolean rarrays)
+                                                    boolean rarrays,
+                                                    boolean hasInvariants)
     throws CodeGenerationException
   {
     if ( method == null ) { return; }
@@ -1350,11 +1353,13 @@ public class Cxx {
       writer.println();
       writer.backTab();
       writer.println(")");
-      generateThrowsList( writer, method, canThrowNullIORException, context );
+      generateThrowsList( writer, method, canThrowNullIORException, 
+                          hasInvariants, context );
       writer.println("{");
     } else { 
       writer.print("() ");
-      generateThrowsList( writer, method, canThrowNullIORException, context );
+      generateThrowsList( writer, method, canThrowNullIORException, 
+                          hasInvariants, context );
       writer.print("{");
     }
     writer.println();
@@ -1385,56 +1390,135 @@ public class Cxx {
   public static void generateThrowsList( LanguageWriterForCxx writer, 
 					 Method method, 
 					 boolean canThrowNullIORException,
+                                         boolean hasInvariants,
                                          Context context) { 
-    Set exceptions = method.getThrows();
+    Set     exceptions = method.getThrows();
+    boolean hasPre     = method.hasPreClause();
+    boolean hasPost    = method.hasPostClause();
 
     if ( exceptions == null || exceptions.isEmpty() ) { 
-	if ( canThrowNullIORException && context.getConfig().makeCxxCheckNullIOR()) { 
-          writer.writeCommentLine("throws: ::sidl::NullIORException");
-	} 
-        else { 
-	    writer.println( "throw () " );
-	}
-      return;
-    } else { 
-      if (!method.getExplicitThrows().isEmpty()) {
+      if (  canThrowNullIORException 
+         && context.getConfig().makeCxxCheckNullIOR()) 
+      { 
+        writer.writeCommentLine("throws:");
+        writer.writeCommentLine("   ::sidl::NullIORException");
+        if (hasPre) {
+          writer.writeCommentLine("   ::sidl::PreViolation");
+        }
+        if (hasPost) {
+          writer.writeCommentLine("   ::sidl::PostViolation");
+        }
+        if (hasInvariants) {
+          writer.writeCommentLine("   ::sidl::InvViolation");
+        }
+      } else { 
+	writer.println( "throw () " );
+      }
+    } else {  /* Throws implicit and/or explicit exceptions */
+      boolean hasPreExc  = false;
+      boolean hasPostExc = false;
+      boolean hasInvExc  = false;
+      if (method.hasExplicitExceptions()) {
         writer.writeCommentLine("throws:");
         if ( canThrowNullIORException ) { 
-          writer.writeCommentLine("    ::sidl::NullIORException");
+          writer.writeCommentLine("       ::sidl::NullIORException");
         }
         for( Iterator e = exceptions.iterator(); e.hasNext(); ) { 
-          SymbolID id = (SymbolID) e.next();
+          SymbolID id    = (SymbolID) e.next();
+          String excName = Cxx.getObjectName(id);
+          hasPreExc  = hasPreExc  || excName.equals("::sidl::PreViolation");
+          hasPostExc = hasPostExc || excName.equals("::sidl::PostViolation");
+          hasInvExc  = hasInvExc  || excName.equals("::sidl::InvViolation");
           
-          writer.writeCommentLine("    " + Cxx.getObjectName(id));
+          writer.writeCommentLine("   " + excName);
+        }
+      } else { 
+        /*
+         * Should at least output the "standard" implicit set...
+         */
+        if (hasPre || hasPost || hasInvariants) {
+          writer.writeCommentLine("throws:");
         }
       }
+      
+      if (hasPre && (!hasPreExc)) {
+        writer.writeCommentLine("   ::sidl::PreViolation");
+      }
+      if (hasPost && (!hasPostExc)) {
+        writer.writeCommentLine("   ::sidl::PostViolation");
+      }
+      if (hasInvariants && (!hasInvExc)) {
+        writer.writeCommentLine("   ::sidl::InvViolation");
+      }
     }
+    return;
   }
 
   public static String generateThrowsList(Method method, 
                                           boolean canThrowNullIORException,
+                                          boolean hasInvariants,
                                           Context context) 
   { 
-    Set exceptions = method.getThrows();
+    Set exceptions   = method.getThrows();
     StringBuffer ret = new StringBuffer();
+    boolean hasPre   = method.hasPreClause();
+    boolean hasPost  = method.hasPostClause();
     
-    if ( exceptions == null || exceptions.isEmpty() ) { 
-      if ( canThrowNullIORException && context.getConfig().makeCxxCheckNullIOR()) { 
-        return "// throws: ::sidl::NullIORException\n";
+    if ( exceptions == null || exceptions.isEmpty() ) 
+    {
+      if (  canThrowNullIORException 
+         && context.getConfig().makeCxxCheckNullIOR()) 
+      {
+        ret.append("// throws:\n");
+        ret.append("//     ::sidl::NullIORException\n");
+        if (hasPre) {
+          ret.append("//   ::sidl::PreViolation\n");
+        }
+        if (hasPost) {
+          ret.append("//   ::sidl::PostViolation\n");
+        }
+        if (hasInvariants) {
+          ret.append("//   ::sidl::InvViolation\n");
+        }
       } else { 
-        return "throw () \n";
+        ret.append("throw () \n");
       }
- 
     } else { 
+      boolean hasPreExc  = false;
+      boolean hasPostExc = false;
+      boolean hasInvExc  = false;
       if (!method.getExplicitThrows().isEmpty()) {
         ret.append("// throws:\n");
-        if ( canThrowNullIORException && context.getConfig().makeCxxCheckNullIOR()) { 
-          ret.append("//       ::sidl::NullIORException\n");
+        if (  canThrowNullIORException 
+           && context.getConfig().makeCxxCheckNullIOR()) 
+        { 
+          ret.append("//   ::sidl::NullIORException\n");
         }
         for( Iterator e = exceptions.iterator(); e.hasNext(); ) { 
           SymbolID id = (SymbolID) e.next();
-          ret.append("//       ").append(Cxx.getObjectName(id)).append('\n');
+          String excName = Cxx.getObjectName(id);
+          hasPreExc  = hasPreExc  || excName.equals("::sidl::PreViolation");
+          hasPostExc = hasPostExc || excName.equals("::sidl::PostViolation");
+          hasInvExc  = hasInvExc  || excName.equals("::sidl::InvViolation");
+          ret.append("//   ").append(excName).append('\n');
         }
+      } else {
+        /*
+         * Should at least output the "standard" implicit set...
+         */
+        if (hasPre || hasPost || hasInvariants) {
+          ret.append("// throws:\n");
+        }
+      }
+
+      if (hasPre && (!hasPreExc)) {
+        ret.append("//   ::sidl::PreViolation\n");
+      }
+      if (hasPost && (!hasPostExc)) {
+        ret.append("//   ::sidl::PostViolation\n");
+      }
+      if (hasInvariants && (!hasInvExc)) {
+        ret.append("//   ::sidl::InvViolation\n");
       }
     }
     return ret.toString();
@@ -1443,8 +1527,10 @@ public class Cxx {
   /**
    *  Generate Signature for the exception throwing method
    */
-  public static void generateExceptionSetSignature( LanguageWriterForCxx writer, SymbolID id,
-                                                    Set throw_set, int index, boolean stub, boolean isSuper) 
+  public static void generateExceptionSetSignature( LanguageWriterForCxx writer,
+                                                    SymbolID id, Set throw_set,
+                                                    int index, boolean stub, 
+                                                    boolean isSuper) 
     throws CodeGenerationException {
     
 
@@ -1466,7 +1552,7 @@ public class Cxx {
       for(Iterator set_i = throw_set.iterator(); set_i.hasNext();) {
         SymbolID ex_id = (SymbolID) set_i.next();;
         
-        writer.writeCommentLine("  " + Cxx.getObjectName(ex_id) );
+        writer.writeCommentLine("   " + Cxx.getObjectName(ex_id) );
         
       }
     }
@@ -1600,7 +1686,8 @@ public class Cxx {
       }
     }
     func_decl.append(" )\n");
-    func_decl.append(Cxx.generateThrowsList(m, true, context ));
+    func_decl.append(Cxx.generateThrowsList(m, true, d_ext.hasInvClause(true), 
+                     context ));
     
     return func_decl.toString().trim();
   }
