@@ -1,12 +1,12 @@
 //
 // File:        CxxImplHeader.java
 // Package:     gov.llnl.babel.backend.ucxx
-// Revision:    @(#) $Revision: 6201 $
-// Date:        $Date: 2007-10-25 23:15:35 -0700 (Thu, 25 Oct 2007) $
+// Revision:    @(#) $Revision: 6494 $
+// Date:        $Date: 2008-09-04 16:24:45 -0700 (Thu, 04 Sep 2008) $
 // Description: Write Cxx extension header file for a BABEL extendable
 // 
 // This is typically directed by GenCxxClient.
-// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -142,7 +142,7 @@ public class CxxImplHeader {
         d_writer.println(CodeConstants.C_UNREFERENCED_COMMENT1);
         d_writer.println(CodeConstants.C_UNREFERENCED_COMMENT2);
         d_writer.println(CodeConstants.C_UNREFERENCED_COMMENT3);
-        d_writer.println(CodeConstants.C_END_UNREFERENCED_METHODS);
+        d_writer.endBlockComment(true);
         d_splicer.outputUnusedSymbolEdits( d_writer );
         d_writer.writeCommentLine(CodeConstants.C_END_UNREFERENCED_METHODS);
       }
@@ -336,20 +336,21 @@ public class CxxImplHeader {
 
     Iterator m = null;
     m = d_ext.getStaticMethods(false).iterator();
+    boolean hasInvariants = d_ext.hasInvClause(true);
     while (m.hasNext()) {
       Method method = (Method) m.next();
       Cxx.generateMethodSignature( d_writer, d_context, method, 
                                    "user defined static method", 
-                                   Cxx.FILE_ROLE_IMPL, true);
-      if(IOR.supportHooks(d_ext, d_context)) {
-        Method pre = method.spawnPreHook();
+                                   Cxx.FILE_ROLE_IMPL, true, hasInvariants);
+      if (IOR.generateHookMethods(d_ext, d_context)) {
+        Method pre  = method.spawnPreHook();
         Method post = method.spawnPostHook();
         Cxx.generateMethodSignature( d_writer, d_context,  pre, 
                                      "User Defined static method pre-hook", 
-                                     Cxx.FILE_ROLE_IMPL, true);
+                                     Cxx.FILE_ROLE_IMPL, true, false);
         Cxx.generateMethodSignature( d_writer, d_context, post, 
                                      "user defined static method post-hook", 
-                                     Cxx.FILE_ROLE_IMPL, true);
+                                     Cxx.FILE_ROLE_IMPL, true, false);
       }
     }
     d_writer.println();
@@ -360,16 +361,16 @@ public class CxxImplHeader {
       if ( !method.isAbstract() ) { 
         Cxx.generateMethodSignature( d_writer, d_context, method, 
                                      "user defined non-static method.", 
-                                     Cxx.FILE_ROLE_IMPL, true);
-        if(IOR.supportHooks(d_ext, d_context)) {
-          Method pre = method.spawnPreHook();
+                                     Cxx.FILE_ROLE_IMPL, true, hasInvariants);
+        if (IOR.generateHookMethods(d_ext, d_context)) {
+          Method pre  = method.spawnPreHook();
           Method post = method.spawnPostHook();
           Cxx.generateMethodSignature( d_writer, d_context, pre, 
                                        "User Defined static method pre-hook", 
-                                       Cxx.FILE_ROLE_IMPL, true);
+                                       Cxx.FILE_ROLE_IMPL, true, false);
           Cxx.generateMethodSignature( d_writer, d_context, post, 
                                        "user defined static method post-hook", 
-                                       Cxx.FILE_ROLE_IMPL, true);
+                                       Cxx.FILE_ROLE_IMPL, true, false);
         }
       }
     }
