@@ -1,12 +1,12 @@
 //
 // File:        CxxImplSource.java
 // Package:     gov.llnl.babel.backend.ucxx
-// Revision:    @(#) $Revision: 6201 $
-// Date:        $Date: 2007-10-25 23:15:35 -0700 (Thu, 25 Oct 2007) $
+// Revision:    @(#) $Revision: 6494 $
+// Date:        $Date: 2008-09-04 16:24:45 -0700 (Thu, 04 Sep 2008) $
 // Description: Write Cxx extension header file for a BABEL extendable
 // 
 // This is typically directed by GenCxxClient.
-// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -252,19 +252,20 @@ public class CxxImplSource {
   }
 
   private void writeSIDLDefinedMethods() throws CodeGenerationException { 
+    boolean hasInvariants = d_ext.hasInvClause(true);
     List static_methods = (List) d_ext.getStaticMethods(false);
     if ( static_methods.size() > 0 ) { 
       d_writer.writeCommentLine("user defined static methods:");
       for (Iterator m = static_methods.iterator(); m.hasNext(); ) {
         Method method = (Method) m.next();
-        if (IOR.supportHooks(d_ext, d_context)) {
+        if (IOR.generateHookMethods(d_ext, d_context)) {
           Method pre = method.spawnPreHook();
           Method post = method.spawnPostHook();
-          generateMethod(pre, false);
-          generateMethod(method, true);
-          generateMethod(post, false);
+          generateMethod(pre, false, false);
+          generateMethod(method, true, hasInvariants);
+          generateMethod(post, false, false);
         } else {
-          generateMethod(method, true);
+          generateMethod(method, true, hasInvariants);
         }
       }
     } else { 
@@ -278,14 +279,14 @@ public class CxxImplSource {
       for (Iterator m = nonstatic_methods.iterator(); m.hasNext(); ) {
         Method method = (Method) m.next();
 	if ( !method.isAbstract() ) { 	    
-          if (IOR.supportHooks(d_ext, d_context)) {
+          if (IOR.generateHookMethods(d_ext, d_context)) {
             Method pre = method.spawnPreHook();
             Method post = method.spawnPostHook();
-            generateMethod(pre,false);
-            generateMethod(method,true);
-            generateMethod(post,false);
+            generateMethod(pre, false, false);
+            generateMethod(method, true, hasInvariants);
+            generateMethod(post, false, false);
           } else {
-            generateMethod(method,true);
+            generateMethod(method, true, hasInvariants);
           }
 	}
       }
@@ -296,7 +297,8 @@ public class CxxImplSource {
 
   }
 
-  private void generateMethod( Method method, boolean defaultSplicer ) 
+  private void generateMethod( Method method, boolean defaultSplicer,
+                               boolean hasInvariants ) 
     throws CodeGenerationException 
   { 
     if ( method == null ) { return ; }
@@ -316,11 +318,13 @@ public class CxxImplSource {
       Cxx.generateArgumentList( d_writer, method, d_context, true, false );
       d_writer.println(" ) ");
       d_writer.backTab();
-      Cxx.generateThrowsList( d_writer, method, false , d_context);
+      Cxx.generateThrowsList( d_writer, method, false, hasInvariants, 
+                              d_context);
       d_writer.println("{");
     } else { 
       d_writer.println(" () ");      
-      Cxx.generateThrowsList( d_writer, method, false, d_context);
+      Cxx.generateThrowsList( d_writer, method, false, hasInvariants, 
+                              d_context);
       d_writer.println();
       d_writer.println("{");
     }
