@@ -1,12 +1,12 @@
 //
 // File:        CxxSkelSource.java
 // Package:     gov.llnl.babel.backend.ucxx
-// Revision:    @(#) $Revision: 6212 $
-// Date:        $Date: 2007-10-31 17:07:17 -0700 (Wed, 31 Oct 2007) $
+// Revision:    @(#) $Revision: 6482 $
+// Date:        $Date: 2008-08-21 15:50:53 -0700 (Thu, 21 Aug 2008) $
 // Description: Write Cxx extension header file for a BABEL extendable
 // 
 // This is typically directed by GenCxxClient.
-// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -135,6 +135,7 @@ public class CxxSkelSource {
       d_writer.generateInclude( "sidl_String.h", false );
 
       d_writer.printlnUnformatted("#include <stddef.h>");
+      d_writer.printlnUnformatted("#include <cstring>");
  
       Class cls = (Class) d_ext;
       Set serializeSIDs = IOR.getStructSymbolIDs(cls, true);
@@ -176,36 +177,32 @@ public class CxxSkelSource {
   private void writeInitializeEPV() throws CodeGenerationException {
     SymbolID id = d_ext.getSymbolID();
     d_writer.println("void");
-    if(IOR.supportHooks(id, d_context)) {
-      d_writer.println(C.getSetEPVName(id) + "(" + IOR.getEPVName(id) + " *"+s_epv+",");
+    d_writer.print(C.getSetEPVName(id) + "(" + IOR.getEPVName(id) + " *");
+    d_writer.print(s_epv);
+    if (IOR.generateHookEPVs(d_ext, d_context)) {
+      d_writer.println(",");
       d_writer.tab();
-      d_writer.println(IOR.getPreEPVName(id) + " *"+s_pre_epv+", "+
-                     IOR.getPostEPVName(id) + " *"+s_post_epv+")");
+      d_writer.print(IOR.getPreEPVName(id) + " *" + s_pre_epv + ", ");
+      d_writer.print(IOR.getPostEPVName(id) + " *" + s_post_epv + ")");
       d_writer.backTab();
     } else {
-      d_writer.println(C.getSetEPVName(id) + "(" + IOR.getEPVName(id) + " *"+s_epv+")");
+      d_writer.println(")");
     }
     d_writer.println("{");
     d_writer.tab();
     d_writer.writeCommentLine("initialize builtin methods");
     initializeMethodPointer
-      (IOR.getBuiltinMethod(IOR.CONSTRUCTOR, id, d_context), id, s_epv); 
+      (IOR.getBuiltinMethod(IOR.CONSTRUCTOR, id, d_context), id);
     initializeMethodPointer
-      (IOR.getBuiltinMethod(IOR.CONSTRUCTOR2, id, d_context), id, s_epv); 
+      (IOR.getBuiltinMethod(IOR.CONSTRUCTOR2, id, d_context), id);
     initializeMethodPointer
-      (IOR.getBuiltinMethod(IOR.DESTRUCTOR, id, d_context), id, s_epv); 
+      (IOR.getBuiltinMethod(IOR.DESTRUCTOR, id, d_context), id);
     d_writer.writeCommentLine("initialize local methods");
     Iterator i = d_ext.getMethods(false).iterator();
     while (i.hasNext()) {
       Method m = (Method)i.next();
       if ( !m.isAbstract() && !m.isStatic()) { 
-        initializeMethodPointer( m, id, s_epv);
-        if (IOR.supportHooks(d_ext, d_context)) {
-          Method pre = m.spawnPreHook();
-          Method post = m.spawnPostHook();
-          initializeMethodPointer(pre, id, s_pre_epv);
-          initializeMethodPointer(post, id, s_post_epv);
-        }
+        initializeMethodPointer( m, id);
       }
     }
     d_writer.backTab();
@@ -213,6 +210,67 @@ public class CxxSkelSource {
     d_writer.println();
   }
 
+  /**
+   * For non-<code>static</code> methods, write an assignment statement to
+   * initialize a particular membor of the entry point vector.
+   * Initialization of <code>static</code> methods appears elsewhere. 
+   * 
+   * @param m          a description of the method to initialize
+   * @param id         the name of class that owns the method
+   */
+  private void initializeMethodPointer(Method m, SymbolID id) 
+    throws CodeGenerationException
+  {
+    final String methodName = m.getLongMethodName();
+    final String ename      = IOR.getVectorEntry(methodName);
+
+    switch (m.getDefinitionModifier()) {
+    case Method.FINAL:
+    case Method.NORMAL:
+      String sname = Cxx.getMethodSkelName(id, methodName);
+      if (!IOR.isBuiltinMethod(methodName)) {
+        Method pre      = m.spawnPreHook();
+        String preName  = pre.getLongMethodName();
+
+        Method post     = m.spawnPostHook();
+        String postName = post.getLongMethodName();
+
+        /*
+         * Order is (currently) critical!  If hook methods (i.e.,
+         * pre- and post-hooks) are being generated then be sure to
+         * initialize the EPV pointers to the skel/impl methods!
+         * Otherwise, if we're just making sure to include the
+         * basic hook skeletons (but not the implementations), then
+         * initialize the EPV pointers to null.
+         */
+        if (IOR.generateHookMethods(d_ext, d_context)) {
+          writeMethodAssignment(s_pre_epv, IOR.getVectorEntry(preName), 
+                                Cxx.getMethodSkelName(id, preName));
+          writeMethodAssignment(s_epv, ename, sname);
+          writeMethodAssignment(s_post_epv, IOR.getVectorEntry(postName),
+                                Cxx.getMethodSkelName(id, postName));
+        } else if (IOR.generateHookEPVs(d_ext, d_context)) {
+          writeMethodAssignment(s_pre_epv, IOR.getVectorEntry(preName), 
+                                Cxx.NULL);
+          writeMethodAssignment(s_epv, ename, sname);
+          writeMethodAssignment(s_post_epv, IOR.getVectorEntry(postName),
+                                Cxx.NULL);
+        } else {
+          writeMethodAssignment(s_epv, ename, sname);
+        }
+      } else {
+        writeMethodAssignment(s_epv, ename, sname);
+      }
+      break;
+    case Method.ABSTRACT:
+      writeMethodAssignment(s_epv, ename, Cxx.NULL);
+      break;
+    default:
+      /* do nothing */
+      break;
+    }
+  }
+
   /**
    * Write a function to initialize entries in the static entry point vector
    * for a particular class. This will generate an assignment statement for
@@ -222,16 +280,16 @@ public class CxxSkelSource {
   private void writeInitializeSEPV() throws CodeGenerationException { 
     SymbolID id = d_ext.getSymbolID();
     d_writer.println("void");
-    if(IOR.supportHooks(id, d_context)) {
-      d_writer.println(C.getSetSEPVName(id) + "(" + IOR.getSEPVName(id) + " *"+s_sepv+",");
+    d_writer.print(C.getSetSEPVName(id) + "(" + IOR.getSEPVName(id) + " *");
+    d_writer.print(s_sepv);
+    if (IOR.generateHookEPVs(d_ext, d_context)) {
+      d_writer.println(",");
       d_writer.tab();
-      d_writer.println(IOR.getPreSEPVName(id) + " *"+s_pre_sepv+", "+
-                     IOR.getPostSEPVName(id) + " *"+s_post_sepv+")");
+      d_writer.print(IOR.getPreSEPVName(id) + " *" + s_pre_sepv + ", ");
+      d_writer.println(IOR.getPostSEPVName(id) + " *" + s_post_sepv + ")");
       d_writer.backTab();
     } else {
-      d_writer.println(C.getSetSEPVName(id) + "(" + IOR.getSEPVName(id) 
-      + " *" + s_sepv + ")");
-
+      d_writer.println(")");
     }
 
     d_writer.println("{");
@@ -240,13 +298,7 @@ public class CxxSkelSource {
     while (i.hasNext()) {
       Method m = (Method)i.next();
       if (m.isStatic()) {
-        initializeMethodPointer( m, id, s_sepv);
-        if (IOR.supportHooks(d_ext, d_context)) {
-          Method pre = m.spawnPreHook();
-          Method post = m.spawnPostHook();
-          initializeMethodPointer(pre, id, s_pre_sepv);
-          initializeMethodPointer(post, id, s_post_sepv);
-        }
+        initializeStaticMethodPointer(m, id);
       }
     }
     d_writer.backTab();
@@ -255,31 +307,55 @@ public class CxxSkelSource {
   }
 
   /**
-   * For non-<code>static</code> methods, write an assignment statement to
-   * initialize a particular membor of the entry point
-   * vector. Initialization of <code>static</code> methods appears
-   * elsewhere. 
-   * 
+   * For <code>static</code> methods, write an assignment statement to
+   * initialize a particular member of the entry point vector.
+   * Initialization of non-<code>static</code> methods appears elsewhere.
+   *
    * @param m          a description of the method to initialize
    * @param id         the name of class that owns the method
-   * @param epvname    the name of the epv variable, we have 6 now...
    */
-  private void initializeMethodPointer(Method m, SymbolID id, String epvname) {
+  private void initializeStaticMethodPointer(Method m, SymbolID id)
+    throws CodeGenerationException
+  {
     final String methodName = m.getLongMethodName();
+    final String ename      = IOR.getVectorEntry(methodName);
+
     switch (m.getDefinitionModifier()) {
-    case Method.FINAL:
-    case Method.NORMAL:
     case Method.STATIC:
-      d_writer.print(epvname+"->");
-      d_writer.print(IOR.getVectorEntry(methodName));
-      d_writer.print(" = ");
-      d_writer.print(Cxx.getMethodSkelName(id, methodName));
-      d_writer.println(";");
-      break;
-    case Method.ABSTRACT:
-      d_writer.print(epvname+"->");
-      d_writer.print(IOR.getVectorEntry(methodName));
-      d_writer.println(" = NULL;");
+      String sname = Cxx.getMethodSkelName(id, methodName);
+      if (!IOR.isBuiltinMethod(methodName)) {
+        Method pre     = m.spawnPreHook();
+        String preName = pre.getLongMethodName();
+
+        Method post     = m.spawnPostHook();
+        String postName = post.getLongMethodName();
+
+        /*
+         * Order is (currently) critical!  If hook methods (i.e.,
+         * pre- and post-hooks) are being generated then be sure to
+         * initialize the EPV pointers to the skel/impl methods!
+         * Otherwise, if we're just making sure to include the
+         * basic hook skeletons (but not the implementations), then
+         * initialize the EPV pointers to null.
+         */
+        if (IOR.generateHookMethods(d_ext, d_context)) {
+          writeMethodAssignment(s_pre_sepv, IOR.getVectorEntry(preName),
+                                Cxx.getMethodSkelName(id, preName));
+          writeMethodAssignment(s_sepv, ename, sname);
+          writeMethodAssignment(s_post_sepv, IOR.getVectorEntry(postName),
+                                Cxx.getMethodSkelName(id, postName));
+        } else if (IOR.generateHookEPVs(d_ext, d_context)) {
+          writeMethodAssignment(s_pre_sepv, IOR.getVectorEntry(preName),
+                                Cxx.NULL);
+          writeMethodAssignment(s_sepv, ename, sname);
+          writeMethodAssignment(s_post_sepv, IOR.getVectorEntry(postName),
+                                Cxx.NULL);
+        } else {
+          writeMethodAssignment(s_sepv, ename, sname);
+        }
+      } else {
+        writeMethodAssignment(s_sepv, ename, sname);
+      }
       break;
     default:
       /* do nothing */
@@ -287,6 +363,20 @@ public class CxxSkelSource {
     }
   }
 
+
+  /**
+   * Write the function assignment for the specified var and method name
+   * to the specified value.
+   *
+   * @param var        the pointer variable representing the EPV.
+   * @param mname      the method name.
+   * @param value      the desired value, or RHS.
+   */
+  private void writeMethodAssignment(String var, String mname, String value) {
+    d_writer.println(var + "->" + mname + " = " + value + ";");
+  }
+
+
   private void writeConstructor() { 
     SymbolID id = d_ext.getSymbolID();
     d_writer.println("static void");
@@ -449,7 +539,7 @@ public class CxxSkelSource {
       Method m = (Method)i.next();
       if ( !m.isAbstract() ) { 
         writeImplDispatch( m );
-        if (IOR.supportHooks(d_ext, d_context)) {
+        if (IOR.generateHookMethods(d_ext, d_context)) {
           Method pre = m.spawnPreHook();
           Method post = m.spawnPostHook();
           writeImplDispatch( pre );
