@@ -3,17 +3,19 @@
 // Package:     gov.llnl.babel.backend.ucxx
 // Copyright:   (c) 2007 Lawrence Livermore National Security, LLC
 // Release:     $Name$
-// Revision:    @(#) $Revision: 6201 $
-// Date:        $Date: 2007-10-25 23:15:35 -0700 (Thu, 25 Oct 2007) $
+// Revision:    @(#) $Revision: 6584 $
+// Date:        $Date: 2008-11-06 16:44:29 -0800 (Thu, 06 Nov 2008) $
 // Description: 
 // 
 
 package gov.llnl.babel.backend.ucxx;
 import gov.llnl.babel.Context;
 import gov.llnl.babel.backend.CodeGenerationException;
+import gov.llnl.babel.backend.fortran.ImplSource;
 import gov.llnl.babel.backend.IOR;
 import gov.llnl.babel.backend.Utilities;
 import gov.llnl.babel.backend.writers.LanguageWriterForCxx;
+import gov.llnl.babel.symbols.AssertionExpression;
 import gov.llnl.babel.symbols.Struct;
 import gov.llnl.babel.symbols.SymbolID;
 import gov.llnl.babel.symbols.Type;
@@ -43,10 +45,21 @@ public class CxxStructSource {
       case Type.STRING:
       case Type.INTERFACE:
       case Type.CLASS:
-      case Type.ARRAY:
       case Type.OPAQUE:
         d_writer.println(prefix + item.getName() + " = 0;");
         break;
+      case Type.ARRAY:
+        if (!t.isRarray()){
+          d_writer.println(prefix + item.getName() + " = 0;");
+        } else {
+          Iterator indexVar = t.getArrayIndexExprs().iterator();
+          AssertionExpression ae = (AssertionExpression)indexVar.next();
+          String checkInt = ae.toString();
+          if (!ImplSource.isInt(checkInt)){
+            d_writer.println(prefix + item.getName() + " = 0;");
+          }
+        }
+        break;
       case Type.STRUCT:
         Struct embedded = (Struct)Utilities.lookupSymbol(d_context, 
                                                          t.getSymbolID());
@@ -86,10 +99,19 @@ public class CxxStructSource {
         d_writer.println("}");
         break;
       case Type.ARRAY:
-        d_writer.println(dest + iname + " = (" + 
-                         CxxStructHeader.typeString(t, d_context) +
-                         ")sidl__array_smartCopy((struct sidl__array *)(" +
-                         source + iname + "));");
+        if (!t.isRarray()){
+          d_writer.println(dest + iname + " = (" + 
+                          CxxStructHeader.typeString(t, d_context) +
+                          ")sidl__array_smartCopy((struct sidl__array *)(" +
+                          source + iname + "));");
+        } else {
+          Iterator indexVar = t.getArrayIndexExprs().iterator();
+          AssertionExpression ae = (AssertionExpression)indexVar.next();
+          String checkInt = ae.toString();
+          if (!ImplSource.isInt(checkInt)){
+            d_writer.println(dest + iname + " = " + source + iname + ";");
+          }
+        }
         break;
       case Type.STRUCT:
         Struct s = (Struct)Utilities.lookupSymbol(d_context,
@@ -146,9 +168,20 @@ public class CxxStructSource {
         d_writer.println("}");
         break;
       case Type.ARRAY:
-        d_writer.println("sidl__array_deleteRef((struct sidl__array *)" +
-                         iname + ");");
-        d_writer.println(iname + " = 0;");
+        if (!t.isRarray()){
+          d_writer.println("sidl__array_deleteRef((struct sidl__array *)" +
+                          iname + ");");
+          d_writer.println(iname + " = 0;");
+        } else {
+          Iterator indexVar = t.getArrayIndexExprs().iterator();
+          AssertionExpression ae = (AssertionExpression)indexVar.next();
+          String checkInt = ae.toString();
+          if (!ImplSource.isInt(checkInt)){
+            d_writer.println("// "+iname+" skipped since user will manage");
+          } else {
+            d_writer.println("// "+iname+" skipped since it is static");
+          }
+        }
         break;
       case Type.STRUCT:
         d_writer.println("((" +
@@ -229,10 +262,19 @@ public class CxxStructSource {
         d_writer.println("}");
         break;
       case Type.ARRAY:
-        d_writer.println("dest." + iname + 
-                         " = (" + IOR.getReturnString(t, d_context) + 
-                         ")sidl__array_smartCopy((struct sidl__array *)" + 
-                         iname + ");");
+        if (!t.isRarray()){
+          d_writer.println("dest." + iname + 
+                          " = (" + IOR.getReturnString(t, d_context) + 
+                          ")sidl__array_smartCopy((struct sidl__array *)" + 
+                          iname + ");");
+        } else {
+          Iterator indexVar = t.getArrayIndexExprs().iterator();
+          AssertionExpression ae = (AssertionExpression)indexVar.next();
+          String checkInt = ae.toString();
+          if (!ImplSource.isInt(checkInt)){
+            d_writer.println("dest." + iname + " = " + iname + ";");
+          }
+        }
         break;
       case Type.STRUCT:
         d_writer.println("reinterpret_cast< const " + Cxx.getReturnString(t, d_context) + " * > (&" +
@@ -291,9 +333,20 @@ public class CxxStructSource {
         d_writer.println(iname + " = rhs." + iname + "; // DO NOT ADD REF");
         break;
       case Type.ARRAY:
-        d_writer.println("sidl__array_deleteRef((struct sidl__array *)("
-                         + iname + "));");
-        d_writer.println(iname + " = rhs." + iname + ";");
+          if (!t.isRarray()){
+            d_writer.println("sidl__array_deleteRef((struct sidl__array *)("
+                            + iname + "));");
+            d_writer.println(iname + " = rhs." + iname + ";");
+          } else {
+            Iterator indexVar = t.getArrayIndexExprs().iterator();
+            AssertionExpression ae = (AssertionExpression)indexVar.next();
+            String checkInt = ae.toString();
+            if (!ImplSource.isInt(checkInt)){
+              d_writer.println("sidl__array_deleteRef((struct sidl__array *)("
+                              + iname + "));");
+              d_writer.println(iname + " = rhs." + iname + ";");
+            }
+          }
         break;
       case Type.STRUCT:
         d_writer.println("*reinterpret_cast< " + Cxx.getReturnString(t, d_context) + " * > (&" +
@@ -380,12 +433,23 @@ public class CxxStructSource {
         d_writer.println("}");
         break;
       case Type.ARRAY:
-        d_writer.println("sidl__array_deleteRef((struct sidl__array *)("
-                         + iname + "));");
-        d_writer.println(iname + " = (" + 
-                         CxxStructHeader.typeString(t, d_context) +
-                         ")sidl__array_smartCopy((struct sidl__array *)(rhs."
+        if (!t.isRarray()){
+          d_writer.println("sidl__array_deleteRef((struct sidl__array *)("
                          + iname + "));");
+          d_writer.println(iname + " = (" + 
+                          CxxStructHeader.typeString(t, d_context) +
+                          ")sidl__array_smartCopy((struct sidl__array *)(rhs."
+                          + iname + "));");
+        } else {
+          Iterator indexVar = t.getArrayIndexExprs().iterator();
+          AssertionExpression ae = (AssertionExpression)indexVar.next();
+          String checkInt = ae.toString();
+          if (!ImplSource.isInt(checkInt)){
+            d_writer.println("sidl__array_deleteRef((struct sidl__array *)("
+                          + iname + "));");
+            d_writer.println(iname + " = " + "rhs." + iname + ";");
+          }
+        }
         break;
       case Type.STRUCT:
         d_writer.println("*reinterpret_cast< " +
@@ -575,6 +639,7 @@ public class CxxStructSource {
         }
         d_writer.backTab();
         d_writer.println("}");
+        d_writer.printlnUnformatted("#ifdef WITH_RMI");
         d_writer.println("else {");
         d_writer.tab();
         if (serialize) {
@@ -602,6 +667,7 @@ public class CxxStructSource {
         }
         d_writer.backTab();
         d_writer.println("}");
+        d_writer.printlnUnformatted("#endif");
         break;
       case Type.STRUCT:
         d_writer.println("reinterpret_cast< " +
