@@ -1,12 +1,12 @@
 //
 // File:        CxxStubSource.java
 // Package:     gov.llnl.babel.backend.ucxx
-// Revision:    @(#) $Revision: 6201 $
-// Date:        $Date: 2007-10-25 23:15:35 -0700 (Thu, 25 Oct 2007) $
+// Revision:    @(#) $Revision: 6582 $
+// Date:        $Date: 2008-11-06 16:03:18 -0800 (Thu, 06 Nov 2008) $
 // Description: Write Cxx extension header file for a BABEL extendable
 // 
 // This is typically directed by GenCxxClient.
-// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -255,6 +255,27 @@ public class CxxStubSource {
     
     d_writer.generateInclude( "sidl_String.h", false );
     d_writer.generateInclude("sidl_rmi_ConnectRegistry.h", false);
+    if (d_ext.hasInvClause(true)) {
+      d_writer.generateInclude(Cxx.generateFilename(
+                                   BabelConfiguration.getInvariantViolation(),
+                                   Cxx.FILE_ROLE_STUB, 
+                                   Cxx.FILE_TYPE_CXX_HEADER),
+                               true );
+    }
+    if (d_ext.hasPreconditions()) {
+      d_writer.generateInclude(Cxx.generateFilename( 
+                                 BabelConfiguration.getPostconditionViolation(),
+                                 Cxx.FILE_ROLE_STUB, 
+                                 Cxx.FILE_TYPE_CXX_HEADER),
+                               true );
+    }
+    if (d_ext.hasPostconditions()) {
+      d_writer.generateInclude(Cxx.generateFilename( 
+                                  BabelConfiguration.getPreconditionViolation(),
+                                  Cxx.FILE_ROLE_STUB, 
+                                  Cxx.FILE_TYPE_CXX_HEADER),
+                               true );
+    }
     if (!BabelConfiguration.isSIDLBaseClass(id)) {
       d_writer.generateInclude("babel_config.h", false);
       d_writer.printlnUnformatted("#ifdef SIDL_DYNAMIC_LIBRARY");
@@ -329,14 +350,15 @@ public class CxxStubSource {
       d_writer.println(fullNameWithoutLeadingColons + "::_create() {");
       d_writer.tab();
 
-      d_writer.println(IOR.getExceptionFundamentalType() + " _exception;");
+      d_writer.println(IOR.getExceptionFundamentalType() 
+        + " _exception = NULL;");
       d_writer.println( Cxx.getObjectName(id) 
-        + " self( (*_get_ext()->createObject)(NULL,&_exception), false );");
-      d_writer.println("if (_exception) {");
+        + " self( (*_get_ext()->createObject)(NULL, &_exception), false );");
+      d_writer.println("if (_exception != NULL) {");
       d_writer.tab();
       d_writer.println("throwException" +index+"(\""+fullName+"\"");
       d_writer.tab();
-      d_writer.println("\"static constructor\",_exception);");
+      d_writer.println("\"static constructor\", _exception);");
       d_writer.backTab();
       d_writer.backTab();
       d_writer.println("}");
@@ -351,13 +373,16 @@ public class CxxStubSource {
         d_writer.println(fullNameWithoutLeadingColons + 
                          "::_wrapObj(void* private_data) {");
         d_writer.tab();
-        d_writer.println(IOR.getExceptionFundamentalType() + "_exception;");
+        d_writer.println(IOR.getExceptionFundamentalType() 
+          + "_exception = NULL;");
         d_writer.print(fullName+"::ior_t* returnValue = ");
-        d_writer.println("(*_get_ext()->createObject)(private_data,&_exception);");
+        d_writer.println("(*_get_ext()->createObject)"
+          + "(private_data ,&_exception);");
      
         d_writer.println("if (_exception) {");
         d_writer.tab();
-        d_writer.println("throwException" +index+"(\""+fullName+"._wrap\", _exception);");
+        d_writer.println("throwException" + index + "(\"" + fullName 
+          + "._wrap\", _exception);");
         d_writer.backTab();
         d_writer.println("}");
         d_writer.println("return returnValue;"); 
@@ -366,44 +391,62 @@ public class CxxStubSource {
         d_writer.println();
       }
 
+      d_writer.printlnUnformatted("#ifdef WITH_RMI");
       d_writer.writeCommentLine("remote constructor");
       d_writer.println(fullName);
-      d_writer.println(fullNameWithoutLeadingColons + "::_create(" + 
-		       "const std::string& url) {");
+      d_writer.println(fullNameWithoutLeadingColons + "::_create(" 
+        + "const std::string& url) {");
       d_writer.tab();
 
       d_writer.println( "ior_t* ior_self;");
-      d_writer.println(IOR.getExceptionFundamentalType() + "_exception;");
-      d_writer.println( "ior_self = " + ior_name + "__remoteCreate( url.c_str(), &_exception );" );
-      d_writer.println( "if (_exception != 0 ) {");
+      d_writer.println(IOR.getExceptionFundamentalType() 
+        + "_exception = NULL;");
+      d_writer.println( "ior_self = " + ior_name 
+        + "__remoteCreate( url.c_str(), &_exception );" );
+      d_writer.println( "if (_exception != NULL ) {");
       d_writer.tab();
-      d_writer.println("throwException" +index+"(\""+fullName+" remoteCreate\", _exception);");
+      try {
+        d_writer.pushLineBreak(false);
+          d_writer.println("throwException" +index+"(\""+fullName+" remoteCreate\", _exception);");
+      }
+      finally {
+        d_writer.popLineBreak();
+      }
       d_writer.backTab();
       d_writer.println("}");
       d_writer.println( "return " + fullName + "( ior_self, false );" );
       d_writer.backTab();
       d_writer.println("}");
+      d_writer.printlnUnformatted("#endif");
       d_writer.println();
     }
 
+    d_writer.printlnUnformatted("#ifdef WITH_RMI");
     d_writer.writeCommentLine("remote connector");
     d_writer.println(fullName);
-    d_writer.println(fullNameWithoutLeadingColons + "::_connect(" + 
-		     "const std::string& url, const bool ar ) {");
+    d_writer.println(fullNameWithoutLeadingColons + "::_connect(" 
+      + "const std::string& url, const bool ar ) {");
     d_writer.tab();
     d_writer.println( "ior_t* ior_self;");
-    d_writer.println(IOR.getExceptionFundamentalType() + "_exception;");
+    d_writer.println(IOR.getExceptionFundamentalType() + "_exception = NULL;");
             
-    d_writer.println( "ior_self = " + ior_name + 
-                      "__remoteConnect( url.c_str(), ar?TRUE:FALSE, &_exception );" );
-    d_writer.println( "if (_exception != 0 ) {");
+    d_writer.println( "ior_self = " + ior_name 
+      + "__remoteConnect( url.c_str(), ar?TRUE:FALSE, &_exception );" );
+    d_writer.println( "if (_exception != NULL ) {");
     d_writer.tab();
-    d_writer.println("throwException" +index+"(\""+fullName+" connect\",_exception);");
+    try {
+      d_writer.pushLineBreak(false);
+      d_writer.println("throwException" +index+"(\""+fullName+" connect\",_exception);");
+    }
+    finally {
+      d_writer.popLineBreak();
+    }
     d_writer.backTab();
     d_writer.println("}");
     d_writer.println( "return " + fullName + "( ior_self, false );" );
     d_writer.backTab();
     d_writer.println("}");
+    d_writer.printlnUnformatted("#endif");
     d_writer.println();
       
     d_writer.writeCommentLine("copy constructor");
@@ -567,20 +610,52 @@ public class CxxStubSource {
     d_writer.println("}");
     d_writer.println();
 
-    generateMethodDispatch(IOR.getBuiltinMethod(IOR.GETURL, d_ext.getSymbolID(), d_context, false),
-                           "get URL of the implementation of this object", false,false);
+    generateMethodDispatch(IOR.getBuiltinMethod(IOR.GETURL, 
+                           d_ext.getSymbolID(), d_context, false),
+                           "get URL of the implementation of this object", 
+                           false, false);
 
     boolean hasStatic = d_ext.hasStaticMethod(true);
-    //generate _set_hooks (if we should)
 
-    generateMethodDispatch(IOR.getBuiltinMethod(IOR.HOOKS,id, d_context, false),
-                           "Set calling hooks on or off for this object", 
-                           false,false);
+    /* Generate built-in hook methods. */
+    /* Instance version of enable/disable hooks. */
+    generateMethodDispatch(
+            IOR.getBuiltinMethod(IOR.HOOKS, id, d_context, false),
+            "Method to enable/disable hooks execution.", false, false);
     if (hasStatic) {
-      //Static set_hooks
-      generateMethodDispatch(IOR.getBuiltinMethod(IOR.HOOKS,id, d_context, true),
-                             "Set calling hooks on or off for class methods", 
-                             false,false);
+      /* Static version of enable/disable hooks. */
+      generateMethodDispatch(
+             IOR.getBuiltinMethod(IOR.HOOKS, id, d_context, true),
+             "Method to enable/disable static hooks execution.",
+             false, false);
+    }
+    
+
+    /* Generate built-in contract methods. */
+    /* Instance version of enable/disable. */
+    generateMethodDispatch(
+      IOR.getBuiltinMethod(IOR.CONTRACTS, id, d_context, false),
+      "Method to enable/disable interface contract enforcement.",
+      false, false);
+    if (hasStatic) {
+      /* Static version of enable/disable. */
+      generateMethodDispatch(
+        IOR.getBuiltinMethod(IOR.CONTRACTS, id, d_context, true),
+        "Method to enable/disable static interface contract enforcement.",
+        false, false);
+    }
+  
+    /* Instance version of stats dump. */
+    generateMethodDispatch(
+      IOR.getBuiltinMethod(IOR.DUMP_STATS, id, d_context, false),
+      "Method to dump interface contract enforcement statistics.",
+      false, false);
+    if (hasStatic) {
+      /* Static version of stats dump. */
+      generateMethodDispatch(
+        IOR.getBuiltinMethod(IOR.DUMP_STATS, id, d_context, true),
+        "Method to dump static interface contract enforcement statistics.",
+        false, false);
     }
     
 
@@ -590,9 +665,10 @@ public class CxxStubSource {
     d_writer.println("{");
 
     d_writer.tab();
-    d_writer.println("static int connect_loaded = 0;");
 
     d_writer.println("ior_t* cast = NULL;"); 
+    d_writer.printlnUnformatted("#ifdef WITH_RMI");
+    d_writer.println("static int connect_loaded = 0;");
  
     d_writer.println();
     d_writer.println("if(!connect_loaded) {");
@@ -604,6 +680,7 @@ public class CxxStubSource {
     d_writer.println("connect_loaded = 1;");
     d_writer.backTab();
     d_writer.println("}");
+    d_writer.printlnUnformatted("#endif");
 
     d_writer.println("if ( src != 0 ) {");
     d_writer.tab();
@@ -827,7 +904,7 @@ public class CxxStubSource {
       }
       Integer num = (Integer)sets_to_nums.get(m.getThrows());
       int index = num.intValue();
-      d_writer.println("if (_exception != 0 ) {");      
+      d_writer.println("if (_exception != NULL ) {");      
       d_writer.tab();
       d_writer.println(Cxx.generatePostIORCleanup(m, rarrays));
       
