@@ -1,10 +1,10 @@
 //
 // File:        PrettyWriter.java
 // Package:     gov.llnl.babel.backend.writers
-// Revision:    @(#) $Id: PrettyWriter.java 5885 2007-02-28 22:54:54Z epperly $
+// Revision:    @(#) $Id: PrettyWriter.java 6482 2008-08-21 22:50:53Z epperly $
 // Description: a pretty writer class to aid in formatting backend output
 //
-// Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -175,7 +175,7 @@ public class PrettyWriter {
    */
   public void pushLineBreak(boolean linebreak)
   {
-    final Boolean b = new Boolean(d_break_enabled);
+    final Boolean b = Boolean.valueOf(d_break_enabled);
     d_lineBreakStack.addFirst(b);
     d_break_enabled = linebreak;
   }
