@@ -1,10 +1,10 @@
 //
 // File:        ParseSymbolXML.java
 // Package:     gov.llnl.babel.parsers.xml
-// Revision:    @(#) $Id: ParseSymbolXML.java 6201 2007-10-26 06:15:35Z kumfert $
+// Revision:    @(#) $Id: ParseSymbolXML.java 6482 2008-08-21 22:50:53Z epperly $
 // Description: convert sidl symbol XML/DOM into a sidl symbol object
 //
-// Copyright (c) 2000-2004, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2004, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -308,9 +308,10 @@ public class ParseSymbolXML {
       parseMethodsBlock(element, cls);
 
       /*
-       * Parse the Assertion List, if any, and add the invariants.
+       * Parse the Contract, if present, and add the assertions to the 
+       * invariant clause.
        */
-      parseInvariantAssertionList(element, cls);
+      parseInvContractClause(element, cls);
    }
 
    /**
@@ -376,9 +377,10 @@ public class ParseSymbolXML {
       parseMethodsBlock(element, ifc);
 
       /*
-       * Parse the Assertion List, if any, and add the invariants.
+       * Parse the Contract, if present, and add the assertions to the 
+       * invariant clause.
        */
-      parseInvariantAssertionList(element, ifc);
+      parseInvContractClause(element, ifc);
    }
 
   /**
@@ -602,31 +604,32 @@ public class ParseSymbolXML {
     }
 
    /**
-    * Recover the invariant assertion list, if any, from its DOM representation
-    * adding them to the extendable.
+    * Recover the invariant contract (clause), if present, from its DOM 
+    * representation adding its assertions to the extendable.
     *
     * Assumptions:
-    * 1) The extendable implementation of addInvariant() will raise an
+    * 1) The extendable implementation of addInvAssertions() will raise an
     *    AssertionException if the assertion is not an invariant.
     *
     * @param   e    The element block being read.
-    * @param   ext  The extendable to which the invariants are to be added.
+    * @param   ext  The extendable to which the invariant clause's assertions
+    *               are to be added.
     * @throws  gov.llnl.babel.parsers.xml.ParseSymbolException
     *               The exception raised if a parsing error is encountered.
     */
-   private void parseInvariantAssertionList(Element e, Extendable ext)
+   private void parseInvContractClause(Element e, Extendable ext)
       throws ParseSymbolException
    {
-      if (hasChildElement(e, "AssertionList")) {
-         Element  list = getElement(e, "AssertionList");
+      if (hasChildElement(e, "Contract")) {
+         Element  list = getElement(e, "Contract");
          Iterator iter = new ElementIterator(list, "Assertion");
          while (iter.hasNext()) {
             try {
                Assertion as  = parseAssertion((Element) iter.next(), 
                                               ext.getFullName());
-               ext.addInvariant(as);
+               ext.addInvAssertion(as);
             } catch (AssertionException aex) {
-               error("parseInvariantAssertionList: AssertionException: "
+               error("parseInvContractClause: AssertionException: "
                    + aex.getMessage());
             }
          }
@@ -694,23 +697,23 @@ public class ParseSymbolXML {
 
 
       /*
-       * Parse the Assertion List, if any, and add any preconditions
-       * and/or postconditions to this method.
+       * Parse the method's Contract, if present, and add any assertions to 
+       * the appropriate precondition or postcondition clause.
        *
        * Assumption:
-       * 1) It is assumed method's addAssertion() will ensure the
-       *    assertion is appropriate for a method and put it on
-       *    the proper assertion list (i.e., pre or post condition).
+       * 1) It is assumed method's addClauseAssertion() will ensure the
+       *    assertion is appropriate for a method and put it in
+       *    the appropriate clause (i.e., precondition or postcondition).
        */
-      if (hasChildElement(e, "AssertionList")) {
-         Element  alist = getElement(e, "AssertionList");
+      if (hasChildElement(e, "Contract")) {
+         Element  alist = getElement(e, "Contract");
          Iterator iter  = new ElementIterator(alist, "Assertion");
          while (iter.hasNext()) {
             try {
                Assertion as  = parseAssertion((Element) iter.next(), name);
-               method.addAssertion(as);
+               method.addClauseAssertion(as);
             } catch (AssertionException aex) {
-               error("parseMethod: AssertionException: "+aex.getMessage());
+               error("parseMethod: AssertionException: " + aex.getMessage());
             }
          }
       }
@@ -839,8 +842,8 @@ public class ParseSymbolXML {
    }
 
    /**
-    * Convert a DOM assertion expression element into an 
-    * <code>AssertionExpression</code> object.  
+    * Convert a DOM expression element into an <code>AssertionExpression</code>
+    * object.  
     *
     * @param   e  The assertion expression element.
     * @throws  gov.llnl.babel.parsers.xml.ParseSymbolException
