@@ -1,10 +1,10 @@
 //
 // File:        SymbolToDOM.java
 // Package:     gov.llnl.babel.parsers.xml
-// Revision:    @(#) $Id: SymbolToDOM.java 6212 2007-11-01 00:07:17Z epperly $
+// Revision:    @(#) $Id: SymbolToDOM.java 6482 2008-08-21 22:50:53Z epperly $
 // Description: convert sidl symbols into an XML DOM document
 //
-// Copyright (c) 2000-2004, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2007, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -30,6 +30,7 @@
 
 package gov.llnl.babel.parsers.xml;
 
+import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.Context;
 import gov.llnl.babel.parsers.xml.DTDManager;
 import gov.llnl.babel.parsers.xml.StringXML;
@@ -249,9 +250,9 @@ public class SymbolToDOM {
       convertMethods(cls, class_element);
 
       /*
-       * Write the invariant assertions list.
+       * Write the invariant contract clause.
        */
-      convertInvariants(cls, class_element);
+      convertInvClause(cls, class_element);
       
       appendChild(class_element, parent);
    }
@@ -322,7 +323,7 @@ public class SymbolToDOM {
       /*
        * Write the invariant assertions list.
        */
-      convertInvariants(ifc, interface_element);
+      convertInvClause(ifc, interface_element);
       
       appendChild(interface_element, parent);
    }
@@ -375,15 +376,22 @@ public class SymbolToDOM {
    }
 
    /**
-    * Convert a SIDL invariant assertions list, if any, into its DOM 
-    * representation.
+    * Convert a SIDL invariants clause, if present, into its DOM representation.
     *
-    * @param  ext     The class or interface whose invariants are being 
+    * @param  ext     The class or interface whose invariants clause is being 
     *                 converted.
     * @param  parent  The parent node of the corresponding DOM representation.
     */
-   private void convertInvariants(Extendable ext, Node parent) {
-      convertAssertions(ext.getInvariants(), parent);
+   private void convertInvClause(Extendable ext, Node parent) {
+      try {
+        convertContract(ext.getInvClause(), parent);
+      } catch (CodeGenerationException cge) {
+        /*
+         * ToDo:  What should we do with this if is is raised?
+         *   It should only happen if we have to get invariants
+         *   from parent(s).
+         */
+      }
    }
 
    /**
@@ -576,7 +584,7 @@ public class SymbolToDOM {
       /*
        * Create the lists of preconditions followed by postconditions, if any.
        */
-      convertAssertions(m, method_element);
+      convertContract(m, method_element);
 
       appendChild(method_element, parent);
    }
@@ -684,28 +692,29 @@ public class SymbolToDOM {
    }
 
    /**
-    * Convert a SIDL list of method assertions, if any, into its DOM 
-    * representation.
+    * Convert the method's precondition and postcondition clauses into
+    * their DOM representation.
     *
-    * @param  meth    The method whose assertions are being converted.
+    * @param  meth    The method whose contracts are being converted.
     * @param  parent  The parent node of the corresponding DOM representation.
     */
-   private void convertAssertions(Method meth, Node parent) {
-      LinkedList list = new LinkedList(meth.getPreconditions());
-      list.addAll(meth.getPostconditions());
-      convertAssertions(list, parent);
+   private void convertContract(Method meth, Node parent) {
+      LinkedList list = new LinkedList(meth.getPreClause());
+      list.addAll(meth.getPostClause());
+      convertContract(list, parent);
    }
 
    /**
-    * Convert an <code>List</code> of assertions, if any, into its DOM 
-    * representation.
+    * Convert a contract [clause], specified as a <code>List</code> of 
+    * assertions, into its DOM representation.
     *
-    * @param  list    The list of SIDL <code>Assertion</code> entries.
+    * @param  list    The list of SIDL <code>Assertion</code> entries
+    *                 that form the basis for the contract [clause].
     * @param  parent  The parent node of the corresponding DOM representation.
     */
-   private void convertAssertions(List list, Node parent) {
+   private void convertContract(List list, Node parent) {
       if (!list.isEmpty()) {
-         Element list_elem = createElement("AssertionList");
+         Element list_elem = createElement("Contract");
          for (Iterator i = list.iterator(); i.hasNext(); ) {
             convertAssertion((Assertion) i.next(), list_elem);
          }
@@ -728,8 +737,7 @@ public class SymbolToDOM {
          try {
            convertAssertionExpression(as.getExpression(), e);
          } catch (AssertionException ex) {
-           System.err.println("FATAL: " + ex.getMessage());
-           System.exit(1);
+           throw new RuntimeException(ex);
          }
 
          appendChild(e, parent);
