@@ -3,7 +3,7 @@
   // Package:     gov.llnl.babel.symbols
   // Copyright:   (c) 2005 Lawrence Livermore National Security, LLC
   // Release:     $Name$
-  // Revision:     @(#) $Id: ASTNode.java 6212 2007-11-01 00:07:17Z epperly $
+  // Revision:     @(#) $Id: ASTNode.java 6426 2008-07-03 16:56:09Z epperly $
   // Description: Base class for all nodes in the AST
 
 package gov.llnl.babel.symbols;
@@ -14,7 +14,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
-public class ASTNode { 
+public class ASTNode implements Cloneable { 
   protected boolean d_frozen = false;
 
   public void freeze() {
