@@ -4,7 +4,7 @@
 // Revision:    @(#) $Id $
 // Description: An assertion clause.
 //
-// Copyright (c) 2003-2004, The Regents of the University of Calfornia.
+// Copyright (c) 2003-2007, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -80,7 +80,7 @@ public class Assertion extends ASTNode 	{
        throws AssertionException
   {
     String prefix = (tag == null) ? "Assertion: " : "Assertion: " + tag;
-    if ( (type < MIN_VALID_TYPE) || (type > MAX_VALID_TYPE) ) {
+    if (!isValidAssertionType(type)) {
       throw new AssertionException(prefix, "Invalid assertion type value \"" 
                 + type + "\", must be in " + MIN_VALID_TYPE + "..." 
                 + MAX_VALID_TYPE + ".");
@@ -97,6 +97,13 @@ public class Assertion extends ASTNode 	{
   }
 
 
+  /**
+   * Return TRUE if the type is valid; otherwise, return FALSE.
+   */
+  public static boolean isValidAssertionType(int type) {
+    return ( (MIN_VALID_TYPE <= type) && (type <= MAX_VALID_TYPE) );
+  }
+
   /**
    * Return the type of the assertion.
    */
@@ -124,7 +131,7 @@ public class Assertion extends ASTNode 	{
 
   
   /**
-   * Return TRUE if a precondition; otherwise, returns FALSE.
+   * Return TRUE if a precondition; otherwise, return FALSE.
    */
   public boolean isPrecondition() {
     return ( (d_type==REQUIRE) || (d_type==REQUIRE_ELSE) );
@@ -132,7 +139,16 @@ public class Assertion extends ASTNode 	{
 
 
   /**
-   * Return TRUE if a postcondition; otherwise, returns FALSE.
+   * Return TRUE if the specified type is a precondition; otherwise, 
+   * return FALSE.
+   */
+  public static boolean isPrecondition(int type) {
+    return ( (type==REQUIRE) || (type==REQUIRE_ELSE) );
+ }
+
+
+  /**
+   * Return TRUE if a postcondition; otherwise, return FALSE.
    */
   public boolean isPostcondition() {
     return ( (d_type==ENSURE) || (d_type==ENSURE_THEN) );
@@ -140,13 +156,31 @@ public class Assertion extends ASTNode 	{
 
 
   /**
-   * Return TRUE if an invariant; otherwise, returns FALSE.
+   * Return TRUE if the specified type is a postcondition; otherwise, 
+   * return FALSE.
+   */
+  public static boolean isPostcondition(int type) {
+    return ( (type==ENSURE) || (type==ENSURE_THEN) );
+  }
+
+
+  /**
+   * Return TRUE if an invariant; otherwise, return FALSE.
    */
   public boolean isInvariant() {
     return (d_type==INVARIANT);
   }
 
 
+  /**
+   * Return TRUE if the specified type is an invariant; otherwise, 
+   * return FALSE.
+   */
+  public static boolean isInvariant(int type) {
+    return (type==INVARIANT);
+  }
+
+
   /**
    * Return the name of the type of the assertion.  It is assumed the
    * type is valid/known thanks to the check in the constructor.
@@ -156,6 +190,15 @@ public class Assertion extends ASTNode 	{
   }
 
 
+  /**
+   * Return the name of the specified type of the assertion, if valid;
+   * otherwise, return NULL.
+   */
+  public static String getTypeName(int type) {
+    return isValidAssertionType(type) ? s_names[type] : null;
+  }
+
+
   /**
    * Return the source associated with this assertion.
    */
@@ -214,6 +257,10 @@ public class Assertion extends ASTNode 	{
       throw new AssertionException("Assertion", "Cannot specify RESULT in a(n) "
                 + getTypeName() + " assertion.");
     }
+    if (expr.hasResultOrOutArg(true)) {
+      throw new AssertionException("Assertion", "Cannot specify output-only "
+                + "arguments in a(n) " + getTypeName() + " assertion.");
+    }
   }
 
 
@@ -326,8 +373,28 @@ public class Assertion extends ASTNode 	{
 
 
   /**
-   * Return TRUE if a method call is found within the expression; otherwise,
-   * return FALSE.
+   * Return TRUE if a result clause or method argument is found within the
+   * expression; otherwise, return FALSE.
+   */
+  public boolean hasResultOrArg() {
+    return d_expression.hasResultOrArg();
+  }
+
+  /**
+   * Return TRUE if a result clause or output argument is found within the
+   * expression; otherwise, return FALSE.
+   *
+   * @param  outOnly  TRUE if only concerned with output arguments that are
+   *                  out only; FALSE otherwise.
+   */
+  public boolean hasResultOrOutArg(boolean outOnly) {
+    return d_expression.hasResultOrOutArg(outOnly);
+  }
+
+
+  /**
+   * Return TRUE if the expression is, or has, at least one method call;
+   * otherwise, return FALSE.
    */
   public boolean hasMethodCall() {
     return d_expression.hasMethodCall();
@@ -335,21 +402,29 @@ public class Assertion extends ASTNode 	{
 
 
   /**
-   * Return TRUE if the specified reserved method call is found within the 
-   * expression; otherwise, return FALSE.
+   * Returns the default complexity of the expression.
    */
-  public boolean hasReservedMethod(int type) {
-    return d_expression.hasReservedMethod(type);
+  public int getDefaultComplexity() {
+    return d_expression.getDefaultComplexity();
   }
 
 
   /**
-   * Return TRUE if the method is any unreserved method (when any is TRUE)
-   * or it is an unreserved method with a throws clause (if any is FALSE);
-   * otherwise, return FALSE.
+   * Return TRUE if the assertion's expression is, or has, the specified
+   * built-in method call; otherwise, return FALSE.
    */
-  public boolean hasUnreservedMethod(boolean any) {
-    return d_expression.hasUnreservedMethod(any);
+  public boolean hasBuiltinMethod(int type) {
+    return d_expression.hasBuiltinMethod(type);
+  }
+
+                                                                                
+  /**
+   * Return TRUE if the assertion's expression has a method AND the method 
+   * is any user-defined method (when any is TRUE) or it is an user-defined 
+   * method with a throws clause (if any is FALSE); otherwise, return FALSE.
+   */
+  public boolean hasUserDefinedMethod(boolean any) {
+    return d_expression.hasUserDefinedMethod(any);
   }
 
 
@@ -362,6 +437,17 @@ public class Assertion extends ASTNode 	{
   }
 
 
+  /**
+   * Return the error message associated with a failure of this assertion,
+   * appending the given mehod name to the error message.  (ToDo...This is 
+   * only necessary until it gets added properly by the parser!)
+   */
+  public String errorMessage(String methName) {
+    return getTypeName().toUpperCase() + " VIOLATION: " + d_source + ": "
+      + methName + ": " + toString() + ".";
+  }
+
+
   /**
    * Return the list of array iteration macro messages, if any.  Each message
    * is a string where the first character indicates the return type associated
