@@ -20,7 +20,7 @@
 // 
 //*************************************************************
 //
-// Copyright (c) 2003-2004, The Regents of the University of Calfornia.
+// Copyright (c) 2003-2007, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -110,10 +110,10 @@ public abstract class AssertionExpression {
   protected void setReturnType(int return_type) throws AssertionException {
     if (return_type != Type.ARRAY) {
       if (  (return_type < Type.BOOLEAN) 
-         || (return_type > Type.ENUM ) ) {
+         || (return_type > Type.INTERFACE ) ) {
         throw new AssertionException("Return type value \"" + return_type 
                     + "\" invalid, must be in " + Type.BOOLEAN + "..." 
-                    + Type.ENUM + " or " + Type.ARRAY + ".");
+                    + Type.INTERFACE + " or " + Type.ARRAY + ".");
       } else {
         d_return_type = new Type(return_type);
       }
@@ -291,6 +291,33 @@ public abstract class AssertionExpression {
    }
 
 
+  /**
+   * Return TRUE if the return type is an enumeration; otherwise, return FALSE.
+   */
+  public boolean returnIsEnum() {
+    return (  (d_return_type                != null        )
+           && (d_return_type.getBasicType() == Type.ENUM) );
+   }
+
+
+  /**
+   * Return TRUE if the return type is a class; otherwise, return FALSE.
+   */
+  public boolean returnIsClass() {
+    return (  (d_return_type                != null        )
+           && (d_return_type.getBasicType() == Type.CLASS) );
+   }
+
+
+  /**
+   * Return TRUE if the return type is an interface; otherwise, return FALSE.
+   */
+  public boolean returnIsInterface() {
+    return (  (d_return_type                != null        )
+           && (d_return_type.getBasicType() == Type.INTERFACE) );
+   }
+
+
   /**
    * Set the return type to boolean.
    */
@@ -371,6 +398,30 @@ public abstract class AssertionExpression {
   }
 
 
+  /**
+   * Set the return type to enumeration.
+   */
+  protected void setReturnToEnum() throws AssertionException {
+    setReturnType(Type.ENUM);
+  }
+
+
+  /**
+   * Set the return type to class.
+   */
+  protected void setReturnToClass() throws AssertionException {
+    setReturnType(Type.CLASS);
+  }
+
+
+  /**
+   * Set the return type to interface.
+   */
+  protected void setReturnToInterface() throws AssertionException {
+    setReturnType(Type.INTERFACE);
+  }
+
+
   /**
    * Return TRUE if the expression has been marked as having passed the
    * validation checks; otherwise, return FALSE.
@@ -393,6 +444,23 @@ public abstract class AssertionExpression {
   abstract public boolean hasResult();
 
 
+  /**
+   * Return TRUE if a result clause or method argument is found within the
+   * expression; otherwise, return FALSE.
+   */
+  abstract public boolean hasResultOrArg();
+
+
+  /**
+   * Return TRUE if a result clause or output argument is found within the
+   * expression; otherwise, return FALSE.
+   *
+   * @param  outOnly  TRUE if only concerned with output arguments that are
+   *                  out only; FALSE otherwise.
+   */
+  abstract public boolean hasResultOrOutArg(boolean outOnly);
+
+
   /**
    * Return TRUE if the expression is, or has, at least one method call;
    * otherwise, return FALSE.
@@ -401,18 +469,24 @@ public abstract class AssertionExpression {
 
 
   /**
-   * Return TRUE if the expression is, or has, the specified reserved method
-   * call; otherwise, return FALSE.
+   * Return the default complexity of the expression.
    */
-  abstract public boolean hasReservedMethod(int type);
+  abstract public int getDefaultComplexity();
 
 
   /**
-   * Return TRUE if the method is any unreserved method (when any is TRUE)
-   * or it is an unreserved method with a throws clause (if any is FALSE);
-   * otherwise, return FALSE.
+   * Return TRUE if the expression is, or has, the specified built-in method
+   * call; otherwise, return FALSE.
    */
-  abstract public boolean hasUnreservedMethod(boolean any);
+  abstract public boolean hasBuiltinMethod(int type);
+
+
+  /**
+   * Return TRUE if the expression has a method AND the method is any 
+   * user-defined method (when any is TRUE) or it is an user-defined 
+   * method with a throws clause (if any is FALSE); otherwise, return FALSE.
+   */
+  abstract public boolean hasUserDefinedMethod(boolean any);
 
 
   /**
