@@ -4,7 +4,7 @@
 // Revision:    @(#) $Id $
 // Description: The binary expression interface for assertions.
 //
-// Copyright (c) 2003-2004, The Regents of the University of Calfornia.
+// Copyright (c) 2003-2004, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -256,6 +256,27 @@ public class BinaryExpression extends AssertionExpression {
   }
 
 
+  /**
+   * Return TRUE if a result clause or method argument is found within the
+   * expression; otherwise, return FALSE.
+   */
+  public boolean hasResultOrArg() {
+    return d_lhs.hasResultOrArg() || d_rhs.hasResultOrArg();
+  }
+
+
+  /**
+   * Return TRUE if a result clause or output argument is found within the
+   * expression; otherwise, return FALSE.
+   *
+   * @param  outOnly  TRUE if only concerned with output arguments that are
+   *                  out only; FALSE otherwise.
+   */
+  public boolean hasResultOrOutArg(boolean outOnly) {
+    return d_lhs.hasResultOrOutArg(outOnly) || d_rhs.hasResultOrOutArg(outOnly);
+  }
+
+
   /**
    * Return TRUE if the expression has at least one method call; otherwise, 
    * return FALSE.
@@ -266,21 +287,31 @@ public class BinaryExpression extends AssertionExpression {
 
 
   /**
-   * Return TRUE if the expression is, or has, the specified reserved method
-   * call; otherwise, return FALSE.
+   * Return the default complexity of the expression (0 = constant, 1 = linear,
+   * etc.).
    */
-  public boolean hasReservedMethod(int type) {
-    return d_lhs.hasReservedMethod(type) || d_rhs.hasReservedMethod(type);
+  public int getDefaultComplexity() {
+    return (d_lhs.getDefaultComplexity() >= d_rhs.getDefaultComplexity())
+         ?  d_lhs.getDefaultComplexity() :  d_rhs.getDefaultComplexity();
   }
 
 
   /**
-   * Return TRUE if the method is any unreserved method (when any is TRUE)
-   * or it is an unreserved method with a throws clause (if any is FALSE);
-   * otherwise, return FALSE.
+   * Return TRUE if the expression is, or has, the specified built-in method
+   * call; otherwise, return FALSE.
    */
-  public boolean hasUnreservedMethod(boolean any) {
-    return d_lhs.hasUnreservedMethod(any) || d_rhs.hasUnreservedMethod(any);
+  public boolean hasBuiltinMethod(int type) {
+    return d_lhs.hasBuiltinMethod(type) || d_rhs.hasBuiltinMethod(type);
+  }
+
+
+  /**
+   * Return TRUE if the expression has a method AND the method is any
+   * user-defined method (when any is TRUE) or it is an user-defined
+   * method with a throws clause (if any is FALSE); otherwise, return FALSE.
+   */
+  public boolean hasUserDefinedMethod(boolean any) {
+    return d_lhs.hasUserDefinedMethod(any) || d_rhs.hasUserDefinedMethod(any);
   }
 
 
@@ -577,6 +608,66 @@ public class BinaryExpression extends AssertionExpression {
   }
 
 
+  /**
+   * Return TRUE if the return value is or should be opaque; otherwise,
+   * return FALSE.
+   */
+  private boolean returnOpaque(int lType, int rType) {
+    boolean isOpaque = false;
+    if (returnIsOpaque()) {
+      isOpaque = true;
+    } else if ( (lType == Type.OPAQUE) && (rType == Type.OPAQUE) ) {
+      isOpaque = true;
+    }
+    return isOpaque;
+  }
+
+
+  /**
+   * Return TRUE if the return value is or should be enumeration; otherwise,
+   * return FALSE.
+   */
+  private boolean returnEnumeration(int lType, int rType) {
+    boolean isEnum = false;
+    if (returnIsEnum()) {
+      isEnum = true;
+    } else if ( (lType == Type.ENUM) && (rType == Type.ENUM) ) {
+      isEnum = true;
+    }
+    return isEnum;
+  }
+
+
+  /**
+   * Return TRUE if the return value is or should be class; otherwise,
+   * return FALSE.
+   */
+  private boolean returnClass(int lType, int rType) {
+    boolean isClass = false;
+    if (returnIsClass()) {
+      isClass = true;
+    } else if ( (lType == Type.CLASS) && (rType == Type.CLASS) ) {
+      isClass = true;
+    }
+    return isClass;
+  }
+
+
+  /**
+   * Return TRUE if the return value is or should be interface; otherwise,
+   * return FALSE.
+   */
+  private boolean returnInterface(int lType, int rType) {
+    boolean isIfc = false;
+    if (returnIsInterface()) {
+      isIfc = true;
+    } else if ( (lType == Type.INTERFACE) && (rType == Type.INTERFACE) ) {
+      isIfc = true;
+    }
+    return isIfc;
+  }
+
+
   /**
    * Return a string containing the standard return type error message.
    *
@@ -745,6 +836,30 @@ public class BinaryExpression extends AssertionExpression {
     } 
   }
 
+
+ /**
+   * Set the return type to either boolean or integer based on the
+   * return types of the subexpressions.
+   *
+   * @throws  gov.llnl.babel.symbols.AssertionException
+   *                   The exception raised if type not suitable.
+   */
+  private void setBooleanOrIntegerType() throws AssertionException {
+    int lType = d_lhs.getReturnTypeValue();
+    int rType = d_rhs.getReturnTypeValue();
+    if (returnBoolean(lType, rType)) {
+      setReturnToBoolean();
+    } else if (returnInteger(lType, rType)) {
+      setReturnToInteger();
+    } else if (returnLong(lType, rType)) {
+      setReturnToLong();
+    } else {
+      throw new AssertionException(
+                 getReturnExceptionMessage("either boolean, integer, or long"));
+    }
+  }
+
+
   /**
    * Set the return type to either integer or long based on the return types
    * of the subexpressions.
@@ -857,11 +972,11 @@ public class BinaryExpression extends AssertionExpression {
       if (returnString(d_lhs.getReturnTypeValue(), 
                        d_rhs.getReturnTypeValue())) {
         throw new AssertionException("Unable to validate expression semantics"
-                                     + " for \"" + getExceptionPrefix(ext, m) 
-                                     + "\" since do not currently support string concatentation"
-                                     + ", which is being attempted with the expressions \""
-                                     + d_lhs.toString() + "\" and \"" + d_rhs.toString() 
-                                     + "\".");
+                   + " for \"" + getExceptionPrefix(ext, m) 
+                   + "\" since do not currently support string concatentation"
+                   + ", which is being attempted with the expressions \""
+                   + d_lhs.toString() + "\" and \"" + d_rhs.toString() 
+                   + "\".");
       } else {
         setNumericType();
       }
