@@ -4,7 +4,7 @@
 // Revision:    @(#) $Id $
 // Description: The character literal assertion expression interface.
 //
-// Copyright (c) 2003-2004, The Regents of the University of Calfornia.
+// Copyright (c) 2003-2007, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -34,6 +34,7 @@ import gov.llnl.babel.Context;
 import gov.llnl.babel.symbols.AssertionException;
 import gov.llnl.babel.symbols.ExprVisitor;
 import gov.llnl.babel.symbols.Extendable;
+import gov.llnl.babel.symbols.Literal;
 import gov.llnl.babel.symbols.Method;
 import java.util.ArrayList;
 
@@ -83,6 +84,27 @@ public class CharacterLiteral extends Literal {
   }
 
 
+  /**
+   * Return TRUE if a result clause or method argument is found within the
+   * expression; otherwise, return FALSE.
+   */
+  public boolean hasResultOrArg() {
+    return false;
+  }
+
+
+  /**
+   * Return TRUE if a result clause or output argument is found within the
+   * expression; otherwise, return FALSE.
+   *
+   * @param  outOnly  TRUE if only concerned with output arguments that are
+   *                  out only; FALSE otherwise.
+   */
+  public boolean hasResultOrOutArg(boolean outOnly) {
+    return false;
+  }
+
+
   /**
    * Return TRUE if the expression is, or has, at least one method call;
    * otherwise, return FALSE.
@@ -93,20 +115,29 @@ public class CharacterLiteral extends Literal {
 
 
   /**
-   * Return TRUE if the expression is, or has, the specified reserved method
+   * Return the default complexity of the expression (0 = constant, 1 = linear,
+   * etc.).
+   */
+  public int getDefaultComplexity() {
+    return 0;
+  }
+
+
+  /**
+   * Return TRUE if the expression is, or has, the specified built-in method
    * call; otherwise, return FALSE.
    */
-  public boolean hasReservedMethod(int type) {
+  public boolean hasBuiltinMethod(int type) {
     return false;
   }
 
 
   /**
-   * Return TRUE if the method is any unreserved method (when any is TRUE)
-   * or it is an unreserved method with a throws clause (if any is FALSE);
-   * otherwise, return FALSE.
+   * Return TRUE if the expression has a method AND the method is any
+   * user-defined method (when any is TRUE) or it is an user-defined
+   * method with a throws clause (if any is FALSE); otherwise, return FALSE.
    */
-  public boolean hasUnreservedMethod(boolean any) {
+  public boolean hasUserDefinedMethod(boolean any) {
     return false;
   }
 
