@@ -4,7 +4,7 @@
 // Revision:    @(#) $Id $
 // Description: The identifier literal/catch-all expression for assertions.
 //
-// Copyright (c) 2003-2004, The Regents of the University of Calfornia.
+// Copyright (c) 2003-2007, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -37,6 +37,7 @@ import gov.llnl.babel.symbols.Argument;
 import gov.llnl.babel.symbols.AssertionException;
 import gov.llnl.babel.symbols.ExprVisitor;
 import gov.llnl.babel.symbols.Extendable;
+import gov.llnl.babel.symbols.Literal;
 import gov.llnl.babel.symbols.Method;
 import java.util.ArrayList;
 
@@ -58,6 +59,9 @@ public class IdentifierLiteral extends Literal {
   public final static int PURE     = 2;
   public final static int RESULT   = 3;
 
+  private final static int MIN_TYPE_VALUE  = 0;  // Minimum valid value
+  private final static int MAX_TYPE_VALUE  = 3;  // Maximum valid value
+
   private final static String s_type_name[] = {
     "argument", "null", "pure", "result"
   };
@@ -164,7 +168,14 @@ public class IdentifierLiteral extends Literal {
    * for the type is valid since it is set internally in the constructor.
    */
   public String getIdentifierTypeName() {
-    return s_type_name[d_type];
+    String nm = null;
+
+    if (  (MIN_TYPE_VALUE <= d_type        )
+       && (d_type         <= MAX_TYPE_VALUE) ) {
+      nm = s_type_name[d_type];
+    } 
+
+    return nm;
   }
 
 
@@ -185,6 +196,38 @@ public class IdentifierLiteral extends Literal {
   }
 
 
+  /**
+   * Return TRUE if a result clause or method argument is found within the
+   * expression; otherwise, return FALSE.
+   */
+  public boolean hasResultOrArg() {
+    return (d_type == RESULT) || (d_type == ARGUMENT);
+  }
+
+
+  /**
+   * Return TRUE if a result clause or output argument is found within the
+   * expression; otherwise, return FALSE.
+   *
+   * @param  outOnly  TRUE if only concerned with output arguments that are
+   *                  out only; FALSE otherwise.
+   */
+  public boolean hasResultOrOutArg(boolean outOnly) {
+    boolean has;
+
+    if (outOnly) {
+      has =    (d_type == RESULT)
+          || ( (d_type == ARGUMENT) && (d_argument_mode == OUT) );
+    } else {
+      has =    (d_type == RESULT)
+          || (  (d_type == ARGUMENT)
+             && ( (d_argument_mode == OUT) || (d_argument_mode == INOUT) ) );
+    }
+
+    return has;
+  }
+
+
   /**
    * Return TRUE if the expression has at least one method call; otherwise, 
    * return FALSE.  Since this is an identifier it cannot be a method call.
@@ -195,20 +238,29 @@ public class IdentifierLiteral extends Literal {
 
 
   /**
-   * Return TRUE if the expression is, or has, the specified reserved method
+   * Return the default complexity of the expression (0 = constant, 1 = linear,
+   * etc.).
+   */
+  public int getDefaultComplexity() {
+    return 0;
+  }
+
+
+  /**
+   * Return TRUE if the expression is, or has, the specified built-in method
    * call; otherwise, return FALSE.
    */
-  public boolean hasReservedMethod(int type) {
+  public boolean hasBuiltinMethod(int type) {
     return false;
   }
 
 
   /**
-   * Return TRUE if the method is any unreserved method (when any is TRUE)
-   * or it is an unreserved method with a throws clause (if any is FALSE);
-   * otherwise, return FALSE.
+   * Return TRUE if the expression has a method AND the method is any
+   * user-defined method (when any is TRUE) or it is an user-defined
+   * method with a throws clause (if any is FALSE); otherwise, return FALSE.
    */
-  public boolean hasUnreservedMethod(boolean any) {
+  public boolean hasUserDefinedMethod(boolean any) {
     return false;
   }
 
@@ -312,6 +364,13 @@ public class IdentifierLiteral extends Literal {
         /* 
          * This corresponds to an argument of the specified method, so set
          * the return type if it hasn't already been done.
+         *
+         * ToDo:  In order to support things like enumeration constants,
+         * it will be necessary to add more context information (or
+         * access the symbol table).  Given a suitable convention for
+         * their specification, then the symbol(s) must be looked up to
+         * determine if it is a proper enumeration constant.  (Should
+         * also be marked as an ENUMERATION instead of an ARGUMENT.)
          */
         if (getReturnType() == null) {
           if (ext != null) {
@@ -405,6 +464,7 @@ public class IdentifierLiteral extends Literal {
   public String toString() {
     return hasParens() ?  "(" + d_identifier + ")"  :  d_identifier;
   }
+
   /**
    * Implement the "visitor pattern".
    */
