@@ -1,9 +1,9 @@
 // File:        Method.java
 // Package:     gov.llnl.babel.symbols
-// Revision:    @(#) $Id: Method.java 6212 2007-11-01 00:07:17Z epperly $
+// Revision:    @(#) $Id: Method.java 6494 2008-09-04 23:24:45Z dahlgren $
 // Description: sidl method (modifiers, return type, name, args, and throws)
 //
-// Copyright (c) 2000-2003, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2007, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -75,9 +75,12 @@ public class Method extends ASTNode
   public static final int STATIC   = 3; // definition modifier
   public static final int LOCAL    = 1; // communication modifier
   public static final int ONEWAY   = 2; // communication modifier
-  public static final int NONBLOCKING = 3; // communication modifier (set by parser)
-  public static final int NONBLOCKING_SEND = 4; // communication modifier (set by spawning a NONBLOCKING instance)
-  public static final int NONBLOCKING_RECV = 5; // communication modifier (set by spawning a NONBLOCKING instance)
+  // communication modifier (set by parser)
+  public static final int NONBLOCKING = 3; 
+  // communication modifier (set by spawning a NONBLOCKING instance)
+  public static final int NONBLOCKING_SEND = 4; 
+  // communication modifier (set by spawning a NONBLOCKING instance)
+  public static final int NONBLOCKING_RECV = 5; 
 
   public static final String[] s_def_mod = {
     "", "abstract", "final", "static",
@@ -87,53 +90,58 @@ public class Method extends ASTNode
     "", "local", "oneway", "nonblocking", "nonblocking_send", "nonblocking_recv",
   };
 
-  private Map   d_attributes = new HashMap();
-  private Metadata  d_metadata = new Metadata(new Date());
-  private List d_arguments;
-  private boolean   d_is_builtin;
-  private Comment   d_comment;
-  private String    d_short_name;      // for langs that support overloading
-  private String    d_extension;       // short + extension = long name
-  private String    d_long_name;       // keep for convenience
-  private String    d_blocking_name;   // if NONBLOCKING_SEND or NONBLOCKING_RECV, this has the original name
-  private Set   d_references;
-  private Set   d_references_no_exceptions;  //subset of references, no exceptions
-  private Set   d_basicarrays;     // basic array types references
-  private Type      d_return_type;
-  private Set   d_throws;
-  private Set   d_explicit_throws;  // subset of d_throws
-  private List d_preconditions;	// Local preconditions only
-  private List d_postconditions;	// Local postconditions only
-  private Set   d_rarray_indices; //All the rarray indices required in the arg list	
-  private Map   d_rarray_index_map; //A Hashmap of rarray index names to
-                                        //a collection of RarrayInfo objects.  And
-                                        //RarrayInfo objects represents
-                                        //what rarray uses the index, and where.
+  private Map         d_attributes = new HashMap();
+  private Metadata    d_metadata   = new Metadata(new Date());
+  private List        d_arguments;
+  private boolean     d_is_builtin;
+  private Comment     d_comment;
+  private String      d_short_name;      // for langs that support overloading
+  private String      d_extension;       // short + extension = long name
+  private String      d_long_name;       // keep for convenience
+  // if NONBLOCKING_SEND or NONBLOCKING_RECV, this has the original name
+  private String      d_blocking_name;   
+  private Set         d_references;
+  //subset of references, no exceptions
+  private Set         d_references_no_exceptions;  
+  private Set         d_basicarrays;     // basic array types references
+  private Type        d_return_type;
+  private Set         d_throws;
+  private Set         d_explicit_throws; // subset of d_throws
+  private List        d_pre_clause;      // Local precondition clause only
+  private List        d_post_clause;     // Local postcondition clause only
+  //All the rarray indices required in the arg list	
+  private Set         d_rarray_indices;  
+
+  /*
+   * A Hashmap of rarray index names to a collection of RarrayInfo objects.  
+   * And RarrayInfo objects represents what rarray uses the index, and where.
+   */
+  private Map         d_rarray_index_map; 
   private SplicerList d_splicers = null;	// Optional splicer blocks
-  private Context d_context;
+  private Context     d_context;
 
   /**
    * Create an empty <code>Method</code> object that will be built by
    * calls to other member functions.
    */
   public Method(Context context) {
-    d_context = context;
-    d_arguments              = new ArrayList();
-    d_is_builtin             = false;  /* NOT built-in by default */
-    d_comment                = null;
-    d_short_name             = null;
-    d_extension              = null;
-    d_long_name              = null;
-    d_references             = new HashSet();
+    d_context                  = context;
+    d_arguments                = new ArrayList();
+    d_is_builtin               = false;  /* NOT built-in by default */
+    d_comment                  = null;
+    d_short_name               = null;
+    d_extension                = null;
+    d_long_name                = null;
+    d_references               = new HashSet();
     d_references_no_exceptions = new HashSet();
-    d_basicarrays            = new HashSet();
-    d_return_type            = null;
-    d_throws                 = new TreeSet();
-    d_explicit_throws        = new TreeSet();
-    d_preconditions          = new ArrayList();
-    d_postconditions         = new ArrayList();
-    d_rarray_indices	     = null;	
-    d_splicers               = null;
+    d_basicarrays              = new HashSet();
+    d_return_type              = null;
+    d_throws                   = new TreeSet();
+    d_explicit_throws          = new TreeSet();
+    d_pre_clause               = new ArrayList();
+    d_post_clause              = new ArrayList();
+    d_rarray_indices           = null;	
+    d_splicers                 = null;
   }
 
   /**
@@ -192,13 +200,18 @@ public class Method extends ASTNode
     m.d_explicit_throws = new TreeSet(d_explicit_throws);
     
     /*
-     * There is NO (good) reason to clone the method's contracts.
+     * If copying is simply used to produce the pre-method hooks signatures, 
+     * there does not appear to be any (good) reason to include contracts.
+     * However, must at least initialize it or null pointer exceptions 
+     * can be raised (if don't make sure it's not null).
      */
-    m.d_preconditions  = null;
-    m.d_postconditions = null;
+    m.d_pre_clause  = new ArrayList();
+    m.d_post_clause = new ArrayList();
           
-    m.d_rarray_indices = ((d_rarray_indices == null ) ? null : new HashSet(d_rarray_indices));
-    m.d_rarray_index_map = ((d_rarray_index_map == null ) ? null : new HashMap(d_rarray_index_map));
+    m.d_rarray_indices   = ((d_rarray_indices == null ) ? null 
+                                      : new HashSet(d_rarray_indices));
+    m.d_rarray_index_map = ((d_rarray_index_map == null ) ? null 
+                                      : new HashMap(d_rarray_index_map));
 
     m.d_arguments = new ArrayList();
     m.d_arguments.addAll(d_arguments);
@@ -214,9 +227,11 @@ public class Method extends ASTNode
       if ( arg.getMode() != Argument.OUT ) { 
         Argument newarg = null;
         if(withinout) {
-          newarg = new Argument(arg.getMode(), arg.getType(), arg.getFormalName() );
+          newarg = new Argument(arg.getMode(), arg.getType(), 
+                                arg.getFormalName() );
         } else {
-          newarg = new Argument(Argument.IN, arg.getType(), arg.getFormalName() );
+          newarg = new Argument(Argument.IN, arg.getType(), 
+                                arg.getFormalName() );
         }
         copyAttributes(arg, newarg);
         m.d_arguments.add(newarg);
@@ -243,7 +258,8 @@ public class Method extends ASTNode
    * @return new Method or null;
    * @see #spawnNonblockingRecv()
    */
-  public Method spawnPostHook(boolean useCReturn, boolean without) throws CodeGenerationException
+  public Method spawnPostHook(boolean useCReturn, boolean without) 
+    throws CodeGenerationException
   {
     Method m = new Method(d_context);
     m.d_is_builtin = true;
@@ -262,18 +278,22 @@ public class Method extends ASTNode
     // FIXME: May need to downselect later.
     m.d_basicarrays = new HashSet(d_basicarrays);
     
-
     m.d_throws = new TreeSet( d_throws);
     m.d_explicit_throws = new TreeSet( d_explicit_throws);
 
     /*
-     * There is NO (good) reason to clone the method's contracts.
+     * If copying is simply used to produce the post-method hook signatures, 
+     * there does not appear to be any (good) reason to copy contracts.
+     * However, must at least initialize it or null pointer exceptions 
+     * can be raised (if don't make sure it's not null).
      */
-    m.d_preconditions  = null;
-    m.d_postconditions = null;
+    m.d_pre_clause  = new ArrayList();
+    m.d_post_clause = new ArrayList();
           
-    m.d_rarray_indices = ((d_rarray_indices == null ) ? null : new HashSet(d_rarray_indices));
-    m.d_rarray_index_map = ((d_rarray_index_map == null ) ? null : new HashMap(d_rarray_index_map));
+    m.d_rarray_indices   = ((d_rarray_indices == null ) ? null 
+                                      : new HashSet(d_rarray_indices));
+    m.d_rarray_index_map = ((d_rarray_index_map == null ) ? null 
+                                      : new HashMap(d_rarray_index_map));
 
     m.d_arguments = new ArrayList();
     m.d_arguments.addAll(d_arguments);
@@ -285,10 +305,11 @@ public class Method extends ASTNode
      */
     m.d_arguments = new ArrayList();
     for( Iterator i = d_arguments.iterator(); i.hasNext(); ) { 
-      Argument arg = (Argument) i.next();
+      Argument arg    = (Argument) i.next();
       Argument newarg = null;
       if(without) {
-        newarg = new Argument(arg.getMode(), arg.getType(), arg.getFormalName() );
+        newarg = new Argument(arg.getMode(), arg.getType(), 
+                              arg.getFormalName() );
       } else {
         newarg = new Argument(Argument.IN, arg.getType(), arg.getFormalName() );
       }
@@ -325,44 +346,50 @@ public class Method extends ASTNode
     if ( getCommunicationModifier() != NONBLOCKING ) { 
       return null;
     }
-    Method m = new Method(d_context);
+    Method m       = new Method(d_context);
     m.d_is_builtin = true;
 
     // now largely copy everything... but there are important differences.
-    m.d_comment = d_comment;
+    m.d_comment                  = d_comment;
     m.d_attributes.putAll(d_attributes);
     
     // change NONBLOCKING to NONBLOCKING_SEND
     m.setCommunicationModifier(NONBLOCKING_SEND);
-    m.d_blocking_name = d_long_name;
-    m.d_short_name = d_short_name + "_send";
-    m.d_extension = d_extension;
-    m.d_long_name = d_long_name + "_send";
+    m.d_blocking_name            = d_long_name;
+    m.d_short_name               = d_short_name + "_send";
+    m.d_extension                = d_extension;
+    m.d_long_name                = d_long_name + "_send";
     // FIXME: May need to downselect later.
-    m.d_references = new HashSet ( d_references);
-    m.d_references_no_exceptions = new HashSet( d_references_no_exceptions);
+    m.d_references               = new HashSet (d_references);
+    m.d_references_no_exceptions = new HashSet(d_references_no_exceptions);
     // FIXME: May need to downselect later.
     m.d_basicarrays = new HashSet( d_basicarrays);
     
     // send will always return a local Ticket
-    Symbol tmpsym = Utilities.lookupSymbol(d_context, "sidl.rmi.Ticket");
+    Symbol tmpsym   = Utilities.lookupSymbol(d_context, "sidl.rmi.Ticket");
     m.d_return_type = new Type( tmpsym.getSymbolID(), d_context );  
     
     // send can return exceptions if the send fails
     // (not any exceptions from the recipient)
     TreeSet default_throws = new TreeSet();
-    default_throws.add( Utilities.lookupSymbol(d_context, BabelConfiguration.getRuntimeException())
-                        .getSymbolID());
-    m.d_throws = default_throws;
-    m.d_explicit_throws = new TreeSet();
+    default_throws.add(Utilities.lookupSymbol(d_context, 
+                       BabelConfiguration.getRuntimeException()).getSymbolID());
+    m.d_throws             = default_throws;
+    m.d_explicit_throws    = new TreeSet();
 
-    // FIXME: ask Tammy if this is right?
-    m.d_preconditions = new ArrayList( d_preconditions);
-    m.d_postconditions = new ArrayList( d_postconditions);
-          
-    m.d_rarray_indices = ((d_rarray_indices == null ) ? null : new HashSet(d_rarray_indices));
-    m.d_rarray_index_map = ((d_rarray_index_map == null ) ? null : new HashMap(d_rarray_index_map));
+    /*
+     * TBD:  Are contracts (to be) generated for the clone?  If so, then
+     * also need to copy method-specific contract clauses.
+     * Must at least initialize it or null pointer exceptions can be raised 
+     * (if don't make sure it's not null).
+     */
+    m.d_pre_clause       = new ArrayList();
+    m.d_post_clause      = new ArrayList();
 	  
+    m.d_rarray_indices   = ((d_rarray_indices == null ) ? null 
+                                      : new HashSet(d_rarray_indices));
+    m.d_rarray_index_map = ((d_rarray_index_map == null ) ? null 
+                                      : new HashMap(d_rarray_index_map));
     /* 
      * now the key part... create a new arglist for the send
      * this new arglist contains only the in and inout args
@@ -372,7 +399,8 @@ public class Method extends ASTNode
     for( Iterator i = d_arguments.iterator(); i.hasNext(); ) { 
       Argument arg = (Argument) i.next();
       if ( arg.getMode() != Argument.OUT ) { 
-        Argument newarg = new Argument(Argument.IN, arg.getType(), arg.getFormalName() );
+        Argument newarg = new Argument(Argument.IN, arg.getType(), 
+                                       arg.getFormalName() );
         copyAttributes(arg, newarg);
         m.d_arguments.add(newarg);
       }
@@ -391,32 +419,41 @@ public class Method extends ASTNode
     if ( getCommunicationModifier() != NONBLOCKING ) { 
       return null;
     }
-    Method m = new Method(d_context);
+    Method m       = new Method(d_context);
     m.d_is_builtin = true;
 
     m.d_attributes.putAll(d_attributes);
 
     // now largely copy everything... but there are important differences.
-    m.d_comment = d_comment;
-    m.setCommunicationModifier(NONBLOCKING_RECV); // change NONBLOCKING to NONBLOCKING_RECV
-    m.d_blocking_name = d_long_name;
-
-    m.d_short_name = d_short_name + "_recv";
-    m.d_extension = d_extension;
-
-    m.d_long_name = d_long_name + "_recv";
+    m.d_comment                  = d_comment;
+    // change NONBLOCKING to NONBLOCKING_RECV
+    m.setCommunicationModifier(NONBLOCKING_RECV); 
+    m.d_blocking_name            = d_long_name;
+    m.d_short_name               = d_short_name + "_recv";
+    m.d_extension                = d_extension;
+    m.d_long_name                = d_long_name + "_recv";
     m.d_references_no_exceptions = new HashSet( d_references_no_exceptions);
 
-    m.d_basicarrays = new HashSet( d_basicarrays); // FIXME: May need to downselect later
+    // FIXME: May need to downselect later
+    m.d_basicarrays     = new HashSet( d_basicarrays); 
     m.setReturnCopy(isReturnCopy());
-    m.d_return_type = d_return_type;
-    m.d_throws = new TreeSet( d_throws);
-    m.d_explicit_throws = new TreeSet( d_explicit_throws);
-    m.d_preconditions = new ArrayList( d_preconditions);        // FIXME: ask Tammy if this is right?
-    m.d_postconditions = new ArrayList( d_postconditions); 
-    m.d_rarray_indices = ((d_rarray_indices == null ) ? null : new HashSet(d_rarray_indices));
-    m.d_rarray_index_map = ((d_rarray_index_map == null ) ? null : new HashMap(d_rarray_index_map));
-	  
+    m.d_return_type     = d_return_type;
+    m.d_throws          = new TreeSet(d_throws);
+    m.d_explicit_throws = new TreeSet(d_explicit_throws);
+
+    /*
+     * TBD:  Are contracts (to be) generated for the clone?  If so, then
+     * also need to copy method-specific contract clauses.
+     * Must at least initialize it or null pointer exceptions  can be 
+     * raised (if don't make sure it's not null).
+     */
+    m.d_pre_clause       = new ArrayList();
+    m.d_post_clause      = new ArrayList();
+
+    m.d_rarray_indices   = ((d_rarray_indices == null ) ? null 
+                                      : new HashSet(d_rarray_indices));
+    m.d_rarray_index_map = ((d_rarray_index_map == null ) ? null 
+                                      : new HashMap(d_rarray_index_map));
     /* 
      * now the key part... create a new arglist for the send
      * this new arglist contains only the inout and out args
@@ -466,7 +503,7 @@ public class Method extends ASTNode
   }
 
   /**
-   * Return the array of arguments in an <code>ArrayList</code>, where each
+   * Return the array of arguments in an <code>List</code>, where each
    * element is an <code>Argument</code>.
    */
   public List getArgumentList() {
@@ -474,14 +511,14 @@ public class Method extends ASTNode
   }
 
   /**
-   * Return the array of arguments in an <code>ArrayList</code> container.
+   * Return the array of arguments in an <code>List</code> container.
    * This method returns only arguments that are not used as indices to 
    * an Rarray
    */
   public List getArgumentListWithOutIndices() {
     if(d_rarray_indices != null) {
-      ArrayList ret = new ArrayList(Math.max(d_arguments.size() -
-                                             d_rarray_indices.size(),0));
+      List ret = new ArrayList(Math.max(d_arguments.size() -
+                                        d_rarray_indices.size(),0));
       for(Iterator a = d_arguments.iterator(); a.hasNext();) {
         Argument arg = (Argument) a.next();
         if (!d_rarray_indices.contains(arg.getFormalName())) {
@@ -495,7 +532,7 @@ public class Method extends ASTNode
   }
 
   /**
-   * Return the array of arguments in an <code>ArrayList</code> container.
+   * Return the array of arguments in an <code>List</code> container.
    * This returns all the arguments in the argument list, including those
    * that are indices to an Rarray.
    */
@@ -1191,6 +1228,10 @@ public class Method extends ASTNode
     return sameArguments(m);
   }
 
+  public boolean overloadCollision(Method m) {
+    return d_short_name.equals(m.d_short_name) && similarArguments(m);
+  }
+
   /**
    * Return TRUE if the arguments of the specified method match those of
    * this method; otherwise, return FALSE.  Note that ordering counts!  
@@ -1217,6 +1258,18 @@ public class Method extends ASTNode
     return true;
   }
 
+  /** Used to detect overload collisions */
+  private boolean similarArguments(Method m) {
+    final int size = d_arguments.size();
+    if (size != m.d_arguments.size()) return false;
+    for (int i = 0; i < size; ++i) {
+      Argument arg1 = (Argument)d_arguments.get(i);
+      Argument arg2 = (Argument)m.d_arguments.get(i);
+      if (!arg1.similar(arg2)) return false;
+    }
+    return true;
+  }
+
   /**
    * Return an the argument whose formal name matches the passed in string.
    * Return null if the string does not match anything.
@@ -1259,68 +1312,235 @@ public class Method extends ASTNode
   }
 
   /**
-   * Return TRUE if PURE clause appears in the assertions associated
-   * with this method; otherwise, return FALSE.
-   */
-  public boolean hasPureAssertion() {
-    boolean   found = false;
-    List list  = getPostconditions();
-    for (Iterator iter = list.iterator();
-         iter.hasNext() && !found; ) {
-      Assertion item = (Assertion) iter.next();
-      if (item.hasPureClause()) {
-        found = true;
-      }
-    }
-    return found;
-  }
-
-  /**
-   * Add the specified assertion to the proper assertion list for this method.
+   * Add the specified assertion to the proper contract clause for this method.
    *
    * @param   assertion  The assertion to be added.
    * @throws  gov.llnl.babel.symbols.AssertionException
    *                     The exception thrown if the assertion is not valid.
    */
-  public void addAssertion(Assertion assertion) throws AssertionException 
+  public void addClauseAssertion(Assertion assertion) throws AssertionException 
   {
     checkFrozen();
     if ( assertion.isPrecondition() ) {
-      d_preconditions.add(assertion);
+      d_pre_clause.add(assertion);
+      try {
+        /*
+         * ToDo:  This should be added as an implicit exception but
+         * more work is needed to get C++ to properly handle additional
+         * implicit exceptions beyond RuntimeException.
+         */
+        if (d_pre_clause.size() <= 0) {
+          addThrows(Utilities.lookupSymbol(d_context, 
+             BabelConfiguration.getPreconditionViolation()).getSymbolID());
+        }
+      } catch (CodeGenerationException preCGEx) {
+        System.err.println("Babel: Warning: Unable to add sidl.PreViolation "
+          + "as implicit exception for " + getLongMethodName() + ".");
+      }
     } else if ( assertion.isPostcondition() ) {
-      d_postconditions.add(assertion);
+      d_post_clause.add(assertion);
+      try {
+        /*
+         * ToDo:  Would prefer to add this as an implicit exception but
+         * more work is needed to get C++ to properly handle additional
+         * implicit exceptions beyond RuntimeException.
+         */
+        if (d_post_clause.size() <= 0) {
+          addThrows(Utilities.lookupSymbol(d_context, 
+             BabelConfiguration.getPostconditionViolation()).getSymbolID());
+        }
+      } catch (CodeGenerationException postCGEx) {
+        System.err.println("Babel: Warning: Unable to add sidl.PostViolation "
+          + "as implicit exception for " + getLongMethodName() + ".");
+      }
     } else {
       throw new AssertionException("Method: " + d_long_name, "Cannot add a(n) "
-                                   + "\"" + assertion.getTypeName() + "\" assertion to a "
-                                   + "method\'s assertion list.");
+                  + "\"" + assertion.getTypeName() + "\" assertion to a "
+                  + "contract clause for a method.");
     }
   }
 
   /**
-   * Return the list of preconditions as an <code>ArrayList</code> of
-   * <code>Assertion</code> elements.  
+   * Return an <code>List</code> of <code>Assertion</code>s that make up
+   * the precondition clause.
    */
-  public List getPreconditions() {
-    return protectList(d_preconditions);
+  public List getPreClause() {
+    return protectList(d_pre_clause);
   }
 
   /**
-   * Return the list of postconditions as an <code>ArrayList</code> of
-   * <code>Assertion</code> elements.
+   * Returns TRUE if the method has assertions within its precondition clause; 
+   * otherwise, returns FALSE.
    */
-  public List getPostconditions() {
-    return protectList(d_postconditions);
+  public boolean hasPreClause() {
+    return d_pre_clause.size() > 0;
   }
 
   /**
-   * Return TRUE if the list of assertions has the specified reserved method 
-   * assertion; FALSE otherwise.
+   * Return an <code>List</code> of <code>Assertion</code>s that make up
+   * the postcondition clause.
    */
-  public boolean hasReservedMethodAssertion(List list, int type) {
+  public List getPostClause() {
+    return protectList(d_post_clause);
+  }
+
+  /**
+   * Returns TRUE if the method has assertions within its postcondition clause;
+   * otherwise, returns FALSE.
+   */
+  public boolean hasPostClause() {
+    return d_post_clause.size() > 0;
+  }
+
+  /**
+   * Returns TRUE if the clause (provided as a list of assertions) contains
+   * at least one method call; returns FALSE otherwise.
+   *
+   * @param list  Contract clause (as a list of assertions).
+   */
+  private static boolean contractHasMethodCall(List list) {
+    boolean has = false;
+    for (Iterator iter = list.iterator(); iter.hasNext() && !has; ) {
+      Assertion item = (Assertion)iter.next();
+      if (item.hasMethodCall()) {
+        has = true;
+      }
+    }
+
+    return has;
+  }
+
+  /**
+   * Returns TRUE if the precondition clause includes at least one method 
+   * call; returns FALSE otherwise.
+   */
+  public boolean preHasMethodCall() {
+    return contractHasMethodCall(d_pre_clause);
+  }
+
+  /**
+   * Returns TRUE if the postcondition clause includes at least one method 
+   * call; returns FALSE otherwise.
+   */
+  public boolean postHasMethodCall() {
+    return contractHasMethodCall(d_post_clause);
+  }
+
+  /**
+   * Returns TRUE if the clause (provided as a list of assertions) includes
+   * at least one result or output argument; returns FALSE otherwise.
+   *
+   * @param list  Contract clause (as a list of assertions).
+   */
+  private static boolean clauseHasResultOrOutArg(List list, boolean outOnly) {
+    boolean has = false;
+    for (Iterator iter = list.iterator(); iter.hasNext() && !has; ) {
+      Assertion item = (Assertion)iter.next();
+      if (item.hasResultOrOutArg(outOnly)) {
+        has = true;
+      }
+    }
+    return has;
+  }
+
+  /**
+  /**
+   * Returns TRUE if the precondition clause includes at least one result 
+   * or output argument; returns FALSE otherwise. 
+   *
+   * Note:  Preconditions should _not_ have such arguments (unless the out
+   * is inout).
+   */
+  public boolean preHasResultOrOutArg() {
+    return clauseHasResultOrOutArg(d_pre_clause, true);
+  }
+
+  /**
+   * Returns TRUE if the postcondition clause includes at least one result 
+   * or output argument; returns FALSE otherwise.
+   */
+  public boolean postHasResultOrOutArg() {
+    return clauseHasResultOrOutArg(d_post_clause, false);
+  }
+
+  /**
+   * Returns TRUE if the postcondition clause includes a result argument; 
+   * returns FALSE otherwise.
+   */
+  public boolean postHasResult() {
+    boolean has = false;
+    for (Iterator iter = d_post_clause.iterator(); iter.hasNext() && !has; ) {
+      Assertion post = (Assertion)iter.next();
+      if (post.hasResult()) {
+        has = true;
+      }
+    }
+
+    return has;
+  }
+
+  /**
+   * Return TRUE if PURE clause appears in the assertions associated
+   * with this method; otherwise, return FALSE.
+   */
+  public boolean hasPureAssertion() {
+    boolean found = false;
+    for (Iterator iter = d_post_clause.iterator(); iter.hasNext() && !found; ) {
+      Assertion post = (Assertion)iter.next();
+      if (post.hasPureClause()) {
+        found = true;
+      }
+    }
+
+    return found;
+  }
+
+  /**
+   * Returns the default complexity of the contract clause (provided as a
+   * list of assertions); returns 0 (for constant-time) if none.
+   *
+   * @param list  Contract clause (as a list of assertions).
+   */
+  private static int getDefaultClauseComplexity(List list) {
+    int def  = 0;
+    int comp = 0;
+
+    for (Iterator iter = list.iterator(); iter.hasNext(); ) {
+      Assertion item = (Assertion)iter.next();
+      comp           = item.getDefaultComplexity();
+      if (def < comp) {
+        def = comp;
+      }
+    }
+
+    return def;
+  }
+
+  /**
+   * Returns the default complexity of the precondition clause; returns 0 
+   * (for constant-time) if none.
+   */
+  public int getPreDefaultComplexity() {
+    return getDefaultClauseComplexity(d_pre_clause);
+  }
+
+  /**
+   * Returns the default complexity of the postcondition clause; returns 0 
+   * (for constant-time) if none.
+   */
+  public int getPostDefaultComplexity() {
+    return getDefaultClauseComplexity(d_post_clause);
+  }
+
+  /**
+   * Return TRUE if the contract clause (provided as a list of assertions) has 
+   * the specified built-in method call; FALSE otherwise.
+   */
+  private static boolean clauseHasBuiltinMethod(List list, int type) {
     boolean hasIt = false;
     for (Iterator iter = list.iterator(); iter.hasNext() && !hasIt; ) {
-      Assertion item = (Assertion) iter.next();
-      if (item.hasReservedMethod(type)) {
+      Assertion item = (Assertion)iter.next();
+      if (item.hasBuiltinMethod(type)) {
         hasIt = true;
       }
     }
@@ -1328,8 +1548,48 @@ public class Method extends ASTNode
   }
 
   /**
-   * Return the number of array iteration macros in the assertions of the 
-   * method of the specified return type.  Valid types are given in 
+   * Returns TRUE if the a built-in assertion method call of the specified type
+   * is included in either contract clause; returns FALSE otherwise.
+   */
+  public boolean contractHasBuiltinMethod(int type) {
+    boolean hasIt = clauseHasBuiltinMethod(d_pre_clause, type);
+    if (!hasIt) {
+      hasIt = clauseHasBuiltinMethod(d_post_clause, type);
+    }
+    return hasIt;
+  }
+
+  /**
+   * Returns TRUE if the contract clause (specified as a list of assertions)
+   * has user-defined method call(s); returns FALSE otherwise.
+   */
+  private boolean clauseHasUserDefinedMethod(List list, boolean any) {
+    boolean hasIt = false;
+    for (Iterator iter = list.iterator(); iter.hasNext() && !hasIt; ) {
+      Assertion item = (Assertion) iter.next();
+      if (item.hasUserDefinedMethod(any)) {
+        hasIt = true;
+      }
+    }
+    return hasIt;
+  }
+
+  /**
+   * Returns TRUE if the contract calls any user-defined methods (when any is 
+   * TRUE) or has a user-defined method with a throws clause (if any is FALSE) 
+   * in either of its contract clauses; otherwise, returns FALSE.
+   */
+  public boolean contractHasUserDefinedMethod(boolean any) {
+    boolean hasIt = clauseHasUserDefinedMethod(d_pre_clause, any);
+    if (!hasIt) {
+      hasIt = clauseHasUserDefinedMethod(d_post_clause, any);
+    }
+    return hasIt;
+  }
+
+  /**
+   * Return the number of array iteration macros --- contained in the contract
+   * --- that return the specified type.  Valid types are given in 
    * MethodCall.java as MACRO_RETURNS_* values.
    */
   public int getMaxArrayIterMacros(char type) {
@@ -1337,7 +1597,7 @@ public class Method extends ASTNode
     Assertion as;
     int max = 0;
 
-    Iterator i = d_preconditions.iterator();
+    Iterator i = d_pre_clause.iterator();
     while (i.hasNext()) {
       as  = (Assertion)i.next();
       num = as.getNumArrayIterMacrosByType(type);
@@ -1346,7 +1606,7 @@ public class Method extends ASTNode
       }
     }
 
-    i = d_postconditions.iterator();
+    i = d_post_clause.iterator();
     while (i.hasNext()) {
       as  = (Assertion)i.next();
       num = as.getNumArrayIterMacrosByType(type);
@@ -1358,46 +1618,6 @@ public class Method extends ASTNode
     return max;
   }
 
-  /**
-   * Return TRUE if the method has the specified reserved method assertion; 
-   * FALSE otherwise.
-   */
-  public boolean hasReservedMethodAssertion(int type) {
-    boolean hasIt = hasReservedMethodAssertion(d_preconditions, type);
-    if (!hasIt) {
-      hasIt = hasReservedMethodAssertion(d_postconditions, type);
-    }
-    return hasIt;
-  }
-
-  /**
-   * Return TRUE if the list of assertions has the type of unreserved method 
-   * assertion; FALSE otherwise.
-   */
-  public boolean hasUnreservedMethodAssertion(List list, boolean any) {
-    boolean hasIt = false;
-    for (Iterator iter = list.iterator(); iter.hasNext() && !hasIt; ) {
-      Assertion item = (Assertion) iter.next();
-      if (item.hasUnreservedMethod(any)) {
-        hasIt = true;
-      }
-    }
-    return hasIt;
-  }
-
-  /**
-   * Return TRUE if the method has any unreserved method (when any is TRUE)
-   * assertion or has an unreserved method assertion with a throws clause 
-   * (if any is FALSE); otherwise, return FALSE.
-   */
-  public boolean hasUnreservedMethodAssertion(boolean any) {
-    boolean hasIt = hasUnreservedMethodAssertion(d_preconditions, any);
-    if (!hasIt) {
-      hasIt = hasUnreservedMethodAssertion(d_postconditions, any);
-    }
-    return hasIt;
-  }
-
   /**
    * This returns an array list of RarrayInfo objects that contains the info
    * for every rarray index used in this method.  
@@ -1471,7 +1691,7 @@ public class Method extends ASTNode
   }
 
   
-  public class RarrayInfo {
+  public static class RarrayInfo {
     
     public final Argument rarray;
     public final Argument index;
@@ -1484,7 +1704,8 @@ public class Method extends ASTNode
     }
     
     public int hashCode() {
-      return rarray.getFormalName().hashCode() + index.getFormalName().hashCode() + dim;
+      return rarray.getFormalName().hashCode() 
+           + index.getFormalName().hashCode() + dim;
     }
 
     public boolean equals(Object o) {
@@ -1509,7 +1730,8 @@ public class Method extends ASTNode
     if (hasAttribute(key)) {
       return (String)d_attributes.get(key);
     }
-    throw new UnknownAttributeException("Method does not have attribute " + key);
+    throw new UnknownAttributeException("Method does not have attribute " 
+                                       + key);
   }
 
   public void setAttribute(String key)
@@ -1531,7 +1753,8 @@ public class Method extends ASTNode
       d_attributes.remove(key);
     }
     else {
-      throw new UnknownAttributeException("Method does not have attribute " + key);
+      throw new UnknownAttributeException("Method does not have attribute " 
+                                        + key);
     }
   }
   
@@ -1598,17 +1821,17 @@ public class Method extends ASTNode
         SymbolID id = (SymbolID)i.next();
         id.freeze();
       }
-      d_attributes = protectMap(d_attributes);
-      d_arguments = protectList(d_arguments);
-      d_references = protectSet(d_references);
+      d_attributes               = protectMap(d_attributes);
+      d_arguments                = protectList(d_arguments);
+      d_references               = protectSet(d_references);
       d_references_no_exceptions = protectSet(d_references_no_exceptions);
-      d_basicarrays = protectSet(d_basicarrays);
-      d_throws = protectSet(d_throws);
-      d_explicit_throws = protectSet(d_explicit_throws);
-      d_preconditions = protectList(d_preconditions);
-      d_postconditions = protectList(d_postconditions);
-      d_rarray_indices = protectSet(d_rarray_indices);
-      d_rarray_index_map = protectMap(d_rarray_index_map);
+      d_basicarrays              = protectSet(d_basicarrays);
+      d_throws                   = protectSet(d_throws);
+      d_explicit_throws          = protectSet(d_explicit_throws);
+      d_pre_clause               = protectList(d_pre_clause);
+      d_post_clause              = protectList(d_post_clause);
+      d_rarray_indices           = protectSet(d_rarray_indices);
+      d_rarray_index_map         = protectMap(d_rarray_index_map);
     }
   }
 }
