@@ -4,7 +4,7 @@
 // Revision:    @(#) $Id $
 // Description: The assertion expression interface for a method call..
 //
-// Copyright (c) 2003-2004, The Regents of the University of Calfornia.
+// Copyright (c) 2003-2007, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -53,36 +53,36 @@ public class MethodCall extends AssertionExpression {
   /** Method name attribute.  */
   private String    d_name                  = null;
 
-  /** Unreserved method is static */
+  /** User-defined method is static */
   private boolean   d_is_static             = true;
 
-  /** Unreserved method throws exceptions */
+  /** User-defined method throws exceptions */
   private boolean   d_throws_exceptions     = false;
 
   /** Argument list attribute.  */
   private ArrayList d_arguments             = null;
 
-  /** Reserved method type attribute. */
-  private int       d_reserved_method       = 0;
+  /** Method type attribute. */
+  private int       d_method_type        = 0;
 
-  /** Relation association for reserved methods with relations */
-  private int       d_reserved_relation     = METHOD_RELATION_NONE;
+  /** Relation association for methods with relations */
+  private int       d_method_relation      = METHOD_RELATION_NONE;
 
   /** 
    * Relation string that is either "<op> <val>" or "<val> <op>" depending
    * upon whether the array is on the left or right of the binary expression.
-   * This information is saved from the validation process for reserved
-   * methods that require a relation argument to be passed to the built-in
+   * This information is saved from the validation process for built-in
+   * methods that require a relation argument to be passed to the corresponding
    * macro to avoid extra argument processing during code generation.
    */
-  private String    d_reserved_relation_rel = null;
+  private String    d_method_relation_rel  = null;
 
   /**
-   * The basic type information associated with an (or the last) array
-   * variable detected in the argument list.  It is saved off from the 
-   * validation process to facilitate code generation.
+   * The basic type information associated with the last array variable
+   * detected in the called method's argument list.  It is saved off during
+   * the validation process to facilitate code generation.
    */
-  private Type      d_reserved_array_type   = null;
+  private Type      d_builtin_array_type   = null;
 
   /**
    **********************************************************************
@@ -115,16 +115,16 @@ public class MethodCall extends AssertionExpression {
    */
 
   /*
-   * ...To facilitate checking on reserved methods...
+   * ...To facilitate checking on built-in methods...
    */
   public static final int  METHOD_ANY_ARRAY        = -3;
   public static final int  METHOD_NUMERIC_ARRAY    = -2;
   public static final int  METHOD_ANY_BUILTIN      = -1;
 
   /*
-   * ...Actual reserved method attribute types...
+   * ...Actual built-in method attribute types...
    */
-  public static final int  METHOD_NOT_RESERVED     = 0;
+  public static final int  METHOD_IS_USER_DEFINED  = 0;
   public static final int  METHOD_ARRAY_ALL        = 1;
   public static final int  METHOD_ARRAY_ANY        = 2;
   public static final int  METHOD_ARRAY_COUNT      = 3;
@@ -150,7 +150,7 @@ public class MethodCall extends AssertionExpression {
   public static final int  MAXIMUM_ARRAY_METHOD    = 17;
   public static final int  MAXIMUM_METHOD          = 20;
 
-  private static final String s_reserved_name[] = {
+  private static final String s_builtin_name[] = {
     "",
     "all",
     "any",
@@ -249,10 +249,10 @@ public class MethodCall extends AssertionExpression {
       throw new AssertionException("Cannot instantiate a method call without "
                 + "the name of the method to be called.");
     }
-    for (int i=MINIMUM_METHOD; (i<=MAXIMUM_METHOD) && (d_reserved_method==0); 
+    for (int i=MINIMUM_METHOD; (i<=MAXIMUM_METHOD) && (d_method_type==0); 
          i++) {
-      if (name.toLowerCase().equals(s_reserved_name[i])) {
-        d_reserved_method = i;
+      if (name.toLowerCase().equals(s_builtin_name[i])) {
+        d_method_type = i;
       }
     }
     d_name      = name;
@@ -269,7 +269,7 @@ public class MethodCall extends AssertionExpression {
 
 
   /**
-   * Return TRUE if the (unreserved) method is static; otherwise, return
+   * Return TRUE if the user-defined method is static; otherwise, return
    * FALSE.
    *
    * WARNING:  This is not known until _after_ the method has been validated!
@@ -280,7 +280,7 @@ public class MethodCall extends AssertionExpression {
 
 
   /**
-   * Return TRUE if the (unreserved) method throws exceptions; otherwise, 
+   * Return TRUE if the user-defined method throws exceptions; otherwise, 
    * return FALSE.
    *
    * WARNING:  This is not known until _after_ the method has been validated!
@@ -315,8 +315,8 @@ public class MethodCall extends AssertionExpression {
    */
   public boolean isArrayMethod() {
     boolean is = false;
-    if (  (MINIMUM_METHOD  <= d_reserved_method     ) 
-       && (d_reserved_method <= MAXIMUM_ARRAY_METHOD) ) {
+    if (  (MINIMUM_METHOD  <= d_method_type     ) 
+       && (d_method_type <= MAXIMUM_ARRAY_METHOD) ) {
        is = true;
     }
     return is; 
@@ -329,11 +329,11 @@ public class MethodCall extends AssertionExpression {
    */
   public boolean isArrayMacroMethod() {
     boolean is = false;
-    if (  (d_reserved_method == METHOD_ARRAY_DIMEN) 
-       || (d_reserved_method == METHOD_ARRAY_LOWER)
-       || (d_reserved_method == METHOD_ARRAY_SIZE)
-       || (d_reserved_method == METHOD_ARRAY_STRIDE)
-       || (d_reserved_method == METHOD_ARRAY_UPPER) ) {
+    if (  (d_method_type == METHOD_ARRAY_DIMEN) 
+       || (d_method_type == METHOD_ARRAY_LOWER)
+       || (d_method_type == METHOD_ARRAY_SIZE)
+       || (d_method_type == METHOD_ARRAY_STRIDE)
+       || (d_method_type == METHOD_ARRAY_UPPER) ) {
        is = true;
     }
     return is; 
@@ -359,10 +359,10 @@ public class MethodCall extends AssertionExpression {
    */
   public boolean isBuiltinRelationMethod() {
     boolean is = false;
-    if (  (d_reserved_method == METHOD_ARRAY_ALL) 
-       || (d_reserved_method == METHOD_ARRAY_ANY)
-       || (d_reserved_method == METHOD_ARRAY_COUNT)
-       || (d_reserved_method == METHOD_ARRAY_NONE) ) {
+    if (  (d_method_type == METHOD_ARRAY_ALL) 
+       || (d_method_type == METHOD_ARRAY_ANY)
+       || (d_method_type == METHOD_ARRAY_COUNT)
+       || (d_method_type == METHOD_ARRAY_NONE) ) {
        is = true;
     }
     return is; 
@@ -383,7 +383,70 @@ public class MethodCall extends AssertionExpression {
    * Return TRUE if the expression contains RESULT; otherwise, return FALSE.
    */
   public boolean hasResult() {
-    return false;
+    boolean   has  = false;
+    ArrayList args = d_arguments;
+
+    if ( (args != null) && (!args.isEmpty()) ) {
+      ListIterator i = args.listIterator();
+
+      while (i.hasNext() && !has) {
+        AssertionExpression arg = (AssertionExpression)i.next();
+        if (arg.hasResult()) {
+          has = true;
+        }
+      }
+    }
+
+    return has;
+  }
+
+
+  /**
+   * Return TRUE if a result clause or method argument is found within the
+   * expression; otherwise, return FALSE.
+   */
+  public boolean hasResultOrArg() {
+    boolean   has = false;
+    ArrayList args = d_arguments;
+
+    if ( (args != null) && (!args.isEmpty()) ) {
+      ListIterator i = args.listIterator();
+
+      while (i.hasNext() && !has) {
+        AssertionExpression arg = (AssertionExpression)i.next();
+        if (arg.hasResultOrArg()) {
+          has = true;
+        }
+      }
+    }
+
+    return has;
+  }
+
+
+  /**
+   * Return TRUE if a result clause or output argument is found within the
+   * expression; otherwise, return FALSE.
+   *
+   * @param  outOnly  TRUE if only concerned with output arguments that are
+   *                  out only; FALSE otherwise.
+   */
+  public boolean hasResultOrOutArg(boolean outOnly) {
+    boolean   has  = false;
+    ArrayList args = d_arguments;
+
+    if ( (args != null) && (!args.isEmpty()) ) {
+      ListIterator i = args.listIterator();
+
+      while (i.hasNext() && !has) {
+        AssertionExpression arg = (AssertionExpression)i.next();
+        if (arg.hasResultOrOutArg(outOnly)) {
+          has = true;
+        }
+      }
+    }
+
+    return has;
   }
 
 
@@ -399,15 +462,28 @@ public class MethodCall extends AssertionExpression {
 
 
   /**
-   * Return TRUE if the expression is, or has, the specified reserved method; 
+   * Return the default complexity of the expression (0 = constant, 1 = linear,
+   * etc.).
+   */
+  public int getDefaultComplexity() {
+    /*
+     * ToDo...Need to revisit this once distinguish between one- and multi-
+     * dimensional arrays.
+     */
+    return isBuiltinNumericArrayMethod() ? 1 : 0;
+  }
+
+
+  /**
+   * Return TRUE if the expression is, or has, the specified built-in method; 
    * otherwise, return FALSE.  To facilitate checking, the following options
    * for type can also be used to check if the method is one of many:
    * 
-   *   METHOD_ANY_BUILTIN       TRUE for any reserved method
-   *   METHOD_NUMERIC_ARRAY     TRUE for any reserved, numeric array method
-   *   METHOD_ANY_ARRAY         TRUE for any reserved array method
+   *   METHOD_ANY_BUILTIN       TRUE for any built-in method
+   *   METHOD_NUMERIC_ARRAY     TRUE for any built-in, numeric array method
+   *   METHOD_ANY_ARRAY         TRUE for any built-in array method
    */
-  public boolean hasReservedMethod(int type) {
+  public boolean hasBuiltinMethod(int type) {
     boolean hasIt = false;
     switch (type) {
       case METHOD_ANY_BUILTIN:
@@ -427,7 +503,7 @@ public class MethodCall extends AssertionExpression {
         break;
       default:
         if (  ( (MINIMUM_METHOD <= type) && (type <= MAXIMUM_METHOD) )
-           && (type == d_reserved_method) ) {
+           && (type == d_method_type) ) {
           hasIt = true;
         }
         break;
@@ -437,14 +513,14 @@ public class MethodCall extends AssertionExpression {
 
 
   /**
-   * Return TRUE if the method is any unreserved method (when any is TRUE)
-   * or it is an unreserved method with a throws clause (if any is FALSE);
-   * otherwise, return FALSE.
+   * Return TRUE if the expression has a method AND the method is any
+   * user-defined method (when any is TRUE) or it is an user-defined
+   * method with a throws clause (if any is FALSE); otherwise, return FALSE.
    */
-  public boolean hasUnreservedMethod(boolean any) {
+  public boolean hasUserDefinedMethod(boolean any) {
     boolean hasIt = false;
-    if (  (d_reserved_method == METHOD_NOT_RESERVED) 
-       && ( (any) || (d_throws_exceptions) ) ) {
+    if (  (d_method_type == METHOD_IS_USER_DEFINED) 
+       && ( any || d_throws_exceptions ) ) {
       hasIt = true;
     }
     return hasIt;
@@ -603,7 +679,7 @@ public class MethodCall extends AssertionExpression {
   private Method getMethod(Extendable ext) throws AssertionException {
     Method meth = null;
 
-    if (d_reserved_method == 0) {
+    if (d_method_type == 0) {
       /*
        * Try to see if extendable has this method based on the long
        * names of it's methods.
@@ -627,7 +703,7 @@ public class MethodCall extends AssertionExpression {
       throw new AssertionException("Will not attempt to look up method " 
                 + d_name + " in " + getExceptionPrefix(ext) 
                 + " since the method has been "
-                + "marked as one of the reserved assertion methods.");
+                + "marked as one of the built-in assertion methods.");
     }
     return meth;
   }
@@ -639,7 +715,7 @@ public class MethodCall extends AssertionExpression {
   private String getInvalidBuiltinMsg(Extendable ext) {
      return ERROR_SEMANTIC_VALIDATION + "Invalid call to " + d_name + "() in " 
             + getExceptionPrefix(ext) + " since signature is incompatible with "
-            + "that of the built-in " + s_reserved_name[d_reserved_method] 
+            + "that of the built-in " + s_builtin_name[d_method_type] 
             + " method.";
   }
 
@@ -694,7 +770,7 @@ public class MethodCall extends AssertionExpression {
           arg.validateExpression(ext, m);
           if (isBuiltinNumericArrayMethod() && idArg.returnIsArray()) {
             if (idArg.returnIsNumericArray()) {
-              d_reserved_array_type = getReturnType();
+              d_builtin_array_type = getReturnType();
               numArrays++;
               if (numArrays > MAX_ARRAY_ARGS) {
                 throw new AssertionException(ERROR_SEMANTIC_VALIDATION 
@@ -725,13 +801,13 @@ public class MethodCall extends AssertionExpression {
           beArg.validateExpression(ext, m);
 
           if (beArg.arrayOnLeft() && beArg.arrayOnRight()) {
-            d_reserved_relation = METHOD_RELATION_BOTH;
+            d_method_relation = METHOD_RELATION_BOTH;
           } else if (beArg.arrayOnLeft()) {
-            d_reserved_relation = METHOD_RELATION_LEFT;
+            d_method_relation = METHOD_RELATION_LEFT;
           } else {
-            d_reserved_relation = METHOD_RELATION_RIGHT;
+            d_method_relation = METHOD_RELATION_RIGHT;
           }
-          d_reserved_relation_rel = beArg.getArrayRelation();
+          d_method_relation_rel = beArg.getArrayRelation();
         } else {
           arg.validateExpression(ext, m);
         }
@@ -740,12 +816,12 @@ public class MethodCall extends AssertionExpression {
         }
       }
       if (!hasArray) {
-         if (d_reserved_method == METHOD_ARRAY_IRANGE) {
-           d_reserved_method = METHOD_IRANGE;
-         } else if (d_reserved_method == METHOD_ARRAY_NEAR_EQUAL) {
-           d_reserved_method = METHOD_NEAR_EQUAL;
-         } else if (d_reserved_method == METHOD_ARRAY_RANGE) {
-           d_reserved_method = METHOD_RANGE;
+         if (d_method_type == METHOD_ARRAY_IRANGE) {
+           d_method_type = METHOD_IRANGE;
+         } else if (d_method_type == METHOD_ARRAY_NEAR_EQUAL) {
+           d_method_type = METHOD_NEAR_EQUAL;
+         } else if (d_method_type == METHOD_ARRAY_RANGE) {
+           d_method_type = METHOD_RANGE;
          }
       }
     }
@@ -770,7 +846,7 @@ public class MethodCall extends AssertionExpression {
      * ...Now ensure it's an available method.
      */
     ArrayList rList = null;
-    switch (d_reserved_method) {
+    switch (d_method_type) {
       case METHOD_ARRAY_ALL: 
       case METHOD_ARRAY_ANY: 
       case METHOD_ARRAY_NONE: 
@@ -1007,7 +1083,7 @@ public class MethodCall extends AssertionExpression {
       argList.append(Utilities.s_exception);
     }
     if (addRelation) {
-      argList.append(", " + d_reserved_relation_rel);
+      argList.append(", " + d_method_relation_rel);
     }
     if (addArrayIterVars) {
       argList.append(", " + ARRAY_ITER_VAR + ", " + ARRAY_SIZE_VAR);
@@ -1030,35 +1106,42 @@ public class MethodCall extends AssertionExpression {
     ArrayList    list = null;
     StringBuffer call = new StringBuffer();
     
-    switch (d_reserved_method) {
+    switch (d_method_type) {
       case METHOD_ARRAY_ALL: 
       case METHOD_ARRAY_NONE: 
-        call.append(s_builtin_macro[d_reserved_method]);
-        call.append(s_builtin_relation[d_reserved_relation] + "(");
+        call.append(s_builtin_macro[d_method_type]);
+        call.append(s_builtin_relation[d_method_relation] + "(");
         call.append(getCArgList(epvVar, null, true, true, true));
         call.append(", " + ARRAY_COUNT_VAR);
         call.append(", " + ARRAY_BOOLEAN_RESULT_VAR + "[" 
             + startInd[MACRO_RETURNS_BOOLEAN_IND] + "])");
         break;
       case METHOD_ARRAY_ANY: 
-        call.append(s_builtin_macro[d_reserved_method]);
-        call.append(s_builtin_relation[d_reserved_relation] + "(");
+        call.append(s_builtin_macro[d_method_type]);
+        call.append(s_builtin_relation[d_method_relation] + "(");
         call.append(getCArgList(epvVar, null, true, true, true));
         call.append(", " + ARRAY_BOOLEAN_RESULT_VAR + "[" 
             + startInd[MACRO_RETURNS_BOOLEAN_IND] + "])");
         break;
       case METHOD_ARRAY_COUNT: 
-        call.append(s_builtin_macro[d_reserved_method]);
-        call.append(s_builtin_relation[d_reserved_relation] + "(");
+        call.append(s_builtin_macro[d_method_type]);
+        call.append(s_builtin_relation[d_method_relation] + "(");
         call.append(getCArgList(epvVar, null, true, true, true));
         call.append(", " + ARRAY_INTEGER_RESULT_VAR + "[" 
             + startInd[MACRO_RETURNS_INTEGER_IND] + "])");
         break;
       case METHOD_ARRAY_NON_DECR: 
       case METHOD_ARRAY_NON_INCR: 
-        call.append(s_builtin_macro[d_reserved_method] + "(");
+        call.append(s_builtin_macro[d_method_type] + "(");
         call.append(getCArgList(epvVar, null, true, false, true));
+/*
+ * ToDo...Originally just this...but wasn't factoring in properly.
+ *        Is this still preferrable?
+ *
         call.append(", " + ARRAY_DOUBLE_RESULT_VAR);
+ */
+        call.append(", " + ARRAY_DOUBLE_RESULT_VAR + "["
+            + startInd[MACRO_RETURNS_DOUBLE_IND] + "] ");
         call.append(", " + ARRAY_BOOLEAN_RESULT_VAR + "[" 
             + startInd[MACRO_RETURNS_BOOLEAN_IND] + "])");
         break;
@@ -1070,23 +1153,23 @@ public class MethodCall extends AssertionExpression {
          * better to leave as-is (i.e., as a double).  Of course,
          * this won't work with complex results!
          */
-        call.append(s_builtin_macro[d_reserved_method] + "(");
+        call.append(s_builtin_macro[d_method_type] + "(");
         call.append(getCArgList(epvVar, null, true, false, true));
-        call.append(", " + d_reserved_array_type.getArrayTypeName());
+        call.append(", " + d_builtin_array_type.getArrayTypeName());
         call.append(", " + ARRAY_DOUBLE_RESULT_VAR + "[" 
             + startInd[MACRO_RETURNS_DOUBLE_IND] + "])");
         break;
       case METHOD_ARRAY_IRANGE: 
       case METHOD_ARRAY_NEAR_EQUAL: 
       case METHOD_ARRAY_RANGE: 
-        call.append(s_builtin_macro[d_reserved_method] + "(");
+        call.append(s_builtin_macro[d_method_type] + "(");
         call.append(getCArgList(epvVar, null, true, false, true));
         call.append(", " + ARRAY_COUNT_VAR);
         call.append(", " + ARRAY_BOOLEAN_RESULT_VAR + "[" 
             + startInd[MACRO_RETURNS_BOOLEAN_IND] + "])");
         break;
       case METHOD_ARRAY_SUM: 
-        call.append(s_builtin_macro[d_reserved_method] + "(");
+        call.append(s_builtin_macro[d_method_type] + "(");
         call.append(getCArgList(epvVar, null, true, false, true));
         call.append(", " + ARRAY_DOUBLE_RESULT_VAR + "[" 
             + startInd[MACRO_RETURNS_DOUBLE_IND] + "])");
@@ -1111,24 +1194,26 @@ public class MethodCall extends AssertionExpression {
     int num = 0;
 
     if (type == MACRO_RETURN_TYPE[MACRO_RETURNS_BOOLEAN_IND]) {
-      if (  (d_reserved_method == METHOD_ARRAY_ALL)
-         || (d_reserved_method == METHOD_ARRAY_ANY)
-         || (d_reserved_method == METHOD_ARRAY_IRANGE)
-         || (d_reserved_method == METHOD_ARRAY_NEAR_EQUAL)
-         || (d_reserved_method == METHOD_ARRAY_NON_DECR)
-         || (d_reserved_method == METHOD_ARRAY_NON_INCR)
-         || (d_reserved_method == METHOD_ARRAY_NONE)
-         || (d_reserved_method == METHOD_ARRAY_RANGE) ) {
+      if (  (d_method_type == METHOD_ARRAY_ALL)
+         || (d_method_type == METHOD_ARRAY_ANY)
+         || (d_method_type == METHOD_ARRAY_IRANGE)
+         || (d_method_type == METHOD_ARRAY_NEAR_EQUAL)
+         || (d_method_type == METHOD_ARRAY_NON_DECR)
+         || (d_method_type == METHOD_ARRAY_NON_INCR)
+         || (d_method_type == METHOD_ARRAY_NONE)
+         || (d_method_type == METHOD_ARRAY_RANGE) ) {
         num = 1;
       }
     } else if (type == MACRO_RETURN_TYPE[MACRO_RETURNS_DOUBLE_IND]) {
-      if (  (d_reserved_method == METHOD_ARRAY_MAX)
-         || (d_reserved_method == METHOD_ARRAY_MIN)
-         || (d_reserved_method == METHOD_ARRAY_SUM) ) {
+      if (  (d_method_type == METHOD_ARRAY_MAX)
+         || (d_method_type == METHOD_ARRAY_MIN)
+         || (d_method_type == METHOD_ARRAY_NON_DECR)
+         || (d_method_type == METHOD_ARRAY_NON_INCR)
+         || (d_method_type == METHOD_ARRAY_SUM) ) {
           num = 1;
       }
     } else if (type == MACRO_RETURN_TYPE[MACRO_RETURNS_INTEGER_IND]) {
-      if (d_reserved_method == METHOD_ARRAY_COUNT) {
+      if (d_method_type == METHOD_ARRAY_COUNT) {
         num = 1;
       }
     } else {
@@ -1138,7 +1223,6 @@ public class MethodCall extends AssertionExpression {
   }
 
 
-
   /**
    * Return the C version of the expression.
    *
@@ -1148,7 +1232,7 @@ public class MethodCall extends AssertionExpression {
   public String cExpression(String epvVar, int[] startInd) {
     StringBuffer call = new StringBuffer();
     
-    switch (d_reserved_method) {
+    switch (d_method_type) {
       case METHOD_ARRAY_ALL: 
       case METHOD_ARRAY_ANY: 
       case METHOD_ARRAY_IRANGE: 
@@ -1169,7 +1253,7 @@ public class MethodCall extends AssertionExpression {
       case METHOD_ARRAY_SIZE: 
       case METHOD_ARRAY_STRIDE: 
       case METHOD_ARRAY_UPPER: 
-        call.append(s_builtin_macro[d_reserved_method] + "(");
+        call.append(s_builtin_macro[d_method_type] + "(");
         call.append(getCArgList(epvVar, startInd, true, false, false));
         call.append(")");
         break;
@@ -1182,7 +1266,7 @@ public class MethodCall extends AssertionExpression {
       case METHOD_IRANGE: 
       case METHOD_NEAR_EQUAL: 
       case METHOD_RANGE: 
-        call.append(s_builtin_macro[d_reserved_method] + "(");
+        call.append(s_builtin_macro[d_method_type] + "(");
         call.append(getCArgList(epvVar, startInd, true, false, false));
         call.append(")");
         break;
