@@ -1,10 +1,10 @@
 //
 // File:        Type.java
 // Package:     gov.llnl.babel.symbols
-// Revision:    @(#) $Id: Type.java 6212 2007-11-01 00:07:17Z epperly $
+// Revision:    @(#) $Id: Type.java 6482 2008-08-21 22:50:53Z epperly $
 // Description: class that represents a sidl type
 //
-// Copyright (c) 2000-2003, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2007, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -282,6 +282,14 @@ public class Type extends ASTNode {
     return d_type;
   }
 
+  /**
+   * Return TRUE if the type is one of the standard primitive types,
+   * used for F03 Function return types; otherwise, return FALSE.
+   */
+  public boolean isF03FuncRetType() {
+     return d_type < OPAQUE && d_type != VOID && d_type !=CHAR ;
+  }
+
   /**
    * Return TRUE if the type is one of the standard primitive types,
    * including strings; otherwise, return FALSE.
@@ -562,6 +570,21 @@ public class Type extends ASTNode {
       return eq;
    }
 
+  /**
+   * Used to detect overloading collisions.
+   */
+  public boolean similar(Type t1)
+  {
+    if (equals(t1)) return true;
+    if ((getType() == ARRAY) && (t1.getType() == ARRAY)) {
+      if (d_array_type != null){
+        /* do not require dimension & order match */
+        return d_array_type.equals(t1.d_array_type);
+      }
+    }
+    return false;
+  }
+
   public void freeze()
   {
     if (!d_frozen) {
