@@ -4,7 +4,7 @@
 // Revision:    @(#) $Id $
 // Description: The unary expression interface for assertions.
 //
-// Copyright (c) 2003-2004, The Regents of the University of Calfornia.
+// Copyright (c) 2003-2007, Lawrence Livermore National Security, LLC
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -140,6 +140,28 @@ public class UnaryExpression extends AssertionExpression {
   }
 
 
+                                                                                
+  /**
+   * Return TRUE if a result clause or method argument is found within the
+   * expression; otherwise, return FALSE.
+   */
+  public boolean hasResultOrArg() {
+    return d_expression.hasResultOrArg();
+  }
+
+
+  /**
+   * Return TRUE if a result clause or output argument is found within the
+   * expression; otherwise, return FALSE.
+   *
+   * @param  outOnly  TRUE if only concerned with output arguments that are
+   *                  out only; FALSE otherwise.
+   */
+  public boolean hasResultOrOutArg(boolean outOnly) {
+    return d_expression.hasResultOrOutArg(outOnly);
+  }
+
+
   /**
    * Return TRUE if the expression has at least one method call; otherwise, 
    * return FALSE.  Clearly this expression isn't a method call BUT the
@@ -150,22 +172,31 @@ public class UnaryExpression extends AssertionExpression {
   }
 
 
-  /**
-   * Return TRUE if the expression is, or has, the specified reserved method
-   * call; otherwise, return FALSE.
+ /**
+   * Return the default complexity of the expression (0 = constant, 1 = linear,
+   * etc.).
    */
-  public boolean hasReservedMethod(int type) {
-    return d_expression.hasReservedMethod(type);
+  public int getDefaultComplexity() {
+    return d_expression.getDefaultComplexity();
   }
 
 
   /**
-   * Return TRUE if the method is any unreserved method (when any is TRUE)
-   * or it is an unreserved method with a throws clause (if any is FALSE);
-   * otherwise, return FALSE.
+   * Return TRUE if the expression is, or has, the specified built-in method
+   * call; otherwise, return FALSE.
    */
-  public boolean hasUnreservedMethod(boolean any) {
-    return d_expression.hasUnreservedMethod(any);
+  public boolean hasBuiltinMethod(int type) {
+    return d_expression.hasBuiltinMethod(type);
+  }
+
+
+  /**
+   * Return TRUE if the expression has a method AND the method is any
+   * user-defined method (when any is TRUE) or it is an user-defined
+   * method with a throws clause (if any is FALSE); otherwise, return FALSE.
+   */
+  public boolean hasUserDefinedMethod(boolean any) {
+    return d_expression.hasUserDefinedMethod(any);
   }
 
 
