@@ -191,7 +191,7 @@ public class CollisionChecker extends Visitor {
     s_reservedWords.put("xor_eq",           "C++");
   }
   
-  class Tuple { 
+  static class Tuple { 
     ASTNode node;
     HashMap members;
     public Tuple( ASTNode n, HashMap m) { 
