@@ -658,7 +658,7 @@ public class SymbolTableDecorator extends Visitor {
       gov.llnl.babel.symbols.Assertion a = (gov.llnl.babel.symbols.Assertion)
         n.accept(this, aType);
       try {
-        method.addAssertion(a);
+        method.addClauseAssertion(a);
       }
       catch (AssertionException ae) {
         d_msgs.addMsg(new UserMsg(UserMsg.ERROR, "unable to convert assertion",
@@ -679,7 +679,7 @@ public class SymbolTableDecorator extends Visitor {
       gov.llnl.babel.symbols.Assertion a = (gov.llnl.babel.symbols.Assertion)
         n.accept(this, aType);
       try {
-        method.addAssertion(a);
+        method.addClauseAssertion(a);
       }
       catch (AssertionException ae) {
         d_msgs.addMsg(new UserMsg(UserMsg.ERROR, "unable to convert assertion",
