@@ -1,6 +1,10 @@
-#include "decaf_Framework.hh"
-#include "gov_cca_ComponentID.hh"
-#include "gov_cca_ports_GoPort.hh"
+#include "decaf_Framework.hxx"
+#include "gov_cca_ComponentID.hxx"
+#include "gov_cca_ConnectionID.hxx"
+#include "gov_cca_ports_GoPort.hxx"
+#include "gov_cca_TypeMap.hxx"
+#include <string>
+using namespace std;
 
 int main() { 
   decaf::Framework fwk = decaf::Framework::_create();
@@ -9,7 +13,8 @@ int main() {
 
   fwk.connect( client, "HelloServer", server, "HelloServer" );
   gov::cca::Port p = fwk.lookupPort( client, "GoPort" );
-  gov::cca::ports::GoPort go = p;
-    
-  go.go();
+  gov::cca::ports::GoPort go = sidl::babel_cast< gov::cca::ports::GoPort > (p);
+  if (go._not_nil()) {
+    go.go();
+  }
 } 
