@@ -1,8 +1,8 @@
 c
 c     File:       HelloDriver.f
-c     Copyright:  (c) 2001 The Regents of the University of California
-c     Revision:   @(#) $Revision: 4434 $
-c     Date:       $Date: 2005-03-17 09:05:29 -0800 (Thu, 17 Mar 2005) $
+c     Copyright:  (c) 2001 Lawrence Livermore National Security, LLC
+c     Revision:   @(#) $Revision: 6171 $
+c     Date:       $Date: 2007-10-08 16:39:28 -0700 (Mon, 08 Oct 2007) $
 c     Description:Simple CCA Hello World F77 client
 c
 c
@@ -17,7 +17,7 @@ c
       integer *8 properties
       integer *8 connectionID
 
-      call decaf_Framework__create_f(decaf)
+      call decaf_Framework__create_f(decaf, ex)
 
       call decaf_Framework_createTypemap_f( decaf, properties, ex )
 
@@ -37,7 +37,7 @@ c
      &  client,
      &  ex)
 
-      call gov_cca_TypeMap_deleteRef_f(properties)
+      call gov_cca_TypeMap_deleteRef_f(properties, ex)
 
       call decaf_Framework_connect_f(
      &  decaf,
@@ -47,19 +47,20 @@ c
      &  'HelloServer',
      &  connectionID,
      &  ex )
-      call gov_cca_ConnectionID_deleteRef_f(connectionID)
+      call gov_cca_ConnectionID_deleteRef_f(connectionID, ex)
 
-      call decaf_Framework_lookupPort_f(decaf, client, 'GoPort', port)
-      call gov_cca_ports_GoPort__cast_f(port, go)
-      call gov_cca_ports_GoPort_go_f(go, retval)
-      call gov_cca_ports_GoPort_deleteRef_f(go)
+      call decaf_Framework_lookupPort_f(decaf, client, 'GoPort',
+     $     port, ex)
+      call gov_cca_ports_GoPort__cast_f(port, go, ex)
+      call gov_cca_ports_GoPort_go_f(go, retval, ex)
+      call gov_cca_ports_GoPort_deleteRef_f(go, ex)
+      call gov_cca_Port_deleteRef_f(port, ex)
 
       call decaf_Framework_destroyInstance_f(decaf, server, 0.0, ex)
       call decaf_Framework_destroyInstance_f(decaf, client, 0.0, ex)
 
-      call gov_cca_ComponentID_deleteRef_f(server)
-      call gov_cca_ComponentID_deleteRef_f(client)
-      call decaf_Framework_deleteRef_f(decaf)
+      call gov_cca_ComponentID_deleteRef_f(server, ex)
+      call gov_cca_ComponentID_deleteRef_f(client, ex)
+      call decaf_Framework_deleteRef_f(decaf, ex)
 
-      return
       end
