@@ -1,8 +1,8 @@
 /*
  * File:        arraytests.c
  * Copyright:   (c) 2001-2003 Lawrence Livermore National Security, LLC
- * Revision:    @(#) $Revision: 6171 $
- * Date:        $Date: 2007-10-08 16:39:28 -0700 (Mon, 08 Oct 2007) $
+ * Revision:    @(#) $Revision: 6479 $
+ * Date:        $Date: 2008-08-21 07:38:07 -0700 (Thu, 21 Aug 2008) $
  * Description: Simple test on the ArrayTest static methods
  *
  */
@@ -91,7 +91,7 @@ int main(int argc, char **argv) {
   sidl_BaseInterface exception = NULL;
   int part_no = 0;
   int magicNumber = 1;
-  synch_RegOut tracker = synch_RegOut__create(&exception);SIDL_REPORT(exception);
+  synch_RegOut tracker = synch_RegOut__create(&exception);
   
   struct sidl_bool__array     *barray;
   struct sidl_char__array     *carray;
@@ -105,6 +105,7 @@ int main(int argc, char **argv) {
   struct sidl__array          *garray;
 
   struct ArrayTest_ArrayOps__array *oarray;
+  SIDL_REPORT(exception);
   srand(time(NULL));
 
   synch_RegOut_setExpectations(tracker, 161,&exception);SIDL_REPORT(exception);
