@@ -1,8 +1,8 @@
 // 
 // File:        ArrayTestDriver.java
-// Copyright:   (c) 2001 The Regents of the University of California
-// Revision:    @(#) $Revision: 4434 $
-// Date:        $Date: 2005-03-17 09:05:29 -0800 (Thu, 17 Mar 2005) $
+// Copyright:   (c) 2001 Lawrence Livermore National Security, LLC
+// Revision:    @(#) $Revision: 6173 $
+// Date:        $Date: 2007-10-10 20:53:56 -0700 (Wed, 10 Oct 2007) $
 // Description: array regression test case for Java calling other languages
 // 
 
@@ -29,7 +29,7 @@ public class ArrayTestDriver {
   private final static int TEST_DIM2 = 16;
 
   private static int s_part = 0;
-  private static int s_result = synch.ResultType.PASS;
+  private static long s_result = synch.ResultType.PASS;
 
   private static synch.RegOut tracker; 
   /**
@@ -42,7 +42,7 @@ public class ArrayTestDriver {
       tracker.writeComment(test);
   }
   
-  private static void check(int expected, boolean pass, String test ) {
+  private static void check(long expected, boolean pass, String test ) {
     if (test != null)
       tracker.writeComment(test);
     
@@ -60,23 +60,9 @@ public class ArrayTestDriver {
       tracker.endPart(s_part, synch.ResultType.FAIL);
   }
 
-
-  /**
-   * The main test driver takes no command-line arguments and runs the
-   * regression tests.
-   */
-  public static void main(String args[]) {
-    try {
-
-      /*
-       * Begin the test
-       */
-      tracker = new synch.RegOut();
-      tracker.setExpectations(143);
-      
-      s_part   = 0;
-      s_result = synch.ResultType.PASS;
-
+  private static void tests(synch.RegOut tracker)
+    throws InterruptedException
+  {
       /*
        * Boolean array tests
        */
@@ -141,6 +127,8 @@ public class ArrayTestDriver {
               (ArrayTest.ArrayOps.checkBool(array) == true),
               "makeBool9: (ArrayTest.ArrayOps.checkBool(array) == true)");
 
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -186,6 +174,8 @@ public class ArrayTestDriver {
               (ArrayTest.ArrayOps.checkChar(array) == false),
               "makeChar: (ArrayTest.ArrayOps.checkChar(array) == false)");
 
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -229,6 +219,8 @@ public class ArrayTestDriver {
               (ArrayTest.ArrayOps.checkInt(array) == false),
               "makeInt: (ArrayTest.ArrayOps.checkInt(array) == false)");
 
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
             
@@ -275,6 +267,8 @@ public class ArrayTestDriver {
               (ArrayTest.ArrayOps.checkLong(array) == false),
               "makeLong: (ArrayTest.ArrayOps.checkLong(array) == false)");
 
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -320,6 +314,8 @@ public class ArrayTestDriver {
               (ArrayTest.ArrayOps.checkString(array) == false),
               "makeString: (ArrayTest.ArrayOps.checkString(array) == false)");
      
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -365,6 +361,8 @@ public class ArrayTestDriver {
               (ArrayTest.ArrayOps.checkDouble(array) == false),
               "makeDouble: (ArrayTest.ArrayOps.checkDouble(array) == false)");
 
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -411,6 +409,8 @@ public class ArrayTestDriver {
               (ArrayTest.ArrayOps.checkFloat(array) == false),
               "makeFloat: (ArrayTest.ArrayOps.checkFloat(array) == false)");
 
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -461,6 +461,8 @@ public class ArrayTestDriver {
               "makeFcomplex: " +
               "(ArrayTest.ArrayOps.checkFcomplex(array) == false)");
    
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -510,6 +512,8 @@ public class ArrayTestDriver {
               "makeDcomplex: " +
               "(ArrayTest.ArrayOps.checkDcomplex(array) == false)");
      
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -525,10 +529,11 @@ public class ArrayTestDriver {
         startTest(null);
         check(synch.ResultType.PASS, (array.length() == 10), "Correctly retrived length");
         array.set(5, new ArrayTest.ArrayOps());
-        ArrayTest.ArrayOps recieved = null;
-        recieved = array.get(5);
+        ArrayTest.ArrayOps received = null;
+        received = array.get(5);
+        ArrayTest.ArrayOps foo = array.get(4);
         startTest(null);
-        check(synch.ResultType.PASS, (recieved != null), "Successfully get an object from an Array");
+        check(synch.ResultType.PASS, (received != null), "Successfully get an object from an Array");
        
 
         /*
@@ -547,14 +552,16 @@ public class ArrayTestDriver {
 	  
         startTest(null);
         check(synch.ResultType.PASS, (sliced1._get(0) != null), "Sliced array element 0 is not null");
-        sidl.BaseClass testBase = (sidl.BaseClass) sidl.BaseClass._cast(recieved);
+        sidl.BaseClass testBase = (sidl.BaseClass) sidl.BaseClass._cast(received);
         ArrayTest.ArrayOps testA = (ArrayTest.ArrayOps) ArrayTest.ArrayOps._cast(testBase);
         startTest(null);
         check(synch.ResultType.PASS, (testBase != null), "Element cast to baseClass is not null");
         startTest(null);
         check(synch.ResultType.PASS, (testBase._get_ior() == testA._get_ior()), "Pre cast and post cast elements are equal");  
 
-
+        array = null; sliced = null; sliced1 = null; 
+        received = null; testA = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -574,6 +581,8 @@ public class ArrayTestDriver {
         startTest(null);
         check(synch.ResultType.PASS, (array.length(1) == TEST_DIM2), "2nd Dimension has the correct length");
         
+        array = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -592,6 +601,8 @@ public class ArrayTestDriver {
         check(synch.ResultType.PASS, (array.length(0) == TEST_DIM1), "1st Dimension has the correct length");
         startTest(null);
         check(synch.ResultType.PASS, (array.length(1) == TEST_DIM2), "2nd Dimension has the correct length");
+        array = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -605,6 +616,8 @@ public class ArrayTestDriver {
         check(synch.ResultType.PASS,
               (ArrayTest.ArrayOps.check2Float(array) == true),
               "create2Float: (ArrayTest.ArrayOps.check2Float(array) == true)");
+        array = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -619,6 +632,8 @@ public class ArrayTestDriver {
               (ArrayTest.ArrayOps.check2Dcomplex(array) == true),
               "create2Dcomplex:" + 
               "(ArrayTest.ArrayOps.check2Dcomplex(array) == true)");
+        array = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -633,6 +648,8 @@ public class ArrayTestDriver {
               (ArrayTest.ArrayOps.check2Fcomplex(array) == true),
               "create2Fcomplex: " +
               "(ArrayTest.ArrayOps.check2Fcomplex(array) == true)");
+        array = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -646,6 +663,8 @@ public class ArrayTestDriver {
               (ArrayTest.ArrayOps.check3Int(array) == true),
               "create3Int: " +
               "(ArrayTest.ArrayOps.check3Int(array) == true)");
+        array = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -659,6 +678,8 @@ public class ArrayTestDriver {
               (ArrayTest.ArrayOps.check4Int(array) == true),
               "create4Int: " +
               "(ArrayTest.ArrayOps.check4Int(array) == true)");
+        array = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -672,6 +693,8 @@ public class ArrayTestDriver {
               (ArrayTest.ArrayOps.check5Int(array) == true),
               "create5Int: " +
               "(ArrayTest.ArrayOps.check5Int(array) == true)");
+        array = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -685,6 +708,8 @@ public class ArrayTestDriver {
               (ArrayTest.ArrayOps.check6Int(array) == true),
               "create6Int: " +
               "(ArrayTest.ArrayOps.check6Int(array) == true)");
+        array = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -698,6 +723,8 @@ public class ArrayTestDriver {
               (ArrayTest.ArrayOps.check7Int(array) == true),
               "create7Int: " +
               "(ArrayTest.ArrayOps.check7Int(array) == true)");
+        array = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
 
@@ -717,6 +744,8 @@ public class ArrayTestDriver {
         check(synch.ResultType.PASS, 
               (ArrayTest.ArrayOps.checkBool(array) == true),
               "makeInOutBool: (ArrayTest.ArrayOps.checkBool(array) == true)"); 
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -735,6 +764,8 @@ public class ArrayTestDriver {
               (ArrayTest.ArrayOps.checkChar(array) == true),
               "makeInOutChar: (ArrayTest.ArrayOps.checkChar(array) == true)");
 	  
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -753,6 +784,8 @@ public class ArrayTestDriver {
               (ArrayTest.ArrayOps.checkInt(array) == true),
               "makeInOutInt: (ArrayTest.ArrayOps.checkInt(array) == true)");
 	
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -771,6 +804,8 @@ public class ArrayTestDriver {
               (ArrayTest.ArrayOps.checkLong(array) == true),
               "makeInOutLong: (ArrayTest.ArrayOps.checkLong(array) == true)");
 	  
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -790,6 +825,8 @@ public class ArrayTestDriver {
               "makeInOutString: " + 
               "(ArrayTest.ArrayOps.checkString(array) == true)");
 
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -809,6 +846,8 @@ public class ArrayTestDriver {
               "makeInOutDouble: " +
               "(ArrayTest.ArrayOps.checkDouble(array) == true)");
 
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -827,6 +866,8 @@ public class ArrayTestDriver {
               (ArrayTest.ArrayOps.checkFloat(array) == true),
               "makeInOutFloat: (ArrayTest.ArrayOps.checkFloat(array) == true)");
 
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -846,6 +887,8 @@ public class ArrayTestDriver {
               "makeInOutFcomplex: " +
               "(ArrayTest.ArrayOps.checkFcomplex(array) == true)");
 
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -865,6 +908,8 @@ public class ArrayTestDriver {
               "makeInOutDcomplex: " +
               "(ArrayTest.ArrayOps.checkDcomplex(array) == true)");
 
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -884,6 +929,8 @@ public class ArrayTestDriver {
               "makeInOut2Int: " +
               "(ArrayTest.ArrayOps.check2Int(array) == true)");
 
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -903,6 +950,8 @@ public class ArrayTestDriver {
               "makeInOut2Double: " +
               "(ArrayTest.ArrayOps.check2Double(array) == true)");
 	  
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -922,6 +971,8 @@ public class ArrayTestDriver {
               "makeInOut2Float: " +
               "(ArrayTest.ArrayOps.check2Float(array) == true)");
 
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -941,6 +992,8 @@ public class ArrayTestDriver {
               "makeInOut2Dcomplex: " +
               "(ArrayTest.ArrayOps.check2Dcomplex(array) == true)");
 
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -960,6 +1013,8 @@ public class ArrayTestDriver {
               "makeInOut2Fcomplex: " +
               "(ArrayTest.ArrayOps.check2Fcomplex(array) == true)");
 
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -979,6 +1034,8 @@ public class ArrayTestDriver {
               "makeInOut3Int: " +
               "(ArrayTest.ArrayOps.check3Int(array) == true)");
 	  
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -998,6 +1055,8 @@ public class ArrayTestDriver {
               "makeInOut4Int: " +
               "(ArrayTest.ArrayOps.check4Int(array) == true)");
 	  
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -1016,6 +1075,8 @@ public class ArrayTestDriver {
               "makeInOut5Int: " +
               "(ArrayTest.ArrayOps.check5Int(array) == true)");
 	  
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
       /*
        * inout null 6D integer array test
@@ -1033,6 +1094,8 @@ public class ArrayTestDriver {
               "makeInOut6Int: " +
               "(ArrayTest.ArrayOps.check6Int(array) == true)");
 	  
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       /*
@@ -1050,7 +1113,8 @@ public class ArrayTestDriver {
               (ArrayTest.ArrayOps.check7Int(array) == true),
               "makeInOut7Int: " +
               "(ArrayTest.ArrayOps.check7Int(array) == true)");
-	  
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       if (true) {
@@ -1118,7 +1182,10 @@ public class ArrayTestDriver {
         startTest(null);
         check(synch.ResultType.PASS, ArrayTest.ArrayOps.check2Int(iarray2) == true, 
               "Generic inout is correct" );
-
+        holdDim = null; holdTp = null; holdOut = null; holdInout = null;
+        garray = null; garrayret = null; garrayout = null; 
+        garrayinout = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       
@@ -1157,6 +1224,8 @@ public class ArrayTestDriver {
         check(synch.ResultType.PASS,
               (ArrayTest.ArrayOps.checkRarray1Int(array) == true),
               "rarrayInt: (ArrayTest.ArrayOps.checkRarray1Int(array) == true)");
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       if (true) {
@@ -1184,6 +1253,8 @@ public class ArrayTestDriver {
         check(synch.ResultType.PASS,
               (ArrayTest.ArrayOps.checkRarray3Int(array) == true),
               "rarray3Int: (ArrayTest.ArrayOps.checkRarray3Int(array) == true)");
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       if (true) {
@@ -1210,6 +1281,8 @@ public class ArrayTestDriver {
         check(synch.ResultType.PASS,
               (ArrayTest.ArrayOps.checkRarray7Int(array) == true),
               "rarray7Int: (ArrayTest.ArrayOps.checkRarray7Int(array) == true)");
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       if (true) {
@@ -1238,6 +1311,8 @@ public class ArrayTestDriver {
         check(synch.ResultType.PASS,
               (ArrayTest.ArrayOps.checkRarray1Double(array) == true),
               "rarray1Double: (ArrayTest.ArrayOps.checkRarray1Double(array) == true)");
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
 
       if (true) {
@@ -1266,6 +1341,8 @@ public class ArrayTestDriver {
         check(synch.ResultType.PASS,
               (ArrayTest.ArrayOps.checkRarray1Dcomplex(array) == true),
               "rarray1Dcomplex: (ArrayTest.ArrayOps.checkRarray1Dcomplex(array) == true)");
+        array = null; hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
       
       if (true) {
@@ -1293,14 +1370,61 @@ public class ArrayTestDriver {
               (ArrayTest.ArrayOps.checkMatrixMultiply(a,b,x) == true),
               "checkMatrixMultiply: (ArrayTest.ArrayOps.checkMatrixMultiply(a,b,x) == true)");
 
+        a = null; b = null; x = null; h = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
       }
       
 
+      /*
+       * 2D String array tests
+       */
+
+      if (true) {
+        sidl.String.Array2 array = ArrayTest.ArrayOps.create2String(12,15);
+        sidl.String.Array2.Holder hold = new sidl.String.Array2.Holder();
+        startTest(null);
+        check(synch.ResultType.PASS,
+              (array != null),
+              "create2String: (array != null)");
+        startTest(null);
+        check(synch.ResultType.PASS,
+              (ArrayTest.ArrayOps.check2String(array) == true),
+              "create2String: (ArrayTest.ArrayOps.checkString(array) == true)");
+        array = null;
+        hold = null;
+        System.gc(); Thread.sleep(5); // try to force garbage collector to run
+      }
+  }
+
+
+  /**
+   * The main test driver takes no command-line arguments and runs the
+   * regression tests.
+   */
+  public static void main(String args[]) {
+    try {
+
+      /*
+       * Begin the test
+       */
+      tracker = new synch.RegOut();
+      tracker.setExpectations(145);
+      
+      s_part   = 0;
+      s_result = synch.ResultType.PASS;
+
+      tests(tracker);
+      System.gc(); Thread.sleep(5); // try to force garbage collector to run
 
       /*
        * Output final test results
        */
       tracker.close();
+      tracker = null;
+      System.gc(); Thread.sleep(50); // try to force garbage collector to run
+      System.gc(); Thread.sleep(50); // try to force garbage collector to run
+      System.gc(); Thread.sleep(50); // try to force garbage collector to run
+
       //System.out.println("TEST_RESULT " + s_results[s_result]);
 
       Runtime.getRuntime().exit(0); /* workaround for Linux JVM 1.3.1 bug */
@@ -1308,9 +1432,11 @@ public class ArrayTestDriver {
        * Catch any unexpected exceptions and return a test failure
        */
     } catch (Throwable ex) {
+      tracker.forceFailure();
       tracker.close();
       System.out.println("TEST_RESULT FAIL");
       System.out.println(ex.toString());
+      ex.printStackTrace();
       System.exit(1);
     }
   }
