@@ -1,8 +1,8 @@
 // 
 // File:        ExceptionTestDriver.java
-// Copyright:   (c) 2001 The Regents of the University of California
-// Revision:    @(#) $Revision: 4434 $
-// Date:        $Date: 2005-03-17 09:05:29 -0800 (Thu, 17 Mar 2005) $
+// Copyright:   (c) 2001 Lawrence Livermore National Security, LLC
+// Revision:    @(#) $Revision: 6173 $
+// Date:        $Date: 2007-10-10 20:53:56 -0700 (Wed, 10 Oct 2007) $
 // Description: exception regression test case for Java calling other languages
 // 
 
@@ -12,7 +12,7 @@
 public class ExceptionTestDriver {
 
   private static int s_part = 0;
-  private static int s_result = synch.ResultType.PASS;
+  private static long s_result = synch.ResultType.PASS;
 
   private static synch.RegOut tracker; 
   /**
@@ -25,7 +25,7 @@ public class ExceptionTestDriver {
       tracker.writeComment(test);
   }
 
-  private static void check(int expected, boolean pass, String test ) {
+  private static void check(long expected, boolean pass, String test ) {
     if (test != null)
       tracker.writeComment(test);
 
@@ -43,6 +43,99 @@ public class ExceptionTestDriver {
       tracker.endPart(s_part, synch.ResultType.FAIL);
   }
 
+  private static void tests(synch.RegOut tracker)
+  {
+    /*
+     * No exception thrown
+     */
+    ExceptionTest.Fib fib = new ExceptionTest.Fib();
+    try {
+      fib.getFib(10, 25, 200, 0);
+      startTest(null);
+      check(synch.ResultType.PASS, 
+            true, 
+            "no exception thrown (none expected)");
+    } catch (java.lang.Exception ex) {
+      startTest(null);
+      check(synch.ResultType.PASS, 
+            false, 
+            "unexpected exception thrown");
+    }
+
+    /*
+     * Throw a NegativeValueException
+     */
+
+    try {
+      fib.getFib(-1, 10, 10, 0);
+      startTest(null);
+      check(synch.ResultType.PASS, 
+            false, 
+            "no exception (NegativeValueException expected)");
+    } catch (java.lang.Exception ex) {
+      
+      if (((sidl.BaseException)ex).isType("ExceptionTest.NegativeValueException")) {
+        startTest(null);
+        check(synch.ResultType.PASS, 
+              true, 
+              "NegativeValueException thrown (as expected)");
+      } else {
+        startTest(null);
+        check(synch.ResultType.PASS, 
+              false, 
+              "unexpected sidl exception thrown");
+      }
+    }
+
+    /*
+     * Throw a TooDeepException 
+     */
+
+    try {
+      fib.getFib(10, 1, 1000, 0);
+      startTest(null);
+      check(synch.ResultType.PASS, 
+            false, 
+            "no exception (TooDeepException expected)");
+    } catch (java.lang.Exception ex) {
+      if (((sidl.SIDLException)ex).isType("ExceptionTest.TooDeepException")) {
+        startTest(null);
+        check(synch.ResultType.PASS, 
+              true, 
+              "TooDeepException thrown (as expected)");
+      } else {
+        startTest(null);
+        check(synch.ResultType.PASS, 
+              false, 
+              "unexpected exception thrown");
+      }
+    }
+
+    /*
+     * Throw a TooBigException 
+     */
+
+    try {
+      fib.getFib(10, 1000, 1, 0);
+      startTest(null);
+      check(synch.ResultType.PASS, 
+            false, 
+            "no exception (TooBigException expected)");
+    } catch (java.lang.Exception ex) {
+      if (((sidl.SIDLException)ex).isType("ExceptionTest.TooBigException")) {
+        startTest(null);
+        check(synch.ResultType.PASS, 
+              true, 
+              "TooBigException thrown (as expected)");
+      } else {
+        startTest(null);
+        check(synch.ResultType.PASS, 
+              false, 
+              "unexpected exception thrown");
+      }
+    }
+  }
+
 
   /**
    * The main test driver takes no command-line arguments and runs the
@@ -58,100 +151,17 @@ public class ExceptionTestDriver {
       s_part   = 0;
       s_result = synch.ResultType.PASS;
 
-      /*
-       * No exception thrown
-       */
-
-      ExceptionTest.Fib fib = new ExceptionTest.Fib();
-      try {
-        fib.getFib(10, 25, 200, 0);
-        startTest(null);
-        check(synch.ResultType.PASS, 
-              true, 
-              "no exception thrown (none expected)");
-      } catch (java.lang.Exception ex) {
-        startTest(null);
-        check(synch.ResultType.PASS, 
-              false, 
-              "unexpected exception thrown");
-      }
-
-      /*
-       * Throw a NegativeValueException
-       */
-
-      try {
-        fib.getFib(-1, 10, 10, 0);
-        startTest(null);
-        check(synch.ResultType.PASS, 
-              false, 
-              "no exception (NegativeValueException expected)");
-      } catch (java.lang.Exception ex) {
-        if (((sidl.SIDLException)ex).isType("ExceptionTest.NegativeValueException")) {
-          startTest(null);
-          check(synch.ResultType.PASS, 
-                true, 
-                "NegativeValueException thrown (as expected)");
-        } else {
-          startTest(null);
-          check(synch.ResultType.PASS, 
-                false, 
-                "unexpected sidl exception thrown");
-        }
-      }
-
-      /*
-       * Throw a TooDeepException 
-       */
-
-      try {
-        fib.getFib(10, 1, 1000, 0);
-        startTest(null);
-        check(synch.ResultType.PASS, 
-              false, 
-              "no exception (TooDeepException expected)");
-      } catch (java.lang.Exception ex) {
-        if (((sidl.SIDLException)ex).isType("ExceptionTest.TooDeepException")) {
-          startTest(null);
-          check(synch.ResultType.PASS, 
-                true, 
-                "TooDeepException thrown (as expected)");
-        } else {
-          startTest(null);
-          check(synch.ResultType.PASS, 
-                false, 
-                "unexpected exception thrown");
-        }
-      }
-
-      /*
-       * Throw a TooBigException 
-       */
-
-      try {
-        fib.getFib(10, 1000, 1, 0);
-        startTest(null);
-        check(synch.ResultType.PASS, 
-              false, 
-              "no exception (TooBigException expected)");
-      } catch (java.lang.Exception ex) {
-        if (((sidl.SIDLException)ex).isType("ExceptionTest.TooBigException")) {
-          startTest(null);
-          check(synch.ResultType.PASS, 
-                true, 
-                "TooBigException thrown (as expected)");
-        } else {
-          startTest(null);
-          check(synch.ResultType.PASS, 
-                false, 
-                "unexpected exception thrown");
-        }
-      }
-
+      tests(tracker);
+      System.gc(); Thread.sleep(5); // try to force the garbage collector 
+      
       /*
        * Output final test results
        */
       tracker.close();
+      tracker = null;
+      System.gc(); Thread.sleep(50); // try to force the garbage collector 
+      System.gc(); Thread.sleep(50); // try to force the garbage collector 
+      System.gc(); Thread.sleep(50); // try to force the garbage collector 
       //System.out.println("TEST_RESULT " + s_results[s_result]);
 
       Runtime.getRuntime().exit(0); /* workaround for Linux JVM 1.3.1 bug */
