@@ -50,6 +50,7 @@ import getpass
 import poplib
 import string
 import StringIO
+import xml.sax
 
 #import gantlet.config
 from gantlet.session_email_parser import session_email_parser
@@ -111,6 +112,7 @@ class session_pop_client:
         removed from the POP server in the process.
 
         """
+        n_failed=0
         server = self.connect( )
         try:
             if self.debug > 1 : print server.list()
@@ -122,10 +124,15 @@ class session_pop_client:
                 (hdr, content, bytes) = server.retr(i)
                 content = string.join( content, '\n')            
                 strfile = StringIO.StringIO(content)  # make string look like file
-                msgs.append( session_email_parser( strfile, i, content ) )
+                try:
+                    msg =  session_email_parser( strfile, i, content )
+                    msgs.append(msg)
+                except xml.sax._exceptions.SAXParseException:
+                    print "Message %d had parse exception" %(i,)
+                    n_failed = n_failed + 1
         finally:
             server.quit()
-        assert len(msgs) == (n_msgs - loadfrom + 1)
+        assert len(msgs) == (n_msgs - loadfrom - n_failed + 1)
         return msgs 
 
     def del_msgs( self, msgs ):
@@ -205,21 +212,28 @@ def _commandline():
         passwd = getpass.getpass( "Password for %s@%s : " % (popuser, popserver) )
 
     # Now do the interesting stuff
-    mail_reader = session_pop_client( popserver, popuser, passwd )
+    mail_reader = session_pop_client( popserver, popuser, passwd, debug=3 )
     all_msgs = mail_reader.load_msgs( )
     (msgs, junk) = mail_reader.split_msgs( all_msgs )
     
-    if count:
+    if count or verbose :
         print "%d messages total:  %d gantlet, %d other" % (
+
             len( all_msgs), len( msgs ), len (junk ) )
 
     if verbose:
         for msg in msgs:
+            #print (msg.id, msg.attr)
+            if '/' in msg.attr['session']:
+                print '*********************'
+                msg.attr['session']='ask_epperly'
+            else:
+                print '**** NO RENAMING NECESSARY ****'
             print (msg.id, msg.attr)
 
     if repository and len(msgs):
         if verbose: print "Loading into repository ('%s')" % repository
-        mgr = session_archiver(repository , 1)
+        mgr = session_archiver(repository)
         for msg in msgs:
             mgr.insert( msg )
         mgr.store_index()
