@@ -1,8 +1,8 @@
 C     
 C File:        inherittest.f
 C Copyright:   (c) 2001 Lawrence Livermore National Security, LLC
-C Revision:    $Revision: 6183 $
-C Date:        $Date: 2007-10-15 14:41:51 -0700 (Mon, 15 Oct 2007) $
+C Revision:    $Revision: 6322 $
+C Date:        $Date: 2008-02-05 16:08:25 -0800 (Tue, 05 Feb 2008) $
 C Description: Regression test to test FORTRAN calls to BABEL
 C
       subroutine catch(exc)
@@ -98,7 +98,7 @@ C
       call synch_RegOut_getInstance_f(tracker, tae)
       partno = 0
 
-      call synch_RegOut_setExpectations_f(tracker, 71, tae)
+      call synch_RegOut_setExpectations_f(tracker, 73, tae)
       call catch(tae)
 
       call Inherit_C__create_f(object, tae)
@@ -217,6 +217,10 @@ C
       call Inherit_E2_deleteRef_f(object, tae)
       call catch(tae)
 
+      call Inherit_E2_m_f(strresult, tae)
+      call catch(tae)
+      call reporttest(partno, 'E2_m', 'E2.m', strresult)
+
       call Inherit_F__create_f(object, tae)
       call catch(tae)
 
@@ -579,6 +583,10 @@ C
       call Inherit_J_deleteRef_f(object, tae)
       call catch(tae)
 
+      call Inherit_J_m_f(strresult, tae)
+      call catch(tae)
+      call reporttest(partno, 'J_m', 'E2.m', strresult)
+
 
 c     begin K test
       call Inherit_K__create_f(object, tae)
