@@ -1,4 +1,8 @@
-from numpy import *
+import sidlPyArrays
+if sidlPyArrays.type == "numpy":
+    from numpy import *
+else:
+    from Numeric import *
 from Tkinter import *
 
 class arena( Canvas ):
