@@ -3,7 +3,11 @@
 import babelenv
 import sys
 import time
-from Numeric import *
+import sidlPyArrays
+if sidlPyArrays.type == "numpy":
+    from numpy import *
+else:
+    from Numeric import *
 from f90.Environment import Environment
 # Those of you with a supported F90 compiler
 # can uncomment this line
