@@ -3,11 +3,15 @@
 import babelenv
 import sys
 import time
-from Numeric import *
+import sidlPyArrays
+if sidlPyArrays.type == "numpy":
+    from numpy import zeros
+else:
+    from Numeric import zeros
 from cxx.Environment import Environment
 # Those of you with a supported F90 compiler
 # can uncomment this line
-# from f90.TimeStepper import TimeStepper
+#from f90.TimeStepper import TimeStepper
 # and comment out this one.
 from cxx.TimeStepper import TimeStepper
 from c.Ruleset import Ruleset
