@@ -2,15 +2,15 @@
  * File:        sidlObjA.c
  * Package:     sidl Python Object Adaptor
  * Copyright:   (c) 2001 Lawrence Livermore National Security, LLC
- * Revision:    @(#) $Revision: 6192 $
- * Date:        $Date: 2007-10-22 16:12:59 -0700 (Mon, 22 Oct 2007) $
+ * Revision:    @(#) $Revision: 6482 $
+ * Date:        $Date: 2008-08-21 15:50:53 -0700 (Thu, 21 Aug 2008) $
  * Description: Python extension type written in C for sidl object/interface
  *
  * This is a Python extension type written in C to wrap instances of
  * sidl objects or interfaces in a Python object.  If this looks
  * mysterious to you, look at Programming Python and the BABEL
  * documentation. 
- * Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+ * Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
  * Produced at the Lawrence Livermore National Laboratory.
  * Written by the Components Team <components@llnl.gov>
  * UCRL-CODE-2002-054
@@ -40,6 +40,7 @@
 #include "sidl_BaseClass_IOR.h"
 #include "sidl_BaseException.h"
 #include "sidl_String.h"
+#include "sidl_Python.h"
 
 #include <string.h>
 
@@ -61,7 +62,15 @@ sidl_Object_Init sidl_Object_Init_PROTO {
     sidlObject->d_ior = ior;
     if (refType == sidl_PyNewRef) {
       struct sidl_BaseInterface__object *throwaway_exception;
+#if (PY_VERSION_HEX >= 0x02040000)
+      Py_BEGIN_ALLOW_THREADS
+        sidl_Python_LogUnlock(__func__, __FILE__, __LINE__);
+#endif /* Python 2.4 or later */
       (*ior->d_epv->f_addRef)(ior, &throwaway_exception);
+#if (PY_VERSION_HEX >= 0x02040000)
+      Py_END_ALLOW_THREADS
+      sidl_Python_LogRelock(__func__, __FILE__, __LINE__);
+#endif /* Python 2.4 or later */
     }
     sidlObject->d_refType = refType;
     return 0;
@@ -85,8 +94,18 @@ static sidl_Cast_RETURN
 sidl_Cast sidl_Cast_PROTO {
   struct sidl_BaseInterface__object *ior = sidl_Get_IOR(obj);
   struct sidl_BaseInterface__object *throwaway_exception;
-  return (ior) ? ((*(ior->d_epv->f__cast))(ior->d_object, name, 
-                                           &throwaway_exception)) : NULL;
+  void *result;
+#if (PY_VERSION_HEX >= 0x02040000)
+  Py_BEGIN_ALLOW_THREADS
+  sidl_Python_LogUnlock(__func__, __FILE__, __LINE__);
+#endif /* Python 2.4 or later */
+  result = (ior) ? ((*(ior->d_epv->f__cast))(ior->d_object, name, 
+                                             &throwaway_exception)) : NULL;
+#if (PY_VERSION_HEX >= 0x02040000)
+  Py_END_ALLOW_THREADS
+  sidl_Python_LogRelock(__func__, __FILE__, __LINE__);
+#endif /* Python 2.4 or later */
+  return result;
 }
 
 static sidl_Opaque_Create_RETURN
@@ -120,7 +139,17 @@ sidl_PyType sidl_PyType_PROTO {
 static sidl_PyExceptionCast_RETURN
 sidl_PyExceptionCast sidl_PyExceptionCast_PROTO {
   struct sidl_BaseInterface__object *throwaway_exception;
-  return sidl_BaseInterface__cast2(ex, name, &throwaway_exception);
+  void *result;
+#if (PY_VERSION_HEX >= 0x02040000)
+  Py_BEGIN_ALLOW_THREADS
+  sidl_Python_LogUnlock(__func__, __FILE__, __LINE__);
+#endif /* Python 2.4 or later */
+  result = sidl_BaseInterface__cast2(ex, name, &throwaway_exception);
+#if (PY_VERSION_HEX >= 0x02040000)
+  Py_END_ALLOW_THREADS
+  sidl_Python_LogRelock(__func__, __FILE__, __LINE__);
+#endif /* Python 2.4 or later */
+  return result;
 }
 
 static void copyValueMsg(PyObject *exc, PyObject *exvalue, PyObject *extype)
@@ -141,14 +170,16 @@ static void copyValueMsg(PyObject *exc, PyObject *exvalue, PyObject *extype)
     free(msg);
     msg = next;
   }
-  PyObject *setNote = PyObject_CallMethod(exc, "setNote", "(s)", msg);
-  if (!setNote) {
-    if (PyErr_Occurred()) {
-      PyErr_Print();
+  {
+    PyObject *setNote = PyObject_CallMethod(exc, "setNote", "(s)", msg);
+    if (!setNote) {
+      if (PyErr_Occurred()) {
+	PyErr_Print();
+      }
     }
+    free(msg);
+    Py_XDECREF(setNote);
   }
-  free(msg);
-  Py_XDECREF(setNote);
   Py_DECREF(valuestr);
 }
 
@@ -279,7 +310,15 @@ spoa_self_destruct(struct sidlPythonObject *self)
   /* remove Python's reference to this sidl object */
   if (self->d_refType != sidl_PyWeakRef) {
     struct sidl_BaseInterface__object *throwaway_exception;
+#if (PY_VERSION_HEX >= 0x02040000)
+    Py_BEGIN_ALLOW_THREADS
+    sidl_Python_LogUnlock(__func__, __FILE__, __LINE__);
+#endif /* Python 2.4 or later */
     (*(ior->d_epv->f_deleteRef))(ior->d_object, &throwaway_exception);
+#if (PY_VERSION_HEX >= 0x02040000)
+    Py_END_ALLOW_THREADS
+    sidl_Python_LogRelock(__func__, __FILE__, __LINE__);
+#endif /* Python 2.4 or later */
   }
   self->d_ior = NULL;
   self->d_refType = sidl_PyWeakRef;
@@ -353,7 +392,10 @@ initsidlObjA(void)
 {
   PyObject *module, *dict, *c_api;
   static void *spoa_api[sidl_API_pointers];
-
+#if  defined(WITH_THREAD) && (PY_VERSION_HEX >= 0x02040000)
+  sidl_Python_LogControlling(__func__, __FILE__, __LINE__);
+  PyEval_InitThreads();
+#endif
   module = Py_InitModule("sidlObjA", spoa_methods);
   dict = PyModule_GetDict(module);
 
