@@ -2,11 +2,11 @@
  * File:        sidlObjA.h
  * Package:     sidl Python Object Adaptor
  * Copyright:   (c) 2001 Lawrence Livermore National Security, LLC
- * Revision:    @(#) $Revision: 6171 $
- * Date:        $Date: 2007-10-08 16:39:28 -0700 (Mon, 08 Oct 2007) $
+ * Revision:    @(#) $Revision: 6482 $
+ * Date:        $Date: 2008-08-21 15:50:53 -0700 (Thu, 21 Aug 2008) $
  * Description: A Python C extension type to wrap up sidl objects/interfaces
  *
- * Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+ * Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
  * Produced at the Lawrence Livermore National Laboratory.
  * Written by the Components Team <components@llnl.gov>
  * UCRL-CODE-2002-054
@@ -296,8 +296,9 @@ sidl_Object_Adaptor_API[sidl_AddTrace_NUM])
 #define import_SIDLObjA() \
 { \
   pthread_mutex_t __sidl_pyapi_mutex = PTHREAD_MUTEX_INITIALIZER; \
+  PyObject *module = NULL; \
   pthread_mutex_lock(&__sidl_pyapi_mutex); \
-  PyObject *module = PyImport_ImportModule("sidlObjA"); \
+  module = PyImport_ImportModule("sidlObjA"); \
   if (module != NULL) { \
     PyObject *module_dict = PyModule_GetDict(module); \
     PyObject *c_api_object = PyDict_GetItemString(module_dict, "_C_API"); \
