@@ -1,8 +1,8 @@
 /*
  * File:        sidlPyArrays.c
  * Copyright:   (c) 2001 Lawrence Livermore National Security, LLC
- * Revision:    @(#) $Revision: 6235 $
- * Date:        $Date: 2007-11-04 08:53:49 -0800 (Sun, 04 Nov 2007) $
+ * Revision:    @(#) $Revision: 6344 $
+ * Date:        $Date: 2008-03-19 14:31:01 -0700 (Wed, 19 Mar 2008) $
  * Description: Runtime support routines to convert sidl arrays to/from Python
  *
  * This file provides functions to convert sidl arrays to Python with or
@@ -51,6 +51,7 @@
 #include "sidl_string_IOR.h"
 #include "sidl_interface_IOR.h"
 #include "sidl_BaseInterface_IOR.h"
+#include "sidl_Python.h"
 
 /* include Numerical Python header file */
 #ifdef SIDL_HAVE_NUMPY
@@ -67,6 +68,18 @@
 #include <stdlib.h>
 #include <string.h>
 
+#if (PY_VERSION_HEX >= 0x02040000)
+#define SIDL_RECORD_BEGIN Py_BEGIN_ALLOW_THREADS sidl_Python_LogUnlock(__func__, __FILE__, __LINE__)
+#define SIDL_RECORD_END Py_END_ALLOW_THREADS sidl_Python_LogRelock(__func__, __FILE__, __LINE__)
+#define SIDL_RECORD_ENSURE _gstate = PyGILState_Ensure(); sidl_Python_LogGILEnsure(__func__, __FILE__, __LINE__, (int)_gstate)
+#define SIDL_RECORD_RELEASE PyGILState_Release(_gstate); sidl_Python_LogGILRelease(__func__, __FILE__, __LINE__, (int)_gstate)
+#else
+#define SIDL_RECORD_BEGIN
+#define SIDL_RECORD_END
+#define SIDL_RECORD_ENSURE
+#define SIDL_RECORD_RELEASE
+#endif
+
 static void
 addPythonArrayModule(PyObject *module)
 {
@@ -101,7 +114,10 @@ void sidl_python_## sidlType ##_destroy(struct sidl__array *array)      \
   if (array) {                                                          \
     struct sidl_python_## sidlType ##_array *parray =                   \
       (struct sidl_python_## sidlType ##_array *)array;                 \
+    PyGILState_STATE _gstate;                                           \
+    SIDL_RECORD_ENSURE;                                                 \
     Py_XDECREF(parray->d_numarray);                                     \
+    SIDL_RECORD_RELEASE;                                                \
     sidl__array_remove(array);                                          \
     free((void *)array);                                                \
   }                                                                     \
@@ -175,7 +191,11 @@ typedef struct {
 static void
 sao_dealloc(SIDLArrayObject* self)
 {
-  if (self->d_array) sidl__array_deleteRef(self->d_array);
+  if (self->d_array) {
+    SIDL_RECORD_BEGIN;
+    sidl__array_deleteRef(self->d_array);
+    SIDL_RECORD_END;
+  }
   self->ob_type->tp_free((PyObject*)self);
 }
 
@@ -351,6 +371,7 @@ static void copy_ ## name ## _data(char * /* restrict */ dest, \
 { \
   size_t num = arraySize(numelem, dim, sizeof(type)); \
   if (num > 0) { \
+    SIDL_RECORD_BEGIN; \
     if ((columnMajorOrder(sstride, numelem, dim, sizeof(type)) && \
          columnMajorOrder(dstride, numelem, dim, sizeof(type))) || \
         (rowMajorOrder(sstride, numelem, dim, sizeof(type)) && \
@@ -377,6 +398,7 @@ static void copy_ ## name ## _data(char * /* restrict */ dest, \
         } \
       } \
     } \
+    SIDL_RECORD_END; \
   } \
 }
 
@@ -1591,42 +1613,252 @@ sidl_python_copy sidl_python_copy_PROTO
 {
   const int32_t src_type = sidl__array_type(src);
   const int32_t dest_type = sidl__array_type(dest);
-  if (src_type == dest_type) {
+  const int32_t srcDimen = sidlArrayDim(src);
+  const int32_t destDimen = sidlArrayDim(dest);
+  if ((src != dest) && (src_type == dest_type) && (srcDimen == destDimen)) {
+    int i;
+    int32_t numElem[SIDL_MAX_ARRAY_DIMENSION];
+    int32_t newStart[SIDL_MAX_ARRAY_DIMENSION];
+    for(i = 0 ; i < srcDimen; ++i) {
+      newStart[i] = 0;
+      numElem[i] = ((sidlLength(src, i) < sidlLength(dest,i)) ?
+                    sidlLength(src, i) : sidlLength(dest, i));
+    }
     switch(src_type) {
     case sidl_bool_array:
-      sidl_bool__array_copy((const struct sidl_bool__array *)src, 
-                            (struct sidl_bool__array *)dest); break;
+      {
+        struct sidl_bool__array *psrc = 
+          sidl_bool__array_slice((struct sidl_bool__array *)src,
+                                 srcDimen, numElem, NULL, NULL,
+                                 newStart);
+        struct sidl_bool__array *pdest =
+          sidl_bool__array_slice((struct sidl_bool__array *)dest,
+                                destDimen, numElem, NULL, NULL,
+                                newStart);
+        sidl_bool__array_copy(psrc, pdest);
+        sidl_bool__array_deleteRef(psrc);
+        sidl_bool__array_deleteRef(pdest);
+                                 
+      }
+      break;
     case sidl_char_array:
-      sidl_char__array_copy((const struct sidl_char__array *)src,
-                            (struct sidl_char__array *)dest); break;
+      {
+        struct sidl_char__array *psrc = 
+          sidl_char__array_slice((struct sidl_char__array *)src,
+                                 srcDimen, numElem, NULL, NULL,
+                                 newStart);
+        struct sidl_char__array *pdest =
+          sidl_char__array_slice((struct sidl_char__array *)dest,
+                                destDimen, numElem, NULL, NULL,
+                                newStart);
+        sidl_char__array_copy(psrc, pdest);
+        sidl_char__array_deleteRef(psrc);
+        sidl_char__array_deleteRef(pdest);
+                                 
+      }
+      break;
     case sidl_dcomplex_array:
-      sidl_dcomplex__array_copy((const struct sidl_dcomplex__array *)src,
-                                (struct sidl_dcomplex__array *)dest); break;
+      {
+        struct sidl_dcomplex__array *psrc = 
+          sidl_dcomplex__array_slice((struct sidl_dcomplex__array *)src,
+                                 srcDimen, numElem, NULL, NULL,
+                                 newStart);
+        struct sidl_dcomplex__array *pdest =
+          sidl_dcomplex__array_slice((struct sidl_dcomplex__array *)dest,
+                                destDimen, numElem, NULL, NULL,
+                                newStart);
+        sidl_dcomplex__array_copy(psrc, pdest);
+        sidl_dcomplex__array_deleteRef(psrc);
+        sidl_dcomplex__array_deleteRef(pdest);
+                                 
+      }
+      break;
     case sidl_double_array:
-      sidl_double__array_copy((const struct sidl_double__array *)src,
-                              (struct sidl_double__array *)dest); break;
+      {
+        struct sidl_double__array *psrc = 
+          sidl_double__array_slice((struct sidl_double__array *)src,
+                                 srcDimen, numElem, NULL, NULL,
+                                 newStart);
+        struct sidl_double__array *pdest =
+          sidl_double__array_slice((struct sidl_double__array *)dest,
+                                destDimen, numElem, NULL, NULL,
+                                newStart);
+        sidl_double__array_copy(psrc, pdest);
+        sidl_double__array_deleteRef(psrc);
+        sidl_double__array_deleteRef(pdest);
+                                 
+      }
+      break;
     case sidl_fcomplex_array:
-      sidl_fcomplex__array_copy((const struct sidl_fcomplex__array *)src,
-                                (struct sidl_fcomplex__array *)dest); break;
+      {
+        struct sidl_fcomplex__array *psrc = 
+          sidl_fcomplex__array_slice((struct sidl_fcomplex__array *)src,
+                                 srcDimen, numElem, NULL, NULL,
+                                 newStart);
+        struct sidl_fcomplex__array *pdest =
+          sidl_fcomplex__array_slice((struct sidl_fcomplex__array *)dest,
+                                destDimen, numElem, NULL, NULL,
+                                newStart);
+        sidl_fcomplex__array_copy(psrc, pdest);
+        sidl_fcomplex__array_deleteRef(psrc);
+        sidl_fcomplex__array_deleteRef(pdest);
+                                 
+      }
+      break;
     case sidl_float_array:
-      sidl_float__array_copy((const struct sidl_float__array *)src,
-                             (struct sidl_float__array *)dest); break;
+      {
+        struct sidl_float__array *psrc = 
+          sidl_float__array_slice((struct sidl_float__array *)src,
+                                 srcDimen, numElem, NULL, NULL,
+                                 newStart);
+        struct sidl_float__array *pdest =
+          sidl_float__array_slice((struct sidl_float__array *)dest,
+                                destDimen, numElem, NULL, NULL,
+                                newStart);
+        sidl_float__array_copy(psrc, pdest);
+        sidl_float__array_deleteRef(psrc);
+        sidl_float__array_deleteRef(pdest);
+                                 
+      }
+      break;
     case sidl_int_array:
-      sidl_int__array_copy((const struct sidl_int__array *)src,
-                           (struct sidl_int__array *)dest); break;
+      {
+        struct sidl_int__array *psrc = 
+          sidl_int__array_slice((struct sidl_int__array *)src,
+                                 srcDimen, numElem, NULL, NULL,
+                                 newStart);
+        struct sidl_int__array *pdest =
+          sidl_int__array_slice((struct sidl_int__array *)dest,
+                                destDimen, numElem, NULL, NULL,
+                                newStart);
+        sidl_int__array_copy(psrc, pdest);
+        sidl_int__array_deleteRef(psrc);
+        sidl_int__array_deleteRef(pdest);
+                                 
+      }
+      break;
     case sidl_long_array:
-      sidl_long__array_copy((const struct sidl_long__array *)src,
-                            (struct sidl_long__array *)dest); break;
+      {
+        struct sidl_long__array *psrc = 
+          sidl_long__array_slice((struct sidl_long__array *)src,
+                                 srcDimen, numElem, NULL, NULL,
+                                 newStart);
+        struct sidl_long__array *pdest =
+          sidl_long__array_slice((struct sidl_long__array *)dest,
+                                destDimen, numElem, NULL, NULL,
+                                newStart);
+        sidl_long__array_copy(psrc, pdest);
+        sidl_long__array_deleteRef(psrc);
+        sidl_long__array_deleteRef(pdest);
+                                 
+      }
+      break;
     case sidl_opaque_array:
-      sidl_opaque__array_copy((const struct sidl_opaque__array *)src,
-                              (struct sidl_opaque__array *)dest); break;
+      {
+        struct sidl_opaque__array *psrc = 
+          sidl_opaque__array_slice((struct sidl_opaque__array *)src,
+                                 srcDimen, numElem, NULL, NULL,
+                                 newStart);
+        struct sidl_opaque__array *pdest =
+          sidl_opaque__array_slice((struct sidl_opaque__array *)dest,
+                                destDimen, numElem, NULL, NULL,
+                                newStart);
+        sidl_opaque__array_copy(psrc, pdest);
+        sidl_opaque__array_deleteRef(psrc);
+        sidl_opaque__array_deleteRef(pdest);
+                                 
+      }
+      break;
     case sidl_string_array:
-      sidl_string__array_copy((const struct sidl_string__array *)src,
-                              (struct sidl_string__array *)dest); break;
+      {
+        struct sidl_string__array *psrc = 
+          sidl_string__array_slice((struct sidl_string__array *)src,
+                                 srcDimen, numElem, NULL, NULL,
+                                 newStart);
+        struct sidl_string__array *pdest =
+          sidl_string__array_slice((struct sidl_string__array *)dest,
+                                destDimen, numElem, NULL, NULL,
+                                newStart);
+        sidl_string__array_copy(psrc, pdest);
+        sidl_string__array_deleteRef(psrc);
+        sidl_string__array_deleteRef(pdest);
+                                 
+      }
+      break;
     }
   }
 }
 
+sidl_python_ensure_RETURN
+sidl_python_ensure sidl_python_ensure_PROTO
+{
+  struct sidl__array *result = NULL;
+  if (src) {
+    const int order =(columnOrdering 
+                      ? sidl_column_major_order 
+                      : sidl_row_major_order);
+    switch(sidl__array_type(src)){
+    case sidl_bool_array:
+      result = (struct sidl__array*)
+        sidl_bool__array_ensure((struct sidl_bool__array*)src,
+                                sidlArrayDim(src),order);
+      break;
+    case sidl_char_array:
+      result = (struct sidl__array*)
+        sidl_char__array_ensure((struct sidl_char__array*)src,
+                                sidlArrayDim(src),order);
+      break;
+    case sidl_dcomplex_array:
+      result = (struct sidl__array*)
+        sidl_dcomplex__array_ensure((struct sidl_dcomplex__array*)src,
+                                    sidlArrayDim(src),order);
+      break;
+    case sidl_double_array:
+      result = (struct sidl__array*)
+        sidl_double__array_ensure((struct sidl_double__array*)src,
+                                  sidlArrayDim(src),order);
+      break;
+    case sidl_fcomplex_array:
+      result = (struct sidl__array*)
+        sidl_fcomplex__array_ensure((struct sidl_fcomplex__array*)src,
+                                    sidlArrayDim(src),order);
+      break;
+    case sidl_float_array:
+      result = (struct sidl__array*)
+        sidl_float__array_ensure((struct sidl_float__array*)src,
+                                 sidlArrayDim(src),order);
+      break;
+    case sidl_int_array:
+      result = (struct sidl__array*)
+        sidl_int__array_ensure((struct sidl_int__array*)src,
+                               sidlArrayDim(src),order);
+      break;
+    case sidl_long_array:
+      result = (struct sidl__array*)
+        sidl_long__array_ensure((struct sidl_long__array*)src,
+                                sidlArrayDim(src),order);
+      break;
+    case sidl_opaque_array:
+      result = (struct sidl__array*)
+        sidl_opaque__array_ensure((struct sidl_opaque__array*)src,
+                                  sidlArrayDim(src),order);
+      break;
+    case sidl_string_array:
+      result = (struct sidl__array*)
+        sidl_string__array_ensure((struct sidl_string__array*)src,
+                                  sidlArrayDim(src),order);
+      break;
+    case sidl_interface_array:
+      result = (struct sidl__array*)
+        sidl_interface__array_ensure((struct sidl_interface__array*)src,
+                                     sidlArrayDim(src),order);
+      break;
+    }
+    sidl__array_deleteRef(src);
+  }
+  return result;
+}
+
 sidl_array__convert_sidl_RETURN
 sidl_array__convert_sidl sidl_array__convert_sidl_PROTO
 {
@@ -2407,11 +2639,14 @@ sidl_createPy ## sidltype (PyObject *_ignored,  \
   PyObject *result = NULL; \
   PyObject *initialValue = NULL; \
   int columnOrder = 0, dimen; \
-  static const int32_t lower[SIDL_MAX_ARRAY_DIMENSION] = {1,1,1,1,1,1,1}; \
+  static const int32_t lower[SIDL_MAX_ARRAY_DIMENSION] = {0,0,0,0,0,0,0}; \
   int32_t extents[SIDL_MAX_ARRAY_DIMENSION]; \
   if (parseShapeAndOrder(_args, _kwdict, extents,  \
                          &dimen, &columnOrder, &initialValue)) { \
-    struct sidl_ ## sidltype ## __array *array =  \
+    struct sidl_ ## sidltype ## __array *array; \
+    int i; \
+    for(i = 0 ; i < dimen; ++i) --extents[i]; /* covert extents to upper */ \
+    array =  \
       (columnOrder ? sidl_ ## sidltype ##__array_createCol(dimen, lower, extents) \
        : sidl_ ## sidltype ## __array_createRow(dimen, lower, extents)); \
     if (initialValue && array) { \
@@ -2621,6 +2856,8 @@ initsidlPyArrays(void)
     (void *)sidl_generic_borrow_python_array;
   spa_api[sidl_python_copy_NUM] =
     (void *)sidl_python_copy;
+  spa_api[sidl_python_ensure_NUM] =
+    (void *)sidl_python_ensure;
   spa_api[sidl_bool__borrow_python_array_NUM] =
     (void *)sidl_bool__borrow_python_array;
   spa_api[sidl_bool__clone_python_array_column_NUM] =
