@@ -1,11 +1,11 @@
 /*
  * File:        sidlPyArrays.h
  * Copyright:   (c) 2001 Lawrence Livermore National Security, LLC
- * Revision:    @(#) $Revision: 6171 $
- * Date:        $Date: 2007-10-08 16:39:28 -0700 (Mon, 08 Oct 2007) $
+ * Revision:    @(#) $Revision: 6482 $
+ * Date:        $Date: 2008-08-21 15:50:53 -0700 (Thu, 21 Aug 2008) $
  * Description: Runtime support for routines to convert arrays to/from Python
  *
- * Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+ * Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
  * Produced at the Lawrence Livermore National Laboratory.
  * Written by the Components Team <components@llnl.gov>
  * UCRL-CODE-2002-054
@@ -64,10 +64,20 @@
  * PyObject *
  * sidl_python_clone_array(struct sidl__array *array);
  *
- * Copy from src to dest if they're the same type.
+ * Copy from src to dest if they're the same type.  This ignores the
+ * lower and upper bounds on the indices in each dimension and just copies
+ * based on the extents.
  * void
  * sidl_python_copy(struct sidl__array *src, struct sidl__array *dest);
  *
+ * Ensure the array has the required ordering.  The intended usage is:
+ *  foo = sidl_python_ensure(foo, FALSE);
+ * or
+ *  foo = sidl_python_ensure(foo, TRUE);
+ * If ensure must make a copy of src, it deleteRef's the incoming array.
+ * struct sidl__array *
+ * sidl_python_ensure(struct sidl__array *src, int columnOrdering);
+ *
  *
  * For each type, 
  *      bool
@@ -266,231 +276,236 @@ typedef int (*sidl_array_get_py_elem)(void *, const int32_t *, PyObject **);
 #define sidl_python_copy_NUM 5
 #define sidl_python_copy_RETURN void
 #define sidl_python_copy_PROTO \
-  (const struct sidl__array *src, struct sidl__array *dest)
+  (struct sidl__array *src, struct sidl__array *dest)
+
+#define sidl_python_ensure_NUM 6
+#define sidl_python_ensure_RETURN struct sidl__array *
+#define sidl_python_ensure_PROTO \
+  (struct sidl__array *src, int columnOrdering)
 
 
-#define sidl_bool__borrow_python_array_NUM 6
+#define sidl_bool__borrow_python_array_NUM 7
 #define sidl_bool__borrow_python_array_RETURN int
 #define sidl_bool__borrow_python_array_PROTO \
   (PyObject *obj, struct sidl_bool__array **array)
 
-#define sidl_bool__clone_python_array_column_NUM 7
+#define sidl_bool__clone_python_array_column_NUM 8
 #define sidl_bool__clone_python_array_column_RETURN int
 #define sidl_bool__clone_python_array_column_PROTO \
   (PyObject *obj, struct sidl_bool__array **array)
 
-#define sidl_bool__clone_python_array_row_NUM 8
+#define sidl_bool__clone_python_array_row_NUM 9
 #define sidl_bool__clone_python_array_row_RETURN int
 #define sidl_bool__clone_python_array_row_PROTO \
   (PyObject *obj, struct sidl_bool__array **array)
 
-#define sidl_bool__clone_python_array_NUM 9
+#define sidl_bool__clone_python_array_NUM 10
 #define sidl_bool__clone_python_array_RETURN int
 #define sidl_bool__clone_python_array_PROTO \
   (PyObject *obj, struct sidl_bool__array **array)
 
 
 
-#define sidl_char__borrow_python_array_NUM 10
+#define sidl_char__borrow_python_array_NUM 11
 #define sidl_char__borrow_python_array_RETURN int
 #define sidl_char__borrow_python_array_PROTO \
   (PyObject *obj, struct sidl_char__array **array)
 
-#define sidl_char__clone_python_array_column_NUM 11
+#define sidl_char__clone_python_array_column_NUM 12
 #define sidl_char__clone_python_array_column_RETURN int
 #define sidl_char__clone_python_array_column_PROTO \
   (PyObject *obj, struct sidl_char__array **array)
 
-#define sidl_char__clone_python_array_row_NUM 12
+#define sidl_char__clone_python_array_row_NUM 13
 #define sidl_char__clone_python_array_row_RETURN int
 #define sidl_char__clone_python_array_row_PROTO \
   (PyObject *obj, struct sidl_char__array **array)
 
-#define sidl_char__clone_python_array_NUM 13
+#define sidl_char__clone_python_array_NUM 14
 #define sidl_char__clone_python_array_RETURN int
 #define sidl_char__clone_python_array_PROTO \
   (PyObject *obj, struct sidl_char__array **array)
 
-#define sidl_dcomplex__borrow_python_array_NUM 14
+#define sidl_dcomplex__borrow_python_array_NUM 15
 #define sidl_dcomplex__borrow_python_array_RETURN int
 #define sidl_dcomplex__borrow_python_array_PROTO \
   (PyObject *obj, struct sidl_dcomplex__array **array)
 
-#define sidl_dcomplex__clone_python_array_column_NUM 15
+#define sidl_dcomplex__clone_python_array_column_NUM 16
 #define sidl_dcomplex__clone_python_array_column_RETURN int
 #define sidl_dcomplex__clone_python_array_column_PROTO \
   (PyObject *obj, struct sidl_dcomplex__array **array)
 
-#define sidl_dcomplex__clone_python_array_row_NUM 16
+#define sidl_dcomplex__clone_python_array_row_NUM 17
 #define sidl_dcomplex__clone_python_array_row_RETURN int
 #define sidl_dcomplex__clone_python_array_row_PROTO \
   (PyObject *obj, struct sidl_dcomplex__array **array)
 
-#define sidl_dcomplex__clone_python_array_NUM 17
+#define sidl_dcomplex__clone_python_array_NUM 18
 #define sidl_dcomplex__clone_python_array_RETURN int
 #define sidl_dcomplex__clone_python_array_PROTO \
   (PyObject *obj, struct sidl_dcomplex__array **array)
 
 
-#define sidl_double__borrow_python_array_NUM 18
+#define sidl_double__borrow_python_array_NUM 19
 #define sidl_double__borrow_python_array_RETURN int
 #define sidl_double__borrow_python_array_PROTO \
   (PyObject *obj, struct sidl_double__array **array)
 
-#define sidl_double__clone_python_array_column_NUM 19
+#define sidl_double__clone_python_array_column_NUM 20
 #define sidl_double__clone_python_array_column_RETURN int
 #define sidl_double__clone_python_array_column_PROTO \
   (PyObject *obj, struct sidl_double__array **array)
 
-#define sidl_double__clone_python_array_row_NUM 20
+#define sidl_double__clone_python_array_row_NUM 21
 #define sidl_double__clone_python_array_row_RETURN int
 #define sidl_double__clone_python_array_row_PROTO \
   (PyObject *obj, struct sidl_double__array **array)
 
-#define sidl_double__clone_python_array_NUM 21
+#define sidl_double__clone_python_array_NUM 22
 #define sidl_double__clone_python_array_RETURN int
 #define sidl_double__clone_python_array_PROTO \
   (PyObject *obj, struct sidl_double__array **array)
 
 
 
-#define sidl_fcomplex__borrow_python_array_NUM 22
+#define sidl_fcomplex__borrow_python_array_NUM 23
 #define sidl_fcomplex__borrow_python_array_RETURN int
 #define sidl_fcomplex__borrow_python_array_PROTO \
   (PyObject *obj, struct sidl_fcomplex__array **array)
 
-#define sidl_fcomplex__clone_python_array_column_NUM 23
+#define sidl_fcomplex__clone_python_array_column_NUM 24
 #define sidl_fcomplex__clone_python_array_column_RETURN int
 #define sidl_fcomplex__clone_python_array_column_PROTO \
   (PyObject *obj, struct sidl_fcomplex__array **array)
 
-#define sidl_fcomplex__clone_python_array_row_NUM 24
+#define sidl_fcomplex__clone_python_array_row_NUM 25
 #define sidl_fcomplex__clone_python_array_row_RETURN int
 #define sidl_fcomplex__clone_python_array_row_PROTO \
   (PyObject *obj, struct sidl_fcomplex__array **array)
 
-#define sidl_fcomplex__clone_python_array_NUM 25
+#define sidl_fcomplex__clone_python_array_NUM 26
 #define sidl_fcomplex__clone_python_array_RETURN int
 #define sidl_fcomplex__clone_python_array_PROTO \
   (PyObject *obj, struct sidl_fcomplex__array **array)
 
 
 
-#define sidl_float__borrow_python_array_NUM 26
+#define sidl_float__borrow_python_array_NUM 27
 #define sidl_float__borrow_python_array_RETURN int
 #define sidl_float__borrow_python_array_PROTO \
   (PyObject *obj, struct sidl_float__array **array)
 
-#define sidl_float__clone_python_array_column_NUM 27
+#define sidl_float__clone_python_array_column_NUM 28
 #define sidl_float__clone_python_array_column_RETURN int
 #define sidl_float__clone_python_array_column_PROTO \
   (PyObject *obj, struct sidl_float__array **array)
 
-#define sidl_float__clone_python_array_row_NUM 28
+#define sidl_float__clone_python_array_row_NUM 29
 #define sidl_float__clone_python_array_row_RETURN int
 #define sidl_float__clone_python_array_row_PROTO \
   (PyObject *obj, struct sidl_float__array **array)
 
-#define sidl_float__clone_python_array_NUM 29
+#define sidl_float__clone_python_array_NUM 30
 #define sidl_float__clone_python_array_RETURN int
 #define sidl_float__clone_python_array_PROTO \
   (PyObject *obj, struct sidl_float__array **array)
 
 
-#define sidl_int__borrow_python_array_NUM 30
+#define sidl_int__borrow_python_array_NUM 31
 #define sidl_int__borrow_python_array_RETURN int
 #define sidl_int__borrow_python_array_PROTO \
   (PyObject *obj, struct sidl_int__array **array)
 
-#define sidl_int__clone_python_array_column_NUM 31
+#define sidl_int__clone_python_array_column_NUM 32
 #define sidl_int__clone_python_array_column_RETURN int
 #define sidl_int__clone_python_array_column_PROTO \
   (PyObject *obj, struct sidl_int__array **array)
 
-#define sidl_int__clone_python_array_row_NUM 32
+#define sidl_int__clone_python_array_row_NUM 33
 #define sidl_int__clone_python_array_row_RETURN int
 #define sidl_int__clone_python_array_row_PROTO \
   (PyObject *obj, struct sidl_int__array **array)
 
-#define sidl_int__clone_python_array_NUM 33
+#define sidl_int__clone_python_array_NUM 34
 #define sidl_int__clone_python_array_RETURN int
 #define sidl_int__clone_python_array_PROTO \
   (PyObject *obj, struct sidl_int__array **array)
 
 
 
-#define sidl_long__borrow_python_array_NUM 34
+#define sidl_long__borrow_python_array_NUM 35
 #define sidl_long__borrow_python_array_RETURN int
 #define sidl_long__borrow_python_array_PROTO \
   (PyObject *obj, struct sidl_long__array **array)
 
-#define sidl_long__clone_python_array_column_NUM 35
+#define sidl_long__clone_python_array_column_NUM 36
 #define sidl_long__clone_python_array_column_RETURN int
 #define sidl_long__clone_python_array_column_PROTO \
   (PyObject *obj, struct sidl_long__array **array)
 
-#define sidl_long__clone_python_array_row_NUM 36
+#define sidl_long__clone_python_array_row_NUM 37
 #define sidl_long__clone_python_array_row_RETURN int
 #define sidl_long__clone_python_array_row_PROTO \
   (PyObject *obj, struct sidl_long__array **array)
 
-#define sidl_long__clone_python_array_NUM 37
+#define sidl_long__clone_python_array_NUM 38
 #define sidl_long__clone_python_array_RETURN int
 #define sidl_long__clone_python_array_PROTO \
   (PyObject *obj, struct sidl_long__array **array)
 
 
 
-#define sidl_opaque__borrow_python_array_NUM 38
+#define sidl_opaque__borrow_python_array_NUM 39
 #define sidl_opaque__borrow_python_array_RETURN int
 #define sidl_opaque__borrow_python_array_PROTO \
   (PyObject *obj, struct sidl_opaque__array **array)
 
-#define sidl_opaque__clone_python_array_column_NUM 39
+#define sidl_opaque__clone_python_array_column_NUM 40
 #define sidl_opaque__clone_python_array_column_RETURN int
 #define sidl_opaque__clone_python_array_column_PROTO \
   (PyObject *obj, struct sidl_opaque__array **array)
 
-#define sidl_opaque__clone_python_array_row_NUM 40
+#define sidl_opaque__clone_python_array_row_NUM 41
 #define sidl_opaque__clone_python_array_row_RETURN int
 #define sidl_opaque__clone_python_array_row_PROTO \
   (PyObject *obj, struct sidl_opaque__array **array)
 
-#define sidl_opaque__clone_python_array_NUM 41
+#define sidl_opaque__clone_python_array_NUM 42
 #define sidl_opaque__clone_python_array_RETURN int
 #define sidl_opaque__clone_python_array_PROTO \
   (PyObject *obj, struct sidl_opaque__array **array)
 
 
 
-#define sidl_string__borrow_python_array_NUM 42
+#define sidl_string__borrow_python_array_NUM 43
 #define sidl_string__borrow_python_array_RETURN int
 #define sidl_string__borrow_python_array_PROTO \
   (PyObject *obj, struct sidl_string__array **array)
 
-#define sidl_string__clone_python_array_column_NUM 43
+#define sidl_string__clone_python_array_column_NUM 44
 #define sidl_string__clone_python_array_column_RETURN int
 #define sidl_string__clone_python_array_column_PROTO \
   (PyObject *obj, struct sidl_string__array **array)
 
-#define sidl_string__clone_python_array_row_NUM 44
+#define sidl_string__clone_python_array_row_NUM 45
 #define sidl_string__clone_python_array_row_RETURN int
 #define sidl_string__clone_python_array_row_PROTO \
   (PyObject *obj, struct sidl_string__array **array)
 
-#define sidl_string__clone_python_array_NUM 45
+#define sidl_string__clone_python_array_NUM 46
 #define sidl_string__clone_python_array_RETURN int
 #define sidl_string__clone_python_array_PROTO \
   (PyObject *obj, struct sidl_string__array **array)
 
 
-#define sidl_array__convert_python_NUM 46
+#define sidl_array__convert_python_NUM 47
 #define sidl_array__convert_python_RETURN int
 #define sidl_array__convert_python_PROTO \
   (PyObject *pya_src, const int32_t dimen, void *sidl_dest, \
    sidl_array_set_py_elem setfunc)
 
-#define sidl_array__convert_sidl_NUM 47
+#define sidl_array__convert_sidl_NUM 48
 #define sidl_array__convert_sidl_RETURN int
 #define sidl_array__convert_sidl_PROTO \
   (PyObject *pya_dest, \
@@ -501,13 +516,13 @@ typedef int (*sidl_array_get_py_elem)(void *, const int32_t *, PyObject **);
    void *sidl_src, \
    sidl_array_get_py_elem getfunc)
 
-#define sidl_array__extract_python_info_NUM 48
+#define sidl_array__extract_python_info_NUM 49
 #define sidl_array__extract_python_info_RETURN int
 #define sidl_array__extract_python_info_PROTO \
   (PyObject *pya, int32_t *dimension, int32_t lower[], int32_t upper[],\
    int32_t stride[])
 
-#define sidlPyArrays_API_pointers 49
+#define sidlPyArrays_API_pointers 50
 
 
 #ifdef sidlPyArrays_MODULE
@@ -527,6 +542,8 @@ sidl_generic_borrow_python_array sidl_generic_borrow_python_array_PROTO;
 static sidl_python_copy_RETURN
 sidl_python_copy sidl_python_copy_PROTO;
 
+static sidl_python_ensure_RETURN
+sidl_python_ensure sidl_python_ensure_PROTO;
 
 static sidl_bool__borrow_python_array_RETURN
 sidl_bool__borrow_python_array sidl_bool__borrow_python_array_PROTO;
@@ -689,6 +706,11 @@ static void **sidlPyArrays_API;
    sidl_python_copy_PROTO) \
  sidlPyArrays_API[sidl_python_copy_NUM])
 
+#define sidl_python_ensure \
+(*( sidl_python_ensure_RETURN (*) \
+   sidl_python_ensure_PROTO) \
+ sidlPyArrays_API[sidl_python_ensure_NUM])
+
 #define sidl_bool__borrow_python_array  \
 (*( sidl_bool__borrow_python_array_RETURN (*) \
    sidl_bool__borrow_python_array_PROTO) \
@@ -909,8 +931,9 @@ static void **sidlPyArrays_API;
 #define import_SIDLPyArrays() \
 { \
   pthread_mutex_t __sidl_pyapi_mutex = PTHREAD_MUTEX_INITIALIZER; \
+  PyObject *module = NULL; \
   pthread_mutex_lock(&__sidl_pyapi_mutex); \
-  PyObject *module = PyImport_ImportModule("sidlPyArrays"); \
+  module = PyImport_ImportModule("sidlPyArrays"); \
   if (module != NULL) { \
     PyObject *module_dict = PyModule_GetDict(module); \
     PyObject *c_api_object = PyDict_GetItemString(module_dict, "_C_API"); \
