@@ -1,10 +1,10 @@
 /*
  * File:        sidlAsserts.h
- * Revision:    @(#) $Revision: 4682 $
- * Date:        $Date: 2005-06-02 09:04:38 -0700 (Thu, 02 Jun 2005) $
+ * Revision:    @(#) $Revision: 6482 $
+ * Date:        $Date: 2008-08-21 15:50:53 -0700 (Thu, 21 Aug 2008) $
  * Description: convenience C macros for managing SIDL Assertions
  *
- * Copyright (c) 2004, The Regents of the University of Calfornia.
+ * Copyright (c) 2004-2007, Lawrence Livermore National Security, LLC
  * Produced at the Lawrence Livermore National Laboratory.
  * Written by the Components Team <components@llnl.gov>
  * UCRL-CODE-2002-054
@@ -68,108 +68,8 @@
 #endif
 
 /****************************************************************************
- * SIDL Assertion Checking option, or level, masks
- *
- * There are three types of assertions supported for interfaces and five 
- * frequencies that can be combined with an optional adaptation.
- *
- * The assertion types are:
- *      preconditions,
- *      postconditions, and
- *      invariants.
- * The first set of masks are used to specify combinations of these checks.
- * You can "or" together the desired combination of these types OR simply
- * use one of the combinations given for your convenience.
- *
- * The five basic frequency, or sampling, options are:
- *      always       => always check the specified type(s) of assertions
- *      periodically => only check the assertions on a periodic basis
- *      proportional => check assertions as often as possibly such that
- *                      the amount of time spent checking adds no more than
- *                      a user-defined fraction of the time spent executing
- *                      the method dispatch.
- *                      (WARNING: This needs to be adjusted to factor in the
- *                      overhead of up to three gettimeofday() calls!)
- *      randomly     => check assertions on a random basis
- *
- * In all cases, the assertions are checked the first time the method
- * is invoked.  As long as assertions fail for a method, regardless of
- * frequency option, it's assertions will be checked upon invocation
- * until such time as the assertions pass.  
- *
- * NOTE that an alternative checking frequency is NEVER, which means that
- * assertions are never checked.  In fact, the annotated functions are 
- * completely by-passed.
- ****************************************************************************/
-
-/*
- * ... Basic assertion types to be checked
- */
-#ifndef CHECK_NO_TYPES
-#define CHECK_NO_TYPES       0
-#endif
-#ifndef CHECK_PRECONDITIONS
-#define CHECK_PRECONDITIONS  1
-#endif
-#ifndef CHECK_POSTCONDITIONS
-#define CHECK_POSTCONDITIONS 2
-#endif
-#ifndef CHECK_INVARIANTS
-#define CHECK_INVARIANTS     4
-#endif
-
-/*
- * ...... PLUS combinations of basic assertion types to be checked
- */
-#ifndef CHECK_PRE_POST_ONLY
-#define CHECK_PRE_POST_ONLY  3
-#endif
-#ifndef CHECK_PRE_INV_ONLY
-#define CHECK_PRE_INV_ONLY   5
-#endif
-#ifndef CHECK_POST_INV_ONLY
-#define CHECK_POST_INV_ONLY  6
-#endif
-#ifndef CHECK_ALL_TYPES
-#define CHECK_ALL_TYPES      7
-#endif
-
-/*
- * ... Adaptive checking enforcement
- */
-#ifndef CHECK_ADAPTIVELY
-#define CHECK_ADAPTIVELY     8
-#endif
- 
-/*
- * ... Basic checking frequency levels
- */
-
-#ifndef CHECK_ALWAYS
-#define CHECK_ALWAYS         16
-#endif
-#ifndef CHECK_PERIODICALLY
-#define CHECK_PERIODICALLY   32
-#endif
-#ifndef CHECK_TIMING
-#define CHECK_TIMING         64
-#endif
-#ifndef CHECK_RANDOMLY
-#define CHECK_RANDOMLY       128
-#endif
-
-/*
- * ...... PLUS a mask for determining if any FREQUENCY is specified (e.g.,
- *        (MY_CHECKING & CHECK_ASSERTIONS) will be TRUE if (at least) one of 
- *        the options is set in MY_CHECKING) -- though ONLY one should be!
- */
-#ifndef CHECK_ASSERTIONS
-#define CHECK_ASSERTIONS     240
-#endif
-
-
-/****************************************************************************
- * SIDL Assertion macros
+ * SIDL Assertion macros originally intended for use in generated interface
+ * contract enforcement routines.
  ****************************************************************************/
 
 /*
@@ -413,17 +313,17 @@
 
 /*
  * SIDL_ARRAY_NON_DECR	nonDecr(a), where a is array of numeric values
- *                      to be checked for being in increasing order.
+ *                              to be checked for being in increasing order.
  */
 #define SIDL_ARRAY_NON_DECR(AC, AV, I, NUM, V, BRES) {\
    BRES = TRUE;\
-   V    = ((AV) != NULL) ? (double) AC##_get1((AV),0) : 0.0;\
+   V    = ((AV) != NULL) ? (double) AC##_get1((AV), 0) : 0.0;\
    NUM  = SIDL_ARRAY_SIZE(AC, (AV));\
    for (I=0; (I<NUM) && (BRES); I++) {\
      if ((double)AC##_get1((AV),I) < V) {\
        BRES = FALSE; \
      } else {\
-       V = (double) AC##_get1((AV),0);\
+       V = (double) AC##_get1((AV), 0);\
      }\
    }\
 }
@@ -459,7 +359,7 @@
 
 
 /****************************************************************************
- * SIDL Assertion Enforcement Macros
+ * Additional macros
  ****************************************************************************/
 
 /*
@@ -480,62 +380,4 @@
  */
 #define SIDL_INCR_IF_TRUE(EXPR, V)  if (EXPR) { (V) += 1; } 
 
-/*
- *  SIDL_SET_COUNTDOWN		Assertion Checking Policy Enforcement
- *
- *  Assumption(s):
- *  1) Exceptions encountered during adaptive checking imply that assertions 
- *     should ALWAYS be checked on next invocation of the method regardless of 
- *     policy.
- */
-#define SIDL_SET_COUNTDOWN(LVL, R, CD, AOK, MOK, MT, TT) {\
-  if ((LVL) & CHECK_ADAPTIVELY) {\
-    if ((AOK) && (MOK)) {\
-      SIDL_SET_POLICY_CD(LVL, R, CD, MT, TT)\
-    } else {\
-      (CD) = 0;\
-    }\
-  } else {\
-    SIDL_SET_POLICY_CD(LVL, R, CD, MT, TT)\
-  }\
-}
-
-/*
- *  SIDL_SET_POLICY_CD  Set the countdown based only on the policy
- *
- *  Assumption(s):  N/A
- */
-#define SIDL_SET_POLICY_CD(LVL, R, CD, MT, TT) {\
-  if ((LVL) & CHECK_TIMING) {\
-    SIDL_SET_TIMING_CD(R, MT, TT, CD)\
-  } else if ((LVL) & CHECK_PERIODICALLY) {\
-    (CD) = (R);\
-  } else if ((LVL) & CHECK_RANDOMLY) {\
-    (CD) = ceil(((double)rand()/(RAND_MAX))*(R));\
-  } else { /* CHECK_ALWAYS */ \
-    (CD) = 0;\
-  }\
-}
-
-/*
- *  SIDL_SET_TIMING_CD	Set the proportional checking time countdown.
- *
- *  Assumption(s):
- *  1) In the highly unlikely event that the amount of time it takes to
- *     execute a method is under 1 microsecond, the TIMING checking policy
- *     assumes the method takes 1 microsecond.  (Note that this avoids a 
- *     divide-by-zero issue in the calculation of the countdown.)
- */
-#define SIDL_SET_TIMING_CD(R, MT, TT, CD) {\
-  if ( ((TT-MT) == 0) || ((CD) < 0) ) {\
-    (CD) = -1;\
-  } else {\
-    if ((MT) > 0) {\
-      (CD) = ceil((((double)((TT)-(MT)))/(double)((MT)?(MT):1))*(1.0/((R)?(R):1.0))) - 1;\
-    } else {\
-      (CD) = ceil(((double)(TT)-1.0)*(1.0/((R)?(R):1.0))) - 1;\
-    }\
-  }\
-}
-
 #endif /* included_sidlAsserts_h */
