@@ -1,8 +1,8 @@
 /*
  * File:        sidlMemory.c
- * Copyright:   (c) 2004 The Regents of the University of California
- * Revision:    @(#) $Revision: 4434 $
- * Date:        $Date: 2005-03-17 09:05:29 -0800 (Thu, 17 Mar 2005) $
+ * Copyright:   (c) 2004 Lawrence Livermore National Security, LLC
+ * Revision:    @(#) $Revision: 6171 $
+ * Date:        $Date: 2007-10-08 16:39:28 -0700 (Mon, 08 Oct 2007) $
  * Description: Replacement memory allocation functions
  *
  */
@@ -27,6 +27,13 @@ extern "C"
 #endif
 void *realloc(void *, size_t);
 
+#ifdef __cplusplus
+extern "C"
+#endif
+void free(void *);
+
+#include <stddef.h>
+
 void *rpl_realloc(void *ptr, size_t n)
 {
   if (ptr) {
