@@ -1,11 +1,11 @@
 /*
  * File:        sidl_Python.h
  * Copyright:   (c) 2001 Lawrence Livermore National Security, LLC
- * Revision:    @(#) $Revision: 6171 $
- * Date:        $Date: 2007-10-08 16:39:28 -0700 (Mon, 08 Oct 2007) $
+ * Revision:    @(#) $Revision: 6482 $
+ * Date:        $Date: 2008-08-21 15:50:53 -0700 (Thu, 21 Aug 2008) $
  * Description: API to initialize a Python language interpretter
  *
- * Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+ * Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
  * Produced at the Lawrence Livermore National Laboratory.
  * Written by the Components Team <components@llnl.gov>
  * UCRL-CODE-2002-054
@@ -51,6 +51,37 @@ void sidl_Python_Init(void);
 void sidl_Python_IncGlobalRef(void);
 
 void sidl_Python_DecGlobalRef(void);
+
+void sidl_Python_LogGILEnsure(const char *func,
+                                 const char *file,
+                                 const int  line,
+                                 const int state);
+
+void sidl_Python_LogGILRelease(const char *func,
+                                  const char *file,
+                                  const int   line,
+                                  const int   state);
+
+void sidl_Python_LogUnlock(const char *func,
+                           const char *file,
+                           const int   line);
+
+void sidl_Python_LogRelock(const char *func,
+                           const char *file,
+                           const int   line);
+
+void sidl_Python_LogControlling(const char *func,
+                                const char *file,
+                                const int line);
+
+#ifndef SIDL_PYTHON_THREAD_DEBUGGING
+#define sidl_Python_LogGILEnsure(i,j,k,l)
+#define sidl_Python_LogGILRelease(i,j,k,l)
+#define sidl_Python_LogUnlock(i,j,k)
+#define sidl_Python_LogRelock(i,j,k)
+#define sidl_Python_LogControlling(i,j,k)
+#endif
+                             
 #ifdef __cplusplus
 }
 #endif
