@@ -150,6 +150,22 @@ valuetype * fnname (struct hashtable *h, keytype *k) \
 }
 
 
+/*****************************************************************************
+ * hashtable_count
+   
+ * @name         hashtable_iterate
+ * @param   h    the hashtable
+ * @param   iter a function that will be called for each key, value
+ *               pair stored in the hash table.  The arguments
+ *               to iter are iter(key, value, data).
+ * @param   data this value is passed as the first argument to iter.
+ */
+void
+hashtable_iterate(struct hashtable *h,
+                  void (*iter)(void *, void *, void *),
+                  void *data);
+                  
+
 /*****************************************************************************
  * hashtable_count
    
