@@ -1,11 +1,11 @@
 /*
  * File:        sidl_header.h
  * Copyright:   (c) 2001 Lawrence Livermore National Security, LLC
- * Revision:    @(#) $Revision: 6171 $
- * Date:        $Date: 2007-10-08 16:39:28 -0700 (Mon, 08 Oct 2007) $
+ * Revision:    @(#) $Revision: 6482 $
+ * Date:        $Date: 2008-08-21 15:50:53 -0700 (Thu, 21 Aug 2008) $
  * Description: 
  *
- * Copyright (c) 2000-2001, The Regents of the University of Calfornia.
+ * Copyright (c) 2000-2001, Lawrence Livermore National Security, LLC
  * Produced at the Lawrence Livermore National Laboratory.
  * Written by the Components Team <components@llnl.gov>
  * UCRL-CODE-2002-054
@@ -37,24 +37,6 @@
 #include "sidlType.h"
 #endif
 
-#ifndef SIDL_INLINE_DECL
-#ifdef SIDL_C_HAS_INLINE
-#if (defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 199901L)) || defined(__INTEL_COMPILER) || !defined(__GNUC__)
-#define SIDL_C_INLINE_DECL inline
-#define SIDL_C_INLINE_DEFN extern
-#define SIDL_C_INLINE_REPEAT_DEFN 0
-#else
-#define SIDL_C_INLINE_DECL extern inline
-#define SIDL_C_INLINE_DEFN
-#define SIDL_C_INLINE_REPEAT_DEFN 1
-#endif
-#else /* !defined(SIDL_C_HAS_INLINE) */ 
-#define SIDL_C_INLINE_DECL
-#define SIDL_C_INLINE_DEFN
-#define SIDL_C_INLINE_REPEAT_DEFN 1
-#endif /* SIDL_C_HAS_INLINE */
-#endif /* SIDL_INLINE_DECL */
-
 #ifndef included_sidlArray_h
 #include "sidlArray.h"
 #endif
