@@ -5,7 +5,10 @@
 #include "sidlx_common.h"
 #include "sidl_String.h"
 
-#include "sidl_BaseInterface.h"
+#include "sidl_DLL.h"
+#include "sidl_Resolve.h"
+#include "sidl_Scope.h"
+#include "sidl_Loader.h"
 
 #include "sidl_rmi_NetworkException.h"
 #include "sidl_rmi_MalformedURLException.h"
@@ -26,9 +29,7 @@
 #include "sidlx_rmi_UnrecognizedNetworkException.h"
 #include "sidlx_rmi_RecoverableException.h"
 
-#if SIZEOF_LONG < 8
 #include <limits.h>
-#endif
 
 static struct sidlx_stats my_stats = {0,0,0,0,0,0,0,0,0,0};
 
@@ -36,6 +37,7 @@ struct sidlx_stats * get_sidlx_stats_struct(void) {
   return &my_stats;
 }
 
+
 void sidlx_throwException(const int errval, sidl_BaseInterface* _ex) {
   const char * const errmsg = strerror(errval);
   char *note = NULL;
@@ -100,7 +102,9 @@ void sidlx_throwException(const int errval, sidl_BaseInterface* _ex) {
       }
 
     case ETIMEDOUT:
+#ifdef ETIME
     case ETIME:
+#endif
       {
 	sidlx_rmi_TimeoutException tmpEx = 
 	  sidlx_rmi_TimeoutException__create(&_throwaway_exception);
@@ -187,7 +191,7 @@ void sidlx_throwException(const int errval, sidl_BaseInterface* _ex) {
       {
 	sidlx_rmi_UnrecognizedNetworkException tmpEx = 
 	  sidlx_rmi_UnrecognizedNetworkException__create(&_throwaway_exception);        note = sidl_String_concat2
-          ("Fatal Error, unexpected and unrecognized error: ", errmsg);
+                                                                                          ("Fatal Error, unexpected and unrecognized error: ", errmsg);
 	sidlx_rmi_UnrecognizedNetworkException_setNote(tmpEx, note, &_throwaway_exception);
 	sidlx_rmi_UnrecognizedNetworkException_setErrno(tmpEx, errval, &_throwaway_exception);
 	sidlx_rmi_UnrecognizedNetworkException_add(tmpEx, __FILE__, __LINE__, __FUNC__, &_throwaway_exception);
@@ -200,6 +204,70 @@ void sidlx_throwException(const int errval, sidl_BaseInterface* _ex) {
 }
 
 
+sidl_BaseClass sidlx_createClass(const char* objName, sidl_BaseInterface* _ex) {
+  sidl_BaseInterface _throwaway_exception = NULL; /*(TODO: a way to not throw these away? */
+  sidl_BaseClass baseClass;
+
+  void* (*_locCreateObject)(void*,sidl_BaseInterface*);
+  sidl_DLL dll = sidl_DLL__create(_ex); 
+
+  /* HMMM, would it be faster or better to do this directly rather than using sidl_DLL_createClass?*/
+
+  /* Holds the name for the object creation call. objName+__createObject */
+  /*  int objNameLength = strlen(objName);
+  char* createObjName = malloc( objNameLength + 24);
+  char* createObjName_ptr = createObjName;
+  int ii;
+
+  for(ii = 0; ii < objNameLength; ++ii) {
+    if(objName[ii] == '.') {
+      createObjName[ii] = '_';
+    } else {
+      createObjName[ii] = objName[ii];
+    }
+  }
+  memcpy(createObjName+ii, "__createObject", 15);
+  */
+  /* check global namespace for symbol first */
+  if (dll && sidl_DLL_loadLibrary(dll, "main:", TRUE, FALSE,&_throwaway_exception)) {
+    baseClass = sidl_DLL_createClass(dll, objName,_ex); SIDL_CHECK(*_ex); 
+
+      /*_locCreateObject =
+        (  void*(*)(void)) sidl_DLL_lookupSymbol(
+        dll, createObjName,_ex); SIDL_CHECK(*_ex);*/
+        /*_locCreateObject = (dll_f ? (*dll_f)() : NULL);*/
+  }
+  if (dll) sidl_DLL_deleteRef(dll,_ex); SIDL_CHECK(*_ex);
+  if (!baseClass) {
+    dll = sidl_Loader_findLibrary(objName,
+				  "ior/impl", sidl_Scope_SCLSCOPE,
+				  sidl_Resolve_SCLRESOLVE,_ex); SIDL_CHECK(*_ex);
+    if (dll) {
+      baseClass = sidl_DLL_createClass(dll, objName,_ex); SIDL_CHECK(*_ex); 
+      /*_locCreateObject =
+        (  void*(*)(void)) sidl_DLL_lookupSymbol( 
+        dll, createObjName,_ex); SIDL_CHECK(*_ex);*/
+      /*      _locCreateObject = (dll_f ? (*dll_f)() : NULL);*/
+      sidl_DLL_deleteRef(dll,_ex); SIDL_CHECK(*_ex);
+    }
+  }
+  if (!baseClass) {
+    char ex_msg[1024];      
+    sprintf(ex_msg, 
+            "sidlx_createObject: Unable to load DLL for class %s. check SIDL_DLL_PATH.", 
+            objName);
+    SIDL_THROW(*_ex, sidl_rmi_ObjectDoesNotExistException, 
+               ex_msg);
+  }
+  
+  /*create class and return sidl_BaseClass */
+  /*  baseClass = (*_locCreateObject)(NULL,_ex); SIDL_CHECK(*_ex);*/
+  return baseClass;
+  
+ EXIT:
+  return NULL;
+  
+}
 
 
 int s_socket( int family, int type, int protocol, sidl_BaseInterface *_ex) { 
@@ -893,4 +961,21 @@ void sidlx_parseURL(const char* url, char** protocol, char** server,
 }
 
 
+/* This function converts an int to a dot format IP address.
+   params[in] int32_t   : iAddress (host format)
+   params[out] char*    : sAddress, but point to a buffer of at least 16 char.
+*/
+
+void int2ip(int32_t iAddress, char* sAddress) {
+
+  int32_t nAddress = htonl(iAddress);
+  sprintf(sAddress, "%u.%u.%u.%u",
+          (0x000000FF & nAddress),
+          0x000000FF & (nAddress >> 8),
+          0x000000FF & (nAddress >> 16),
+          0x000000FF & (nAddress >> 24));
+  
+}
+   
+
 
